@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: data_out_base.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: data_out_base.h 21203 2010-06-11 19:48:01Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -82,7 +82,7 @@ class ParameterHandler;
  * concept, the following two sections are kept from a previous
  * version of this documentation.
  *
- * 
+ *
  * <h4>Patches</h4>
  *
  * Grids can be thought of as a collection of cells; if you want to write out
@@ -164,7 +164,7 @@ class ParameterHandler;
  * entirely free applications from knowledge which formats the library
  * presently allows to output; several of the example programs show how to do
  * this.
- * 
+ *
  * <h3>Output parameters</h3>
  *
  * All functions take a parameter which is a structure of type
@@ -214,7 +214,7 @@ class ParameterHandler;
  * @ingroup output
  * @author Wolfgang Bangerth, Guido Kanschat 1999, 2000, 2001, 2002, 2005, 2006.
  */
-class DataOutBase 
+class DataOutBase
 {
   public:
 				     /**
@@ -258,7 +258,7 @@ class DataOutBase
 					  * parameter available.
 					  */
 	static const unsigned int space_dim=spacedim;
-	
+
 					 /**
 					  * Corner points of a patch.
 					  * Inner points are computed by
@@ -293,7 +293,7 @@ class DataOutBase
 					  * we better store this.
 					  */
 	unsigned int patch_index;
-	
+
 					 /**
 					  * Number of subdivisions with
 					  * which this patch is to be
@@ -303,7 +303,7 @@ class DataOutBase
 					  * etc.
 					  */
 	unsigned int n_subdivisions;
-	
+
 					 /**
 					  * Data vectors. The format is
 					  * as follows:
@@ -366,7 +366,7 @@ class DataOutBase
 					  * output stream.
 					  */
 	bool points_are_available;
-	
+
 					 /**
 					  * Default constructor. Sets
 					  * <tt>n_subdivisions</tt> to one.
@@ -381,7 +381,7 @@ class DataOutBase
 					  * our testsuite.
 					  */
 	bool operator == (const Patch &patch) const;
-	
+
 					 /**
 					  * Determine an estimate for
 					  * the memory consumption (in
@@ -399,7 +399,7 @@ class DataOutBase
 					  * value.
 					  */
 	unsigned int memory_consumption () const;
-	
+
 					 /**
 					  * Value to be used if this
 					  * patch has no neighbor on
@@ -408,7 +408,7 @@ class DataOutBase
 	static const unsigned int no_neighbor = numbers::invalid_unsigned_int;
 					 /** @addtogroup Exceptions
 					  * @{ */
-	
+
 					 /**
 					  * Exception
 					  */
@@ -426,7 +426,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct DXFlags 
+    struct DXFlags
     {
 					 /**
 					  * Write neighbor
@@ -451,20 +451,20 @@ class DataOutBase
 					  * binary format.
 					  */
 	bool coordinates_binary;
-	
+
 					 /**
 					  * Write data vectors in
 					  * binary format.
 					  */
 	bool data_binary;
-	
+
 					 /**
 					  * Write binary coordinate
 					  * vectors as double (64 bit)
 					  * numbers instead of float (32 bit).
 					  */
 	bool data_double;
-	
+
 					 /**
 					  * Constructor.
 					  */
@@ -472,7 +472,7 @@ class DataOutBase
 		 const bool int_binary = false,
 		 const bool coordinates_binary = false,
 		 const bool data_binary = false);
-        
+
 					 /**
 					  * Declare all flags with name
 					  * and type as offered by this
@@ -507,7 +507,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct UcdFlags 
+    struct UcdFlags
     {
 					 /**
 					  * Write a comment at the
@@ -528,7 +528,7 @@ class DataOutBase
 					  * Default: <code>false</code>.
 					  */
 	bool write_preamble;
-	
+
 					 /**
 					  * Constructor.
 					  */
@@ -578,7 +578,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct GnuplotFlags 
+    struct GnuplotFlags
     {
       private:
 					 /**
@@ -592,6 +592,11 @@ class DataOutBase
 	int dummy;
 
       public:
+					 /**
+					  * Constructor.
+					  */
+	GnuplotFlags ();
+
 					 /**
 					  * Declare all flags with name
 					  * and type as offered by this
@@ -608,7 +613,7 @@ class DataOutBase
 					  * The flags thus obtained overwrite
 					  * all previous contents of this object.
 					  */
-	void parse_parameters (const ParameterHandler &prm);
+	void parse_parameters (const ParameterHandler &prm) const;
 
 					 /**
 					  * Determine an estimate for
@@ -638,7 +643,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct PovrayFlags 
+    struct PovrayFlags
     {
 					 /**
 					  * Normal vector interpolation,
@@ -647,7 +652,7 @@ class DataOutBase
 					  * default = false
 					  */
 	bool smooth;
-	
+
 					 /**
 					  * Use bicubic patches (b-splines)
 					  * instead of triangles.
@@ -665,14 +670,14 @@ class DataOutBase
 					  * default = false
 					  */
 	bool external_data;
-	
+
 					 /**
 					  * Constructor.
 					  */
 	PovrayFlags (const bool smooth = false,
 		     const bool bicubic_patch = false,
 		     const bool external_data = false);
-	
+
 					 /**
 					  * Declare all flags with name
 					  * and type as offered by this
@@ -718,7 +723,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct EpsFlags 
+    struct EpsFlags
     {
 					 /**
 					  * This denotes the number of the
@@ -740,7 +745,7 @@ class DataOutBase
 					  * <tt>height_vector</tt>.
 					  */
 	unsigned int color_vector;
-	
+
 					 /**
 					  * Enum denoting the possibilities
 					  * whether the scaling should be done
@@ -759,7 +764,7 @@ class DataOutBase
 					  * See above. Default is <tt>width</tt>.
 					  */
 	SizeType size_type;
-	
+
 					 /**
 					  * Width or height of the output
 					  * as given in postscript units
@@ -779,7 +784,7 @@ class DataOutBase
 					  * units. Default is 0.5.
 					  */
 	double line_width;
-	
+
 					 /**
 					  * Angle of the line origin-viewer
 					  * against the z-axis in degrees.
@@ -806,11 +811,11 @@ class DataOutBase
 					  * the following:
 					  *
 					  * @verbatim
-					  *  
+					  *
 					  *          3________7
 					  *          /       /|
 					  *         /       / |
-					  *       2/______6/  |   
+					  *       2/______6/  |
 					  *       |   |   |   |
 					  * O-->  |   0___|___4
 					  *       |  /    |  /
@@ -964,7 +969,7 @@ class DataOutBase
         grey_scale_color_function (const double value,
                                    const double min_value,
                                    const double max_value);
-	
+
 					 /**
 					  * This is one more
 					  * alternative color function
@@ -983,7 +988,7 @@ class DataOutBase
         reverse_grey_scale_color_function (const double value,
                                            const double min_value,
                                            const double max_value);
-	
+
 					 /**
 					  * Constructor.
 					  */
@@ -1049,7 +1054,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct GmvFlags 
+    struct GmvFlags
     {
       private:
 					 /**
@@ -1067,7 +1072,7 @@ class DataOutBase
 					  * Default constructor.
 					  */
 	GmvFlags ();
-	
+
 					 /**
 					  * Declare all flags with name
 					  * and type as offered by this
@@ -1084,7 +1089,7 @@ class DataOutBase
 					  * The flags thus obtained overwrite
 					  * all previous contents of this object.
 					  */
-	void parse_parameters (const ParameterHandler &prm);
+	void parse_parameters (const ParameterHandler &prm) const;
 
 					 /**
 					  * Determine an estimate for
@@ -1112,7 +1117,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct TecplotFlags 
+    struct TecplotFlags
     {
 
       public:
@@ -1134,13 +1139,13 @@ class DataOutBase
 					  * variable stores this name.
 					  */
 	const char* zone_name;
-	
+
 	                                 /**
 	                                  * Constructor
 	                                  **/
 	TecplotFlags (const char* tecplot_binary_file_name = NULL,
 		      const char* zone_name = NULL);
-      
+
 					 /**
 					  * Declare all flags with name
 					  * and type as offered by this
@@ -1157,7 +1162,7 @@ class DataOutBase
 					  * The flags thus obtained overwrite
 					  * all previous contents of this object.
 					  */
-	void parse_parameters (const ParameterHandler &prm);
+	void parse_parameters (const ParameterHandler &prm) const;
 
 					 /**
 					  * Determine an estimate for
@@ -1186,7 +1191,7 @@ class DataOutBase
 				      *
 				      * @ingroup output
 				      */
-    struct VtkFlags 
+    struct VtkFlags
     {
       private:
 					 /**
@@ -1221,7 +1226,7 @@ class DataOutBase
 					  * The flags thus obtained overwrite
 					  * all previous contents of this object.
 					  */
-	void parse_parameters (const ParameterHandler &prm);
+	void parse_parameters (const ParameterHandler &prm) const;
 
 					 /**
 					  * Determine an estimate for
@@ -1241,9 +1246,6 @@ class DataOutBase
 					  */
 	unsigned int memory_consumption () const;
     };
-
-
-
     				     /**
 				      * Flags controlling the details
 				      * of output in deal.II
@@ -1281,6 +1283,11 @@ class DataOutBase
 	int dummy;
 
       public:
+					 /**
+					  * Constructor.
+					  */
+	Deal_II_IntermediateFlags ();
+
 					 /**
 					  * Declare all flags with name
 					  * and type as offered by this
@@ -1297,7 +1304,7 @@ class DataOutBase
 					  * The flags thus obtained overwrite
 					  * all previous contents of this object.
 					  */
-	void parse_parameters (const ParameterHandler &prm);
+	void parse_parameters (const ParameterHandler &prm) const;
 
 					 /**
 					  * Determine an estimate for
@@ -1317,7 +1324,7 @@ class DataOutBase
 					  */
 	unsigned int memory_consumption () const;
     };
-    
+
 				     /**
 				      * Provide a data type specifying
 				      * the presently supported output
@@ -1368,7 +1375,7 @@ class DataOutBase
 					    * SoftwareTecplot in text
 					    * format.
 					    */
-	  
+
 	  tecplot,
 					   /**
 					    * Output for @ref
@@ -1378,13 +1385,19 @@ class DataOutBase
 					    * format.
 					    */
 	  tecplot_binary,
-          
+
 					   /**
 					    * Output in @ref
 					    * SoftwareVTK format.
 					    */
 	  vtk,
-          
+
+					   /**
+					    * Output in @ref
+					    * SoftwareVTK format.
+					    */
+	  vtu,
+
 					   /**
 					    * Output in deal.II
 					    * intermediate format.
@@ -1392,7 +1405,7 @@ class DataOutBase
 	  deal_II_intermediate
     };
 
-    
+
 /**
  * Write the given list of patches to the output stream in @ref
  * SoftwareOpenDX format.
@@ -1561,22 +1574,22 @@ class DataOutBase
  * A <tt>bicubic_patch</tt> is a 3-dimensional Bezier patch. It consists of 16 Points
  * describing the surface. The 4 corner points are touched by the object,
  * while the other 12 points pull and stretch the patch into shape.
- * One <tt>bicubic_patch</tt> is generated on each patch. Therefor the number of 
+ * One <tt>bicubic_patch</tt> is generated on each patch. Therefor the number of
  * subdivisions has to be 3 to provide the patch with 16 points.
  * A bicubic patch is not exact but generates very smooth images.
  *
  * <li> <tt>MESH</tt>
  * The mesh object is used to store large number of triangles.
- * Every square of the patch data is split into one upper-left and one 
+ * Every square of the patch data is split into one upper-left and one
  * lower-right triangle. If the number of subdivisions is three, 32 triangle
  * are generated for every patch.
- * 
+ *
  * Using the smooth flag povray interpolates the normals on the triangles,
  * imitating a curved surface
  * </ul>
  *
  * All objects get one texture definition called Tex. This texture has to be
- * declared somewhere before the object data. This may be in an external 
+ * declared somewhere before the object data. This may be in an external
  * data file or at the beginning of the output file.
  * Setting the <tt>external_data</tt> flag to false, an standard camera, light and
  * texture (scaled to fit the scene) is added to the outputfile. Set to true
@@ -1650,7 +1663,7 @@ class DataOutBase
  * several subdivisions of each cell. These subcells will then,
  * however, also appear as different cells by programs which
  * understand the UCD format.
- * 
+ *
  * No use is made of the possibility to give model data since these
  * are not supported by all UCD aware programs. You may give cell data
  * in derived classes by setting all values of a given data set on a
@@ -1664,11 +1677,9 @@ class DataOutBase
 			   std::ostream                            &out);
 
 /**
- * Write the given list of patches to the output stream in @ref
- * SoftwareVTK format.
- *
- * This is the file format used by @ref SoftwareVTK, as described in
- * their manual, section 14.3.
+ * Write the given list of patches to the output stream in @ref SoftwareVTK
+ * format. The data is written in the traditional VTK format as opposed to the
+ * XML-based format that write_vtu() produces.
  *
  * The vector_data_ranges argument denotes ranges of components in the
  * output that are considered a vector, rather than simply a
@@ -1684,6 +1695,31 @@ class DataOutBase
 			   const VtkFlags                          &flags,
 			   std::ostream                            &out);
 
+
+/**
+ * Write the given list of patches to the output stream in @ref SoftwareVTK
+ * format. The data is written in the XML-based VTK format as opposed to the
+ * traditional format that write_vtk() produces.
+ *
+ * The vector_data_ranges argument denotes ranges of components in the
+ * output that are considered a vector, rather than simply a
+ * collection of scalar fields. The VTK output format has special
+ * provisions that allow these components to be output by a single
+ * name rather than having to group several scalar fields into a
+ * vector later on in the visualization program.
+ *
+ * Some visualization programs, such as ParaView, can read several separate
+ * VTU files to parallelize visualization. In that case, you need a
+ * <code>.pvtu</code> file that describes which VTU files form a group. The
+ * DataOutInterface::write_pvtu_record() function can generate such a master record.
+ */
+    template <int dim, int spacedim>
+    static void write_vtu (const std::vector<Patch<dim,spacedim> > &patches,
+			   const std::vector<std::string>          &data_names,
+			   const std::vector<std_cxx1x::tuple<unsigned int, unsigned int, std::string> > &vector_data_ranges,
+			   const VtkFlags                          &flags,
+			   std::ostream                            &out);
+
 /**
  * Write the given list of patches to the output stream in deal.II
  * intermediate format. This is not a format understood by any other
@@ -1712,7 +1748,7 @@ class DataOutBase
  * generate a single output file from it, which may be again in
  * intermediate format or any of the final formats. This latter option
  * is most helpful for parallel programs: as demonstrated in the
- * @ref step_17 "step-17" example program, it is possible to let only one processor
+ * step-17 example program, it is possible to let only one processor
  * generate the graphical output for the entire parallel program, but
  * this can become vastly inefficient if many processors are involved,
  * because the load is no longer balanced. The way out is to let each
@@ -1754,7 +1790,7 @@ class DataOutBase
     static
     std::pair<unsigned int, unsigned int>
     determine_intermediate_format_dimensions (std::istream &input);
-    
+
 				     /**
 				      * Return the <tt>OutputFormat</tt>
 				      * value corresponding to the
@@ -1806,12 +1842,12 @@ class DataOutBase
 				      * <li> <tt>povray</tt>: <tt>.pov</tt>
 				      * <li> <tt>eps</tt>: <tt>.eps</tt>
 				      * <li> <tt>gmv</tt>: <tt>.gmv</tt>
-				      * <li> <tt>vtk</tt>: <tt>.vtk</tt>.
+				      * <li> <tt>vtk</tt>: <tt>.vtk</tt>
 				      * <li> <tt>deal_II_intermediate</tt>: <tt>.d2</tt>.
 				      * </ul>
 				      */
     static std::string default_suffix (const OutputFormat output_format);
-    
+
 				     /**
 				      * Determine an estimate for
 				      * the memory consumption (in
@@ -1829,10 +1865,10 @@ class DataOutBase
 				      * value.
 				      */
     static unsigned int memory_consumption ();
-    
+
 				     /** @addtogroup Exceptions
 				      * @{ */
-    
+
 				     /**
 				      * Exception
 				      */
@@ -1857,8 +1893,8 @@ class DataOutBase
 		    char*,
 		    << "There was an error opening Tecplot file " << arg1
 		    << " for output");
-    
-				     //@}  
+
+				     //@}
   private:
 				     /**
 				      * Write the coordinates of nodes
@@ -1898,12 +1934,12 @@ class DataOutBase
     class EpsCell2d
     {
       public:
-	
+
 					 /**
 					  * Vector of vertices of this cell.
 					  */
 	Point<2> vertices[4];
-	
+
 					 /**
 					  * Data value from which the actual
 					  * colors will be computed by
@@ -1911,7 +1947,7 @@ class DataOutBase
 					  * in the <tt>EpsFlags</tt> class.
 					  */
 	float color_value;
-	
+
 					 /**
 					  * Depth into the picture, which
 					  * is defined as the distance from
@@ -1919,7 +1955,7 @@ class DataOutBase
 					  * direction of the line of sight.
 					  */
 	float depth;
-	
+
 					 /**
 					  * Comparison operator for
 					  * sorting.
@@ -2017,7 +2053,7 @@ class DataOutBase
  * Likewise, there is a function <tt>parse_parameters</tt> which reads
  * these parameters and stores them in the flags associated with
  * this object (see above).
- * 
+ *
  * Using these functions, you do not have to track which formats are
  * presently implemented.
  *
@@ -2095,16 +2131,17 @@ class DataOutInterface : private DataOutBase
     using DataOutBase::tecplot;
     using DataOutBase::tecplot_binary;
     using DataOutBase::vtk;
+    using DataOutBase::vtu;
     using DataOutBase::deal_II_intermediate;
     using DataOutBase::parse_output_format;
     using DataOutBase::get_output_format_names;
     using DataOutBase::determine_intermediate_format_dimensions;
-    
+
 				     /**
 				      * Constructor.
 				      */
     DataOutInterface ();
-    
+
                                      /**
                                       * Destructor. Does nothing, but is
                                       * declared virtual since this class has
@@ -2116,7 +2153,7 @@ class DataOutInterface : private DataOutBase
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in OpenDX format. See
-				      * DataOut::write_dx.
+				      * DataOutBase::write_dx.
 				      */
     void write_dx (std::ostream &out) const;
 
@@ -2124,7 +2161,7 @@ class DataOutInterface : private DataOutBase
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in EPS format. See
-				      * DataOut::write_eps.
+				      * DataOutBase::write_eps.
 				      */
     void write_eps (std::ostream &out) const;
 
@@ -2132,7 +2169,7 @@ class DataOutInterface : private DataOutBase
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in GMV format. See
-				      * DataOut::write_gmv.
+				      * DataOutBase::write_gmv.
 				      */
     void write_gmv (std::ostream &out) const;
 
@@ -2140,7 +2177,7 @@ class DataOutInterface : private DataOutBase
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in GNUPLOT format. See
-				      * DataOut::write_gnuplot.
+				      * DataOutBase::write_gnuplot.
 				      */
     void write_gnuplot (std::ostream &out) const;
 
@@ -2148,15 +2185,15 @@ class DataOutInterface : private DataOutBase
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in POVRAY format. See
-				      * DataOut::write_povray.
+				      * DataOutBase::write_povray.
 				      */
     void write_povray (std::ostream &out) const;
-    
+
     				     /**
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in Tecplot format. See
-				      * DataOut::write_tecplot.
+				      * DataOutBase::write_tecplot.
 				      */
     void write_tecplot (std::ostream &out) const;
 
@@ -2176,7 +2213,7 @@ class DataOutInterface : private DataOutBase
 				      * get_patches() and write it to
 				      * <tt>out</tt> in UCD format for
 				      * @ref SoftwareAVS. See
-				      * DataOut::write_ucd.
+				      * DataOutBase::write_ucd.
 				      */
     void write_ucd (std::ostream &out) const;
 
@@ -2184,16 +2221,71 @@ class DataOutInterface : private DataOutBase
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in Vtk format. See
-				      * DataOut::write_vtk.
+				      * DataOutBase::write_vtk.
 				      */
     void write_vtk (std::ostream &out) const;
 
+				     /**
+				      * Obtain data through get_patches()
+				      * and write it to <tt>out</tt>
+				      * in Vtu (VTK's XML) format. See
+				      * DataOutBase::write_vtu.
+				      *
+				      * Some visualization programs, such as
+				      * ParaView, can read several separate
+				      * VTU files to parallelize
+				      * visualization. In that case, you need
+				      * a <code>.pvtu</code> file that
+				      * describes which VTU files form a
+				      * group. The
+				      * DataOutInterface::write_pvtu_record()
+				      * function can generate such a master
+				      * record.
+				      */
+    void write_vtu (std::ostream &out) const;
+
+				     /**
+				      * Some visualization programs, such as
+				      * ParaView, can read several separate
+				      * VTU files to parallelize
+				      * visualization. In that case, you need
+				      * a <code>.pvtu</code> file that
+				      * describes which VTU files (written,
+				      * for example, through the write_vtu()
+				      * function) form a group. The current
+				      * function can generate such a master
+				      * record.
+				      *
+				      * The file so written contains a list of
+				      * (scalar or vector) fields whose values
+				      * are described by the individual files
+				      * that comprise the set of parallel VTU
+				      * files along with the names of these
+				      * files. This function gets the names
+				      * and types of fields through the
+				      * get_patches() function of this class
+				      * like all the other write_xxx()
+				      * functions. The second argument to this
+				      * function specifies the names of the
+				      * files that form the parallel set.
+				      *
+				      * @note See DataOutBase::write_vtu for
+				      * writing each piece. Also note that
+				      * only one parallel process needs to
+				      * call the current function, listing the
+				      * names of the files written by all
+				      * parallel processes.
+				      */
+    void write_pvtu_record (std::ostream &out,
+			    const std::vector<std::string> &piece_names) const;
+
+
     				     /**
 				      * Obtain data through get_patches()
 				      * and write it to <tt>out</tt>
 				      * in deal.II intermediate
 				      * format. See
-				      * DataOut::write_deal_II_intermediate.
+				      * DataOutBase::write_deal_II_intermediate.
 				      *
 				      * Note that the intermediate
 				      * format is what its name
@@ -2209,7 +2301,7 @@ class DataOutInterface : private DataOutBase
 				      * that was used for writing.
 				      */
     void write_deal_II_intermediate (std::ostream &out) const;
-    
+
 				     /**
 				      * Write data and grid to <tt>out</tt>
 				      * according to the given data
@@ -2288,7 +2380,7 @@ class DataOutInterface : private DataOutBase
 				      * Set the flags to be used for output in
 				      * deal.II intermediate format.
 				      */
-    void set_flags (const Deal_II_IntermediateFlags &deal_II_intermediate_flags);    
+    void set_flags (const Deal_II_IntermediateFlags &deal_II_intermediate_flags);
 
 				     /**
 				      * A function that returns the same
@@ -2342,7 +2434,7 @@ class DataOutInterface : private DataOutBase
 				      * by the set_flags() function.
 				      */
     void parse_parameters (ParameterHandler &prm);
-    
+
 				     /**
 				      * Determine an estimate for
 				      * the memory consumption (in
@@ -2427,7 +2519,7 @@ class DataOutInterface : private DataOutBase
     virtual
     std::vector<std_cxx1x::tuple<unsigned int, unsigned int, std::string> >
     get_vector_data_ranges () const;
-    
+
 				     /**
 				      * The default number of
 				      * subdivisions for patches. This
@@ -2448,7 +2540,7 @@ class DataOutInterface : private DataOutBase
 				      * or in a parameter file.
 				      */
     OutputFormat default_fmt;
-    
+
 				     /**
 				      * Flags to be used upon output
 				      * of OpenDX data. Can be changed by
@@ -2488,7 +2580,7 @@ class DataOutInterface : private DataOutBase
 				      * using the <tt>set_flags</tt>
 				      * function.
 				      */
-    EpsFlags     eps_flags;    
+    EpsFlags     eps_flags;
 
 				     /**
 				      * Flags to be used upon output
@@ -2556,10 +2648,10 @@ class DataOutInterface : private DataOutBase
  * be merged, while this is almost impossible once the data has been
  * written out in any of the supported established graphics formats.
  *
- * This second use scenario is explained in some detail in the @ref step_18 "step-18"
+ * This second use scenario is explained in some detail in the step-18
  * example program.
  *
- * Both these applications are implemented in the @ref step_19 "step-19" example program.
+ * Both these applications are implemented in the step-19 example program.
  * There, a slight complication is also explained: in order to read data back
  * into this object, you have to know the template parameters for the space
  * dimension which were used when writing the data. If this knowledge is
@@ -2567,7 +2659,7 @@ class DataOutInterface : private DataOutBase
  * (such as in a simple format converter), then it needs to be figured out at
  * run time, even though the compiler already needs it at compile time. A way
  * around using the DataOutBase::determine_intermediate_format_dimensions()
- * function is explained in @ref step_19 "step-19".
+ * function is explained in step-19.
  *
  * Note that the intermediate format is what its name suggests: a
  * direct representation of internal data. It isn't standardized and
@@ -2636,10 +2728,10 @@ class DataOutReader : public DataOutInterface<dim,spacedim>
                                       * patches.
 				      *
 				      * The use of this function is
-				      * demonstrated in @ref step_19 "step-19".
+				      * demonstrated in step-19.
                                       */
     void merge (const DataOutReader<dim,spacedim> &other);
-    
+
                                      /**
                                       * Exception
                                       */
@@ -2661,7 +2753,7 @@ class DataOutReader : public DataOutInterface<dim,spacedim>
 		    << arg2 << "> or the space dimensions <"
 		    << arg3 << "> and <" << arg4
 		    << "> do not match!");
-    
+
   protected:
 				     /**
 				      * This is the function
@@ -2692,7 +2784,7 @@ class DataOutReader : public DataOutInterface<dim,spacedim>
 				      * time we read a file.
 				      */
     virtual std::vector<std::string> get_dataset_names () const;
-    
+
 				     /**
 				      * This functions returns
 				      * information about how the
