@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: function.h 14038 2006-10-23 02:46:34Z bangerth $
+//    $Id: function.h 20602 2010-02-13 17:44:17Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -340,20 +340,6 @@ class Function : public FunctionTime,
 				      * value.
 				      */
     unsigned int memory_consumption () const;
-
-				     /** @addtogroup Exceptions
-				      * @{ */
-
-                                     /**
-                                      * Number of components does not match.
-                                      */
-    DeclException2 (ExcNumberOfComponents,
-                    int, int,
-                    << "You can only assign function objects with the same "
-                    << "number of components. However, here the left operand "
-                    << "has " << arg1 << " components, and the right operand "
-                    << arg2 << " components.");
-				     //@}
 };
 
 
@@ -591,7 +577,7 @@ class ConstantFunction : public ZeroFunction<dim>
  * components are zero.  It is especially useful as a weight function
  * for <tt>VectorTools::integrate_difference</tt>, where it allows to
  * integrate only one or a few vector components, rather than the
- * entire vector-valued solution. See the @ref step_20 "step-20"
+ * entire vector-valued solution. See the step-20
  * tutorial program for a detailed explanation.
  *
  * @ingroup functions
