@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: geometry_info.h 18645 2009-04-17 23:59:29Z bangerth $
+//    $Id: geometry_info.h 19226 2009-08-11 21:37:55Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -300,7 +300,7 @@ class RefinementCase : public RefinementPossibilities<dim>
 				      * explicit.
 				      */
    explicit RefinementCase (const unsigned char refinement_case);
-    
+
 				     /**
 				      * Return the numeric value
 				      * stored by this class. While
@@ -335,7 +335,7 @@ class RefinementCase : public RefinementPossibilities<dim>
 				      * one given as argument.
 				      */
     RefinementCase operator | (const RefinementCase &r) const;
-    
+
 				     /**
 				      * Return the intersection of the
 				      * refinement flags represented
@@ -373,7 +373,7 @@ class RefinementCase : public RefinementPossibilities<dim>
 				      */
     static
     RefinementCase cut_axis (const unsigned int i);
-    
+
 				     /**
 				      * Return the amount of memory
 				      * occupied by an object of this
@@ -388,7 +388,7 @@ class RefinementCase : public RefinementPossibilities<dim>
 		    int,
 		    << "The refinement flags given (" << arg1 << ") contain set bits that do not "
 		    << "make sense for the space dimension of the object to which they are applied.");
-    
+
   private:
 				     /**
 				      * Store the refinement case as a
@@ -579,55 +579,55 @@ namespace internal
  |   0   |    case_none
  |       |
  *-------*
- 
+
  *---*---*
  |   |   |
  | 0 | 1 |    case_x
  |   |   |
  *---*---*
- 
+
  *---*---*
  | 1 |   |
  *---* 2 |    case_x1y
  | 0 |   |
  *---*---*
- 
+
  *---*---*
  |   | 2 |
  | 0 *---*    case_x2y
  |   | 1 |
  *---*---*
- 
+
  *---*---*
  | 1 | 3 |
  *---*---*    case_x1y2y   (successive refinement: first cut_x, then cut_y for both children)
  | 0 | 2 |
  *---*---*
- 
+
  *-------*
  |   1   |
  *-------*    case_y
  |   0   |
  *-------*
- 
+
  *-------*
  |   2   |
  *---*---*    case_y1x
  | 0 | 1 |
  *---*---*
- 
+
  *---*---*
  | 1 | 2 |
  *---*---*    case_y2x
  |   0   |
  *-------*
- 
+
  *---*---*
  | 2 | 3 |
  *---*---*    case_y1x2x   (successive refinement: first cut_y, then cut_x for both children)
  | 0 | 1 |
  *---+---*
- 
+
  *---*---*
  | 2 | 3 |
  *---*---*    case_xy      (one isotropic refinement step)
@@ -693,7 +693,7 @@ namespace internal
 					* specified in the base class.
 					*/
       SubfaceCase (const typename SubfacePossibilities<dim>::Possibilities subface_possibility);
-    
+
 				       /**
 					* Return the numeric value
 					* stored by this class. While
@@ -720,7 +720,7 @@ namespace internal
 					* values (the array indices).
 					*/
       operator unsigned char () const;
-    
+
 				       /**
 					* Return the amount of memory
 					* occupied by an object of this
@@ -735,7 +735,7 @@ namespace internal
 		      int,
 		      << "The subface case given (" << arg1 << ") does not make sense "
 		      << "for the space dimension of the object to which they are applied.");
-    
+
     private:
 				       /**
 					* Store the refinement case as a
@@ -745,7 +745,7 @@ namespace internal
 					*/
       unsigned char value : (dim == 3 ? 4 : 1);
   };
-  
+
 } // namespace internal
 
 
@@ -776,7 +776,7 @@ template <int dim> class GeometryInfo;
  * @author Wolfgang Bangerth, 1998
  */
 template <>
-struct GeometryInfo<0> 
+struct GeometryInfo<0>
 {
 
                                      /**
@@ -819,7 +819,7 @@ struct GeometryInfo<0>
 				      * is equal to one.
 				      */
     static unsigned int n_children(const RefinementCase<0> &refinement_case);
-    
+
 				     /**
 				      * Number of vertices a cell has.
 				      */
@@ -840,7 +840,7 @@ struct GeometryInfo<0>
 				      * Number of lines each face has.
 				      */
     static const unsigned int lines_per_face    = 0;
-    
+
 				     /**
 				      * Number of quads on each face.
 				      */
@@ -891,7 +891,7 @@ struct GeometryInfo<0>
  *
  *
  * <h3>Implementation conventions for two spatial dimensions</h3>
- * 
+ *
  * From version 5.2 onwards deal.II is based on a numbering scheme
  * that uses a lexicographic ordering (with x running fastest)
  * wherever possible, hence trying to adopt a kind of 'canonical'
@@ -910,8 +910,8 @@ struct GeometryInfo<0>
  * N3) the direction of a line is represented by the direction of
  * point 0 towards point 1 and is always in one of the coordinate
  * directions
- * 
- * N4/ face lines in 3d are ordered, such that the induced 2d local
+ *
+ * N4) face lines in 3d are ordered, such that the induced 2d local
  * coordinate system (x,y) implies (right hand rule) a normal in
  * face normal direction, see N2/.
  *
@@ -926,7 +926,7 @@ struct GeometryInfo<0>
  *    0-->--1
  *        2
  * @endverbatim
- * 
+ *
  * Note that the orientation of lines has to be correct upon construction of a
  * grid; however, it is automatically preserved upon refinement.
  *
@@ -957,7 +957,7 @@ struct GeometryInfo<0>
  *   |   0-----1
  *   *------------>x
  * @endverbatim
- * 
+ *
  * Here, vertex 0 is the origin of the coordinate system, vertex 1 has
  * coordinates <tt>(1,0)</tt>, vertex 2 at <tt>(0,1)</tt> and vertex 3 at
  * <tt>(1,1)</tt>. The GeometryInfo<dim>::unit_cell_vertex() function can be
@@ -992,13 +992,13 @@ struct GeometryInfo<0>
  * member functions and variables of the present class.
  *
  * <h4>Vertices</h4>
- *  
+ *
  * The ordering of vertices in 3d is defined by the same rules as in
  * the 2d case, i.e.
  *
  * N1) vertices are numbered in lexicographic ordering.
  *
- * Hence, the vertices are numbered as follows   
+ * Hence, the vertices are numbered as follows
  * @verbatim
  *       6-------7        6-------7
  *      /|       |       /       /|
@@ -1011,14 +1011,14 @@ struct GeometryInfo<0>
  *   |/       /       |       |/
  *   0-------1        0-------1
  * @endverbatim
- * 
+ *
  * We note, that first the vertices on the bottom face (z=0) are numbered
  * exactly the same way as are the vertices on a quadrilateral. Then the
  * vertices on the top face (z=1) are numbered similarly by moving the bottom
  * face to the top. Again, the GeometryInfo<dim>::unit_cell_vertex() function
  * can be used to query this information at run-time.
  *
- * 
+ *
  * <h4>Lines</h4>
  *
  * Here, the same holds as for the vertices:
@@ -1067,7 +1067,7 @@ struct GeometryInfo<0>
  * globally according to above convention. However, the GridReordering
  * class can do this for you, by reorienting cells and edges of an
  * arbitrary list of input cells that need not be already sorted.
- * 
+ *
  * <h4>Faces</h4>
  *
  * The numbering of faces in 3d is defined by a rule analogous to 2d:
@@ -1113,7 +1113,7 @@ struct GeometryInfo<0>
  *    |/       /       |       |/
  *    *-------*        *-------*
  *  @endverbatim
- * 
+ *
  * <li> Faces 2 and 3:
  *  @verbatim
  *        x Face 3           Face 2
@@ -1127,8 +1127,8 @@ struct GeometryInfo<0>
  *    | /       /      2       | /
  *    |/       /       |       |/
  *    *-------*        *---0---*y
- *  @endverbatim 
- * 
+ *  @endverbatim
+ *
  * <li> Faces 4 and 5:
  *  @verbatim
  *          Face 4         y Face 5
@@ -1390,7 +1390,7 @@ struct GeometryInfo<0>
  *    <li> Vertex 7: <tt>(1,1,1)</tt>.
  * </ul>
  *
- * 
+ *
  *
  * @note Instantiations for this template are provided for dimensions 1,2,3,4,
  * and there is a specialization for dim=0 (see the section on @ref
@@ -1402,7 +1402,7 @@ struct GeometryInfo<0>
 template <int dim>
 struct GeometryInfo
 {
-    
+
                                      /**
                                       * Maximum number of children of
                                       * a refined cell, i.e. the
@@ -1450,7 +1450,7 @@ struct GeometryInfo
 				      */
     static const unsigned int lines_per_face
     = GeometryInfo<dim-1>::lines_per_cell;
-    
+
 				     /**
 				      * Number of quads on each face.
 				      */
@@ -1555,13 +1555,15 @@ struct GeometryInfo
                                       * the vertex belongs
                                       */
     static const unsigned int vertex_to_face[vertices_per_cell][dim];
- 
+
 				     /**
 				      * Return the number of children
 				      * of a cell (or face) refined
 				      * with <tt>ref_case</tt>.
 				      */
-    static unsigned int n_children(const RefinementCase<dim> &refinement_case);
+    static
+    unsigned int
+    n_children(const RefinementCase<dim> &refinement_case);
 
 				     /**
 				      * Return the number of subfaces
@@ -1569,7 +1571,9 @@ struct GeometryInfo
 				      * internal::SubfaceCase
 				      * @p face_ref_case.
 				      */
-    static unsigned int n_subfaces(const internal::SubfaceCase<dim> &subface_case);
+    static
+    unsigned int
+    n_subfaces(const internal::SubfaceCase<dim> &subface_case);
 
 				     /**
 				      * Given a face on the reference
@@ -1586,8 +1590,10 @@ struct GeometryInfo
 				      * the ratio is 1/4 for each of
 				      * the subfaces.
 				      */
-    static double subface_ratio(const internal::SubfaceCase<dim> &subface_case,
-				const unsigned int subface_no);
+    static
+    double
+    subface_ratio(const internal::SubfaceCase<dim> &subface_case,
+		  const unsigned int subface_no);
 
 				     /**
 				      * Given a cell refined with the
@@ -1597,12 +1603,13 @@ struct GeometryInfo
 				      * <code>SubfaceCase</code> of
 				      * the @p face_no th face.
 				      */
-    static RefinementCase<dim-1> face_refinement_case(
-      const RefinementCase<dim> &cell_refinement_case,
-      const unsigned int face_no,
-      const bool face_orientation = true,
-      const bool face_flip        = false,
-      const bool face_rotation    = false);
+    static
+    RefinementCase<dim-1>
+    face_refinement_case (const RefinementCase<dim> &cell_refinement_case,
+			  const unsigned int face_no,
+			  const bool face_orientation = true,
+			  const bool face_flip        = false,
+			  const bool face_rotation    = false);
 
 				     /**
 				      * Given the SubfaceCase @p
@@ -1612,12 +1619,14 @@ struct GeometryInfo
 				      * cell, which corresponds to
 				      * that refinement of the face.
 				      */
-    static RefinementCase<dim> min_cell_refinement_case_for_face_refinement(
-      const RefinementCase<dim-1> &face_refinement_case,
-      const unsigned int face_no,
-      const bool face_orientation = true,
-      const bool face_flip        = false,
-      const bool face_rotation    = false);
+    static
+    RefinementCase<dim>
+    min_cell_refinement_case_for_face_refinement
+    (const RefinementCase<dim-1> &face_refinement_case,
+     const unsigned int face_no,
+     const bool face_orientation = true,
+     const bool face_flip        = false,
+     const bool face_rotation    = false);
 
 				     /**
 				      * Given a cell refined with the
@@ -1626,8 +1635,10 @@ struct GeometryInfo
 				      * the RefinementCase of the @p
 				      * line_no th face.
 				      */
-    static RefinementCase<1> line_refinement_case(const RefinementCase<dim> &cell_refinement_case,
-						  const unsigned int line_no);
+    static
+    RefinementCase<1>
+    line_refinement_case(const RefinementCase<dim> &cell_refinement_case,
+			 const unsigned int line_no);
 
 				     /**
 				      * Return the minimal / smallest
@@ -1635,7 +1646,9 @@ struct GeometryInfo
 				      * ensures refinement of line
 				      * @p line_no.
 				      */
-    static RefinementCase<dim> min_cell_refinement_case_for_line_refinement(const unsigned int line_no);
+    static
+    RefinementCase<dim>
+    min_cell_refinement_case_for_line_refinement(const unsigned int line_no);
 
                                      /**
 				      * This field stores which child
@@ -1696,20 +1709,23 @@ struct GeometryInfo
 				      * In the case of anisotropically refined
 				      * cells and faces, the @p RefinementCase of
 				      * the face, <tt>face_ref_case</tt>,
-				      * might have an influence on 
+				      * might have an influence on
 				      * which child is behind which given
 				      * subface, thus this is an additional
 				      * argument, defaulting to isotropic
 				      * refinement of the face.
 				      */
-    static unsigned int child_cell_on_face (const RefinementCase<dim> &ref_case,
-					    const unsigned int face,
-					    const unsigned int subface,
-					    const bool face_orientation = true,
-					    const bool face_flip        = false,
-					    const bool face_rotation    = false,
-					    const RefinementCase<dim-1> &face_refinement_case = RefinementCase<dim-1>::isotropic_refinement);
-    
+    static
+    unsigned int
+    child_cell_on_face (const RefinementCase<dim> &ref_case,
+			const unsigned int face,
+			const unsigned int subface,
+			const bool face_orientation = true,
+			const bool face_flip        = false,
+			const bool face_rotation    = false,
+			const RefinementCase<dim-1> &face_refinement_case
+			= RefinementCase<dim-1>::isotropic_refinement);
+
 				     /**
 				      * Map line vertex number to cell
 				      * vertex number, i.e. give the
@@ -1731,8 +1747,10 @@ struct GeometryInfo
 				      * face_to_cell_vertices()
 				      * function.
 				      */
-    static unsigned int line_to_cell_vertices (const unsigned int line,
-					       const unsigned int vertex);
+    static
+    unsigned int
+    line_to_cell_vertices (const unsigned int line,
+			   const unsigned int vertex);
 
 				     /**
 				      * Map face vertex number to cell
@@ -1740,7 +1758,10 @@ struct GeometryInfo
 				      * cell vertex number of the
 				      * <tt>vertex</tt>th vertex of
 				      * face <tt>face</tt>, e.g.
-				      * <tt>GeometryInfo<2>::face_to_cell_vertices(3,0)=2</tt>.
+				      * <tt>GeometryInfo<2>::face_to_cell_vertices(3,0)=2</tt>,
+				      * see the image under point N4
+				      * in the 2d section of this
+				      * class's documentation.
 				      *
 				      * Through the <tt>face_orientation</tt>,
 				      * <tt>face_flip</tt> and
@@ -1763,11 +1784,13 @@ struct GeometryInfo
 				      * child_cell_on_face() giving it
 				      * a suggestive name.
 				      */
-    static unsigned int face_to_cell_vertices (const unsigned int face,
-					       const unsigned int vertex,
-					       const bool face_orientation = true,
-					       const bool face_flip        = false,
-					       const bool face_rotation    = false);
+    static
+    unsigned int
+    face_to_cell_vertices (const unsigned int face,
+			   const unsigned int vertex,
+			   const bool face_orientation = true,
+			   const bool face_flip        = false,
+			   const bool face_rotation    = false);
 
 				     /**
 				      * Map face line number to cell
@@ -1788,11 +1811,13 @@ struct GeometryInfo
 				      * <tt>false</tt> (standard orientation)
 				      * and has no effect in 2d.
 				      */
-    static unsigned int face_to_cell_lines (const unsigned int face,
-					    const unsigned int line,
-					    const bool face_orientation = true,
-					    const bool face_flip        = false,
-					    const bool face_rotation    = false);
+    static
+    unsigned int
+    face_to_cell_lines (const unsigned int face,
+			const unsigned int line,
+			const bool face_orientation = true,
+			const bool face_flip        = false,
+			const bool face_rotation    = false);
 
 				     /**
 				      * Map the vertex index @p vertex of a face
@@ -1807,11 +1832,13 @@ struct GeometryInfo
 				      *
 				      * This function is only implemented in 3D.
 				      */
-    static unsigned int standard_to_real_face_vertex (const unsigned int vertex,
-						      const bool face_orientation = true,
-						      const bool face_flip        = false,
-						      const bool face_rotation    = false);
-    
+    static
+    unsigned int
+    standard_to_real_face_vertex (const unsigned int vertex,
+				  const bool face_orientation = true,
+				  const bool face_flip        = false,
+				  const bool face_rotation    = false);
+
 				     /**
 				      * Map the vertex index @p vertex of a face
 				      * with arbitrary @p face_orientation, @p
@@ -1825,11 +1852,13 @@ struct GeometryInfo
 				      *
 				      * This function is only implemented in 3D.
 				      */
-    static unsigned int real_to_standard_face_vertex (const unsigned int vertex,
-						      const bool face_orientation = true,
-						      const bool face_flip        = false,
-						      const bool face_rotation    = false);
-    
+    static
+    unsigned int
+    real_to_standard_face_vertex (const unsigned int vertex,
+				  const bool face_orientation = true,
+				  const bool face_flip        = false,
+				  const bool face_rotation    = false);
+
 				     /**
 				      * Map the line index @p line of a face
 				      * in standard orientation to one of a face
@@ -1843,11 +1872,13 @@ struct GeometryInfo
 				      *
 				      * This function is only implemented in 3D.
 				      */
-    static unsigned int standard_to_real_face_line (const unsigned int line,
-						    const bool face_orientation = true,
-						    const bool face_flip        = false,
-						    const bool face_rotation    = false);
-    
+    static
+    unsigned int
+    standard_to_real_face_line (const unsigned int line,
+				const bool face_orientation = true,
+				const bool face_flip        = false,
+				const bool face_rotation    = false);
+
 				     /**
 				      * Map the line index @p line of a face
 				      * with arbitrary @p face_orientation, @p
@@ -1861,11 +1892,13 @@ struct GeometryInfo
 				      *
 				      * This function is only implemented in 3D.
 				      */
-    static unsigned int real_to_standard_face_line (const unsigned int line,
-						    const bool face_orientation = true,
-						    const bool face_flip        = false,
-						    const bool face_rotation    = false);
-    
+    static
+    unsigned int
+    real_to_standard_face_line (const unsigned int line,
+				const bool face_orientation = true,
+				const bool face_flip        = false,
+				const bool face_rotation    = false);
+
 				     /**
 				      * Return the position of the @p ith
 				      * vertex on the unit cell. The order of
@@ -1873,7 +1906,9 @@ struct GeometryInfo
 				      * deal.II, as described in the general
 				      * documentation of this class.
 				      */
-    static Point<dim> unit_cell_vertex (const unsigned int vertex);
+    static
+    Point<dim>
+    unit_cell_vertex (const unsigned int vertex);
 
 				     /**
 				      * Given a point @p p in unit
@@ -1890,7 +1925,9 @@ struct GeometryInfo
 				      * the general documentation of this
 				      * class.
 				      */
-    static unsigned int child_cell_from_point (const Point<dim> &p);
+    static
+    unsigned int
+    child_cell_from_point (const Point<dim> &p);
 
 				     /**
 				      * Given coordinates @p p on the
@@ -1906,9 +1943,12 @@ struct GeometryInfo
 				      * depends on the number of the
 				      * child.
 				      */
-    static Point<dim> cell_to_child_coordinates (const Point<dim>          &p,
-						 const unsigned int         child_index,
-						 const RefinementCase<dim>  refine_case=RefinementCase<dim>::isotropic_refinement);
+    static
+    Point<dim>
+    cell_to_child_coordinates (const Point<dim>          &p,
+			       const unsigned int         child_index,
+			       const RefinementCase<dim>  refine_case
+			       = RefinementCase<dim>::isotropic_refinement);
 
 				     /**
 				      * The reverse function to the
@@ -1918,17 +1958,22 @@ struct GeometryInfo
 				      * coordinate system of the
 				      * mother cell.
 				      */
-    static Point<dim> child_to_cell_coordinates (const Point<dim>          &p,
-						 const unsigned int         child_index,
-						 const RefinementCase<dim>  refine_case=RefinementCase<dim>::isotropic_refinement);
+    static
+    Point<dim>
+    child_to_cell_coordinates (const Point<dim>          &p,
+			       const unsigned int         child_index,
+			       const RefinementCase<dim>  refine_case
+			       = RefinementCase<dim>::isotropic_refinement);
 
 				     /**
 				      * Return true if the given point
 				      * is inside the unit cell of the
 				      * present space dimension.
 				      */
-    static bool is_inside_unit_cell (const Point<dim> &p);
-    
+    static
+    bool
+    is_inside_unit_cell (const Point<dim> &p);
+
 				     /**
 				      * Return true if the given point
 				      * is inside the unit cell of the
@@ -1951,8 +1996,10 @@ struct GeometryInfo
 				      * that the point should be
 				      * safely inside the cell.
 				      */
-    static bool is_inside_unit_cell (const Point<dim> &p,
-                                     const double eps);
+    static
+    bool
+    is_inside_unit_cell (const Point<dim> &p,
+			 const double eps);
 
 				     /**
 				      * Projects a given point onto the
@@ -1960,7 +2007,9 @@ struct GeometryInfo
                                       * outside [0..1] is modified
                                       * to lie within that interval.
 				      */
-    static Point<dim> project_to_unit_cell (const Point<dim> &p);
+    static
+    Point<dim>
+    project_to_unit_cell (const Point<dim> &p);
 
                                      /**
                                       * Returns the infinity norm of
@@ -1970,7 +2019,138 @@ struct GeometryInfo
                                       * For points inside the cell, this is
                                       * defined as zero.
                                       */
-    static double distance_to_unit_cell (const Point<dim> &p);
+    static
+    double
+    distance_to_unit_cell (const Point<dim> &p);
+
+				     /**
+				      * Compute the value of the $i$-th
+				      * $d$-linear (i.e. (bi-,tri-)linear)
+				      * shape function at location $\xi$.
+				      */
+    static
+    double
+    d_linear_shape_function (const Point<dim> &xi,
+			     const unsigned int i);
+
+				     /**
+				      * Compute the gradient of the $i$-th
+				      * $d$-linear (i.e. (bi-,tri-)linear)
+				      * shape function at location $\xi$.
+				      */
+    static
+    Tensor<1,dim>
+    d_linear_shape_function_gradient (const Point<dim> &xi,
+				      const unsigned int i);
+
+				     /**
+				      * For a (bi-, tri-)linear
+				      * mapping from the reference
+				      * cell, face, or edge to the
+				      * object specified by the given
+				      * vertices, compute the
+				      * alternating form of the
+				      * transformed unit vectors
+				      * vertices. For an object of
+				      * dimensionality @p dim, there
+				      * are @p dim vectors with @p
+				      * spacedim components each, and
+				      * the alternating form is a
+				      * tensor of rank spacedim-dim
+				      * that corresponds to the wedge
+				      * product of the @p dim unit
+				      * vectors, and it corresponds to
+				      * the volume and normal vectors
+				      * of the mapping from reference
+				      * element to the element
+				      * described by the vertices.
+				      *
+				      * For example, if dim==spacedim==2, then
+				      * the alternating form is a scalar
+				      * (because spacedim-dim=0) and its value
+				      * equals $\mathbf v_1\wedge \mathbf
+				      * v_2=\mathbf v_1^\perp \cdot\mathbf
+				      * v_2$, where $\mathbf v_1^\perp$ is a
+				      * vector that is rotated to the right by
+				      * 90 degrees from $\mathbf v_1$. If
+				      * dim==spacedim==3, then the result is
+				      * again a scalar with value $\mathbf
+				      * v_1\wedge \mathbf v_2 \wedge \mathbf
+				      * v_3 = (\mathbf v_1\times \mathbf
+				      * v_2)\cdot \mathbf v_3$, where $\mathbf
+				      * v_1, \mathbf v_2, \mathbf v_3$ are the
+				      * images of the unit vectors at a vertex
+				      * of the unit dim-dimensional cell under
+				      * transformation to the dim-dimensional
+				      * cell in spacedim-dimensional space. In
+				      * both cases, i.e. for dim==2 or 3, the
+				      * result happens to equal the
+				      * determinant of the Jacobian of the
+				      * mapping from reference cell to cell in
+				      * real space. Note that it is the actual
+				      * determinant, not its absolute value as
+				      * often used in transforming integrals
+				      * from one coordinate system to
+				      * another. In particular, if the object
+				      * specified by the vertices is a
+				      * parallelogram (i.e. a linear
+				      * transformation of the reference cell)
+				      * then the computed values are the same
+				      * at all vertices and equal the (signed)
+				      * area of the cell; similarly, for
+				      * parallel-epipeds, it is the volume of
+				      * the cell.
+				      *
+				      * Likewise, if we have dim==spacedim-1
+				      * (e.g. we have a quad in 3d space, or a
+				      * line in 2d), then the alternating
+				      * product denotes the normal vector
+				      * (i.e. a rank-1 tensor, since
+				      * spacedim-dim=1) to the object at each
+				      * vertex, where the normal vector's
+				      * magnitude denotes the area element of
+				      * the transformation from the reference
+				      * object to the object given by the
+				      * vertices. In particular, if again the
+				      * mapping from reference object to the
+				      * object under consideration here is
+				      * linear (not bi- or trilinear), then
+				      * the returned vectors are all
+				      * %parallel, perpendicular to the mapped
+				      * object described by the vertices, and
+				      * have a magnitude equal to the
+				      * area/volume of the mapped object. If
+				      * dim=1, spacedim=2, then the returned
+				      * value is $\mathbf v_1^\perp$, where
+				      * $\mathbf v_1$ is the image of the sole
+				      * unit vector of a line mapped to the
+				      * line in 2d given by the vertices; if
+				      * dim=2, spacedim=3, then the returned
+				      * values are $\mathbf v_1 \wedge \mathbf
+				      * v_2=\mathbf v_1 \times \mathbf v_2$
+				      * where $\mathbf v_1,\mathbf v_2$ are
+				      * the two three-dimensional vectors that
+				      * are tangential to the quad mapped into
+				      * three-dimensional space.
+				      *
+				      * This function is used in order to
+				      * determine how distorted a cell is (see
+				      * the entry on
+				      * @ref GlossDistorted "distorted cells"
+				      * in the glossary).
+				      */
+    template <int spacedim>
+    static
+    void
+    alternating_form_at_vertices
+#ifndef DEAL_II_ARRAY_ARG_BUG
+    (const Point<spacedim> (&vertices)[vertices_per_cell],
+     Tensor<spacedim-dim,spacedim> (&forms)[vertices_per_cell])
+#else
+    (const Point<spacedim> *vertices,
+     Tensor<spacedim-dim,spacedim> *forms)
+#endif
+      ;
 
                                      /**
 				      * For each face of the reference
@@ -2041,7 +2221,7 @@ struct GeometryInfo
 		    double,
 		    << "The coordinates must satisfy 0 <= x_i <= 1, "
 		    << "but here we have x_i=" << arg1);
-            
+
                                      /**
                                       * Exception
                                       */
@@ -2088,6 +2268,26 @@ template <>
 const unsigned int GeometryInfo<4>::opposite_face[faces_per_cell];
 #endif
 
+
+template <>
+Tensor<1,1>
+GeometryInfo<1>::
+d_linear_shape_function_gradient (const Point<1> &xi,
+				  const unsigned int i);
+template <>
+Tensor<1,2>
+GeometryInfo<2>::
+d_linear_shape_function_gradient (const Point<2> &xi,
+				  const unsigned int i);
+template <>
+Tensor<1,3>
+GeometryInfo<3>::
+d_linear_shape_function_gradient (const Point<3> &xi,
+				  const unsigned int i);
+
+
+
+
 /* -------------- inline functions ------------- */
 
 namespace internal
@@ -2099,18 +2299,18 @@ namespace internal
 		  :
 		  value (subface_possibility)
   {}
-  
-  
+
+
   template <int dim>
   inline
   SubfaceCase<dim>::operator unsigned char () const
   {
     return value;
   }
-  
-  
+
+
 } // namespace internal
-  
+
 
 template <int dim>
 inline
@@ -2297,7 +2497,7 @@ GeometryInfo<dim>::unit_cell_vertex (const unsigned int)
 {
   Assert(false, ExcNotImplemented());
 
-  return Point<dim> ();  
+  return Point<dim> ();
 }
 
 
@@ -2308,7 +2508,7 @@ unsigned int
 GeometryInfo<1>::child_cell_from_point (const Point<1> &p)
 {
   Assert ((p[0] >= 0) && (p[0] <= 1), ExcInvalidCoordinate(p[0]));
-  
+
   return (p[0] <= 0.5 ? 0 : 1);
 }
 
@@ -2321,7 +2521,7 @@ GeometryInfo<2>::child_cell_from_point (const Point<2> &p)
 {
   Assert ((p[0] >= 0) && (p[0] <= 1), ExcInvalidCoordinate(p[0]));
   Assert ((p[1] >= 0) && (p[1] <= 1), ExcInvalidCoordinate(p[1]));
-  
+
   return (p[0] <= 0.5 ?
 	  (p[1] <= 0.5 ? 0 : 2) :
 	  (p[1] <= 0.5 ? 1 : 3));
@@ -2337,7 +2537,7 @@ GeometryInfo<3>::child_cell_from_point (const Point<3> &p)
   Assert ((p[0] >= 0) && (p[0] <= 1), ExcInvalidCoordinate(p[0]));
   Assert ((p[1] >= 0) && (p[1] <= 1), ExcInvalidCoordinate(p[1]));
   Assert ((p[2] >= 0) && (p[2] <= 1), ExcInvalidCoordinate(p[2]));
-  
+
   return (p[0] <= 0.5 ?
 	  (p[1] <= 0.5 ?
 	   (p[2] <= 0.5 ? 0 : 4) :
@@ -2427,9 +2627,11 @@ GeometryInfo<3>::cell_to_child_coordinates (const Point<3>         &p,
 	  ExcIndexRange (child_index, 0, GeometryInfo<3>::n_children(refine_case)));
 
   Point<3> point=p;
-				   // there might be a cleverer way to do this, but since this function is
-				   // called in very few places for initialization purposes only, I don't
-				   // care at the moment
+				   // there might be a cleverer way to do
+				   // this, but since this function is called
+				   // in very few places for initialization
+				   // purposes only, I don't care at the
+				   // moment
   switch (refine_case)
     {
       case RefinementCase<3>::cut_x:
@@ -2532,9 +2734,11 @@ GeometryInfo<3>::child_to_cell_coordinates (const Point<3>         &p,
 	  ExcIndexRange (child_index, 0, GeometryInfo<3>::n_children(refine_case)));
 
   Point<3> point=p;
-				   // there might be a cleverer way to do this, but since this function is
-				   // called in very few places for initialization purposes only, I don't
-				   // care at the moment
+				   // there might be a cleverer way to do
+				   // this, but since this function is called
+				   // in very few places for initialization
+				   // purposes only, I don't care at the
+				   // moment
   switch (refine_case)
     {
       case RefinementCase<3>::cut_x:
