@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: polynomial.h 14038 2006-10-23 02:46:34Z bangerth $
+//    $Id: polynomial.h 20494 2010-02-04 15:04:13Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -18,6 +18,7 @@
 #include <base/config.h>
 #include <base/exceptions.h>
 #include <base/subscriptor.h>
+#include <base/std_cxx1x/shared_ptr.h>
 
 #include <vector>
 
@@ -43,7 +44,7 @@ namespace Polynomials
  * happens through the Horner scheme which provides both numerical
  * stability and a minimal number of numerical operations.
  *
- * @author Ralf Hartmann, Guido Kanschat, 2000, 2006
+ * @author Ralf Hartmann, Guido Kanschat, 2000, 2006, 2009
  */
   template <typename number>
   class Polynomial : public Subscriptor
@@ -68,12 +69,18 @@ namespace Polynomials
                                         */
       Polynomial (const std::vector<number> &coefficients);
 
+				       /**
+					* Constructor creating a zero
+					* polynomial of degree @p n.
+					*/
+      Polynomial (const unsigned int n);
+
 				       /**
 					* Default constructor creating
 					* an illegal object.
 					*/
       Polynomial ();
-      
+
                                        /**
                                         * Return the value of this
                                         * polynomial at the given point.
@@ -83,7 +90,7 @@ namespace Polynomials
                                         * of the evaluation.
                                         */
       number value (const number x) const;
-    
+
                                        /**
                                         * Return the values and the
                                         * derivatives of the
@@ -187,12 +194,12 @@ namespace Polynomials
 					* Add a second polynomial.
 					*/
       Polynomial<number>& operator += (const Polynomial<number>& p);
-      
+
 				       /**
 					* Subtract a second polynomial.
 					*/
       Polynomial<number>& operator -= (const Polynomial<number>& p);
-      
+
                                        /**
                                         * Print coefficients.
                                         */
@@ -220,7 +227,7 @@ namespace Polynomials
                                         */
       static void multiply (std::vector<number>& coefficients,
                             const number factor);
-    
+
                                        /**
                                         * Coefficients of the polynomial
                                         * $\sum_i a_i x^i$. This vector
@@ -272,7 +279,7 @@ namespace Polynomials
       static
       std::vector<Polynomial<number> >
       generate_complete_basis (const unsigned int degree);
-    
+
     private:
 				       /**
 					* Needed by constructor.
@@ -280,7 +287,7 @@ namespace Polynomials
       static std::vector<number> make_vector(unsigned int n,
 					     const double coefficient);
   };
-  
+
 
 /**
  * Lagrange polynomials with equidistant interpolation points in
@@ -334,7 +341,7 @@ namespace Polynomials
       static
       std::vector<Polynomial<double> >
       generate_complete_basis (const unsigned int degree);
-    
+
     private:
 
                                        /**
@@ -345,10 +352,11 @@ namespace Polynomials
                                         * called in the
                                         * constructor.
                                         */
-      static 
-      std::vector<double> 
+      static
+      void
       compute_coefficients (const unsigned int n,
-                            const unsigned int support_point);
+                            const unsigned int support_point,
+			    std::vector<double>& a);
   };
 
 /**
@@ -373,9 +381,9 @@ namespace Polynomials
       std::vector<Polynomial<double> >
       generate_complete_basis (const std::vector<Point<1> >& points);
   };
-  
-  
-  
+
+
+
 /**
  * Legendre polynomials of arbitrary degree on <tt>[0,1]</tt>.
  *
@@ -412,30 +420,33 @@ namespace Polynomials
       static
       std::vector<Polynomial<double> >
       generate_complete_basis (const unsigned int degree);
-    
+
     private:
                                        /**
                                         * Coefficients for the interval $[0,1]$.
                                         */
-      static std::vector<const std::vector<double> *> shifted_coefficients;
-    
+      static std::vector<std_cxx1x::shared_ptr<const std::vector<double> > > shifted_coefficients;
+
                                        /**
                                         * Vector with already computed
-                                        * coefficients. For each degree
-                                        * of the polynomial, we keep one
-                                        * pointer to the list of
-                                        * coefficients; we do so rather
-                                        * than keeping a vector of
+                                        * coefficients. For each degree of the
+                                        * polynomial, we keep one pointer to
+                                        * the list of coefficients; we do so
+                                        * rather than keeping a vector of
                                         * vectors in order to simplify
-                                        * programming multithread-safe.
+                                        * programming multithread-safe. In
+                                        * order to avoid memory leak, we use a
+                                        * shared_ptr in order to correctly
+                                        * free the memory of the vectors when
+                                        * the global destructor is called.
                                         */
-      static std::vector<const std::vector<double> *> recursive_coefficients;
-    
+      static std::vector<std_cxx1x::shared_ptr<const std::vector<double> > > recursive_coefficients;
+
                                        /**
                                         * Compute coefficients recursively.
                                         */
       static void compute_coefficients (const unsigned int p);
-    
+
                                        /**
                                         * Get coefficients for
                                         * constructor.  This way, it can
@@ -452,23 +463,23 @@ namespace Polynomials
 /**
  * Hierarchical polynomials of arbitrary degree on <tt>[0,1]</tt>.
  *
- * When Constructing a Hierarchical polynomial of degree <tt>p</tt>, 
+ * When Constructing a Hierarchical polynomial of degree <tt>p</tt>,
  * the coefficients will be computed by a recursion formula.  The
  * coefficients are stored in a static data vector to be available
  * when needed next time.
  *
- * These hierarchical polynomials are based on those of Demkowicz, Oden, 
+ * These hierarchical polynomials are based on those of Demkowicz, Oden,
  * Rachowicz, and Hardy (CMAME 77 (1989) 79-112, Sec. 4). The first two
- * polynomials are the standard linear shape functions given by 
+ * polynomials are the standard linear shape functions given by
  * $\phi_{0}(x) = 1 - x$ and $\phi_{1}(x) = x$. For $l \geq 2$
  * we use the definitions $\phi_{l}(x) = (2x-1)^l - 1, l = 2,4,6,...$
- * and $\phi_{l}(x) = (2x-1)^l - (2x-1), l = 3,5,7,...$. These satisfy the 
- * recursion relations $\phi_{l}(x) = (2x-1)\phi_{l-1}, l=3,5,7,...$ and 
- * $\phi_{l}(x) = (2x-1)\phi_{l-1} + \phi_{2}, l=4,6,8,...$. 
+ * and $\phi_{l}(x) = (2x-1)^l - (2x-1), l = 3,5,7,...$. These satisfy the
+ * recursion relations $\phi_{l}(x) = (2x-1)\phi_{l-1}, l=3,5,7,...$ and
+ * $\phi_{l}(x) = (2x-1)\phi_{l-1} + \phi_{2}, l=4,6,8,...$.
  *
- * The degrees of freedom are the values at the vertices and the 
+ * The degrees of freedom are the values at the vertices and the
  * derivatives at the midpoint. Currently, we do not scale the
- * polynomials in any way, although better conditioning of the 
+ * polynomials in any way, although better conditioning of the
  * element stiffness matrix could possibly be achieved with scaling.
  *
  * Calling the constructor with a given index <tt>p</tt> will generate the
@@ -522,7 +533,7 @@ namespace Polynomials
       static
       std::vector<Polynomial<double> >
       generate_complete_basis (const unsigned int degree);
-    
+
     private:
 				     /**
 				      * Compute coefficients recursively.
@@ -538,22 +549,22 @@ namespace Polynomials
 				      */
      static const std::vector<double> &
      get_coefficients (const unsigned int p);
- 
+
      static std::vector<const std::vector<double> *> recursive_coefficients;
-   };  
+   };
 }
 
 /** @} */
 
 /* -------------------------- inline functions --------------------- */
 
-namespace Polynomials 
+namespace Polynomials
 {
   template <typename number>
   inline
-  Polynomial<number>::Polynomial () 
+  Polynomial<number>::Polynomial ()
   {}
-  
+
   template <typename number>
   inline
   unsigned int
@@ -562,6 +573,24 @@ namespace Polynomials
     Assert (coefficients.size()>0, ExcEmptyObject());
     return coefficients.size() - 1;
   }
+
+
+
+  template <typename number>
+  inline
+  number
+  Polynomial<number>::value (const number x) const
+  {
+    Assert (coefficients.size() > 0, ExcEmptyObject());
+    const unsigned int m=coefficients.size();
+
+                                     // Horner scheme
+    number value = coefficients.back();
+    for (int k=m-2; k>=0; --k)
+      value = value*x + coefficients[k];
+
+    return value;
+  }
 }
 DEAL_II_NAMESPACE_CLOSE
 
