@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: qprojector.h 16430 2008-07-08 15:25:01Z hartmann $
+//    $Id: qprojector.h 20163 2009-11-25 04:25:16Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -158,23 +158,18 @@ class QProjector
 				      *
 				      * The weights of the new rule
 				      * are replications of the
-				      * original weights. This is not
-				      * a proper handling, in that the
-				      * sum of weights does not equal
-				      * one, but it is consistent with
-				      * the use of this function,
-				      * namely to generate sets of
-				      * face quadrature points on a
-				      * cell, one set of which will
-				      * then be selected at each
-				      * time. This is used in the
-				      * FEFaceValues class,
-				      * where we initialize the
-				      * values, derivatives, etc on
-				      * all faces at once, while
-				      * selecting the data of one
-				      * particular face only happens
-				      * later.
+				      * original weights. Thus, the
+				      * sum of the weights is not one,
+				      * but the number of faces, which
+				      * is the surface of the
+				      * reference cell.
+				      *
+				      * This in particular allows us
+				      * to extract a subset of points
+				      * corresponding to a single face
+				      * and use it as a quadrature on
+				      * this face, as is done in
+				      * FEFaceValues.
 				      *
 				      * @note In 3D, this function
 				      * produces eight sets of
@@ -187,18 +182,32 @@ class QProjector
     project_to_all_faces (const SubQuadrature &quadrature);
 
 				     /**
-				      * This function is alike the
-				      * previous one, but projects the
-				      * given face quadrature formula
-				      * to the subfaces of a cell,
-				      * i.e. to the children of the
-				      * faces of the unit cell.
+				      * Take a face quadrature formula
+				      * and generate a cell quadrature
+				      * formula from it where the
+				      * quadrature points of the given
+				      * argument are projected on all
+				      * subfaces.
+				      *
+				      * Like in project_to_all_faces(),
+				      * the weights of the new rule
+				      * sum up to the number of faces
+				      * (not subfaces), which
+				      * is the surface of the
+				      * reference cell.
+				      *
+				      * This in particular allows us
+				      * to extract a subset of points
+				      * corresponding to a single subface
+				      * and use it as a quadrature on
+				      * this face, as is done in
+				      * FESubfaceValues.
 				      */
     static Quadrature<dim>
     project_to_all_subfaces (const SubQuadrature &quadrature);
 
 				     /**
-				      * Project a give quadrature
+				      * Project a given quadrature
 				      * formula to a child of a
 				      * cell. You may want to use this
 				      * function in case you want to
@@ -221,6 +230,28 @@ class QProjector
     project_to_child (const Quadrature<dim>  &quadrature,
 		      const unsigned int      child_no);
 
+				     /**
+				      * Project a quadrature rule to
+				      * all children of a
+				      * cell. Similarly to
+				      * project_to_all_subfaces(),
+				      * this function replicates the
+				      * formula generated by
+				      * project_to_child() for all
+				      * children, such that the
+				      * weights sum up to one, the
+				      * volume of the total cell
+				      * again.
+				      *
+				      * The child
+				      * numbering is the same as the
+				      * children would be numbered
+				      * upon refinement of the cell.
+				      */
+    static
+    Quadrature<dim>
+    project_to_all_children (const Quadrature<dim>  &quadrature);
+    
 				     /**
 				      * Project the onedimensional
 				      * rule <tt>quadrature</tt> to
@@ -421,10 +452,51 @@ class QProjector
 /*@}*/
 
 
+// -------------------  inline and template functions ----------------
+
+
+
+template <int dim>
+inline
+QProjector<dim>::DataSetDescriptor::
+DataSetDescriptor (const unsigned int dataset_offset)
+                :
+                dataset_offset (dataset_offset)
+{}
+
+
+template <int dim>
+inline
+QProjector<dim>::DataSetDescriptor::
+DataSetDescriptor ()
+                :
+                dataset_offset (numbers::invalid_unsigned_int)
+{}
+
+
+
+template <int dim>
+typename QProjector<dim>::DataSetDescriptor
+QProjector<dim>::DataSetDescriptor::cell ()
+{
+  return 0;
+}
+
+
+
+template <int dim>
+inline
+QProjector<dim>::DataSetDescriptor::operator unsigned int () const
+{
+  return dataset_offset;
+}
+
+
 /* -------------- declaration of explicit specializations ------------- */
 
 #ifndef DOXYGEN
 
+
 template <>
 void
 QProjector<1>::project_to_face (const Quadrature<0> &,
