@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: quadrature_lib.h 18650 2009-04-18 22:39:02Z bangerth $
+//    $Id: quadrature_lib.h 18907 2009-06-05 03:56:02Z hartmann $
 //    Version: $Name$
 //
 //    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009 by the deal.II authors
@@ -134,7 +134,7 @@ class QGaussLobatto : public Quadrature<dim>
 				      * $ \Gamma(n) = (n-1)! $. 
 				      * @param n  point of evaluation (integer).
 				      */
-    unsigned int gamma(const unsigned int n) const;
+    long double gamma(const unsigned int n) const;
 };
 
   
@@ -513,7 +513,7 @@ template <>
 long double QGaussLobatto<1>::
 JacobiP(const long double, const int, const int, const unsigned int) const;
 template <>
-unsigned int 
+long double 
 QGaussLobatto<1>::gamma(const unsigned int n) const;
 
 template <> std::vector<double> QGaussLog<1>::set_quadrature_points(const unsigned int) const;
