@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: bind.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: bind.h 20202 2009-12-04 15:12:15Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2009 by the deal.II authors
@@ -10,7 +10,7 @@
 //    further information on this license.
 //
 //---------------------------------------------------------------------------
-#ifndef __deal2__std_cxx1x_bin_h
+#ifndef __deal2__std_cxx1x_bind_h
 #define __deal2__std_cxx1x_bind_h
 
 
