@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tensor.h 17866 2008-12-05 22:27:44Z bangerth $
+//    $Id: tensor.h 18982 2009-06-28 19:17:36Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -1330,15 +1330,6 @@ template <int rank>
 inline
 double determinant (const Tensor<rank,1> &t)
 {
-				   // determinant of tensors of
-				   // dimension one and arbitrary rank
-				   // can be computed by recursion. we
-				   // need therefore not try to access
-				   // the number itself, which is
-				   // difficult since it needs <tt>rank</tt>
-				   // indirections, which is not
-				   // computable in the general
-				   // template
   return determinant(t[0]);
 }
 
@@ -1360,6 +1351,22 @@ double determinant (const Tensor<1,1> &t)
 
 
 
+/**
+ * Compute the determinant of a tensor of rank two and dimension
+ * one. Since this is a number, the return value is, of course, the
+ * number itself.
+ *
+ * @relates Tensor
+ * @author Wolfgang Bangerth, 1998
+ */
+inline
+double determinant (const Tensor<2,1> &t)
+{
+  return t[0][0];
+}
+
+
+
 /**
  * Compute the determinant of a tensor or rank 2, here for <tt>dim==2</tt>.
  *
@@ -1401,6 +1408,45 @@ double determinant (const Tensor<2,3> &t)
 
 
 
+/**
+ * Compute the determinant of a tensor or rank 2, here for all dimensions for
+ * which no explicit specialization is available above.
+ *
+ * @relates Tensor
+ * @author Wolfgang Bangerth, 2009
+ */
+template <int dim>
+inline
+double determinant (const Tensor<2,dim> &t)
+{
+				   // compute the determinant using the
+				   // Laplace expansion of the
+				   // determinant. this may not be the most
+				   // efficient algorithm, but it does for
+				   // small n.
+				   //
+				   // for some algorithmic simplicity, we use
+				   // the expansion along the last row
+  double det = 0;
+  
+  for (unsigned int k=0; k<dim; ++k)
+    {
+      Tensor<2,dim-1> minor;
+      for (unsigned int i=0; i<dim-1; ++i)
+	for (unsigned int j=0; j<dim-1; ++j)
+	  minor[i][j] = t[i][j<k ? j : j+1];
+
+      const double cofactor = std::pow (-1., static_cast<double>(k+1)) *
+			      determinant (minor);
+
+      det += t[dim-1][k] * cofactor;
+    }
+  
+  return std::pow (-1., static_cast<double>(dim)) * det;
+}
+
+
+
 /**
  * Compute and return the trace of a tensor of rank 2, i.e. the sum of
  * its diagonal entries.
