@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tensor_product_polynomials.h 14038 2006-10-23 02:46:34Z bangerth $
+//    $Id: tensor_product_polynomials.h 20161 2009-11-24 22:54:51Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -62,6 +62,14 @@ template <int dim>
 class TensorProductPolynomials
 {
   public:
+				     /**
+				      * Access to the dimension of
+				      * this object, for checking and
+				      * automatic setting of dimension
+				      * in other classes.
+				      */
+    static const unsigned int dimension = dim;
+    
 				     /**
 				      * Constructor. <tt>pols</tt> is
 				      * a vector of objects that
