@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: utilities.h 18745 2009-04-26 22:08:40Z bangerth $
+//    $Id: utilities.h 21358 2010-06-24 23:38:14Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -29,6 +29,7 @@ typedef int MPI_Comm;
 #ifdef DEAL_II_USE_TRILINOS
 #  include <Teuchos_RCP.hpp>
 #  include <Epetra_Comm.h>
+#  include <Epetra_Map.h>
 #  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
 #    include <Epetra_MpiComm.h>
 #  else
@@ -145,7 +146,7 @@ namespace Utilities
   std::pair<int, unsigned int>
   get_integer_at_position (const std::string &name,
 			   const unsigned int position);
-    
+
                                    /**
                                     * Generate a random number from a
                                     * normalized Gaussian probability
@@ -193,16 +194,16 @@ namespace Utilities
 				    */
   std::vector<unsigned int>
   invert_permutation (const std::vector<unsigned int> &permutation);
-  
+
                                    /**
                                     * A namespace for utility functions that
                                     * probe system properties.
-                                    * 
+                                    *
                                     * @ingroup utilities
                                     */
   namespace System
   {
-    
+
                                      /**
                                       * Return the CPU load as returned by
                                       * "uptime". Note that the interpretation
@@ -228,6 +229,23 @@ namespace Utilities
                                       */
     std::string get_time ();
 
+				     /**
+				      * Return whether (i) deal.II has
+				      * been compiled to support MPI
+				      * (for example by compiling with
+				      * <code>CXX=mpiCC</code>) and if
+				      * so whether (ii)
+				      * <code>MPI_Init()</code> has
+				      * been called (for example using
+				      * the
+				      * Utilities::System::MPI_InitFinalize
+				      * class). In other words, the
+				      * result indicates whether the
+				      * current job is running under
+				      * MPI.
+				      */
+    bool job_supports_mpi ();
+
 				     /**
 				      * Return the number of MPI processes
 				      * there exist in the given communicator
@@ -249,6 +267,70 @@ namespace Utilities
     unsigned int get_this_mpi_process (const MPI_Comm &mpi_communicator);
 
 
+				     /**
+				      * Consider an unstructured
+				      * communication pattern where
+				      * every process in an MPI
+				      * universe wants to send some
+				      * data to a subset of the other
+				      * processors. To do that, the
+				      * other processors need to know
+				      * who to expect messages
+				      * from. This function computes
+				      * this information.
+				      *
+				      * @param mpi_comm A communicator
+				      * that describes the processors
+				      * that are going to communicate
+				      * with each other.
+				      *
+				      * @param destinations The list
+				      * of processors the current
+				      * process wants to send
+				      * information to. This list need
+				      * not be sorted in any way. If
+				      * it contains duplicate entries
+				      * that means that multiple
+				      * messages are intended for a
+				      * given destination.
+				      *
+				      * @return A list of processors
+				      * that have indicated that they
+				      * want to send something to the
+				      * current processor. The
+				      * resulting list is not
+				      * sorted. It may contain
+				      * duplicate entries if
+				      * processors enter the same
+				      * destination more than once in
+				      * their destinations list.
+				      */
+    std::vector<unsigned int>
+    compute_point_to_point_communication_pattern (const MPI_Comm & mpi_comm,
+						  const std::vector<unsigned int> & destinations);
+
+
+
+				     /**
+				      * Given a communicator, generate a new
+				      * communicator that contains the same
+				      * set of processors but that has a
+				      * different, unique identifier.
+				      *
+				      * This functionality can be used to
+				      * ensure that different objects, such as
+				      * distributed matrices, each have unique
+				      * communicators over which they can
+				      * interact without interfering with each
+				      * other.
+				      *
+				      * When no longer needed, the
+				      * communicator created here needs to
+				      * be destroyed using
+				      * <code>MPI_Comm_free</code>.
+				      */
+    MPI_Comm duplicate_communicator (const MPI_Comm &mpi_communicator);
+
 				     /**
 				      * A class that is used to initialize the
 				      * MPI system at the beginning of a
@@ -283,7 +365,7 @@ namespace Utilities
 					  */
 	MPI_InitFinalize (int    &argc,
 			  char** &argv);
-	
+
 					 /**
 					  * Destructor. Calls
 					  * <tt>MPI_Finalize()</tt> in
@@ -322,10 +404,10 @@ namespace Utilities
    * classes such as matrices, vectors, and preconditioners. The most
    * important function in this class is <tt>comm()</tt>, which is needed
    * for the initialization of Trilinos Epetra_Maps, which design the
-   * parallel distribution of vectors and matrices. Moreover, this class
-   * provides a unified interface to both serial and parallel
+   * %parallel distribution of vectors and matrices. Moreover, this class
+   * provides a unified interface to both serial and %parallel
    * implementations of Trilinos, sets up the MPI communicator in case the
-   * programs are run in parallel, and correctly terminates all processes
+   * programs are run in %parallel, and correctly terminates all processes
    * when the destructor is called.
    */
 #ifdef DEAL_II_USE_TRILINOS
@@ -347,6 +429,120 @@ namespace Utilities
 				      */
     const Epetra_Comm& comm_world();
 
+				     /**
+				      * Returns a Trilinos Epetra_Comm
+				      * object needed for creation of
+				      * Epetra_Maps.
+				      *
+				      * If deal.II has been configured to use
+				      * a compiler that does not support MPI
+				      * then the resulting communicator will
+				      * be a serial one. Otherwise, the
+				      * communicator will correspond to
+				      * MPI_COMM_SELF, i.e. a communicator
+				      * that comprises only this one
+				      * processor.
+				      */
+    const Epetra_Comm& comm_self();
+
+				     /**
+				      * Given a communicator, duplicate it. If
+				      * the given communicator is serial, that
+				      * means to just return a copy of
+				      * itself. On the other hand, if it is
+				      * %parallel, we duplicate the underlying
+				      * MPI_Comm object: we create a separate
+				      * MPI communicator that contains the
+				      * same processors and in the same order
+				      * but has a separate identifier distinct
+				      * from the given communicator. The
+				      * function returns a pointer to a new
+				      * object of a class derived from
+				      * Epetra_Comm. The caller of this
+				      * function needs to assume ownership of
+				      * this function. The returned object
+				      * should be destroyed using the
+				      * destroy_communicator() function.
+				      *
+				      * This facility is used to separate
+				      * streams of communication. For example,
+				      * a program could simply use
+				      * MPI_Comm_World for everything. But it
+				      * is easy to come up with scenarios
+				      * where sometimes not all processors
+				      * participate in a communication that is
+				      * intended to be global -- for example
+				      * if we assemble a matrix on a coarse
+				      * mesh with fewer cells than there are
+				      * processors, some processors may not
+				      * sync their matrices with the rest
+				      * because they haven't written into it
+				      * because they own no cells. That's
+				      * clearly a bug. However, if these
+				      * processors just continue their work,
+				      * and the next %parallel operation
+				      * happens to be a sync on a different
+				      * matrix, then the sync could succeed --
+				      * by accident, since different
+				      * processors are talking about different
+				      * matrices.
+				      *
+				      * This kind of situation can be avoided
+				      * if we use different communicators for
+				      * different matrices which reduces the
+				      * likelihood that communications meant
+				      * to be separate aren't recognized as
+				      * such just because they happen on the
+				      * same communicator. In addition, it is
+				      * conceivable that some MPI operations
+				      * can be parallelized using multiple
+				      * threads because their communicators
+				      * identifies the communication in
+				      * question, not their relative timing as
+				      * is the case in a sequential program
+				      * that just uses a single communicator.
+				      */
+    Epetra_Comm *
+    duplicate_communicator (const Epetra_Comm &communicator);
+
+				     /**
+				      * Given an Epetra communicator that was
+				      * created by the
+				      * duplicate_communicator() function,
+				      * destroy the underlying MPI
+				      * communicator object and reset the
+				      * Epetra_Comm object to a the result of
+				      * comm_self().
+				      *
+				      * It is necessary to call this function
+				      * at the time when the result of
+				      * duplicate_communicator() is no longer
+				      * needed. The reason is that in that
+				      * function, we first create a new
+				      * MPI_Comm object and then create an
+				      * Epetra_Comm around it. While we can
+				      * take care of destroying the latter, it
+				      * doesn't destroy the communicator since
+				      * it can only assume that it may also be
+				      * still used by other objects in the
+				      * program. Consequently, we have to take
+				      * care of destroying it ourselves,
+				      * explicitly.
+				      *
+				      * This function does exactly
+				      * that. Because this has to happen while
+				      * the Epetra_Comm object is still
+				      * around, it first resets the latter and
+				      * then destroys the communicator object.
+				      *
+				      * @note If you call this function on an
+				      * Epetra_Comm object that is not created
+				      * by duplicate_communicator(), you are
+				      * likely doing something quite
+				      * wrong. Don't do this.
+				      */
+    void
+    destroy_communicator (Epetra_Comm &communicator);
 
 				     /**
 				      * Return the number of MPI processes
@@ -367,6 +563,25 @@ namespace Utilities
 				      * get_n_mpi_processes()).
 				      */
     unsigned int get_this_mpi_process (const Epetra_Comm &mpi_communicator);
+
+				     /**
+				      * Given a Trilinos Epetra map, create a
+				      * new map that has the same subdivision
+				      * of elements to processors but uses the
+				      * given communicator object instead of
+				      * the one stored in the first
+				      * argument. In essence, this means that
+				      * we create a map that communicates
+				      * among the same processors in the same
+				      * way, but using a separate channel.
+				      *
+				      * This function is typically used with a
+				      * communicator that has been obtained by
+				      * the duplicate_communicator() function.
+				      */
+    Epetra_Map
+    duplicate_map (const Epetra_BlockMap  &map,
+		   const Epetra_Comm &comm);
   }
 
 #endif
@@ -394,13 +609,13 @@ namespace Utilities
 	case 4:
 	      return n*n*n*n;
 	default:
-	      T result = n;  
+	      T result = n;
 	      for (unsigned int d=1;d<N;++d)
 		result *= n;
 	      return result;
       }
   }
-  
+
 }
 
 
