@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: boost_threads.cc 18259 2009-01-22 00:13:04Z bangerth $
+//    $Id: boost_threads.cc 20951 2010-04-06 14:54:38Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2009 by the deal.II authors
+//    Copyright (C) 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,9 +13,9 @@
 
 #include <base/config.h>
 
-#if (DEAL_II_USE_MT == 1) && !defined(DEAL_II_CAN_USE_CXX0X)
+#if (DEAL_II_USE_MT == 1) && !defined(DEAL_II_CAN_USE_CXX1X)
 
-// of the C++ compiler doesn't completely support the C++0x standard (and
+// if the C++ compiler doesn't completely support the C++1x standard (and
 // consequently we can't use std::thread, std::mutex, etc), then include all
 // the files that form BOOST's thread implementation so that we don't have to
 // build BOOST itself only to get at this small part of it. it also ensures
@@ -31,13 +31,11 @@
 #      define UINTMAX_C(x) x ## ULL
 #    endif
 
-#    include <../libs/thread/src/pthread/once.cpp>
-#    include <../libs/thread/src/pthread/exceptions.cpp>
-#    include <../libs/thread/src/pthread/thread.cpp>
+#    include "../contrib/boost/libs/thread/src/pthread/once.cpp"
+#    include "../contrib/boost/libs/thread/src/pthread/thread.cpp"
 #  else
-#    include <../libs/thread/src/win32/once.cpp>
-#    include <../libs/thread/src/win32/exceptions.cpp>
-#    include <../libs/thread/src/win32/thread.cpp>
+#    include "../contrib/boost/libs/thread/src/win32/once.cpp"
+#    include "../contrib/boost/libs/thread/src/win32/thread.cpp"
 #  endif
 
 #endif
