@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: function_lib.cc 14490 2007-02-17 03:52:57Z kanschat $
+//    $Id: function_lib.cc 19451 2009-09-13 17:47:26Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -412,7 +412,15 @@ namespace Functions
   }
   
 //////////////////////////////////////////////////////////////////////
-  
+
+  template <int dim>
+  CosineFunction<dim>::CosineFunction (const unsigned int n_components)
+		  :
+		Function<dim> (n_components)
+{}
+
+
+
   template<int dim>
   double
   CosineFunction<dim>::value (const Point<dim>   &p,
@@ -441,26 +449,29 @@ namespace Functions
     Assert (values.size() == points.size(),
 	    ExcDimensionMismatch(values.size(), points.size()));
     
+    for (unsigned int i=0;i<points.size();++i)
+      values[i] = value(points[i]);
+  }
+  
+  
+  template<int dim>
+  void
+  CosineFunction<dim>::vector_value_list (
+    const std::vector<Point<dim> > &points,
+    std::vector<Vector<double> >   &values) const
+  {
+    Assert (values.size() == points.size(),
+	    ExcDimensionMismatch(values.size(), points.size()));
+    
     for (unsigned int i=0;i<points.size();++i)
       {
-	const Point<dim>& p = points[i];
-	switch(dim)
-	  {
-	    case 1:
-		  values[i] = std::cos(M_PI_2*p(0));
-		  break;
-	    case 2:
-		  values[i] = std::cos(M_PI_2*p(0)) * std::cos(M_PI_2*p(1));
-		  break;
-	    case 3:
-		  values[i] = std::cos(M_PI_2*p(0)) * std::cos(M_PI_2*p(1)) * std::cos(M_PI_2*p(2));
-		  break;
-	    default:
-		  Assert(false, ExcNotImplemented());
-	  }
+	const double v = value(points[i]);
+	for (unsigned int k=0;k<values[i].size();++k)
+	  values[i](k) = v;
       }
   }
   
+  
   template<int dim>
   double
   CosineFunction<dim>::laplacian (const Point<dim>   &p,
@@ -490,23 +501,7 @@ namespace Functions
 	    ExcDimensionMismatch(values.size(), points.size()));
     
     for (unsigned int i=0;i<points.size();++i)
-      {
-	const Point<dim>& p = points[i];
-	switch(dim)
-	  {
-	    case 1:
-		  values[i] = -M_PI_2*M_PI_2* std::cos(M_PI_2*p(0));
-		  break;
-	    case 2:
-		  values[i] = -2*M_PI_2*M_PI_2* std::cos(M_PI_2*p(0)) * std::cos(M_PI_2*p(1));
-		  break;
-	    case 3:
-		  values[i] = -3*M_PI_2*M_PI_2* std::cos(M_PI_2*p(0)) * std::cos(M_PI_2*p(1)) * std::cos(M_PI_2*p(2));
-		  break;
-	    default:
-		  Assert(false, ExcNotImplemented());
-	  }
-      }
+      values[i] = laplacian(points[i]);
   }
   
   template<int dim>
