@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//      $Id: utilities.cc 17434 2008-10-31 17:49:59Z bangerth $   
+//      $Id: utilities.cc 21162 2010-06-07 20:47:13Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2008 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -32,6 +32,18 @@
 #  include <limits.h>
 #endif
 
+#ifdef DEAL_II_USE_TRILINOS
+#  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+#    include <Epetra_MpiComm.h>
+#    include <lac/vector_memory.h>
+#    include <lac/trilinos_vector.h>
+#    include <lac/trilinos_block_vector.h>
+#  endif
+#  include "Epetra_SerialComm.h"
+#endif
+
+
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -51,7 +63,7 @@ namespace Utilities
 		  << "Can't convert the string " << arg1
                   << " to the desired type");
 
-	  
+
   std::string
   int_to_string (const unsigned int i,
 		 const unsigned int digits)
@@ -60,8 +72,8 @@ namespace Utilities
 				     // not pad the resulting string at all
     if (digits == numbers::invalid_unsigned_int)
       return int_to_string (i, needed_digits(i));
-    
-    
+
+
     AssertThrow ( ! ((digits==1 && i>=10)   ||
 		     (digits==2 && i>=100)  ||
 		     (digits==3 && i>=1000) ||
@@ -69,9 +81,9 @@ namespace Utilities
 		     (digits==5 && i>=100000)||
 		     (i>=1000000)),
 		  ExcInvalidNumber2StringConversersion(i, digits));
-  
+
     std::string s;
-    switch (digits) 
+    switch (digits)
       {
 	case 6:
 	      s += '0' + i/100000;
@@ -114,7 +126,7 @@ namespace Utilities
   }
 
 
-  
+
   int
   string_to_int (const std::string &s)
   {
@@ -125,16 +137,16 @@ namespace Utilities
 #else
     static const int max_int = INT_MAX;
 #endif
-    
+
     int i = max_int;
     ss >> i;
 
                                      // check for errors
     AssertThrow (i != max_int, ExcCantConvertString (s));
-    
+
     return i;
   }
-  
+
 
 
   std::vector<int>
@@ -169,7 +181,7 @@ namespace Utilities
 	  }
 	else
 	  tmp = "";
-      
+
 	while ((name.length() != 0) &&
 	       (name[0] == ' '))
 	  name.erase (0,1);
@@ -196,7 +208,7 @@ namespace Utilities
                                      // remove trailing spaces
     while ((text.length() != 0) && (text[text.length()-1] == delimiter))
       text.erase(text.length()-1,1);
-  
+
                                      // then split the text into lines
     while (text.length() != 0)
       {
@@ -222,7 +234,7 @@ namespace Utilities
             for (; location>0; --location)
               if (text[location] == delimiter)
                 break;
-          
+
                                              // if there are no spaces, then try if
                                              // there are spaces coming up
             if (location == 0)
@@ -231,7 +243,7 @@ namespace Utilities
                    ++location)
                 if (text[location] == delimiter)
                   break;
-          
+
                                              // now take the text up to the found
                                              // location and put it into a single
                                              // line, and remove it from 'text'
@@ -239,10 +251,10 @@ namespace Utilities
             text.erase (0, location);
           }
       }
-  
+
     return lines;
   }
-  
+
 
 
   bool
@@ -259,17 +271,17 @@ namespace Utilities
     return true;
   }
 
-  
+
 
   std::pair<int, unsigned int>
   get_integer_at_position (const std::string &name,
 			   const unsigned int position)
   {
     Assert (position < name.size(), ExcInternalError());
-    
+
     const std::string test_string (name.begin()+position,
 				   name.end());
-    
+
     std::istringstream str(test_string);
 
     int i;
@@ -303,7 +315,7 @@ namespace Utilities
       return std::make_pair (-1, numbers::invalid_unsigned_int);
   }
 
-  
+
 
   double
   generate_normal_random_number (const double a,
@@ -313,7 +325,7 @@ namespace Utilities
     if (sigma == 0)
       return a;
 
-#ifdef HAVE_RAND_R  
+#ifdef HAVE_RAND_R
     static unsigned int seed = 0xabcd1234;
     const double y = 1.0*rand_r(&seed)/RAND_MAX;
 #else
@@ -349,7 +361,7 @@ namespace Utilities
   reverse_permutation (const std::vector<unsigned int> &permutation)
   {
     const unsigned int n = permutation.size();
-    
+
     std::vector<unsigned int> out (n);
     for (unsigned int i=0; i<n; ++i)
       out[i] = n - 1 - permutation[i];
@@ -363,9 +375,9 @@ namespace Utilities
   invert_permutation (const std::vector<unsigned int> &permutation)
   {
     const unsigned int n = permutation.size();
-    
+
     std::vector<unsigned int> out (n, numbers::invalid_unsigned_int);
-    
+
     for (unsigned int i=0; i<n; ++i)
       {
 	Assert (permutation[i] < n, ExcIndexRange (permutation[i], 0, n));
@@ -377,10 +389,10 @@ namespace Utilities
     for (unsigned int i=0; i<n; ++i)
       Assert (out[i] != numbers::invalid_unsigned_int,
 	      ExcMessage ("The given input permutation had duplicate entries!"));
-    
+
     return out;
   }
-  
+
 
 
 
@@ -392,7 +404,7 @@ namespace Utilities
     {
       std::ifstream cpuinfo;
       cpuinfo.open("/proc/loadavg");
-    
+
       AssertThrow(cpuinfo, ExcIO());
 
       double load;
@@ -402,12 +414,12 @@ namespace Utilities
     }
 
 #else
-  
+
     double get_cpu_load ()
     {
       return 0.;
     }
-  
+
 #endif
 
 
@@ -424,7 +436,7 @@ namespace Utilities
     std::string get_time ()
     {
       std::time_t  time1= std::time (0);
-      std::tm     *time = std::localtime(&time1); 
+      std::tm     *time = std::localtime(&time1);
 
       std::ostringstream o;
       o << time->tm_hour << ":"
@@ -434,8 +446,17 @@ namespace Utilities
       return o.str();
     }
 
-    
+
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+
+    bool job_supports_mpi ()
+    {
+      int MPI_has_been_started = 0;
+      MPI_Initialized(&MPI_has_been_started);
+
+      return (MPI_has_been_started > 0);
+    }
+
                                 // Unfortunately, we have to work
                                 // around an oddity in the way PETSc
                                 // and some gcc versions interact. If
@@ -473,30 +494,123 @@ namespace Utilities
     {
       int n_jobs=1;
       (void) MPI_Comm_size (mpi_communicator, &n_jobs);
-      
+
       return n_jobs;
     }
 
-    
+
     unsigned int get_this_mpi_process (const MPI_Comm &mpi_communicator)
     {
       int rank=0;
       (void) MPI_Comm_rank (mpi_communicator, &rank);
-      
+
       return rank;
     }
-    
+
+
+    MPI_Comm duplicate_communicator (const MPI_Comm &mpi_communicator)
+    {
+      MPI_Comm new_communicator;
+      MPI_Comm_dup (mpi_communicator, &new_communicator);
+      return new_communicator;
+    }
+
+
+    std::vector<unsigned int>
+    compute_point_to_point_communication_pattern (const MPI_Comm & mpi_comm,
+						  const std::vector<unsigned int> & destinations)
+    {
+      unsigned int myid = Utilities::System::get_this_mpi_process(mpi_comm);
+      unsigned int n_procs = Utilities::System::get_n_mpi_processes(mpi_comm);
+
+      for (unsigned int i=0; i<destinations.size(); ++i)
+	{
+	  Assert (destinations[i] < n_procs,
+		  ExcIndexRange (destinations[i], 0, n_procs));
+	  Assert (destinations[i] != myid,
+		  ExcMessage ("There is no point in communicating with ourselves."));
+	}
+
+
+				       // let all processors
+				       // communicate the maximal
+				       // number of destinations they
+				       // have
+      unsigned int my_n_destinations = destinations.size();
+      unsigned int max_n_destinations = 0;
+
+      MPI_Allreduce (&my_n_destinations, &max_n_destinations, 1, MPI_UNSIGNED,
+		    MPI_MAX, mpi_comm);
+
+				       // now that we know the number
+				       // of data packets every
+				       // processor wants to send, set
+				       // up a buffer with the maximal
+				       // size and copy our
+				       // destinations in there,
+				       // padded with -1's
+      std::vector<unsigned int> my_destinations(max_n_destinations,
+						numbers::invalid_unsigned_int);
+      std::copy (destinations.begin(), destinations.end(),
+		 my_destinations.begin());
+
+				       // now exchange these (we could
+				       // communicate less data if we
+				       // used MPI_Allgatherv, but
+				       // we'd have to communicate
+				       // my_n_destinations to all
+				       // processors in this case,
+				       // which is more expensive than
+				       // the reduction operation
+				       // above in MPI_Allreduce)
+      std::vector<unsigned int> all_destinations (max_n_destinations * n_procs);
+      MPI_Allgather (&my_destinations[0], max_n_destinations, MPI_UNSIGNED,
+		     &all_destinations[0], max_n_destinations, MPI_UNSIGNED,
+		     mpi_comm);
+
+				       // now we know who is going to
+				       // communicate with
+				       // whom. collect who is going
+				       // to communicate with us!
+      std::vector<unsigned int> origins;
+      for (unsigned int i=0; i<n_procs; ++i)
+	for (unsigned int j=0; j<max_n_destinations; ++j)
+	  if (all_destinations[i*max_n_destinations + j] == myid)
+	    origins.push_back (i);
+	  else if (all_destinations[i*max_n_destinations + j] ==
+		   numbers::invalid_unsigned_int)
+	    break;
+
+      return origins;
+    }
+
 #else
-    
+
+    bool job_supports_mpi ()
+    {
+      return false;
+    }
+
+
+
     unsigned int get_n_mpi_processes (const MPI_Comm &)
     {
       return 1;
     }
 
+
+
     unsigned int get_this_mpi_process (const MPI_Comm &)
     {
       return 0;
     }
+
+
+
+    MPI_Comm duplicate_communicator (const MPI_Comm &mpi_communicator)
+    {
+      return mpi_communicator;
+    }
 #endif
 
 
@@ -509,8 +623,8 @@ namespace Utilities
       static bool constructor_has_already_run = false;
       Assert (constructor_has_already_run == false,
 	      ExcMessage ("You can only create a single object of this class "
-			  "in a program sinc it initialize the MPI system."));
-      
+			  "in a program since it initializes the MPI system."));
+
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
       int MPI_has_been_started = 0;
       MPI_Initialized(&MPI_has_been_started);
@@ -535,9 +649,40 @@ namespace Utilities
     MPI_InitFinalize::~MPI_InitFinalize()
     {
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+
+#  if defined(DEAL_II_USE_TRILINOS) && !defined(__APPLE__)
+				       // make memory pool release all
+				       // vectors that are no longer
+				       // used at this point. this is
+				       // relevant because the static
+				       // object destructors run for
+				       // these vectors at the end of
+				       // the program would run after
+				       // MPI_Finalize is called,
+				       // leading to errors
+				       //
+				       // TODO: On Mac OS X, shared libs can
+ 				       // only depend on other libs listed
+ 				       // later on the command line. This
+ 				       // means that libbase can't depend on
+ 				       // liblac, and we can't destroy the
+ 				       // memory pool here as long as we have
+ 				       // separate libraries. Consequently,
+ 				       // the #ifdef above. Deal will then
+ 				       // just continue to seg fault upon
+ 				       // completion of main()
+      GrowingVectorMemory<TrilinosWrappers::MPI::Vector>
+	::release_unused_memory ();
+      GrowingVectorMemory<TrilinosWrappers::MPI::BlockVector>
+	::release_unused_memory ();
+#  endif
+
       int mpi_err = 0;
 
-      if (program_uses_mpi() == true && owns_mpi == true)
+      int MPI_has_been_started = 0;
+      MPI_Initialized(&MPI_has_been_started);
+      if (program_uses_mpi() == true && owns_mpi == true &&
+	  MPI_has_been_started != 0)
 	mpi_err = MPI_Finalize();
 
       AssertThrow (mpi_err == 0,
@@ -558,12 +703,12 @@ namespace Utilities
 #else
       return false;
 #endif
-    }    
+    }
   }
-  
+
 
 #ifdef DEAL_II_USE_TRILINOS
-  
+
   namespace Trilinos
   {
     const Epetra_Comm&
@@ -576,23 +721,120 @@ namespace Utilities
       static Teuchos::RCP<Epetra_SerialComm>
 	communicator = Teuchos::rcp (new Epetra_SerialComm (), true);
 #endif
-    
+
       return *communicator;
     }
 
 
+
+    const Epetra_Comm&
+    comm_self()
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      static Teuchos::RCP<Epetra_MpiComm>
+	communicator = Teuchos::rcp (new Epetra_MpiComm (MPI_COMM_SELF), true);
+#else
+      static Teuchos::RCP<Epetra_SerialComm>
+	communicator = Teuchos::rcp (new Epetra_SerialComm (), true);
+#endif
+
+      return *communicator;
+    }
+
+
+
+    Epetra_Comm *
+    duplicate_communicator (const Epetra_Comm &communicator)
+    {
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+
+				       // see if the communicator is in fact a
+				       // parallel MPI communicator; if so,
+				       // return a duplicate of it
+      const Epetra_MpiComm
+	*mpi_comm = dynamic_cast<const Epetra_MpiComm *>(&communicator);
+      if (mpi_comm != 0)
+	return new Epetra_MpiComm(Utilities::System::
+				  duplicate_communicator(mpi_comm->GetMpiComm()));
+#endif
+
+				       // if we don't support MPI, or if the
+				       // communicator in question was in fact
+				       // not an MPI communicator, return a
+				       // copy of the same object again
+      Assert (dynamic_cast<const Epetra_SerialComm*>(&communicator)
+	      != 0,
+	      ExcInternalError());
+      return new Epetra_SerialComm(dynamic_cast<const Epetra_SerialComm&>(communicator));
+    }
+
+
+
+    void destroy_communicator (Epetra_Comm &communicator)
+    {
+      Assert (&communicator != 0, ExcInternalError());
+
+				       // save the communicator, reset
+				       // the map, and delete the
+				       // communicator if this whole
+				       // thing was created as an MPI
+				       // communicator
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+      Epetra_MpiComm
+	*mpi_comm = dynamic_cast<Epetra_MpiComm *>(&communicator);
+      if (mpi_comm != 0)
+	{
+	  MPI_Comm comm = mpi_comm->GetMpiComm();
+	  *mpi_comm = Epetra_MpiComm(MPI_COMM_SELF);
+	  MPI_Comm_free (&comm);
+	}
+#endif
+    }
+
+
+
     unsigned int get_n_mpi_processes (const Epetra_Comm &mpi_communicator)
     {
       return mpi_communicator.NumProc();
     }
-    
+
 
     unsigned int get_this_mpi_process (const Epetra_Comm &mpi_communicator)
     {
       return (unsigned int)mpi_communicator.MyPID();
     }
+
+
+
+    Epetra_Map
+    duplicate_map (const Epetra_BlockMap &map,
+		   const Epetra_Comm     &comm)
+    {
+      if (map.LinearMap() == true)
+	{
+					   // each processor stores a
+					   // contiguous range of
+					   // elements in the
+					   // following constructor
+					   // call
+	  return Epetra_Map (map.NumGlobalElements(),
+			     map.NumMyElements(),
+			     map.IndexBase(),
+			     comm);
+	}
+      else
+	{
+					   // the range is not
+					   // contiguous
+	  return Epetra_Map (map.NumGlobalElements(),
+			     map.NumMyElements(),
+			     map.MyGlobalElements (),
+			     0,
+			     comm);
+	}
+    }
   }
-  
+
 #endif
 
 }
