@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_accessor.h 17892 2008-12-08 18:45:16Z bangerth $
+//    $Id: dof_accessor.h 21058 2010-05-03 22:15:00Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -26,6 +26,7 @@ DEAL_II_NAMESPACE_OPEN
 template <typename number> class FullMatrix;
 template <typename number> class SparseMatrix;
 template <typename number> class Vector;
+class ConstraintMatrix;
 
 template <typename Accessor> class TriaRawIterator;
 
@@ -65,11 +66,11 @@ namespace internal
 /**
  * This is a switch class which only declares a @p typedef. It is meant to
  * determine which class a DoFAccessor class is to be derived from. By
- * default, <tt>DoFAccessor@<celldim,dim,spacedim@></tt> derives from the
+ * default, <tt>DoFAccessor@<structdim,dim,spacedim@></tt> derives from the
  * typedef in the general
- * <tt>Inheritance@<celldim,dim,spacedim@></tt> class, which is
- * <tt>TriaAccessor@<celldim,dim,spacedim@></tt>, but if
- * <tt>celldim==dim</tt>, then the specialization
+ * <tt>Inheritance@<structdim,dim,spacedim@></tt> class, which is
+ * <tt>TriaAccessor@<structdim,dim,spacedim@></tt>, but if
+ * <tt>structdim==dim</tt>, then the specialization
  * <tt>Inheritance@<dim,dim,spacedim@></tt> is used which declares
  * its local type to be <tt>CellAccessor@<dim,spacedim@></tt>. Therefore, the
  * inheritance is automatically chosen to be from CellAccessor if the object
@@ -79,7 +80,7 @@ namespace internal
  * @ingroup Accessors 
  * @author Wolfgang Bangerth, 1999
  */
-    template <int celldim, int dim, int spacedim>
+    template <int structdim, int dim, int spacedim>
     struct Inheritance 
     {
 					 /**
@@ -87,7 +88,7 @@ namespace internal
 					  * See the full documentation for
 					  * more information.
 					  */
-	typedef dealii::TriaAccessor<celldim,dim,spacedim> BaseClass;
+	typedef dealii::TriaAccessor<structdim,dim,spacedim> BaseClass;
     };
 
 
@@ -279,6 +280,15 @@ class DoFAccessor : public internal::DoFAccessor::Inheritance<structdim, DH::dim
 				      */
     void copy_from (const DoFAccessor<structdim, DH> &a);
 
+				     /**
+				      * Copy operator used by the
+				      * iterator class. Keeps the
+				      * previously set dof handler,
+				      * but sets the object
+				      * coordinates of the TriaAccessor.
+				      */
+    void copy_from (const TriaAccessorBase<structdim, DH::dimension, DH::space_dimension> &da);
+    
 				     /**
 				      *  @name Accessing sub-objects
 				      */
@@ -764,7 +774,13 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
 				      * simpler.
 				      */
     typedef DoFAccessor<DH::dimension,DH> BaseClass;
-    
+
+    				     /**
+				      * Define the type of the
+				      * container this is part of.
+				      */
+    typedef DH Container;
+
 				     /**
 				      * @name Constructors
 				      */
@@ -917,6 +933,76 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
     void get_dof_values (const InputVector &values,
 			 Vector<number>    &local_values) const;
 
+    				     /**
+				      * Return the values of the given vector
+				      * restricted to the dofs of this
+				      * cell in the standard ordering: dofs
+				      * on vertex 0, dofs on vertex 1, etc,
+				      * dofs on line 0, dofs on line 1, etc,
+				      * dofs on quad 0, etc.
+				      *
+				      * The vector has to have the
+				      * right size before being passed
+				      * to this function. This
+				      * function is only callable for
+				      * active cells.
+				      *
+				      * The input vector may be either
+				      * a <tt>Vector<float></tt>,
+				      * Vector<double>, or a
+				      * BlockVector<double>, or a
+				      * PETSc or Trilinos vector if
+				      * deal.II is compiled to support
+				      * these libraries. It is in the
+				      * responsibility of the caller
+				      * to assure that the types of
+				      * the numbers stored in input
+				      * and output vectors are
+				      * compatible and with similar
+				      * accuracy.
+				      */
+    template <class InputVector, typename ForwardIterator>
+    void get_dof_values (const InputVector &values,
+			 ForwardIterator    local_values_begin,
+			 ForwardIterator    local_values_end) const;
+
+    				     /**
+				      * Return the values of the given vector
+				      * restricted to the dofs of this
+				      * cell in the standard ordering: dofs
+				      * on vertex 0, dofs on vertex 1, etc,
+				      * dofs on line 0, dofs on line 1, etc,
+				      * dofs on quad 0, etc.
+				      *
+				      * The vector has to have the
+				      * right size before being passed
+				      * to this function. This
+				      * function is only callable for
+				      * active cells.
+				      *
+				      * The input vector may be either a
+				      * <tt>Vector<float></tt>,
+				      * Vector<double>, or a
+				      * BlockVector<double>, or a PETSc or
+				      * Trilinos vector if deal.II is
+				      * compiled to support these
+				      * libraries. It is in the
+				      * responsibility of the caller to
+				      * assure that the types of the numbers
+				      * stored in input and output vectors
+				      * are compatible and with similar
+				      * accuracy. The ConstraintMatrix
+				      * passed as an argument to this
+				      * function makes sure that constraints
+				      * are correctly distributed when the
+				      * dof values are calculated.
+				      */
+    template <class InputVector, typename ForwardIterator>
+    void get_dof_values (const ConstraintMatrix &constraints,
+			 const InputVector      &values,
+			 ForwardIterator         local_values_begin,
+			 ForwardIterator         local_values_end) const;
+
 				     /**
 				      * This function is the counterpart to
 				      * get_dof_values(): it takes a vector
@@ -1105,6 +1191,50 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
     distribute_local_to_global (const Vector<number> &local_source,
                                 OutputVector         &global_destination) const;
 
+				     /**
+				      * Distribute a local (cell based)
+				      * vector in iterator format to a
+				      * global one by mapping the local
+				      * numbering of the degrees of freedom
+				      * to the global one and entering the
+				      * local values into the global vector.
+				      *
+				      * The elements are <em>added</em> up
+				      * to the elements in the global
+				      * vector, rather than just set, since
+				      * this is usually what one wants.
+				      */
+    template <typename ForwardIterator, typename OutputVector>
+    void
+    distribute_local_to_global (ForwardIterator   local_source_begin,
+				ForwardIterator   local_source_end,
+				OutputVector     &global_destination) const;
+
+				     /**
+				      * Distribute a local (cell based)
+				      * vector in iterator format to a
+				      * global one by mapping the local
+				      * numbering of the degrees of freedom
+				      * to the global one and entering the
+				      * local values into the global vector.
+				      *
+				      * The elements are <em>added</em> up
+				      * to the elements in the global
+				      * vector, rather than just set, since
+				      * this is usually what one
+				      * wants. Moreover, the
+				      * ConstraintMatrix passed to this
+				      * function makes sure that also
+				      * constraints are eliminated in this
+				      * process.
+				      */
+    template <typename ForwardIterator, typename OutputVector>
+    void
+    distribute_local_to_global (const ConstraintMatrix &constraints,
+                                ForwardIterator         local_source_begin,
+				ForwardIterator         local_source_end,
+				OutputVector           &global_destination) const;
+
 				     /**
 				      * This function does much the
 				      * same as the
@@ -1121,6 +1251,19 @@ class DoFCellAccessor :  public DoFAccessor<DH::dimension,DH>
     distribute_local_to_global (const FullMatrix<number> &local_source,
                                 OutputMatrix             &global_destination) const;
     
+				     /**
+				      * This function does what the two
+				      * <tt>distribute_local_to_global</tt>
+				      * functions with vector and matrix
+				      * argument do, but all at once.
+				      */
+    template <typename number, typename OutputMatrix, typename OutputVector>
+    void
+    distribute_local_to_global (const FullMatrix<number> &local_matrix,
+				const Vector<number>     &local_vector,
+                                OutputMatrix             &global_matrix,
+				OutputVector             &global_vector) const;
+    
 				     /**
 				      * @}
 				      */
