@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_handler.h 17907 2008-12-10 17:36:06Z bangerth $
+//    $Id: dof_handler.h 21278 2010-06-23 13:40:05Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -18,6 +18,7 @@
 #include <base/config.h>
 #include <base/exceptions.h>
 #include <base/smartpointer.h>
+#include <dofs/block_info.h>
 #include <dofs/dof_iterator_selector.h>
 #include <dofs/function_map.h>
 
@@ -41,7 +42,7 @@ namespace internal
   {
     struct Implementation;
   }
-  
+
   namespace DoFCellAccessor
   {
     struct Implementation;
@@ -81,7 +82,7 @@ namespace internal
  * so that this class can be used to solve problems in the finite
  * element method formulation.
  *
- * 
+ *
  * <h3>Distribution of indices for degrees of freedom</h3>
  *
  * The degrees of freedom (`dofs') are distributed on the given triangulation
@@ -128,6 +129,9 @@ class DoFHandler  :  public Subscriptor
 {
     typedef internal::DoFHandler::Iterators<DoFHandler<dim,spacedim> > IteratorSelector;
   public:
+    typedef typename IteratorSelector::CellAccessor         cell_accessor;
+    typedef typename IteratorSelector::FaceAccessor         face_accessor;
+    
     typedef typename IteratorSelector::raw_line_iterator    raw_line_iterator;
     typedef typename IteratorSelector::line_iterator        line_iterator;
     typedef typename IteratorSelector::active_line_iterator active_line_iterator;
@@ -204,18 +208,18 @@ class DoFHandler  :  public Subscriptor
 				      * value.
 				      */
     static const unsigned int default_fe_index = 0;
-    
+
 				     /**
 				      * Constructor. Take @p tria as the
 				      * triangulation to work on.
 				      */
-    DoFHandler ( const Triangulation<dim,spacedim> &tria);    
+    DoFHandler ( const Triangulation<dim,spacedim> &tria);
 
 				     /**
 				      * Destructor.
 				      */
     virtual ~DoFHandler ();
-    
+
 				     /**
 				      * Go through the triangulation and
 				      * distribute the degrees of freedoms
@@ -250,6 +254,19 @@ class DoFHandler  :  public Subscriptor
     virtual void distribute_dofs (const FiniteElement<dim,spacedim> &fe,
 				  const unsigned int        offset = 0);
 
+				     /**
+				      * After distribute_dofs() with
+				      * an FESystem element, the block
+				      * structure of global and level
+				      * vectors is stored in a
+				      * BlockInfo object accessible
+				      * with block_info(). This
+				      * function initializes the local
+				      * block structure on each cell
+				      * in the same object.
+				      */
+    void initialize_local_block_info();
+    
 				     /**
 				      * Clear all data of this object and
 				      * especially delete the lock this object
@@ -257,7 +274,7 @@ class DoFHandler  :  public Subscriptor
 				      * time when @p distribute_dofs was called.
 				      */
     virtual void clear ();
-    
+
                                        /**
                                         * Renumber degrees of freedom based on
                                         * a list of new dof numbers for all the
@@ -292,6 +309,12 @@ class DoFHandler  :  public Subscriptor
     void renumber_dofs (const std::vector<unsigned int> &new_numbers);
 
 				     /**
+				      * @deprecated Use
+				      * CompressedSparsityPattern instead of
+				      * initializing SparsityPattern with this
+				      * value, see the discussion in step-2
+				      * and the @ref Sparsity module.
+				      *
 				      * Return the maximum number of
 				      * degrees of freedom a degree of freedom
 				      * in the given triangulation with the
@@ -332,14 +355,20 @@ class DoFHandler  :  public Subscriptor
 				      * problem is to use an intermediate
 				      * compressed sparsity pattern that only
 				      * allocates memory on demand. Refer to
-				      * the @ref step_11 "step-11" example
-				      * program on how to do this. The problem
+				      * the step-2 and step-11 example
+				      * programs on how to do this. The problem
 				      * is also discussed in the documentation
 				      * of the module on @ref Sparsity.
 				      */
     unsigned int max_couplings_between_dofs () const;
 
 				     /**
+				      * @deprecated Use
+				      * CompressedSparsityPattern
+				      * instead of initializing
+				      * SparsityPattern with this
+				      * value.
+				      *
 				      * Return the number of degrees of freedom
 				      * located on the boundary another dof on
 				      * the boundary can couple with.
@@ -349,7 +378,7 @@ class DoFHandler  :  public Subscriptor
 				      * dimension less.
 				      */
     unsigned int max_couplings_between_boundary_dofs () const;
-				     
+
 				     /**
 				      *  @name Cell iterator functions
 				      */
@@ -401,7 +430,7 @@ class DoFHandler  :  public Subscriptor
 				      * <tt>end()</tt>.
 				      */
     cell_iterator        end (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is the first
 				      * iterator not on level. If @p level is
@@ -615,7 +644,7 @@ class DoFHandler  :  public Subscriptor
 				      * <tt>end()</tt>.
 				      */
     line_iterator        end_line (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is the first
 				      * iterator not on level. If @p level is
@@ -669,7 +698,7 @@ class DoFHandler  :  public Subscriptor
 				      *  active line on level @p level.
 				      */
     active_line_iterator last_active_line (const unsigned int level) const;
-				     /*@}*/	  
+				     /*@}*/
 
 				     /*---------------------------------------*/
 
@@ -712,7 +741,7 @@ class DoFHandler  :  public Subscriptor
 				      * <tt>end()</tt>.
 				      */
     quad_iterator        end_quad (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is the first
 				      * iterator not on level. If @p level is
@@ -813,7 +842,7 @@ class DoFHandler  :  public Subscriptor
 				      * <tt>end()</tt>.
 				      */
     hex_iterator        end_hex (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is the first
 				      * iterator not on level. If @p level is
@@ -918,6 +947,34 @@ class DoFHandler  :  public Subscriptor
     unsigned int
     n_boundary_dofs (const std::set<unsigned char> &boundary_indicators) const;
 
+				     /**
+				      * Access to an object informing
+				      * of the block structure of the
+				      * dof handler.
+				      *
+				      * If an FESystem is used in
+				      * distribute_dofs(), degrees ofd
+				      * freedom naturally split into
+				      * several @ref GlossBlock
+				      * "blocks". For each base element
+				      * as many blocks appear as its
+				      * multiplicity.
+				      *
+				      * At the end of
+				      * distribute_dofs(), the number
+				      * of degrees of freedom in each
+				      * block is counted, and stored
+				      * in a BlockInfo object, which
+				      * can be accessed here. In an
+				      * MGDoFHandler, the same is done
+				      * on each level. Additionally,
+				      * the block structure on each
+				      * cell can be generated in this
+				      * object by calling
+				      * initialize_local_block_indices().
+				      */
+    const BlockInfo& block_info() const;
+    
 				     /**
 				      * Return a constant reference to
 				      * the selected finite element
@@ -931,7 +988,7 @@ class DoFHandler  :  public Subscriptor
 				      * this object.
 				      */
     const Triangulation<dim,spacedim> & get_tria () const;
-    
+
 				     /**
 				      * Determine an estimate for the
 				      * memory consumption (in bytes)
@@ -940,7 +997,7 @@ class DoFHandler  :  public Subscriptor
 				      * This function is made virtual,
 				      * since a dof handler object
 				      * might be accessed through a
-				      * pointers to thisr base class,
+				      * pointers to this base class,
 				      * although the actual object
 				      * might be a derived class.
 				      */
@@ -951,11 +1008,16 @@ class DoFHandler  :  public Subscriptor
 				      */
     DeclException0 (ExcInvalidTriangulation);
 				     /**
-				      * Exception
+				      * No finite element has been
+				      * assigned to this
+				      * DoFHandler. Call the function
+				      * DoFHandler::distribute_dofs()
+				      * before you attemped to do what
+				      * raised this exception.
 				      */
     DeclException0 (ExcNoFESelected);
     				     /**
-				      * Exception
+				      * @todo Replace by ExcInternalError.
 				      */
     DeclException0 (ExcRenumberingIncomplete);
 				     /**
@@ -1000,12 +1062,12 @@ class DoFHandler  :  public Subscriptor
 		    << ", but this level is empty.");
 
   protected:
-    
+
 				     /**
 				      * Address of the triangulation to
 				      * work on.
 				      */
-    SmartPointer<const Triangulation<dim,spacedim> > tria;
+    SmartPointer<const Triangulation<dim,spacedim>,DoFHandler<dim,spacedim> > tria;
 
 				     /**
 				      * Store a pointer to the finite element
@@ -1019,8 +1081,9 @@ class DoFHandler  :  public Subscriptor
 				      * function (this clears all data of
 				      * this object as well, though).
 				      */
-    SmartPointer<const FiniteElement<dim,spacedim> > selected_fe;
-
+    SmartPointer<const FiniteElement<dim,spacedim>,DoFHandler<dim,spacedim> > selected_fe;
+    
+    BlockInfo block_info_object;
   private:
 
 				     /**
@@ -1049,7 +1112,7 @@ class DoFHandler  :  public Subscriptor
 				      * Free all used memory.
 				      */
     void clear_space ();
-           
+
 				     /**
 				      * Space to store the DoF numbers
 				      * for the different
@@ -1082,7 +1145,7 @@ class DoFHandler  :  public Subscriptor
 				      */
     std::vector<unsigned int>      vertex_dofs;
 
-				     /*
+				     /**
 				      * Make accessor objects friends.
 				      */
     template <int, class> friend class DoFAccessor;
@@ -1090,7 +1153,7 @@ class DoFHandler  :  public Subscriptor
     friend class internal::DoFAccessor::Implementation;
     friend class internal::DoFCellAccessor::Implementation;
 
-    friend class internal::DoFHandler::Implementation;    
+    friend class internal::DoFHandler::Implementation;
 };
 
 
@@ -1137,6 +1200,15 @@ DoFHandler<dim,spacedim>::get_tria () const
 }
 
 
+template <int dim, int spacedim>
+inline
+const BlockInfo&
+DoFHandler<dim,spacedim>::block_info () const
+{
+  return block_info_object;
+}
+
+
 
 
 #endif // DOXYGEN
