@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_renumbering.h 18693 2009-04-22 21:23:33Z kanschat $
+//    $Id: dof_renumbering.h 20879 2010-03-22 15:32:22Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -18,6 +18,7 @@
 #include <base/exceptions.h>
 #include <base/point.h>
 #include <dofs/dof_handler.h>
+#include <hp/dof_handler.h>
 #include <multigrid/mg_dof_handler.h>
 
 #include <vector>
@@ -71,7 +72,7 @@ DEAL_II_NAMESPACE_OPEN
  * published in 1984), but certainly not with those used in this
  * library, featuring several 10,000 to a few 100,000 elements.
  *
- * 
+ *
  * <h4>Implementation of renumbering schemes</h4>
  *
  * The renumbering algorithms need quite a lot of memory, since they
@@ -91,7 +92,7 @@ DEAL_II_NAMESPACE_OPEN
  * function. This will then give inferior results, since knots in the
  * graph (representing dofs) are not found to be neighbors even if
  * they would be after condensation.
- * 
+ *
  * The renumbering algorithms work on a purely algebraic basis, due to
  * the isomorphism between the graph theoretical groundwork underlying
  * the algorithms and binary matrices (matrices of which the entries
@@ -108,7 +109,7 @@ DEAL_II_NAMESPACE_OPEN
  * deleted. If no index is allowed, the algorithm will search for its
  * own starting point.
  *
- * 
+ *
  * <h4>Results of renumbering</h4>
  *
  * The renumbering schemes mentioned above do not lead to optimal
@@ -203,7 +204,7 @@ DEAL_II_NAMESPACE_OPEN
  * sparsity patterns produced by the various algorithms when using the
  * $Q_2^d\times Q_1$ element combination typically employed in the
  * discretization of Stokes equations, when used on the mesh obtained
- * in @ref step_22 "step-22" after one adaptive mesh refinement in
+ * in step-22 after one adaptive mesh refinement in
  * 3d. The space dimension together with the coupled finite element
  * leads to a rather dense system matrix with, on average around 180
  * nonzero entries per row. After applying each of the reordering
@@ -215,7 +216,7 @@ DEAL_II_NAMESPACE_OPEN
  * right and bottom left.
  *
  * The goal of reordering strategies is to improve the
- * preconditioner. In @ref step_22 "step-22" we use a SparseILU to
+ * preconditioner. In step-22 we use a SparseILU to
  * preconditioner for the velocity-velocity block at the top left. The
  * quality of the preconditioner can then be measured by the number of
  * CG iterations required to solve a linear system with this
@@ -311,7 +312,7 @@ DEAL_II_NAMESPACE_OPEN
  *     of iterations is only slightly less than with BOOST's Cuthill-McKee
  *     implementation, runtime is significantly less. This, again, may be due
  *     to cache effects. As a consequence, this is the algorithm best suited
- *     to the testcase, and is in fact used in @ref step_22 "step-22".
+ *     to the testcase, and is in fact used in step-22.
  *   </td>
  *   <td>
  *     Minimum degree enumeration as produced by calling the BOOST implementation
@@ -363,7 +364,7 @@ DEAL_II_NAMESPACE_OPEN
  * @ingroup dofs
  * @author Wolfgang Bangerth, Guido Kanschat, 1998, 1999, 2000, 2004, 2007, 2008
  */
-namespace DoFRenumbering 
+namespace DoFRenumbering
 {
 				   /**
 				    * Direction based comparator for
@@ -382,7 +383,7 @@ namespace DoFRenumbering
 					*/
       CompareDownstream (const Point<dim> &dir)
 		      :
-		      dir(dir) 
+		      dir(dir)
 	{}
 				       /**
 					* Return true if c1 less c2.
@@ -392,14 +393,52 @@ namespace DoFRenumbering
 	  const Point<dim> diff = c2->center() - c1->center();
 	  return (diff*dir > 0);
 	}
-      
+
     private:
 				       /**
 					* Flow direction.
 					*/
       const Point<dim> dir;
   };
-  
+
+
+				   /**
+				    * Point based comparator for downstream
+				    * directions: it returns @p true if the
+				    * second point is downstream of the first
+				    * one with respect to the direction given
+				    * to the constructor. If the points are
+				    * the same with respect to the downstream
+				    * direction, the point with the lower DoF
+				    * number is considered smaller.
+				    */
+  template <int dim>
+  struct ComparePointwiseDownstream
+  {
+				       /**
+					* Constructor.
+					*/
+      ComparePointwiseDownstream (const Point<dim> &dir)
+		      :
+		      dir(dir)
+	{}
+				       /**
+					* Return true if c1 less c2.
+					*/
+      bool operator () (const std::pair<Point<dim>,unsigned int> &c1,
+			const std::pair<Point<dim>,unsigned int> &c2) const
+        {
+	  const Point<dim> diff = c2.first-c1.first;
+	  return (diff*dir > 0 || (diff*dir==0 && c1.second<c2.second));
+	}
+
+    private:
+				       /**
+					* Flow direction.
+					*/
+      const Point<dim> dir;
+  };
+
 				   /**
 				    * A namespace for the
 				    * implementation of some
@@ -446,7 +485,7 @@ namespace DoFRenumbering
 				      * DoFRenumbering namespace.
 				      */
     template <class DH>
-    void 
+    void
     Cuthill_McKee (DH&                              dof_handler,
 		   const bool                       reversed_numbering = false,
 		   const bool                       use_constraints    = false);
@@ -459,7 +498,7 @@ namespace DoFRenumbering
 				      * the DoFHandler dofs but
 				      * returns the renumbering
 				      * vector.
-				      */    
+				      */
     template <class DH>
     void
     compute_Cuthill_McKee (std::vector<unsigned int>& new_dof_indices,
@@ -490,10 +529,10 @@ namespace DoFRenumbering
 				      * DoFRenumbering namespace.
 				      *
 				      * This algorithm is used in
-				      * @ref step_22 "step-22".
+				      * step-22.
 				      */
     template <class DH>
-    void 
+    void
     king_ordering (DH&                              dof_handler,
 		   const bool                       reversed_numbering = false,
 		   const bool                       use_constraints    = false);
@@ -538,7 +577,7 @@ namespace DoFRenumbering
 				      * DoFRenumbering namespace.
 				      */
     template <class DH>
-    void 
+    void
     minimum_degree (DH&                              dof_handler,
 		    const bool                       reversed_numbering = false,
 		    const bool                       use_constraints    = false);
@@ -558,7 +597,7 @@ namespace DoFRenumbering
 			    const bool reversed_numbering = false,
 			    const bool use_constraints    = false);
   }
-    
+
 				   /**
 				    * Renumber the degrees of
 				    * freedom according to the
@@ -578,7 +617,7 @@ namespace DoFRenumbering
 				    * DoFRenumbering namespace.
 				    */
   template <class DH>
-  void 
+  void
   Cuthill_McKee (DH&                              dof_handler,
 		 const bool                       reversed_numbering = false,
 		 const bool                       use_constraints    = false,
@@ -592,7 +631,7 @@ namespace DoFRenumbering
 				    * the DoFHandler dofs but
 				    * returns the renumbering
 				    * vector.
-				    */    
+				    */
   template <class DH>
   void
   compute_Cuthill_McKee (std::vector<unsigned int>& new_dof_indices,
@@ -625,7 +664,7 @@ namespace DoFRenumbering
 				    * the different methods.
 				    */
   template <int dim>
-  void 
+  void
   Cuthill_McKee (MGDoFHandler<dim>          &dof_handler,
 		 const unsigned int          level,
 		 const bool                  reversed_numbering = false,
@@ -661,7 +700,7 @@ namespace DoFRenumbering
 				    * possible and results in a
 				    * blocking of several components
 				    * into one. This is discussed in
-				    * @ref step_22 "step-22". If you
+				    * step-22. If you
 				    * omit this argument, the same
 				    * order as given by the finite
 				    * element is used.
@@ -680,7 +719,7 @@ namespace DoFRenumbering
 				    * are associated with the first
 				    * vector component to which they
 				    * belong.
-				    * 
+				    *
 				    * For finite elements with only
 				    * one component, or a single
 				    * non-primitive base element,
@@ -693,6 +732,18 @@ namespace DoFRenumbering
 		  const std::vector<unsigned int> &target_component
 		  = std::vector<unsigned int>());
 
+
+				   /**
+				    * Sort the degrees of freedom by
+				    * component. It does the same
+				    * thing as the above function.
+				    */
+  template <int dim>
+  void
+  component_wise (hp::DoFHandler<dim>             &dof_handler,
+		  const std::vector<unsigned int> &target_component = std::vector<unsigned int> ());
+
+
 				   /**
 				    * Sort the degrees of freedom by
 				    * component. It does the same
@@ -725,7 +776,6 @@ namespace DoFRenumbering
   component_wise (MGDoFHandler<dim>&               dof_handler,
 		  const std::vector<unsigned int>& target_component = std::vector<unsigned int>());
 
-    
 				   /**
 				    * Computes the renumbering
 				    * vector needed by the
@@ -734,7 +784,7 @@ namespace DoFRenumbering
 				    * the renumbering on the
 				    * DoFHandler dofs but returns
 				    * the renumbering vector.
-				    */    
+				    */
   template <int dim, class ITERATOR, class ENDITERATOR>
   static unsigned int
   compute_component_wise (std::vector<unsigned int>& new_dof_indices,
@@ -818,7 +868,7 @@ namespace DoFRenumbering
   cell_wise (MGDoFHandler<dim>   &dof_handler,
 	     const unsigned int   level,
 	     const std::vector<typename MGDoFHandler<dim>::cell_iterator> &cell_order);
-    
+
 				   /**
 				    * @deprecated Use cell_wise() instead.
 				    *
@@ -832,7 +882,7 @@ namespace DoFRenumbering
   cell_wise_dg (MGDoFHandler<dim>   &dof_handler,
 		const unsigned int   level,
 		const std::vector<typename MGDoFHandler<dim>::cell_iterator> &cell_order);
-    
+
 				   /**
 				    * Computes the renumbering
 				    * vector needed by the
@@ -870,9 +920,13 @@ namespace DoFRenumbering
 
 
 				   /**
-				    * Cell-wise downstream numbering
-				    * with respect to a constant
-				    * flow direction.
+				    * Downstream numbering with respect to a
+				    * constant flow direction. If the
+				    * additional argument @p
+				    * dof_wise_renumbering is set to @p false,
+				    * the numbering is performed cell-wise,
+				    * otherwise it is performed based on the
+				    * location of the support points.
 				    *
 				    * The cells are sorted such that
 				    * the centers of higher numbers
@@ -886,20 +940,33 @@ namespace DoFRenumbering
 				    * grids, this might not be
 				    * guaranteed for all meshes.
 				    *
-				    * This function produces a
-				    * downstream ordering of the
+				    * If the @p dof_wise_renumbering argument
+				    * is set to @p false, this function
+				    * produces a downstream ordering of the
 				    * mesh cells and calls
-				    * cell_wise_dg().  Therefore, it
-				    * only works with Discontinuous
-				    * Galerkin Finite Elements,
-				    * i.e. all degrees of freedom
-				    * have to be associated with the
-				    * interior of the cell.
+				    * cell_wise(). Therefore, the output only
+				    * makes sense for Discontinuous Galerkin
+				    * Finite Elements (all degrees of freedom
+				    * have to be associated with the interior
+				    * of the cell in that case) in that case.
+				    *
+				    * If @p dof_wise_renumbering is set to @p
+				    * true, the degrees of freedom are
+				    * renumbered based on the support point
+				    * location of the individual degrees of
+				    * freedom (obviously, the finite element
+				    * needs to define support points for this
+				    * to work). The numbering of points with
+				    * the same position in downstream location
+				    * (e.g. those parallel to the flow
+				    * direction, or several dofs within a
+				    * FESystem) will be unaffected.
 				    */
   template <class DH, int dim>
   void
   downstream (DH&               dof_handler,
-	      const Point<dim>& direction);
+	      const Point<dim>& direction,
+	      const bool        dof_wise_renumbering = false);
 
 				   /**
 				    * @deprecated Use downstream() instead.
@@ -918,18 +985,20 @@ namespace DoFRenumbering
 				    */
   template <int dim>
   void
-  downstream (MGDoFHandler<dim>  &dof_handler,
+  downstream (MGDoFHandler<dim> &dof_handler,
 	      const unsigned int level,
-	      const Point<dim> &direction);
+	      const Point<dim>  &direction,
+	      const bool         dof_wise_renumbering = false);
+
 				   /**
 				    * @deprecated Use downstream()
 				    * instead.
 				    */
   template <int dim>
   void
-  downstream_dg (MGDoFHandler<dim>  &dof_handler,
+  downstream_dg (MGDoFHandler<dim> &dof_handler,
 		 const unsigned int level,
-		 const Point<dim> &direction);
+		 const Point<dim>  &direction);
 
 				   /**
 				    * @deprecated The new function
@@ -967,8 +1036,9 @@ namespace DoFRenumbering
   compute_downstream (std::vector<unsigned int>& new_dof_indices,
 		      std::vector<unsigned int>& reverse,
 		      const DH&                  dof_handler,
-		      const Point<dim>&          direction);
-    
+		      const Point<dim>&          direction,
+		      const bool                 dof_wise_renumbering);
+
 				   /**
 				    * @deprecated Use
 				    * compute_downstream() instead
@@ -995,8 +1065,9 @@ namespace DoFRenumbering
 		      std::vector<unsigned int>& reverse,
 		      const MGDoFHandler<dim>&   dof_handler,
 		      const unsigned int         level,
-		      const Point<dim>&          direction);
-    
+		      const Point<dim>&          direction,
+		      const bool                 dof_wise_renumbering);
+
 				   /**
 				    * @deprecated Use
 				    * compute_downstream() instead
@@ -1105,7 +1176,7 @@ namespace DoFRenumbering
 				    * renumbering on the DoFHandler
 				    * dofs but returns the
 				    * renumbering vector.
-				    */   
+				    */
   template <class DH>
   void
   compute_random (std::vector<unsigned int> &new_dof_indices,
@@ -1158,12 +1229,12 @@ namespace DoFRenumbering
 				    * renumbering on the @p
 				    * DoFHandler dofs but returns
 				    * the renumbering vector.
-				    */   
+				    */
   template <class DH>
   void
   compute_subdomain_wise (std::vector<unsigned int> &new_dof_indices,
 			  const DH                  &dof_handler);
-    
+
 				   /**
 				    * Exception
 				    *
