@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: dof_tools.h 18745 2009-04-26 22:08:40Z bangerth $
+//    $Id: dof_tools.h 21358 2010-06-24 23:38:14Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -17,6 +17,7 @@
 #include <base/config.h>
 #include <base/exceptions.h>
 #include <base/table.h>
+#include <base/index_set.h>
 #include <lac/constraint_matrix.h>
 #include <dofs/function_map.h>
 
@@ -199,7 +200,7 @@ class DoFTools
                                             */
 	  nonzero
     };
-    
+
 				     /**
 				      * @name Auxiliary functions
 				      * @{
@@ -211,12 +212,12 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     max_dofs_per_cell (const DoFHandler<dim,spacedim> &dh);
-    
+
     template <int dim, int spacedim>
     static unsigned int
     max_dofs_per_cell (const hp::DoFHandler<dim,spacedim> &dh);
-    
-    
+
+
 				     /**
 				      * Maximal number of degrees of
 				      * freedom on a face.
@@ -242,7 +243,7 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     max_dofs_per_face (const hp::DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * Maximal number of degrees of
 				      * freedom on a vertex.
@@ -255,7 +256,7 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     max_dofs_per_vertex (const DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * Maximal number of degrees of
 				      * freedom on a vertex.
@@ -268,7 +269,7 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     max_dofs_per_vertex (const hp::DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * Number of vector components in the
 				      * finite element object used by this
@@ -282,7 +283,7 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     n_components (const DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * Number of vector components in the
 				      * finite element object used by this
@@ -296,7 +297,7 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     n_components (const hp::DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * Find out whether the FiniteElement
 				      * used by this DoFHandler is primitive
@@ -310,7 +311,7 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     fe_is_primitive (const DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * Find out whether the FiniteElement
 				      * used by this DoFHandler is primitive
@@ -324,16 +325,16 @@ class DoFTools
     template <int dim, int spacedim>
     static unsigned int
     fe_is_primitive (const hp::DoFHandler<dim,spacedim> &dh);
-    
+
 				     /**
 				      * @}
 				      */
-    
+
 				     /**
 				      * @name Sparsity Pattern Generation
 				      * @{
 				      */
-    
+
 				     /**
 				      * Locate non-zero entries of the
 				      * system matrix.
@@ -408,7 +409,7 @@ class DoFTools
 				      * function. No call to
 				      * ConstraintMatrix::condense() is then
 				      * necessary. This process is explained
-				      * in @ref step_27 "step-27".
+				      * in step-27.
 				      *
 				      * In case the constraints are
 				      * already taken care of in this
@@ -428,6 +429,23 @@ class DoFTools
 				      * <tt>keep_constrained_dofs</tt>
 				      * needs to be set to
 				      * <tt>false</tt>.
+				      *
+				      * If the @p subdomain_id parameter is
+				      * given, the sparsity pattern is built
+				      * only on cells that have a subdomain_id
+				      * equal to the given argument. This is
+				      * useful in parallel contexts where the
+				      * matrix and sparsity pattern (for
+				      * example a
+				      * TrilinosWrappers::SparsityPattern) may
+				      * be distributed and not every MPI
+				      * process needs to build the entire
+				      * sparsity pattern; in that case, it is
+				      * sufficient if every process only
+				      * builds that part of the sparsity
+				      * pattern that corresponds to the
+				      * subdomain_id for which it is
+				      * responsible.
 				      */
     template <class DH, class SparsityPattern>
     static
@@ -435,7 +453,8 @@ class DoFTools
     make_sparsity_pattern (const DH               &dof,
 			   SparsityPattern        &sparsity_pattern,
 			   const ConstraintMatrix &constraints = ConstraintMatrix(),
-			   const bool              keep_constrained_dofs = true);
+			   const bool              keep_constrained_dofs = true,
+			   const unsigned int      subdomain_id = numbers::invalid_unsigned_int);
 
 				     /**
 				      * Locate non-zero entries for
@@ -565,6 +584,23 @@ class DoFTools
 				      * <tt>keep_constrained_dofs</tt>
 				      * needs to be set to
 				      * <tt>false</tt>.
+				      *
+				      * If the @p subdomain_id parameter is
+				      * given, the sparsity pattern is built
+				      * only on cells that have a subdomain_id
+				      * equal to the given argument. This is
+				      * useful in parallel contexts where the
+				      * matrix and sparsity pattern (for
+				      * example a
+				      * TrilinosWrappers::SparsityPattern) may
+				      * be distributed and not every MPI
+				      * process needs to build the entire
+				      * sparsity pattern; in that case, it is
+				      * sufficient if every process only
+				      * builds that part of the sparsity
+				      * pattern that corresponds to the
+				      * subdomain_id for which it is
+				      * responsible.
 				      */
     template <class DH, class SparsityPattern>
     static
@@ -573,14 +609,20 @@ class DoFTools
 			   const Table<2, Coupling> &coupling,
 			   SparsityPattern          &sparsity_pattern,
 			   const ConstraintMatrix   &constraints = ConstraintMatrix(),
-			   const bool                keep_constrained_dofs = true);
-    
+			   const bool                keep_constrained_dofs = true,
+			   const unsigned int        subdomain_id = numbers::invalid_unsigned_int);
+
 				     /**
 				      * @deprecated This is the old
 				      * form of the previous
 				      * function. It generates a table
-				      * of DoFTools::Coupling values and calls
-				      * it.
+				      * of DoFTools::Coupling values
+				      * (where a <code>true</code>
+				      * value in the mask is
+				      * translated into a
+				      * Coupling::always value in the
+				      * table) and calls the function
+				      * above.
 				      */
     template <class DH, class SparsityPattern>
     static
@@ -628,7 +670,7 @@ class DoFTools
     make_sparsity_pattern (const DH        &dof_row,
 			   const DH        &dof_col,
 			   SparsityPattern &sparsity);
-    
+
     				     /**
 				      * Create the sparsity pattern for
 				      * boundary matrices. See the general
@@ -651,7 +693,7 @@ class DoFTools
     make_boundary_sparsity_pattern (const DH                        &dof,
 				    const std::vector<unsigned int> &dof_to_boundary_mapping,
 				    SparsityPattern                 &sparsity_pattern);
-    
+
 				     /**
 				      * Write the sparsity structure of the
 				      * matrix composed of the basis functions
@@ -709,7 +751,7 @@ class DoFTools
     static void
     make_flux_sparsity_pattern (const DH        &dof_handler,
 				SparsityPattern &sparsity_pattern);
-    
+
 				     /**
 				      * This function does the same as
 				      * the other with the same name,
@@ -742,7 +784,7 @@ class DoFTools
 				      * @name Hanging Nodes
 				      * @{
 				      */
-    
+
 				     /**
 				      * Compute the constraints resulting from
 				      * the presence of hanging nodes. Hanging
@@ -818,7 +860,7 @@ class DoFTools
     make_hanging_node_constraints (const DH         &dof_handler,
 				   ConstraintMatrix &constraints);
 				     //@}
-    
+
 				     /**
 				      * Take a vector of values which live on
 				      * cells (e.g. an error per cell) and
@@ -948,7 +990,7 @@ class DoFTools
 		  const std::vector<bool> &select,
 		  std::vector<bool>       &selected_dofs,
 		  const bool               blocks = false);
-    
+
 				     /**
 				      * Do the same thing as
 				      * extract_dofs() for one level
@@ -1014,11 +1056,49 @@ class DoFTools
 			   const std::vector<bool>    &component_select,
 			   std::vector<bool>          &selected_dofs,
 			   const std::set<unsigned char> &boundary_indicators = std::set<unsigned char>());
+
+				     /**
+				      * This function is similar to
+				      * the extract_boundary_dofs()
+				      * function but it extracts those
+				      * degrees of freedom whose shape
+				      * functions are nonzero on at
+				      * least part of the selected
+				      * boundary. For continuous
+				      * elements, this is exactly the
+				      * set of shape functions whose
+				      * degrees of freedom are defined
+				      * on boundary faces. On the
+				      * other hand, if the finite
+				      * element in used is a
+				      * discontinuous element, all
+				      * degrees of freedom are defined
+				      * in the inside of cells and
+				      * consequently none would be
+				      * boundary degrees of
+				      * freedom. Several of those
+				      * would have shape functions
+				      * that are nonzero on the
+				      * boundary, however. This
+				      * function therefore extracts
+				      * all those for which the
+				      * FiniteElement::has_support_on_face
+				      * function says that it is
+				      * nonzero on any face on one of
+				      * the selected boundary parts.
+				      */
+    template <class DH>
+    static void
+    extract_dofs_with_support_on_boundary (const DH                   &dof_handler,
+					   const std::vector<bool>    &component_select,
+					   std::vector<bool>          &selected_dofs,
+					   const std::set<unsigned char> &boundary_indicators = std::set<unsigned char>());
+
 				     /**
 				      * @name Hanging Nodes
 				      * @{
 				      */
-    
+
 				     /**
 				      * Select all dofs that will be
 				      * constrained by interface
@@ -1036,7 +1116,7 @@ class DoFTools
     extract_hanging_node_dofs (const DoFHandler<dim,spacedim> &dof_handler,
 			       std::vector<bool>              &selected_dofs);
 				     //@}
-    
+
 				     /**
 				      * Flag all those degrees of
 				      * freedom which are on cells
@@ -1062,56 +1142,45 @@ class DoFTools
 			    std::vector<bool>  &selected_dofs);
 
 				     /**
-				      * Extract a vector that 
-				      * represents the constant
-				      * modes of the DoFHandler
-				      * for the components 
-				      * chosen by <tt>component_select</tt>.
-				      * The constant modes on a
-				      * discretization are the 
-				      * null space of a Laplace
-				      * operator on the selected
-				      * components with Neumann
-				      * boundary conditions 
-				      * applied. The null space is
-				      * a necessary ingredient for
-				      * obtaining a good AMG
-				      * preconditioner when using
-				      * the class 
+				      * Extract a vector that represents the
+				      * constant modes of the DoFHandler for
+				      * the components chosen by
+				      * <tt>component_select</tt>.  The
+				      * constant modes on a discretization are
+				      * the null space of a Laplace operator
+				      * on the selected components with
+				      * Neumann boundary conditions
+				      * applied. The null space is a necessary
+				      * ingredient for obtaining a good AMG
+				      * preconditioner when using the class
 				      * TrilinosWrappers::PreconditionAMG.
-				      * Since the ML AMG package
-				      * only works on algebraic
-				      * properties of the respective
-				      * matrix, it has no chance 
-				      * to detect whether the matrix
-				      * comes from a scalar or a
-				      * vector valued problem. However,
-				      * a near null space supplies
-				      * exactly the needed information
-				      * about these components.
-				      * The null space will consist
-				      * of as many vectors as there
-				      * are true arguments in 
-				      * <tt>component_select</tt>, 
-				      * each of which will be one
-				      * in one component and 
-				      * zero in all others. We store 
-				      * this object in a vector of 
-				      * vectors, where the outer 
-				      * vector is of the size of 
-				      * the number of selected 
-				      * components, and each inner
-				      * vector has as many components
-				      * as there are degrees of 
-				      * freedom in the selected 
-				      * components. Note that any
-				      * matrix associated with this
-				      * null space must have been
-				      * constructed using the 
-				      * same
-				      * <tt>component_select</tt>
-				      * argument.
-				      * 
+				      * Since the ML AMG package only works on
+				      * algebraic properties of the respective
+				      * matrix, it has no chance to detect
+				      * whether the matrix comes from a scalar
+				      * or a vector valued problem. However, a
+				      * near null space supplies exactly the
+				      * needed information about these
+				      * components.  The null space will
+				      * consist of as many vectors as there
+				      * are true arguments in
+				      * <tt>component_select</tt>, each of
+				      * which will be one in one component and
+				      * zero in all others. We store this
+				      * object in a vector of vectors, where
+				      * the outer vector is of the size of the
+				      * number of selected components, and
+				      * each inner vector has as many
+				      * components as there are degrees of
+				      * freedom in the selected
+				      * components. Note that any matrix
+				      * associated with this null space must
+				      * have been constructed using the same
+				      * <tt>component_select</tt> argument,
+				      * since the numbering of DoFs is done
+				      * relative to the selected dofs, not to
+				      * all dofs.
+				      *
 				      * The main reason for this
 				      * program is the use of the
 				      * null space with the
@@ -1142,7 +1211,7 @@ class DoFTools
     static void
     get_active_fe_indices (const DH                  &dof_handler,
 			   std::vector<unsigned int> &active_fe_indices);
-    
+
                                      /**
                                       * For each DoF, return in the output
                                       * array to which subdomain (as given by
@@ -1213,6 +1282,20 @@ class DoFTools
     count_dofs_with_subdomain_association (const DH           &dof_handler,
                                            const unsigned int  subdomain);
 
+				     /**
+				      * Similar to the previous
+				      * function, but do not just
+				      * return the number of degrees
+				      * of freedom that are owned by a
+				      * given subdomain, but return a
+				      * set of their indices.
+				      */
+    template <class DH>
+    static
+    IndexSet
+    dof_indices_with_subdomain_association (const DH           &dof_handler,
+					    const unsigned int  subdomain);
+
                                      /**
                                       * Count how many degrees of freedom are
                                       * uniquely associated with the given
@@ -1234,7 +1317,7 @@ class DoFTools
     count_dofs_with_subdomain_association (const DH           &dof_handler,
                                            const unsigned int  subdomain,
 					   std::vector<unsigned int> &n_dofs_on_subdomain);
-    
+
                                      /**
 				      * Count how many degrees of
 				      * freedom out of the total
@@ -1336,8 +1419,8 @@ class DoFTools
 				      * case, an assertion is thrown.
 				      *
 				      * This function is used in the
-				      * @ref step_22 "step-22", and
-				      * @ref step_31 "step-31" tutorial
+				      * step-22 and
+				      * step-31 tutorial
 				      * programs.
 				      */
     template <int dim, int spacedim>
@@ -1346,7 +1429,7 @@ class DoFTools
 			  std::vector<unsigned int>& dofs_per_block,
 			  std::vector<unsigned int>  target_block
 			  = std::vector<unsigned int>());
-    
+
 				     /**
 				      * @deprecated See the previous
 				      * function with the same name
@@ -1360,7 +1443,7 @@ class DoFTools
     count_dofs_per_component (const DoFHandler<dim,spacedim>&     dof_handler,
 			      std::vector<unsigned int>& dofs_per_component,
 			      std::vector<unsigned int>  target_component);
-    
+
 				     /**
 				      * This function can be used when
 				      * different variables shall be
@@ -1616,7 +1699,7 @@ class DoFTools
 					       const unsigned int                  fine_component,
 					       const InterGridMap<DoFHandler<dim,spacedim> > &coarse_to_fine_grid_map,
 					       std::vector<std::map<unsigned int, float> > &transfer_representation);
-    
+
 				     /**
 				      * Create a mapping from degree
 				      * of freedom indices to the
@@ -1648,7 +1731,7 @@ class DoFTools
     static void
     map_dof_to_boundary_indices (const DH                   &dof_handler,
 				 std::vector<unsigned int>  &mapping);
-    
+
 				     /**
 				      * Same as the previous function,
 				      * except that only those parts
@@ -1665,7 +1748,7 @@ class DoFTools
     map_dof_to_boundary_indices (const DH                      &dof_handler,
 				 const std::set<unsigned char> &boundary_indicators,
 				 std::vector<unsigned int>     &mapping);
-    
+
 				     /**
 				      * Return a list of support
 				      * points for all the degrees of
@@ -1687,7 +1770,7 @@ class DoFTools
     static void
     map_dofs_to_support_points (const Mapping<dim,spacedim>       &mapping,
 				const DoFHandler<dim,spacedim>    &dof_handler,
-				std::vector<Point<spacedim> > &support_points);
+				std::vector<Point<spacedim> >     &support_points);
 
 				     /**
 				      * This is the opposite function
@@ -1719,7 +1802,7 @@ class DoFTools
     template <class DH, class Comp>
     static void
     map_support_points_to_dofs (const Mapping<DH::dimension, DH::space_dimension> &mapping,
-				const DH                     &dof_handler,
+				const DH                                          &dof_handler,
 				std::map<Point<DH::space_dimension>, unsigned int, Comp> &point_to_index_map);
 
 				     /**
@@ -1743,14 +1826,17 @@ class DoFTools
 				 const Table<2, Coupling>& table_by_component,
 				 std::vector<Table<2,Coupling> >& tables_by_block);
 
-                                     /** 
-				      * Make a constraint matrix with 
-				      * zero boundary values.
+                                     /**
+				      * Make a constraint matrix for the
+				      * constraints that result from zero
+				      * boundary values. This function is used
+				      * in step-36, for
+				      * example.
 				      */
-    template <int dim, int spacedim, template <int, int> class DH> 
-    static void 
-    make_zero_boundary_constraints (const DH<dim,spacedim> &dof,  
-				    ConstraintMatrix        &zero_boundary_constraints,    
+    template <int dim, int spacedim, template <int, int> class DH>
+    static void
+    make_zero_boundary_constraints (const DH<dim,spacedim> &dof,
+				    ConstraintMatrix        &zero_boundary_constraints,
 				    const std::vector<bool> &component_mask_=std::vector<bool>());
 
 				     /**
@@ -1787,7 +1873,7 @@ class DoFTools
     Table<2,Coupling>
     dof_couplings_from_component_couplings (const FiniteElement<dim,spacedim> &fe,
 					    const Table<2,Coupling> &component_couplings);
-    
+
 				     /**
 				      * Exception
 				      */
@@ -1894,7 +1980,7 @@ DoFTools::max_dofs_per_cell (const DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::max_dofs_per_face (const DoFHandler<dim,spacedim> &dh) 
+DoFTools::max_dofs_per_face (const DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().dofs_per_face;
 }
@@ -1902,7 +1988,7 @@ DoFTools::max_dofs_per_face (const DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::max_dofs_per_vertex (const DoFHandler<dim,spacedim> &dh) 
+DoFTools::max_dofs_per_vertex (const DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().dofs_per_vertex;
 }
@@ -1910,7 +1996,7 @@ DoFTools::max_dofs_per_vertex (const DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::n_components (const DoFHandler<dim,spacedim> &dh) 
+DoFTools::n_components (const DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().n_components();
 }
@@ -1919,7 +2005,7 @@ DoFTools::n_components (const DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::fe_is_primitive (const DoFHandler<dim,spacedim> &dh) 
+DoFTools::fe_is_primitive (const DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().is_primitive();
 }
@@ -1927,7 +2013,7 @@ DoFTools::fe_is_primitive (const DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::max_dofs_per_cell (const hp::DoFHandler<dim,spacedim> &dh) 
+DoFTools::max_dofs_per_cell (const hp::DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().max_dofs_per_cell ();
 }
@@ -1935,7 +2021,7 @@ DoFTools::max_dofs_per_cell (const hp::DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::max_dofs_per_face (const hp::DoFHandler<dim,spacedim> &dh) 
+DoFTools::max_dofs_per_face (const hp::DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().max_dofs_per_face ();
 }
@@ -1943,7 +2029,7 @@ DoFTools::max_dofs_per_face (const hp::DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::max_dofs_per_vertex (const hp::DoFHandler<dim,spacedim> &dh) 
+DoFTools::max_dofs_per_vertex (const hp::DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe().max_dofs_per_vertex ();
 }
@@ -1951,7 +2037,7 @@ DoFTools::max_dofs_per_vertex (const hp::DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::n_components (const hp::DoFHandler<dim,spacedim> &dh) 
+DoFTools::n_components (const hp::DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe()[0].n_components();
 }
@@ -1959,7 +2045,7 @@ DoFTools::n_components (const hp::DoFHandler<dim,spacedim> &dh)
 
 template <int dim, int spacedim>
 inline unsigned int
-DoFTools::fe_is_primitive (const hp::DoFHandler<dim,spacedim> &dh) 
+DoFTools::fe_is_primitive (const hp::DoFHandler<dim,spacedim> &dh)
 {
   return dh.get_fe()[0].is_primitive();
 }
@@ -1968,24 +2054,26 @@ DoFTools::fe_is_primitive (const hp::DoFHandler<dim,spacedim> &dh)
 template <class DH, class SparsityPattern>
 inline
 void
-DoFTools::make_sparsity_pattern (
-  const DH                              &dof,
-  const std::vector<std::vector<bool> > &mask,
-  SparsityPattern                       &sparsity_pattern)
+DoFTools::make_sparsity_pattern (const DH                              &dof,
+				 const std::vector<std::vector<bool> > &mask,
+				 SparsityPattern                       &sparsity_pattern)
 {
   const unsigned int ncomp = dof.get_fe().n_components();
-  
+
   Assert (mask.size() == ncomp,
 	  ExcDimensionMismatch(mask.size(), ncomp));
   for (unsigned int i=0; i<mask.size(); ++i)
     Assert (mask[i].size() == ncomp,
 	    ExcDimensionMismatch(mask[i].size(), ncomp));
 				   // Create a coupling table out of the mask
-  Table<2,DoFTools::Coupling> couplings(ncomp, ncomp);  
+  Table<2,DoFTools::Coupling> couplings(ncomp, ncomp);
   for (unsigned int i=0;i<ncomp;++i)
     for (unsigned int j=0;j<ncomp;++j)
       if (mask[i][j])
 	couplings(i,j) = always;
+      else
+	couplings(i,j) = none;
+
 				   // Call the new function
   make_sparsity_pattern(dof, couplings, sparsity_pattern);
 }
@@ -1994,8 +2082,8 @@ DoFTools::make_sparsity_pattern (
 template <class DH, class Comp>
 void
 DoFTools::map_support_points_to_dofs (
-  const Mapping<DH::dimension,DH::space_dimension> &mapping,
-  const DH                     &dof_handler,
+  const Mapping<DH::dimension,DH::space_dimension>         &mapping,
+  const DH                                                 &dof_handler,
   std::map<Point<DH::space_dimension>, unsigned int, Comp> &point_to_index_map)
 {
 				   // let the checking of arguments be
