@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_dgp_nonparametric.h 18473 2009-03-10 14:53:36Z kronbichler $
+//    $Id: fe_dgp_nonparametric.h 18933 2009-06-12 08:53:35Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2002, 2003, 2004, 2005, 2006 by the deal.II authors
@@ -484,7 +484,7 @@ class FE_DGPNonparametric : public FiniteElement<dim,spacedim>
 		    typename Mapping<dim,spacedim>::InternalDataBase      &mapping_internal,
 		    typename Mapping<dim,spacedim>::InternalDataBase      &fe_internal,
 		    FEValuesData<dim,spacedim>                            &data,
-		    enum CellSimilarity::Similarity                       &cell_similarity) const;
+		    CellSimilarity::Similarity                       &cell_similarity) const;
     
 				     /**
 				      * Implementation of the same
