@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_poly.templates.h 18473 2009-03-10 14:53:36Z kronbichler $
+//    $Id: fe_poly.templates.h 20161 2009-11-24 22:54:51Z kanschat $
 //    Version: $Name$
 //
 //    Copyright (C) 2006, 2007, 2008, 2009 by the deal.II authors
@@ -28,7 +28,9 @@ FE_Poly<POLY,dim,spacedim>::FE_Poly (const POLY& poly_space,
 				    restriction_is_additive_flags,
 				    nonzero_components),
                 poly_space(poly_space)
-{}
+{
+  AssertDimension(dim, POLY::dimension);
+}
 
 
 template <class POLY, int dim, int spacedim>
@@ -245,7 +247,7 @@ FE_Poly<TensorProductPolynomials<1>,1,2>::fill_fe_values
    Mapping<1,2>::InternalDataBase            &mapping_data,
    Mapping<1,2>::InternalDataBase            &fedata,
    FEValuesData<1,2>                         &data,
-   enum CellSimilarity::Similarity           &cell_similarity) const
+   CellSimilarity::Similarity           &cell_similarity) const
 {
 				   // convert data object to internal
 				   // data for this class. fails with
@@ -288,7 +290,7 @@ FE_Poly<TensorProductPolynomials<2>,2,3>::fill_fe_values
    Mapping<2,3>::InternalDataBase          &mapping_data,
    Mapping<2,3>::InternalDataBase          &fedata,
    FEValuesData<2,3>                       &data,
-   enum CellSimilarity::Similarity         &cell_similarity) const
+   CellSimilarity::Similarity         &cell_similarity) const
 {
 
 				   // assert that the following dynamics
@@ -330,7 +332,7 @@ FE_Poly<PolynomialSpace<1>,1,2>::fill_fe_values (
   Mapping<1,2>::InternalDataBase          &mapping_data,
   Mapping<1,2>::InternalDataBase          &fedata,
   FEValuesData<1,2>                       &data,
-  enum CellSimilarity::Similarity         &cell_similarity) const
+  CellSimilarity::Similarity         &cell_similarity) const
 {
 				   // convert data object to internal
 				   // data for this class. fails with
@@ -375,7 +377,7 @@ FE_Poly<PolynomialSpace<2>,2,3>::fill_fe_values
    Mapping<2,3>::InternalDataBase          &mapping_data,
    Mapping<2,3>::InternalDataBase          &fedata,
    FEValuesData<2,3>                       &data,
-   enum CellSimilarity::Similarity         &cell_similarity) const
+   CellSimilarity::Similarity         &cell_similarity) const
 {
   Assert (dynamic_cast<InternalData *> (&fedata) != 0, ExcInternalError());
   InternalData &fe_data = static_cast<InternalData &> (fedata);
@@ -414,7 +416,7 @@ FE_Poly<POLY,dim,spacedim>::fill_fe_values
    typename Mapping<dim,spacedim>::InternalDataBase &mapping_data,
    typename Mapping<dim,spacedim>::InternalDataBase &fedata,
    FEValuesData<dim,spacedim>                       &data,
-   enum CellSimilarity::Similarity                  &cell_similarity) const
+   CellSimilarity::Similarity                  &cell_similarity) const
 {
 				   // convert data object to internal
 				   // data for this class. fails with
