@@ -1,8 +1,7 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_system.h 18473 2009-03-10 14:53:36Z kronbichler $
-//    Version: $Name$
+//    $Id: fe_system.h 21017 2010-04-19 17:39:12Z heister $
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -29,10 +28,93 @@ DEAL_II_NAMESPACE_OPEN
  * one. To the outside world, the resulting object looks just like a usual
  * finite element object, which is composed of several other finite elements
  * that are possibly of different type. The result is then a vector-valued
- * finite element. Vector valued elements are discussed in a number of
- * tutorial programs, for example @ref step_8 "step-8", @ref step_20
- * "step-20", @ref step_21 "step-21", and in particular in the @ref vector_valued
- * module.
+ * finite element. %Vector valued elements are discussed in a number of
+ * tutorial programs, for example step-8, step-20, step-21, and in particular
+ * in the @ref vector_valued module.
+ *
+ * <h3>FESystem, components and blocks</h3>
+ *
+ * An FESystem, except in the most trivial case, produces a vector-valued
+ * finite element with several components. The number of components
+ * corresponds to the dimension of the solution function in the PDE system,
+ * and correspondingly also to the number of equations your PDE system
+ * has. For example, the mixed Laplace system covered in step-20 has $d+1$
+ * components in $d$ space dimensions: the scalar pressure and the $d$
+ * components of the velocity vector. Similarly, the elasticity equation
+ * covered in step-8 has $d$ components in $d$ space dimensions. In general,
+ * the number of components of a FESystem element is the
+ * accumulated number of components of all base elements times their
+ * multiplicities. A bit more on
+ * components is also given in the
+ * @ref GlossComponent "glossary entry on components".
+ *
+ * While the concept of components is important from the viewpoint of a
+ * partial differential equation, the finite element side looks a bit
+ * different Since not only FESystem, but also vector-valued elements like
+ * FE_RaviartThomas, have several components. The concept needed here is a
+ * @ref GlossBlock "block". Each block encompasses the set of degrees of
+ * freedom associated with a single base element of an FESystem, where base
+ * elements with multiplicities count multiple times. These blocks are usually
+ * addressed using the information in DoFHandler::block_info(). The number of
+ * blocks of of a FESystem object is simply the sum of all multiplicities of
+ * base elements.
+ *
+ * For example, the FESystem for the Taylor-Hood element for the
+ * three-dimensional Stokes problem can be built using the code
+ *
+ * @code
+ * FE_Q<3> u(2);
+ * FE_Q<3> p(1);
+ * FESystem<3> sys1(u,3, p,1);
+ * @endcode
+ *
+ * This example creates an FESystem @p sys1 with four components, three for
+ * the velocity components and one for the pressure, and also four blocks with
+ * the degrees of freedom of each of the velocity components and the pressure
+ * in a separate block each. The number of blocks is four since the first base
+ * element is repeated three times.
+ *
+ * On the other hand, a Taylor-Hood element can also be constructed using
+ *
+ * @code
+ * FESystem<3> U(u,3);
+ * FESystem<3> sys2(U,1, p,1);
+ * @endcode
+ *
+ * The FESystem @p sys2 created here has the same four components, but the
+ * degrees of freedom are distributed into only two blocks. The first block
+ * has all velocity degrees of freedom from @p U, while the second block
+ * contains the pressure degrees of freedom. Note that while @p U itself has 3
+ * blocks, the FESystem @p sys2 does not attempt to split @p U into its base
+ * elements but considers it a block of its own. By blocking all velocities
+ * into one system first as in @p sys2, we achieve the sam block structure
+ * that would be generated if instead of using a $Q_2^3$ element for the
+ * velocities we had used vector-valued base elements, for instance like using
+ * a mixed discretization of Darcy's law using
+ *
+ * @code
+ * FE_RaviartThomas<3> u(1);
+ * FE_DGQ<3> p(1);
+ * FESystem<3> sys3(u,1, p,1);
+ * @endcode
+ *
+ * This example also produces a system with four components, but only
+ * two blocks.
+ *
+ * In most cases, the composed element behaves as if it were a usual
+ * element. It just has more degrees of freedom than most of the "common"
+ * elements. However the underlying structure is visible in the restriction,
+ * prolongation and interface constraint matrices, which do not couple the
+ * degrees of freedom of the base elements. E.g. the continuity requirement is
+ * imposed for the shape functions of the subobjects separately; no
+ * requirement exist between shape functions of different subobjects, i.e. in
+ * the above example: on a hanging node, the respective value of the @p u
+ * velocity is only coupled to @p u at the vertices and the line on the larger
+ * cell next to this vertex, but there is no interaction with @p v and @p w of
+ * this or the other cell.
+ *
+ *
+ * <h3>Internal information on numbering of degrees of freedom</h3>
  *
  * The overall numbering of degrees of freedom is as follows: for each
  * subobject (vertex, line, quad, or hex), the degrees of freedom are
@@ -53,25 +135,9 @@ DEAL_II_NAMESPACE_OPEN
  * <li> Third component on the line:
  *   <tt>p2 = s8</tt>.
  * </ul>
- * Do not rely on this numbering in your application as these
- * internals might change in future. Rather use the functions
- * @p system_to_component_index and @p component_to_system_index,
- * instead.
- *
- * In the most cases, the composed element behaves as if it were a usual element
- * with more degrees of freedom. However the underlying structure is visible in
- * the restriction, prolongation and interface constraint matrices, which do not
- * couple the degrees of freedom of the subobject. E.g. the continuity requirement
- * is imposed for the shape functions of the subobjects separately; no requirement
- * exist between shape functions of different subobjects, i.e. in the above
- * example: on a hanging node, the respective value of the @p u velocity is only
- * coupled to @p u at the vertices and the line on the larger cell next to this
- * vertex, there is no interaction with @p v and @p w of this or the other cell.
- *
- * The number of components of such a system element is the
- * accumulated number of components of all base elements times their
- * multiplicity. The number of blocks of the system is simply the sum
- * of all multiplicities.
+ * That said, you should not rely on this numbering in your application as
+ * these %internals might change in future. Rather use the functions
+ * system_to_component_index() and component_to_system_index().
  *
  * For more information on the template parameter <tt>spacedim</tt>
  * see the documentation of Triangulation.
@@ -129,6 +195,31 @@ class FESystem : public FiniteElement<dim,spacedim>
 	      const FiniteElement<dim,spacedim> &fe2, const unsigned int n2,
 	      const FiniteElement<dim,spacedim> &fe3, const unsigned int n3);
 
+				     /** 
+				      * Constructor for mixed
+				      * discretizations with four
+				      * base elements.
+				      *
+				      * See the other constructor.
+				      */
+    FESystem (const FiniteElement<dim,spacedim> &fe1, const unsigned int n1,
+	      const FiniteElement<dim,spacedim> &fe2, const unsigned int n2,
+	      const FiniteElement<dim,spacedim> &fe3, const unsigned int n3,
+	      const FiniteElement<dim,spacedim> &fe4, const unsigned int n4);
+
+				     /** 
+				      * Constructor for mixed
+				      * discretizations with five
+				      * base elements.
+				      *
+				      * See the other constructor.
+				      */
+    FESystem (const FiniteElement<dim,spacedim> &fe1, const unsigned int n1,
+	      const FiniteElement<dim,spacedim> &fe2, const unsigned int n2,
+	      const FiniteElement<dim,spacedim> &fe3, const unsigned int n3,
+	      const FiniteElement<dim,spacedim> &fe4, const unsigned int n4,
+	      const FiniteElement<dim,spacedim> &fe5, const unsigned int n5);
+
 				     /**
 				      * Destructor.
 				      */
@@ -615,7 +706,7 @@ class FESystem : public FiniteElement<dim,spacedim>
 		    typename Mapping<dim,spacedim>::InternalDataBase &mapping_data,
 		    typename Mapping<dim,spacedim>::InternalDataBase &fe_data,
 		    FEValuesData<dim,spacedim>                       &data,
-		    enum CellSimilarity::Similarity                  &cell_similarity) const;
+		    CellSimilarity::Similarity                  &cell_similarity) const;
 
 				     /**
 				      * Implementation of the same
@@ -685,7 +776,7 @@ class FESystem : public FiniteElement<dim,spacedim>
 		       const unsigned int                                face_no,
 		       const unsigned int                                sub_no,
 		       const Quadrature<dim_1>                          &quadrature,
-		       enum CellSimilarity::Similarity                   cell_similarity,
+		       CellSimilarity::Similarity                   cell_similarity,
 		       typename Mapping<dim,spacedim>::InternalDataBase &mapping_data,
 		       typename Mapping<dim,spacedim>::InternalDataBase &fe_data,
 		       FEValuesData<dim,spacedim>                       &data) const ;
@@ -783,6 +874,34 @@ class FESystem : public FiniteElement<dim,spacedim>
 			  const FiniteElementData<dim> &fe3,
 			  const unsigned int            N3);
 
+				   /**
+				    * with 4 different sub-elements
+				    */
+    static FiniteElementData<dim>
+    multiply_dof_numbers (const FiniteElementData<dim> &fe1,
+			  const unsigned int            N1,
+			  const FiniteElementData<dim> &fe2,
+			  const unsigned int            N2,
+			  const FiniteElementData<dim> &fe3,
+			  const unsigned int            N3,
+			  const FiniteElementData<dim> &fe4,
+			  const unsigned int            N4);
+
+
+				   /**
+				    * and with 5 different sub-elements
+				    */
+    static FiniteElementData<dim>
+    multiply_dof_numbers (const FiniteElementData<dim> &fe1,
+			  const unsigned int            N1,
+			  const FiniteElementData<dim> &fe2,
+			  const unsigned int            N2,
+			  const FiniteElementData<dim> &fe3,
+			  const unsigned int            N3,
+			  const FiniteElementData<dim> &fe4,
+			  const unsigned int            N4,
+			  const FiniteElementData<dim> &fe5,
+			  const unsigned int            N5);
 
 				     /**
 				      * Helper function used in the
@@ -796,30 +915,63 @@ class FESystem : public FiniteElement<dim,spacedim>
 				      * of the sub-element @p fe.
 				      */
     static std::vector<bool>
-    compute_restriction_is_additive_flags (const FiniteElement<dim,spacedim> &fe,
-					   const unsigned int        N);
+    compute_restriction_is_additive_flags (
+      const FiniteElement<dim,spacedim> &fe,
+      const unsigned int        N);
     
 				     /**
 				      * Same as above for mixed elements
 				      * with two different sub-elements.
 				      */
     static std::vector<bool>
-    compute_restriction_is_additive_flags (const FiniteElement<dim,spacedim> &fe1,
-					   const unsigned int        N1,
-					   const FiniteElement<dim,spacedim> &fe2,
-					   const unsigned int        N2);
+    compute_restriction_is_additive_flags (
+      const FiniteElement<dim,spacedim> &fe1,
+      const unsigned int        N1,
+      const FiniteElement<dim,spacedim> &fe2,
+      const unsigned int        N2);
 
 				     /**
 				      * Same as above for mixed elements
 				      * with three different sub-elements.
 				      */
     static std::vector<bool>
-    compute_restriction_is_additive_flags (const FiniteElement<dim,spacedim> &fe1,
-					   const unsigned int        N1,
-					   const FiniteElement<dim,spacedim> &fe2,
-					   const unsigned int        N2,
-					   const FiniteElement<dim,spacedim> &fe3,
-					   const unsigned int        N3);
+    compute_restriction_is_additive_flags (
+      const FiniteElement<dim,spacedim> &fe1,
+      const unsigned int        N1,
+      const FiniteElement<dim,spacedim> &fe2,
+      const unsigned int        N2,
+      const FiniteElement<dim,spacedim> &fe3,
+      const unsigned int        N3);
+
+				   /**
+				    *  with four different sub-elements
+				    */
+    static std::vector<bool>
+    compute_restriction_is_additive_flags (
+      const FiniteElement<dim,spacedim> &fe1,
+      const unsigned int        N1,
+      const FiniteElement<dim,spacedim> &fe2,
+      const unsigned int        N2,
+      const FiniteElement<dim,spacedim> &fe3,
+      const unsigned int        N3,
+      const FiniteElement<dim,spacedim> &fe4,
+      const unsigned int        N4);
+
+				   /**
+				    *  and with five different sub-elements
+				    */
+    static std::vector<bool>
+    compute_restriction_is_additive_flags (
+      const FiniteElement<dim,spacedim> &fe1,
+      const unsigned int        N1,
+      const FiniteElement<dim,spacedim> &fe2,
+      const unsigned int        N2,
+      const FiniteElement<dim,spacedim> &fe3,
+      const unsigned int        N3,
+      const FiniteElement<dim,spacedim> &fe4,
+      const unsigned int        N4,
+      const FiniteElement<dim,spacedim> &fe5,
+      const unsigned int        N5);
 
 				     /**
 				      * Compute the named flags for a
@@ -828,10 +980,11 @@ class FESystem : public FiniteElement<dim,spacedim>
 				      * second argument. This function
 				      * is called from all the above
 				      * functions.
-				     */
+				      */
     static std::vector<bool>
-    compute_restriction_is_additive_flags (const std::vector<const FiniteElement<dim,spacedim>*> &fes,
-                                           const std::vector<unsigned int>              &multiplicities);
+    compute_restriction_is_additive_flags (
+      const std::vector<const FiniteElement<dim,spacedim>*> &fes,
+      const std::vector<unsigned int>              &multiplicities);
     
     
 				     /**
@@ -867,6 +1020,36 @@ class FESystem : public FiniteElement<dim,spacedim>
 				const FiniteElement<dim,spacedim> &fe3,
 				const unsigned int        N3);
 
+				   /**
+				    * Compute the non-zero vector
+				    * components of a composed
+				    * finite element.
+				    */
+   static std::vector<std::vector<bool> >
+    compute_nonzero_components (const FiniteElement<dim,spacedim> &fe1,
+				const unsigned int        N1,
+				const FiniteElement<dim,spacedim> &fe2,
+				const unsigned int        N2,
+				const FiniteElement<dim,spacedim> &fe3,
+				const unsigned int        N3,
+				const FiniteElement<dim,spacedim> &fe4,
+				const unsigned int        N4);
+
+				     /**
+				      * With 5 elements.
+				      */
+   static std::vector<std::vector<bool> >
+    compute_nonzero_components (const FiniteElement<dim,spacedim> &fe1,
+				const unsigned int        N1,
+				const FiniteElement<dim,spacedim> &fe2,
+				const unsigned int        N2,
+				const FiniteElement<dim,spacedim> &fe3,
+				const unsigned int        N3,
+				const FiniteElement<dim,spacedim> &fe4,
+				const unsigned int        N4,
+				const FiniteElement<dim,spacedim> &fe5,
+				const unsigned int        N5);
+
 				     /**
 				      * Compute the nonzero components
 				      * of a list of finite elements
@@ -1084,6 +1267,7 @@ class FESystem : public FiniteElement<dim,spacedim>
 
 
 template <> void FESystem<1>::initialize_unit_face_support_points ();
+template <> void FESystem<1,2>::initialize_unit_face_support_points ();
 
 
 DEAL_II_NAMESPACE_CLOSE
@@ -1091,3 +1275,4 @@ DEAL_II_NAMESPACE_CLOSE
 /*----------------------------  fe_system.h  ---------------------------*/
 #endif
 /*----------------------------  fe_system.h  ---------------------------*/
+
