@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_tools.h 18044 2008-12-29 22:12:33Z bangerth $
+//    $Id: fe_tools.h 20602 2010-02-13 17:44:17Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -163,10 +163,17 @@ class FETools
 				     /**
 				      * Compute the vector required to
 				      * renumber the dofs of a cell by
-				      * block. Furthermore,
-				      * compute the vector storing the
-				      * start indices of each local block
-				      * vector.
+				      * block. Furthermore, compute
+				      * the vector storing either the
+				      * start indices or the size of
+				      * each local block vector.
+				      *
+				      * If the @p bool parameter is
+				      * true, @p block_data is filled
+				      * with the start indices of each
+				      * local block. If it is false,
+				      * then the block sizes are
+				      * returned.
 				      *
 				      * @todo Which way does this
 				      * vector map the numbers?
@@ -175,7 +182,8 @@ class FETools
     static void compute_block_renumbering (
       const FiniteElement<dim,spacedim>&  fe,
       std::vector<unsigned int>& renumbering,
-      std::vector<unsigned int>& start_indices);
+      std::vector<unsigned int>& block_data,
+      bool return_start_indices = true);
     
 				     /**
 				      * @name Generation of local matrices
@@ -471,8 +479,6 @@ class FETools
     static void compute_projection_matrices(const FiniteElement<dim,spacedim> &fe,
 					    std::vector<std::vector<FullMatrix<number> > >& matrices);
 
-//TODO:[WB] Replace this documentation by something comprehensible
-    
 				     /**
                                       * Projects scalar data defined in
                                       * quadrature points to a finite element
@@ -559,7 +565,7 @@ class FETools
                                       * quadrature points.
                                       *
                                       * A use for this function is described
-                                      * in the introduction to the @ref step_18 "step-18"
+                                      * in the introduction to the step-18
                                       * example program.
                                       *
                                       * The opposite of this function,
@@ -646,6 +652,31 @@ class FETools
     compute_interpolation_to_quadrature_points_matrix (const FiniteElement<dim,spacedim> &fe,
                                                        const Quadrature<dim>    &quadrature,
                                                        FullMatrix<double>       &I_q);
+
+
+
+
+				     /**
+				      * This method implements the 
+				      * FETools::compute_projection_from_quadrature_points_matrix
+				      * method for faces of a mesh. 
+				      * The matrix that it returns, X, is face specific
+				      * and its size is fe.dofs_per_cell by
+				      * rhs_quadrature.size().
+				      * The dimension, dim must be larger than 1 for this class,
+				      * since Quadrature<dim-1> objects are required. See the 
+				      * documentation on the Quadrature class for more information.
+				      */
+    template <int dim, int spacedim>
+    static
+    void
+    compute_projection_from_face_quadrature_points_matrix (const FiniteElement<dim, spacedim> &fe,
+						const Quadrature<dim-1>    &lhs_quadrature,
+						const Quadrature<dim-1>    &rhs_quadrature,
+						const typename DoFHandler<dim, spacedim>::active_cell_iterator & cell,
+						unsigned int face,
+						FullMatrix<double>       &X);
+
     
 
 				     //@}
@@ -1332,6 +1363,17 @@ class FETools
 				      */
     DeclException1(ExcLeastSquaresError, double,
 		   << "Least squares fit leaves a gap of " << arg1);
+
+				     /**
+				      * Exception thrown if one variable
+				      * may not be greater than another.
+				      *
+				      * @ingroup Exceptions
+				      */
+    DeclException2 (ExcNotGreaterThan,
+	           int,	int,
+		   << arg1 << " must be greater than " << arg2);
+
   private:
 				     /**
 				      * Return a finite element that
