@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping.h 18473 2009-03-10 14:53:36Z kronbichler $
+//    $Id: mapping.h 20159 2009-11-24 03:25:41Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009 by the deal.II authors
@@ -443,6 +443,24 @@ class Mapping : public Subscriptor
     virtual
     Mapping<dim,spacedim> * clone () const = 0;
     
+				     /**
+				      * Returns whether the mapping preserves
+				      * vertex locations, i.e. whether the
+				      * mapped location of the reference cell
+				      * vertices (given by
+				      * GeometryInfo::unit_cell_vertex())
+				      * equals the result of
+				      * <code>cell-@>vertex()</code>.
+				      *
+				      * For example, implementations in
+				      * derived classes return @p true for
+				      * MappingQ, MappingQ1, MappingCartesian,
+				      * but @p false for MappingQEulerian,
+				      * MappingQ1Eulerian.
+				      */
+    virtual
+    bool preserves_vertex_locations () const = 0;
+
 				     /**
 				      * Exception
 				      */
@@ -553,7 +571,7 @@ class Mapping : public Subscriptor
 		    std::vector<Tensor<3,spacedim> >                          &jacobian_grads,
 		    std::vector<Tensor<2,spacedim> >                          &inverse_jacobians,
 		    std::vector<Point<spacedim> >                             &cell_normal_vectors,
-		    enum CellSimilarity::Similarity                           &cell_similarity
+		    CellSimilarity::Similarity                           &cell_similarity
 	           ) const=0;
 
 
