@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: filtered_iterator.h 14893 2007-08-02 22:36:15Z bangerth $
+//    $Id: filtered_iterator.h 21358 2010-06-24 23:38:14Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -48,7 +48,7 @@ namespace IteratorFilters
 				    *
 				    * @ingroup Iterators
 				    */
-  class Active 
+  class Active
   {
     public:
 				       /**
@@ -59,7 +59,7 @@ namespace IteratorFilters
       template <class Iterator>
       bool operator () (const Iterator &i) const;
   };
-  
+
 				   /**
 				    * Filter that evaluates to true if
 				    * either the iterator points to an
@@ -81,7 +81,7 @@ namespace IteratorFilters
       template <class Iterator>
       bool operator () (const Iterator &i) const;
   };
-  
+
 
 				   /**
 				    * Filter that evaluates to true if
@@ -106,7 +106,7 @@ namespace IteratorFilters
       bool operator () (const Iterator &i) const;
   };
 
-  
+
 				   /**
 				    * Filter for iterators that
 				    * evaluates to true if either the
@@ -117,7 +117,7 @@ namespace IteratorFilters
 				    *
 				    * @ingroup Iterators
 				    */
-  class LevelEqualTo 
+  class LevelEqualTo
   {
     public:
 				       /**
@@ -161,7 +161,7 @@ namespace IteratorFilters
 				    *
 				    * @ingroup Iterators
 				    */
-  class SubdomainEqualTo 
+  class SubdomainEqualTo
   {
     public:
 				       /**
@@ -208,16 +208,18 @@ namespace IteratorFilters
  * called with a pair of iterators denoting a range on which they
  * shall act, by choosing a filtered iterator instead of usual ones.
  *
+ * This class is used in step-18.
+ *
  *
  * <h3>Predicates</h3>
  *
  * The object that represent the condition an iterator has to satisfy
  * only have to provide an interface that allows to call the
- * evaluation operator, i.e. <tt>bool operator() (const
- * BaseIterator&)</tt>. This includes function pointers as well as
- * classes that implement an <tt>bool operator ()(const
- * BaseIterator&)</tt>. Then, the FilteredIterator will skip all
- * objects where the return value of this function is <tt>false</tt>.
+ * evaluation operator, i.e. <code>bool operator() (const
+ * BaseIterator&)</code>. This includes function pointers as well as
+ * classes that implement an <code>bool operator ()(const
+ * BaseIterator&)</code>. Then, the FilteredIterator will skip all
+ * objects where the return value of this function is <code>false</code>.
  *
  *
  * An example of a simple valid predicate is the following: given the function
@@ -242,7 +244,7 @@ namespace IteratorFilters
  *   {
  *     return (static_cast<unsigned int>(c->level()) == level);
  *   };
- * @endcode 
+ * @endcode
  * then
  * @code
  *   std::bind2nd (std::ptr_fun(&level_equal_to<active_cell_iterator>), 3)
@@ -254,7 +256,7 @@ namespace IteratorFilters
  *
  * Finally, classes can be predicates. The following class is one:
  * @code
- *   class Active 
+ *   class Active
  *   {
  *     public:
  *       template <class Iterator>
@@ -281,7 +283,7 @@ namespace IteratorFilters
  *       const unsigned int subdomain_id;
  *   };
  * @endcode
- * Objects like <tt>SubdomainEqualTo(3)</tt> can then be used as predicates.
+ * Objects like <code>SubdomainEqualTo(3)</code> can then be used as predicates.
  *
  * Since whenever a predicate is evaluated it is checked that the
  * iterator checked is actually valid (i.e. not past the end), no
@@ -310,8 +312,8 @@ namespace IteratorFilters
  * however, a value is given to the constructor, that value has either
  * to be past the end, or has to satisfy the predicate. For example,
  * if the predicate only evaluates to true if the level of an object
- * is equal to three, then <tt>tria.begin_active(3)</tt> would be a valid
- * choice while <tt>tria.begin()</tt> would not since the latter also
+ * is equal to three, then <code>tria.begin_active(3)</code> would be a valid
+ * choice while <code>tria.begin()</code> would not since the latter also
  * returns iterators to non-active cells which always start at level
  * 0.
  *
@@ -323,7 +325,7 @@ namespace IteratorFilters
  * assign the next or last previous iterator that satisfies the
  * predicate, i.e. they follow the list of iterators in either
  * direction until they find a matching one (or the past-the-end
- * iterator). Like the <tt>operator=</tt> they return the resulting value
+ * iterator). Like the <code>operator=</code> they return the resulting value
  * of the filtered iterator.
  *
  *
@@ -375,7 +377,7 @@ namespace IteratorFilters
  * Since comparison between filtered and unfiltered iterators is
  * defined, we could as well have let the @p endc variable in the
  * last example be of type
- * <tt>Triangulation<dim>::active_cell_iterator</tt> since it is unchanged
+ * <code>Triangulation@<dim@>::active_cell_iterator</code> since it is unchanged
  * and its value does not depend on the filter.
  *
  * @ingroup grid
@@ -406,11 +408,30 @@ class FilteredIterator : public BaseIterator
 				      * Constructor. Use the given
 				      * predicate for filtering and
 				      * initialize the iterator with
-				      * the given value. This initial
-				      * value has to satisfy the
-				      * predicate.
+				      * the given value.
+				      *
+				      * If the initial value @p bi does
+				      * not satisfy the predicate @p p
+				      * then it is advanced until we
+				      * either hit the the
+				      * past-the-end iterator, or the
+				      * predicate is satisfied. This
+				      * allows, for example, to write
+				      * code like
+				      * @code
+				      *   FilteredIterator<typename Triangulation<dim>::active_cell_iterator>
+				      *     cell (IteratorFilters::SubdomainEqualTo(13),
+				      *           triangulation.begin_active());
+				      * @endcode
+				      *
+				      * If the cell
+				      * <code>triangulation.begin_active()</code>
+				      * does not have a subdomain_id
+				      * equal to 13, then the iterator
+				      * will automatically be advanced
+				      * to the first cell that has.
 				      */
-    template <typename Predicate>    
+    template <typename Predicate>
     FilteredIterator (Predicate           p,
 		      const BaseIterator &bi);
 
@@ -425,7 +446,7 @@ class FilteredIterator : public BaseIterator
 				      * Destructor.
 				      */
     ~FilteredIterator ();
-    
+
 				     /**
 				      * Assignment operator. Copy the
 				      * iterator value of the
@@ -468,7 +489,7 @@ class FilteredIterator : public BaseIterator
 				      */
     FilteredIterator &
     set_to_next_positive (const BaseIterator &bi);
-    
+
 				     /**
 				      * As above, but search for the
 				      * previous iterator from @p bi
@@ -485,7 +506,7 @@ class FilteredIterator : public BaseIterator
 				      */
     FilteredIterator &
     set_to_previous_positive (const BaseIterator &bi);
-    
+
 				     /**
 				      * Compare for equality of the
 				      * underlying iterator values of
@@ -537,7 +558,7 @@ class FilteredIterator : public BaseIterator
 				      *
 				      * We do not compare the
 				      * predicates.
-				      */    
+				      */
     bool operator <  (const FilteredIterator &fi) const;
 
 				     /**
@@ -595,8 +616,8 @@ class FilteredIterator : public BaseIterator
 		    BaseIterator,
 		    << "The element " << arg1
 		    << " with which you want to compare or which you want to"
-		    << " assign is invalid since it does not satisfy the predicate.");
-    
+		    << " assign from is invalid since it does not satisfy the predicate.");
+
   private:
 
 				     /**
@@ -683,7 +704,7 @@ class FilteredIterator : public BaseIterator
 					  * type of this pointer.
 					  */
 	virtual PredicateBase * clone () const;
-	
+
       private:
 					 /**
 					  * Copy of the predicate.
@@ -712,6 +733,28 @@ class FilteredIterator : public BaseIterator
 
 
 
+/**
+ * Create an object of type FilteredIterator given the base iterator
+ * and predicate.  This function makes the creation of temporary
+ * objects (for example as function arguments) a lot simpler because
+ * one does not have to explicitly specify the type of the base
+ * iterator by hand -- it is deduced automatically here.
+ *
+ * @author Wolfgang Bangerth
+ * @relates FilteredIterator
+ */
+template <typename BaseIterator, typename Predicate>
+FilteredIterator<BaseIterator>
+make_filtered_iterator (const BaseIterator &i,
+			const Predicate    &p)
+{
+  FilteredIterator<BaseIterator> fi(p);
+  fi.set_to_next_positive (i);
+  return fi;
+}
+
+
+
 /* ------------------ Inline functions and templates ------------ */
 
 
@@ -736,8 +779,9 @@ FilteredIterator (Predicate          p,
 		BaseIterator (bi),
 		predicate (new PredicateTemplate<Predicate>(p))
 {
-  Assert ((this->state() != IteratorState::valid) || (*predicate) (*this),
-	  ExcInvalidElement(bi));
+  if ((this->state() == IteratorState::valid) &&
+      ! (*predicate) (*this))
+    set_to_next_positive (bi);
 }
 
 
@@ -785,7 +829,7 @@ FilteredIterator<BaseIterator>::
 operator = (const BaseIterator &bi)
 {
   Assert ((bi.state() != IteratorState::valid) || (*predicate)(bi),
-	  ExcInvalidElement(bi));  
+	  ExcInvalidElement(bi));
   BaseIterator::operator = (bi);
   return *this;
 }
@@ -802,7 +846,7 @@ set_to_next_positive (const BaseIterator &bi)
   while ((this->state() == IteratorState::valid) &&
 	 ( ! (*predicate)(*this)))
     BaseIterator::operator++ ();
-  
+
   return *this;
 }
 
@@ -818,7 +862,7 @@ set_to_previous_positive (const BaseIterator &bi)
   while ((this->state() == IteratorState::valid) &&
 	 ( ! (*predicate)(*this)))
     BaseIterator::operator-- ();
-  
+
   return *this;
 }
 
@@ -919,7 +963,7 @@ FilteredIterator<BaseIterator>::
 operator ++ (int)
 {
   const FilteredIterator old_state = *this;
-  
+
   if (this->state() == IteratorState::valid)
     do
       BaseIterator::operator++ ();
@@ -954,7 +998,7 @@ FilteredIterator<BaseIterator>::
 operator -- (int)
 {
   const FilteredIterator old_state = *this;
-  
+
   if (this->state() == IteratorState::valid)
     do
       BaseIterator::operator-- ();
@@ -998,7 +1042,7 @@ clone () const
 
 
 
-namespace IteratorFilters 
+namespace IteratorFilters
 {
 
 // ---------------- IteratorFilters::Active ---------
@@ -1034,7 +1078,7 @@ namespace IteratorFilters
   }
 
 
-// ---------------- IteratorFilters::LevelEqualTo ---------  
+// ---------------- IteratorFilters::LevelEqualTo ---------
   inline
   LevelEqualTo::LevelEqualTo (const unsigned int level)
 		  :
