@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: grid_in.h 18632 2009-04-16 15:07:22Z bangerth $
+//    $Id: grid_in.h 21158 2010-06-06 02:50:58Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -30,27 +30,27 @@ class SubCellData;
 
 
 /**
- * This class implements an input mechanism for grid data. It allows to
- * read a grid structure into a triangulation object. At present, UCD
- * (unstructured cell data), DB Mesh, XDA, Gmsh, Tecplot, NetCDF and Cubit
- * are supported as input format for grid data. Any numerical data after
- * the block of topological information is ignored. Notice also that at
- * the moment in the codimension 1 case only UCD and Gmsh format are
- * accepted.
+ * This class implements an input mechanism for grid data. It allows to read a
+ * grid structure into a triangulation object. At present, UCD (unstructured
+ * cell data), DB Mesh, XDA, Gmsh, Tecplot, NetCDF and Cubit are supported as
+ * input format for grid data. Any numerical data after the block of geometric
+ * (vertex locations) and topological (how vertices form cells) information is
+ * ignored. Notice also that at the moment in the codimension 1 case only UCD
+ * and Gmsh format are accepted.
  *
- * Since the coarse mesh fed into a @p Triangulation object cannot
- * have hanging nodes, strange things will happen if the input file
- * has them. This is due to the fact that most mesh description
- * formats do not store neighborship information between cells, so the
- * grid reading functions have to regenerate it. They do so by
- * checking whether two cells have a common face. If there are hanging
- * nodes in a triangulation, adjacent cells have no common face, so
- * the grid reader concludes that the adjacent cells have no neighbors
- * along these faces and must therefore be at the boundary. In effect,
- * an internal crack of the domain is introduced this way. Since such
- * cases are very hard to detect, the library does not make any
- * attempt to abort on such situations, and you will get a
- * triangulation that probably does not do what you want.
+ * Since the coarse mesh fed into a @p Triangulation object cannot have
+ * hanging nodes, strange things will happen if the input file has them. This
+ * is due to the fact that most mesh description formats do not store
+ * neighborship information between cells, so the grid reading functions have
+ * to regenerate it. They do so by checking whether two cells have a common
+ * face. If there are hanging nodes in a triangulation, adjacent cells have no
+ * common (complete) face, so the grid reader concludes that the adjacent
+ * cells have no neighbors along these faces and must therefore be at the
+ * boundary. In effect, an internal crack of the domain is introduced this
+ * way. Since such cases are very hard to detect, the library does not make
+ * any attempt to abort on such situations, and you will get a triangulation
+ * that probably does not do what you want. A solution to this problem is the
+ * PersistantTriangulation class.
  *
  * Note: if you experience unexpected problems with the use of this
  * class, be sure to read the documentation right until the end, and
@@ -142,11 +142,10 @@ class SubCellData;
  * Cubit at this time. However, Cubit can export in UCD format using a simple
  * plugin, and the resulting UCD file can then be read by this class. The
  * plugin script can be found on the deal.II wiki page,
- * http://www-dimat.unipv.it/heltai/wikideal/, in the
- * section on external tools.
+ * http://sourceforge.net/apps/mediawiki/dealii/index.php?title=Mesh_Input_and_Output_Interfaces .
  *
  * There is also a little program, written by Jean-Paul Pelteret, that
- * can be found in the contrib/mesh_conversion directory and that can
+ * can be found in the <code>contrib/mesh_conversion</code> directory and that can
  * convert Cubit ABAQUS files into UCD format that can be read in as
  * discussed above. The program was designed with the intention of
  * exporting geometries with complex boundary condition surfaces and
@@ -217,6 +216,24 @@ class SubCellData;
  * that class if you experience unexpected problems when reading grids
  * through this class.
  *
+ *
+ * <h3>Dealing with distorted mesh cells</h3>
+ *
+ * For each of the mesh reading functions, the last call is always to
+ * Triangulation::create_triangulation(). That function checks whether all the
+ * cells it creates as part of the coarse mesh are distorted or not (where
+ * distortion here means that the Jacobian of the mapping from the reference
+ * cell to the real cell has a non-positive determinant, i.e. the cell is
+ * pinched or twisted; see the entry on @ref GlossDistorted "distorted cells"
+ * in the glossary). If it finds any such cells, it throws an exception. This
+ * exception is not caught in the grid reader functions of the current class,
+ * and so will propagate through to the function that called it. There, you
+ * can catch and ignore the exception if you are certain that there is no harm
+ * in dealing with such cells. If you were not aware that your mesh had such
+ * cells, your results will likely be of dubious quality at best if you ignore
+ * the exception.
+ *
+ *
  * @ingroup grid
  * @ingroup input
  * @author Wolfgang Bangerth, 1998, 2000, Luca Heltai, 2004, 2007
@@ -264,8 +281,8 @@ class GridIn
 
 				     /**
 				      * Read from the given stream. If
-				      * no format is given, the
-				      * #default_format variable is
+				      * no format is given,
+				      * GridIn::Format::Default is
 				      * used.
 				      */
     void read (std::istream &in, Format format=Default);
@@ -415,12 +432,15 @@ class GridIn
 		    << "3 Quadrilateral (4 nodes, 4 edges).\n"
 		    << "5 Hexahedron (8 nodes, 12 edges, 6 faces).\n"
 		    << "15 Point (1 node, ignored when read)");
+
+    
+    DeclException0 (ExcGmshNoCellInformation);
   protected:
 				     /**
 				      * Store address of the triangulation to
 				      * be fed with the data read in.
 				      */
-    SmartPointer<Triangulation<dim,spacedim> > tria;
+    SmartPointer<Triangulation<dim,spacedim>,GridIn<dim,spacedim> > tria;
     
 				     /**
 				      * This function can write the
