@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_accessor.h 18043 2008-12-29 22:11:48Z bangerth $
+//    $Id: tria_accessor.h 21113 2010-05-11 03:47:29Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -36,6 +36,8 @@ template <typename Accessor> class TriaRawIterator;
 template <typename Accessor> class TriaIterator;
 template <typename Accessor> class TriaActiveIterator;
 
+template <int dim, int spacedim> class Boundary;
+
 
 namespace internal
 {
@@ -49,9 +51,69 @@ namespace internal
   namespace TriaAccessor
   {
     struct Implementation;
+
+				     /**
+				      * Implementation of a type with
+				      * which to store the level of an
+				      * accessor object. We only need
+				      * it for the case that
+				      * <tt>structdim ==
+				      * dim</tt>. Otherwise, an empty
+				      * object is sufficient.
+				      */
+    template <int structdim, int dim> struct PresentLevelType
+    {
+	struct type {
+					     /**
+					      * Dummy
+					      * constructor. Only
+					      * level zero is allowed.
+					      */
+	    type (const int level)
+	      {
+		Assert (level == 0, ExcInternalError());
+	      }
+
+					     /**
+					      * Dummy conversion
+					      * operator. Returns
+					      * level zero.
+					      */
+	    operator int () const
+	      {
+		return 0;
+	      }
+
+	    void operator ++ () const
+	      {
+		Assert (false, ExcInternalError());
+	      }
+
+	    void operator -- () const
+	      {
+		Assert (false, ExcInternalError());
+	      }
+	};
+    };
+
+
+				     /**
+				      * Implementation of a type with
+				      * which to store the level of an
+				      * accessor object. We only need
+				      * it for the case that
+				      * <tt>structdim ==
+				      * dim</tt>. Otherwise, an empty
+				      * object is sufficient.
+				      */
+    template <int dim> struct PresentLevelType<dim,dim>
+    {
+	typedef int type;
+    };
+
   }
 }
-template <int celldim, int dim, int spacedim>	class TriaAccessor;
+template <int structdim, int dim, int spacedim>	class TriaAccessor;
 template <int dim, int spacedim>		class TriaAccessor<0, dim, spacedim>;
 
 // note: the file tria_accessor.templates.h is included at the end of
@@ -65,29 +127,36 @@ template <int dim, int spacedim>		class TriaAccessor<0, dim, spacedim>;
  */
 namespace TriaAccessorExceptions
 {
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcCellNotUsed);
 				   /**
+				    * The cell is not an @ref
+				    * GlossActive "active" cell, but
+				    * it already has children. Some
+				    * operations, like setting
+				    * refinement flags or accessing
+				    * degrees of freedom are only
+				    * possible on active cells.
+				    *
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcCellNotActive);
 				   /**
-				    * @ingroup Exceptions
-				    */
-  DeclException1 (ExcInvalidNeighbor,
-		  int,
-		  << "Neighbor indices must be between 0 and 2^dim-1, but "
-		  << "yours was " << arg1);
-				   /**
+				    * Trying to access the children of
+				    * a cell which is in fact active.
+				    *
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcCellHasNoChildren);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcUnusedCellAsChild);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
@@ -96,41 +165,56 @@ namespace TriaAccessorExceptions
 		  << "You can only set the child index if the cell has no "
 		  << "children, or clear it. The given "
 		  << "index was " << arg1 << " (-1 means: clear children)");
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcUnusedCellAsNeighbor);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcUncaughtCase);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcDereferenceInvalidObject);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcCantCompareIterators);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcNeighborIsCoarser);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcNeighborIsNotCoarser);
 				   /**
+				    * You are trying to access the
+				    * level of a face, but faces have
+				    * no inherent level. The level of
+				    * a face can only be determined by
+				    * the level of an adjacent face,
+				    * which in turn implies that a
+				    * face can have several levels.
+				    *
 				    * @ingroup Exceptions
 				    */
   DeclException0 (ExcFacesHaveNoLevel);
+//TODO: Write documentation!
 				   /**
 				    * @ingroup Exceptions
 				    */
   DeclException1 (ExcSetOnlyEvenChildren,
 		  int,
 		  << "You can only set the child index of an even numbered child."
-		  << "The number of the child given was " << arg1 << ".");    
+		  << "The number of the child given was " << arg1 << ".");
 }
 
 
@@ -143,24 +227,62 @@ namespace TriaAccessorExceptions
  * operators and the like), but has no functionality to actually
  * dereference data. This is done in the derived classes.
  *
- * This template is used for faces and edges, which have no level in
- * the triangulation hierarchy associated with them. There exists a
- * partial specialization of the current class template where
- * <tt>structdim</tt> (the dimensionality of the object represented,
- * for example 1 for a line) equals <tt>dim</tt> (the dimensionality
- * of the space the object lives in, for example 3 if we solve
- * three-dimensional problems).
+ * In the implementation, the behavior of this class differs between
+ * the cases where <tt>structdim==dim</tt> (cells of a mesh) and
+ * <tt>structdim&lt;dim</tt> (faces and edges). For the latter,
+ * #present_level is always equal to zero and the constructors may not
+ * receive a positive value there. For cells, any level is
+ * possible, but only those within the range of the levels of the
+ * Triangulation are reasonable. Furthermore, the function objects()
+ * returns either the container with all cells on the same level or
+ * the container with all objects of this dimension (<tt>structdim&lt;dim</tt>).
  *
  * Some internals of this class are discussed in @ref IteratorAccessorInternals .
  *
  * @ingroup grid
  * @ingroup Accessors
- * @author Wolfgang Bangerth, 1998
+ * @author Wolfgang Bangerth, Guido Kanschat, 1998, 2010
  */
 template <int structdim, int dim, int spacedim=dim>
 class TriaAccessorBase
 {
-  protected:
+  public:
+				     /**
+				      *  Dimension of the space the
+				      *  object represented by this
+				      *  accessor lives in. For
+				      *  example, if this accessor
+				      *  represents a quad that is
+				      *  part of a two-dimensional
+				      *  surface in four-dimensional
+				      *  space, then this value is
+				      *  four.
+				      */
+    static const unsigned int space_dimension = spacedim;
+
+				     /**
+				      * Dimensionality of the object
+				      * that the thing represented by
+				      * this accessopr is part of. For
+				      * example, if this accessor
+				      * represents a line that is part
+				      * of a hexahedron, then this
+				      * value will be three.
+				      */
+    static const unsigned int dimension = dim;
+
+				     /**
+				      * Dimensionality of the current
+				      * object represented by this
+				      * accessor. For example, if it
+				      * is line (irrespective of
+				      * whether it is part of a quad
+				      * or hex, and what dimension we
+				      * are in), then this value
+				      * equals 1.
+				      */
+    static const unsigned int structure_dimension = structdim;
+
 				     /**
 				      * Declare the data type that
 				      * this accessor class expects to
@@ -182,6 +304,12 @@ class TriaAccessorBase
 		      const int                 index  = -1,
 		      const AccessorData       *       =  0);
 
+				     /**
+				      *  Copy constructor. Creates an
+				      *  object with exactly the same data.
+				      */
+    TriaAccessorBase (const TriaAccessorBase &);
+
 				     /**
 				      *  Copy operator. Since this is
 				      *  only called from iterators,
@@ -195,6 +323,14 @@ class TriaAccessorBase
 				      */
     void copy_from (const TriaAccessorBase &);
 
+				     /**
+				      *  Copy operator. Creates an
+				      *  object with exactly the same data.
+				      */
+    TriaAccessorBase & operator = (const TriaAccessorBase &);
+
+  protected:
+//TODO: Wolfgang, is this constructor EVER even asked for?
 				     /**
 				      *  Copy operator. This is normally
 				      *  used in a context like
@@ -213,17 +349,12 @@ class TriaAccessorBase
 				      *  used anyway.
 				      */
     void operator = (const TriaAccessorBase *);
-    
-				     /**
-				      *  Same as above.
-				      */
-    TriaAccessorBase & operator = (const TriaAccessorBase &);
 
 				     /**
-				      *  Compare for equality.            
+				      *  Compare for equality.
 				      */
     bool operator == (const TriaAccessorBase &) const;
-	
+
 				     /**
 				      * Compare for inequality.
 				      */
@@ -266,7 +397,7 @@ class TriaAccessorBase
 				     /**
 				      * @}
 				      */
-    
+
 				     /**
 				      * Access to the other objects of
 				      * a Triangulation with same
@@ -274,7 +405,7 @@ class TriaAccessorBase
 				      */
     internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<structdim> > &
     objects () const;
-    
+
   public:
 				     /**
 				      * Data type to be used for passing
@@ -284,7 +415,7 @@ class TriaAccessorBase
 				      * number of these parameters is.
 				      */
     typedef void * LocalData;
-    
+
 				     /**
 				      * @name Iterator address and state
 				      */
@@ -297,245 +428,15 @@ class TriaAccessorBase
 				      *  pointed to belongs to.
 				      *  This is only valid for cells.
 				      */
-    static int level ();
-    
-				     /**
-				      *  Return the index of the
-				      *  element presently pointed to
-				      *  on the present level.
-				      */
-    int index () const;
-    
-				     /**
-				      *  Return the state of the
-				      *  iterator.  For the different
-				      *  states an accessor can be in,
-				      *  refer to the
-				      *  TriaRawIterator
-				      *  documentation.
-				      */
-    IteratorState::IteratorStates state () const;
-
-				     /**
-				      * Return a pointer to the
-				      * triangulation which the object
-				      * pointed to by this class
-				      * belongs to.
-				      */
-    const Triangulation<dim,spacedim> & get_triangulation () const;
-    
-				     /**
-				      * @}
-				      */
-  protected:
-
-				     /**
-				      *  Used to store the index of
-				      *  the element presently pointed
-				      *  to on the level presentl
-				      *  used.
-				      */
-    int present_index;
-    
-				     /**
-				      *  Pointer to the triangulation
-				      *  which we act on.
-				      */
-    const Triangulation<dim,spacedim> *tria;
-    
-  private:
-    
-				     /**
-				      *  Space dimension of the Accessor
-				      */  
-    static const unsigned int space_dimension = spacedim; 
-
-    static const unsigned int dimension = dim;
-
-    static const unsigned int structure_dimension = structdim;
-
-    template <typename Accessor> friend class TriaRawIterator;
-    template <typename Accessor> friend class TriaIterator;
-    template <typename Accessor> friend class TriaActiveIterator;
-};
-
-
-
-/**
- * A base class for the accessor classes used by TriaRawIterator and
- * derived classes.
- *
- * This class offers only the basic functionality required by the
- * iterators (stores the necessary data members, offers comparison
- * operators and the like), but has no functionality to actually
- * dereference data. This is done in the derived classes.
- *
- * This template is a partial specialization of the general template
- * used for cells only, i.e. for the case where <tt>structdim</tt>
- * (the dimensionality of the object represented, for example 1 for a
- * line) equals <tt>dim</tt> (the dimensionality of the triangulation
- * the object lives in, for example 2 if we solve problems on a
- * two-dimensional manifold -- whether this manifold is itself the
- * space $R^2$ or embedded in a higher-dimensional space whose
- * dimensionality is determined by the <code>spacedim</code> template
- * argument). The reason for using a partial specialization rather
- * than the general template is that cells have a level in the
- * triangulation hierarchy associated with them, whereas faces and
- * edges do not.
- *
- * Some internals of this class are discussed in @ref IteratorAccessorInternals .
- *
- * @ingroup grid
- * @ingroup Accessors
- * @author Wolfgang Bangerth, 1998
- */
-template <int dim, int spacedim>
-class TriaAccessorBase<dim,dim,spacedim>
-{
-  protected:
-				     /**
-				      * Declare the data type that
-				      * this accessor class expects to
-				      * get passed from the iterator
-				      * classes. Since the pure
-				      * triangulation iterators need
-				      * no additional data, this data
-				      * type is @p void.
-				      */
-    typedef void AccessorData;
-
-				     /**
-				      *  Constructor. Protected, thus
-				      *  only callable from friend
-				      *  classes.
-				      */
-    TriaAccessorBase (const Triangulation<dim,spacedim> *parent =  0,
-		      const int                 level  = -1,
-		      const int                 index  = -1,
-		      const AccessorData       *       =  0);
-
-				     /**
-				      *  Copy operator. Since this is
-				      *  only called from iterators,
-				      *  do not return anything, since
-				      *  the iterator will return
-				      *  itself.
-				      *
-				      *  This method is protected,
-				      *  since it is only to be called
-				      *  from the iterator class.
-				      */
-    void copy_from (const TriaAccessorBase &);
-
-				     /**
-				      *  Copy operator. This is normally
-				      *  used in a context like
-				      *  <tt>iterator a,b;  *a=*b;</tt>. Since
-				      *  the meaning is to copy the object
-				      *  pointed to by @p b to the object
-				      *  pointed to by @p a and since
-				      *  accessors are not real but
-				      *  virtual objects, this operation
-				      *  is not useful for iterators on
-				      *  triangulations. We declare this
-				      *  function here private, thus it may
-				      *  not be used from outside.
-				      *  Furthermore it is not implemented
-				      *  and will give a linker error if
-				      *  used anyway.
-				      */
-    void operator = (const TriaAccessorBase *);
-        
-				     /**
-				      *  Same as above.
-				      */
-    TriaAccessorBase & operator = (const TriaAccessorBase &);
-
-				     /**
-				      * @name Advancement of iterators
-				      */
-				     /**
-				      * @{
-				      */
-    
-				     /**
-				      *  This operator advances the
-				      *  iterator to the next element.
-				      *
-				      *  For cells only:
-				      *  The next element is next on
-				      *  this level if there are
-				      *  more. If the present element
-				      *  is the last on this level,
-				      *  the first on the next level
-				      *  is accessed.
-				      */
-    void operator ++ ();
-
-    				     /**
-				      *  This operator moves the
-				      *  iterator to the previous
-				      *  element.
-				      *
-				      *  For cells only:
-				      *  The previous element is
-				      *  previous on this level if
-				      *  <tt>index>0</tt>. If the present
-				      *  element is the first on this
-				      *  level, the last on the
-				      *  previous level is accessed.
-				      */
-    void operator -- ();
-
-				     /**
-				      * Access to the other objects of
-				      * a Triangulation with same
-				      * dimension.
-				      */
-    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<dim> > &
-    objects () const;
-
-  protected:
-    
-				     /**
-				      *  Compare for equality.            
-				      */
-    bool operator == (const TriaAccessorBase &) const;
-	
-				     /**
-				      * Compare for inequality.
-				      */
-    bool operator != (const TriaAccessorBase &) const;
-
-  public:
-				     /**
-				      * Data type to be used for passing
-				      * parameters from iterators to the
-				      * accessor classes in a unified
-				      * way, no matter what the type of
-				      * number of these parameters is.
-				      */
-    typedef void * LocalData;
-    
-				     /** @name Iterator address and state
-				      */
-				     /**
-				      * @{
-				      */
-				     /**
-				      *  Return the level the element
-				      *  pointed to belongs to.
-				      *  This is only valid for cells.
-				      */
     int level () const;
-    
+
 				     /**
 				      *  Return the index of the
 				      *  element presently pointed to
 				      *  on the present level.
 				      */
     int index () const;
-    
+
 				     /**
 				      *  Return the state of the
 				      *  iterator.  For the different
@@ -553,19 +454,18 @@ class TriaAccessorBase<dim,dim,spacedim>
 				      * belongs to.
 				      */
     const Triangulation<dim,spacedim> & get_triangulation () const;
-    
+
 				     /**
 				      * @}
 				      */
   protected:
 				     /**
-				      *  Used to store the level
-				      *  presently pointed to. For
-				      *  faces this should always be
-				      *  0.
+				      * The level if this is a cell
+				      * (<tt>structdim==dim</tt>). Else,
+				      * contains zero.
 				      */
-    int present_level;
-    
+    typename internal::TriaAccessor::PresentLevelType<structdim,dim>::type present_level;
+
 				     /**
 				      *  Used to store the index of
 				      *  the element presently pointed
@@ -573,32 +473,22 @@ class TriaAccessorBase<dim,dim,spacedim>
 				      *  used.
 				      */
     int present_index;
-    
+
 				     /**
 				      *  Pointer to the triangulation
 				      *  which we act on.
 				      */
     const Triangulation<dim,spacedim> *tria;
 
-				     /**
-				      *  Space dimension of the Accessor
-				      */  
-    static const unsigned int space_dimension = spacedim; 
-
-    static const unsigned int dimension = dim;
-
-    static const unsigned int structure_dimension = dim;
-
   private:
-    
+
     template <typename Accessor> friend class TriaRawIterator;
     template <typename Accessor> friend class TriaIterator;
     template <typename Accessor> friend class TriaActiveIterator;
-
-    template <int, int, int> friend class TriaAccessor;
 };
 
 
+
 /**
  * A class that represents accessor objects to iterators that don't
  * make sense such as quad iterators in on 1d meshes.  This class can
@@ -618,6 +508,7 @@ class TriaAccessorBase<dim,dim,spacedim>
  * for syntactic correctness, none of the functions do anything but
  * generate errors.
  *
+ * @ingroup Accessors
  * @author Wolfgang Bangerth, 2008
  */
 template <int structdim, int dim, int spacedim=dim>
@@ -647,7 +538,7 @@ class InvalidAccessor :  public TriaAccessorBase<structdim,dim,spacedim>
 		     const int                 level      = -1,
 		     const int                 index      = -1,
 		     const AccessorData       *local_data =  0);
-    
+
 				     /**
 				      * Copy constructor.  This class
 				      * is used for iterators that
@@ -672,7 +563,7 @@ class InvalidAccessor :  public TriaAccessorBase<structdim,dim,spacedim>
 				      */
     template <typename OtherAccessor>
     InvalidAccessor (const OtherAccessor &);
-    
+
 				     /**
 				      * Dummy copy operation.
 				      */
@@ -690,7 +581,7 @@ class InvalidAccessor :  public TriaAccessorBase<structdim,dim,spacedim>
 				      */
     void operator ++ () const;
     void operator -- () const;
-    
+
 				     /**
 				      * Dummy function representing
 				      * whether the accessor points to
@@ -721,6 +612,7 @@ class InvalidAccessor :  public TriaAccessorBase<structdim,dim,spacedim>
  * <code>spacedim@>dim</code> the triangulation is of a manifold
  * embedded in a higher dimensional space).
  *
+ * @ingroup Accessors
  * @author Wolfgang Bangerth and others, 1998, 2000, 2008
  */
 template <int structdim, int dim, int spacedim>
@@ -775,7 +667,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      */
     template <int structdim2, int dim2, int spacedim2>
     TriaAccessor (const TriaAccessor<structdim2,dim2,spacedim2> &);
-    
+
 				     /**
 				      *  Test for the element being
 				      *  used or not.  The return
@@ -798,7 +690,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				     /**
 				      * @{
 				      */
-    
+
 				     /**
 				      *  Return the global index of i-th
 				      *  vertex of the current object. The
@@ -814,7 +706,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *  associated with it. For this, see the
 				      *  @p DoFAccessor::vertex_dof_index
 				      *  functions.
-				      */ 
+				      */
     unsigned int vertex_index (const unsigned int i) const;
 
 				     /**
@@ -840,7 +732,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * generated.
 				      */
     unsigned int line_index (const unsigned int i) const;
-    
+
     				     /**
 				      * Pointer to the @p ith quad
 				      * bounding this object.
@@ -934,7 +826,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
                                       * otherwise. In 1d and 2d, this is always
                                       * @p true, but in 3d it may be different,
                                       * see the respective discussion in the
-                                      * documentation of the 
+                                      * documentation of the
                                       * GeometryInfo classe.
                                       *
                                       * This function is really only
@@ -946,7 +838,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				     /**
 				      * @}
 				      */
-    
+
 				     /**
 				      *  @name Accessing children
 				      */
@@ -1016,7 +908,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * zero.
 				      */
     unsigned int max_refinement_depth () const;
-    
+
 				     /**
 				      *  Return an iterator to the @p ith
 				      *  child.
@@ -1045,7 +937,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * Return the RefinementCase
 				      * of this cell.
 				      */
-    RefinementCase<structdim> refinement_case() const;
+    RefinementCase<structdim> refinement_case () const;
 
 				     /**
 				      *  Index of the @p ith child.
@@ -1066,7 +958,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *  the child does not exist, -1 is
 				      *  returned.
 				      */
-    int isotropic_child_index (const unsigned int i) const;    
+    int isotropic_child_index (const unsigned int i) const;
 				     /**
 				      * @}
 				      */
@@ -1077,7 +969,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				     /**
 				      * @{
 				      */
-    
+
 				     /**
 				      * Boundary indicator of this
 				      * object.
@@ -1147,13 +1039,13 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * current function.
 				      */
     void set_all_boundary_indicators (const unsigned char) const;
-    
+
 				     /**
 				      * Return whether this object is at the
 				      * boundary. Obviously, the use of this
 				      * function is only possible for
 				      * <tt>dim@>structdim</tt>; however, for
-				      * <tt>dim==celldim</tt>, an object is a
+				      * <tt>dim==structdim</tt>, an object is a
 				      * cell and the CellAccessor class offers
 				      * another possibility to determine
 				      * whether a cell is at the boundary or
@@ -1161,10 +1053,22 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      */
     bool at_boundary () const;
 
+				     /**
+				      * Return a constant reference to a
+				      * boundary object used for this
+				      * object. This function is a shortcut to
+				      * retrieving the boundary indicator
+				      * using boundary_indicator() and then
+				      * asking the
+				      * Triangulation::get_boundary() function
+				      * for the boundary object.
+				      */
+    const Boundary<dim,spacedim> & get_boundary () const;
+
 				     /**
 				      * @}
 				      */
-    
+
 
 				     /**
 				      * @name User data
@@ -1195,7 +1099,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 
     				     /**
 				      *  Clear the user flag for this
-				      * and all descendants. 
+				      * and all descendants.
 				      */
     void recursively_clear_user_flag () const;
 
@@ -1204,7 +1108,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * independent if pointer or index.
 				      */
     void clear_user_data () const;
-    
+
 				     /**
 				      * Set the user pointer
 				      * to @p p.
@@ -1216,7 +1120,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * to a @p NULL pointer.
 				      */
     void clear_user_pointer () const;
-    
+
 				     /**
 				      * Access the value of the user
 				      * pointer. It is in the
@@ -1259,7 +1163,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * the triangulation.
 				      */
     void recursively_set_user_pointer (void *p) const;
-    
+
 				     /**
 				      * Clear the user pointer of this
 				      * object and all of its
@@ -1269,18 +1173,18 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * function.
 				      */
     void recursively_clear_user_pointer () const;
-    
+
 				     /**
 				      * Set the user index
 				      * to @p p.
 				      */
     void set_user_index (const unsigned int p) const;
-    
+
 				     /**
 				      * Reset the user index to 0.
 				      */
     void clear_user_index () const;
-    
+
 				     /**
 				      * Access the value of the user
 				      * index.
@@ -1319,7 +1223,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				     /**
 				      * @}
 				      */
-    
+
 				     /**
 				      * @name Geometric information about an object
 				      */
@@ -1339,6 +1243,24 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      */
     double diameter () const;
 
+				     /** 
+				      * Length of an object in the direction
+				      * of the given axis, specified in the
+				      * local coordinate system. See the
+				      * documentation of GeometryInfo for the
+				      * meaning and enumeration of the local
+				      * axes.
+				      *
+				      * Note that the "length" of an object
+				      * can be interpreted in a variety of
+				      * ways. Here, we choose it as the
+				      * maximal length of any of the edges of
+				      * the object that are parallel to the
+				      * chosen axis on the reference cell.
+				      */
+    double extent_in_direction (const unsigned int axis) const;
+
+
     				     /**
 				      * Center of the object. The center of an
 				      * object is defined to be the average of
@@ -1351,6 +1273,8 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * object if higher order mappings are
 				      * used.
 				      */
+
+
     Point<spacedim> center () const;
 
 				     /**
@@ -1367,11 +1291,41 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * used.
 				      */
     double measure () const;
+
+				     /**
+				      * Return true if the current object is a
+				      * translation of the given argument.
+				      *
+				      * @note For the purpose of a
+				      * triangulation, cells, faces, etc are
+				      * only characterized by their
+				      * vertices. The current function
+				      * therefore only compares the locations
+				      * of vertices. For many practical
+				      * applications, however, it is not only
+				      * the vertices that determine whether
+				      * one cell is a translation of another,
+				      * but also how the cell is mapped from
+				      * the reference cell to its location in
+				      * real space. For example, if we are
+				      * using higher order mappings, then not
+				      * only do the vertices have to be
+				      * translations of each other, but also
+				      * the points along edges. In these
+				      * questions, therefore, it would be
+				      * appropriate to ask the mapping, not
+				      * the current function, whether two
+				      * objects are translations of each
+				      * other.
+				      */
+    bool
+    is_translation_of (const TriaIterator<TriaAccessor<structdim,dim,spacedim> > &o) const;
+
 				     /**
 				      * @}
 				      */
-    
-    
+
+
   private:
 				     /**
 				      *  Copy the data of the given
@@ -1380,7 +1334,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *  triangulation.
 				      */
     void set (const internal::Triangulation::TriaObject<structdim> &o) const;
-    
+
                                      /**
                                       * Set the flag indicating, what
                                       * <code>line_orientation()</code> will
@@ -1388,12 +1342,12 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *
 				      * It is only possible to set the
 				      * line_orientation of faces in 3d
-				      * (i.e. <code>celldim==2 &&
+				      * (i.e. <code>structdim==2 &&
 				      * dim==3</code>).
                                       */
     void set_line_orientation (const unsigned int line,
                                const bool         orientation) const;
-    
+
                                      /**
                                       * Set whether the quad with
                                       * index @p face has its normal
@@ -1423,7 +1377,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *
 				      * It is only possible to set the
 				      * face_orientation of cells in 3d
-				      * (i.e. <code>celldim==3 &&
+				      * (i.e. <code>structdim==3 &&
 				      * dim==3</code>).
                                       */
     void set_face_flip (const unsigned int face,
@@ -1436,12 +1390,12 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *
 				      * It is only possible to set the
 				      * face_orientation of cells in 3d
-				      * (i.e. <code>celldim==3 &&
+				      * (i.e. <code>structdim==3 &&
 				      * dim==3</code>).
                                       */
     void set_face_rotation (const unsigned int face,
 			    const bool         rotation) const;
-    
+
 				     /**
 				      *  Set the @p used flag. Only
 				      *  for internal use in the
@@ -1460,7 +1414,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      * Set the @p RefinementCase<dim> this
 				      * TriaObject is refined with.
 				      * Not defined for
-				      * <tt>celldim=1</tt> as lines
+				      * <tt>structdim=1</tt> as lines
 				      * are always refined resulting
 				      * in 2 children lines (isotropic
 				      * refinement).
@@ -1500,14 +1454,14 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *  not allowed.
 				      */
     void set_children (const unsigned int i, const int index) const;
-	
+
 				     /**
 				      *  Clear the child field,
 				      *  i.e. set it to a value which
 				      *  indicates that this cell has
 				      *  no children.
 				      */
-    void clear_children () const;    
+    void clear_children () const;
 
   private:
     				     /**
@@ -1528,7 +1482,7 @@ class TriaAccessor : public TriaAccessorBase<structdim, dim, spacedim>
 				      *  anyway.
 				      */
     void operator = (const TriaAccessor &);
-    
+
     template <int, int> friend class Triangulation;
 
     friend class internal::Triangulation::Implementation;
@@ -1592,7 +1546,7 @@ class TriaAccessor<0, dim, spacedim> : public TriaAccessorBase<0,dim, spacedim>
       {
 	Assert (false, ExcInternalError());
       }
-    
+
 				     /**
     				      * Boundary indicator of this
 				      * object.
@@ -1631,7 +1585,7 @@ class TriaAccessor<0, dim, spacedim> : public TriaAccessorBase<0,dim, spacedim>
  * in two dimension, etc.
  *
  * The following refers to any dimension:
- * 
+ *
  * This class allows access to a <tt>cell</tt>, which is a line in 1D
  * and a quad in 2D. Cells have more functionality than lines or quads
  * by themselves, for example they can be flagged for refinement, they
@@ -1651,7 +1605,13 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * into the present class.
 				      */
     typedef typename TriaAccessor<dim,dim,spacedim>::AccessorData AccessorData;
-    
+
+				     /**
+				      * Define the type of the
+				      * container this is part of.
+				      */
+    typedef Triangulation<dim, spacedim> Container;
+
 				     /**
 				      *  @name Constructors
 				      */
@@ -1708,7 +1668,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				     /**
 				      * @}
 				      */
-    
+
 				     /**
 				      *  @name Accessing sub-objects and neighbors
 				      */
@@ -1842,13 +1802,13 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
     TriaIterator<CellAccessor<dim, spacedim> >
     neighbor_child_on_subface (const unsigned int face_no,
                                const unsigned int subface_no) const;
-    
+
 				     /**
 				      *  Return a pointer to the
 				      *  @p ith neighbor.  If the
 				      *  neighbor does not exist, an
 				      *  invalid iterator is returned.
-				      *  
+				      *
 				      *  <b>Note</b> (cf. TriaLevel<0>):
 				      *  The neighbor of a cell has at most the
 				      *  same level as this cell, i.e. it may
@@ -1918,7 +1878,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * face.
 				      */
     bool neighbor_is_coarser (const unsigned int neighbor) const;
-    
+
 				     /**
 				      * This function is a generalization of the
 				      * @p neighbor_of_neighbor function for the
@@ -1932,14 +1892,14 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * <tt>cell->neighbor(neighbor)->face(face_no)->child(subface_no)==cell->face(neighbor)</tt>.
 				      * This case in 3d and how it can happen
 				      * is discussed in the introduction of the
-				      * @ref step_30 "step-30" tutorial program.
+				      * step-30 tutorial program.
 				      *
 				      * This function is impossible
 				      * for <tt>dim==1</tt>.
 				      */
     std::pair<unsigned int, unsigned int>
     neighbor_of_coarser_neighbor (const unsigned int neighbor) const;
-    
+
 				     /**
 				      * This function is a generalization of the
 				      * @p neighbor_of_neighbor and the @p
@@ -1950,7 +1910,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * the face_no is returned.
 				      */
     unsigned int neighbor_face_no (const unsigned int neighbor) const;
-    
+
 				     /**
 				      * @}
 				      */
@@ -2077,7 +2037,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * cells in 2d and 3d.
 				      */
     internal::SubfaceCase<dim> subface_case(const unsigned int face_no) const;
-    
+
 				     /**
 				      *  Return whether the coarsen flag
 				      *  is set or not.
@@ -2106,7 +2066,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				     /**
 				      * @{
 				      */
-    
+
 				     /**
 				      * Return the material id of this
 				      * cell.
@@ -2211,12 +2171,12 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      */
     void set_neighbor (const unsigned int i,
 		       const TriaIterator<CellAccessor<dim, spacedim> > &pointer) const;
-    
+
 				     /**
 				      * @}
 				      */
-    
-    
+
+
 				     /**
 				      * @ingroup Exceptions
 				      */
@@ -2229,7 +2189,7 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * @ingroup Exceptions
 				      */
     DeclException0 (ExcCellFlaggedForCoarsening);
-    
+
   protected:
     				     /**
 				      * This function assumes that the
@@ -2265,9 +2225,9 @@ class CellAccessor :  public TriaAccessor<dim,dim,spacedim>
 				      * neighbors.
 				      */
     unsigned int neighbor_of_neighbor_internal (const unsigned int neighbor) const;
-    
+
   private:
-    
+
     				     /**
 				      *  Copy operator. This is
 				      *  normally used in a context
