@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: tria_accessor.templates.h 18035 2008-12-28 04:35:46Z bangerth $
+//    $Id: tria_accessor.templates.h 21036 2010-04-26 04:48:58Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -32,33 +32,73 @@ DEAL_II_NAMESPACE_OPEN
 
 template <int structdim, int dim, int spacedim>
 inline
-TriaAccessorBase<structdim,dim,spacedim>::TriaAccessorBase (const Triangulation<dim,spacedim> *parent,
-					   const int                 level,
-					   const int                 index,
-					   const AccessorData       *)
+TriaAccessorBase<structdim,dim,spacedim>::TriaAccessorBase (
+  const Triangulation<dim,spacedim>* tria,
+  const int                          level,
+  const int                          index,
+  const AccessorData*)
                 :
+		present_level((structdim==dim) ? level : 0),
                 present_index (index),
-                tria (parent)
+                tria (tria)
 {
-
+  
 				   // non-cells have no level, so a 0
 				   // should have been passed, or a -1
 				   // for an end-iterator, or -2 for
 				   // an invalid (default constructed)
 				   // iterator
-  Assert ((level == 0) || (level == -1) || (level == -2),
-	  ExcInternalError());
+  if (structdim != dim)
+    {
+      Assert ((level == 0) || (level == -1) || (level == -2),
+	      ExcInternalError());
+    }
 }
 
 
+template <int structdim, int dim, int spacedim>
+inline
+TriaAccessorBase<structdim,dim,spacedim>::TriaAccessorBase (const TriaAccessorBase<structdim,dim,spacedim> &a)
+		:
+		present_level(a.present_level),
+		present_index(a.present_index),
+		tria(a.tria)
+{}
+
 
 template <int structdim, int dim, int spacedim>
 inline
 void
 TriaAccessorBase<structdim,dim,spacedim>::copy_from (const TriaAccessorBase<structdim,dim,spacedim> &a)
 {
+  present_level = a.present_level;
   present_index = a.present_index;
   tria = a.tria;
+  
+  if (structdim != dim)
+    {
+      Assert ((present_level == 0) || (present_level == -1) || (present_level == -2),
+	      ExcInternalError());
+    }
+}
+
+
+
+template <int structdim, int dim, int spacedim>
+inline
+TriaAccessorBase<structdim,dim,spacedim>&
+TriaAccessorBase<structdim,dim,spacedim>::operator= (const TriaAccessorBase<structdim,dim,spacedim> &a)
+{
+  present_level = a.present_level;
+  present_index = a.present_index;
+  tria = a.tria;
+  
+  if (structdim != dim)
+    {
+      Assert ((present_level == 0) || (present_level == -1) || (present_level == -2),
+	      ExcInternalError());
+    }
+  return *this;
 }
 
 
@@ -69,7 +109,8 @@ bool
 TriaAccessorBase<structdim,dim,spacedim>::operator == (const TriaAccessorBase<structdim,dim,spacedim> &a) const
 {
   Assert (tria == a.tria, TriaAccessorExceptions::ExcCantCompareIterators());
-  return (present_index == a.present_index);
+  return ((present_level == a.present_level) &&
+	  (present_index == a.present_index));
 }
 
 
@@ -80,7 +121,8 @@ bool
 TriaAccessorBase<structdim,dim,spacedim>::operator != (const TriaAccessorBase<structdim,dim,spacedim> &a) const
 {
   Assert (tria == a.tria, TriaAccessorExceptions::ExcCantCompareIterators());
-  return (present_index != a.present_index);
+  return ((present_level != a.present_level) ||
+	  (present_index != a.present_index));
 }
 
 
@@ -88,9 +130,11 @@ TriaAccessorBase<structdim,dim,spacedim>::operator != (const TriaAccessorBase<st
 template <int structdim, int dim, int spacedim>
 inline
 int
-TriaAccessorBase<structdim,dim,spacedim>::level ()
+TriaAccessorBase<structdim,dim,spacedim>::level () const
 {
-  return 0;
+				   // This is always zero or invalid
+				   // if the object is not a cell
+  return present_level;
 }
 
 
@@ -110,7 +154,7 @@ inline
 IteratorState::IteratorStates
 TriaAccessorBase<structdim,dim,spacedim>::state () const
 {
-  if (present_index>=0)
+  if ((present_level>=0) && (present_index>=0))
     return IteratorState::valid;
   else
     if (present_index==-1)
@@ -139,15 +183,38 @@ TriaAccessorBase<structdim,dim,spacedim>::operator ++ ()
 				   // this iterator is used for
 				   // objects without level
   ++this->present_index;
-				   // is index still in the range of
-				   // the vector? (note that we don't
-				   // have to set the level, since
-				   // dim!=1 and the object therefore
-				   // has no level)
-  if (this->present_index
-      >=
-      static_cast<int>(objects().cells.size()))
-    this->present_index = -1;
+
+  if (structdim != dim)
+    {
+				       // is index still in the range of
+				       // the vector? (note that we don't
+				       // have to set the level, since
+				       // dim!=1 and the object therefore
+				       // has no level)
+      if (this->present_index
+	  >=
+	  static_cast<int>(objects().cells.size()))
+	this->present_index = -1;
+    }
+  else
+    {
+      while (this->present_index
+	     >=
+	     static_cast<int>(this->tria->levels[this->present_level]->cells.cells.size()))
+	{
+					   // no -> go one level up until we find
+					   // one with more than zero cells
+	  ++this->present_level;
+	  this->present_index = 0;
+					   // highest level reached?
+	  if (this->present_level >= static_cast<int>(this->tria->levels.size()))
+	    {
+					       // return with past the end pointer
+	      this->present_level = this->present_index = -1;
+	      return;
+	    }
+	}
+    }
 }
 
 
@@ -159,8 +226,28 @@ TriaAccessorBase<structdim,dim,spacedim>::operator -- ()
 				   // same as operator++
   --this->present_index;
 
-  if (this->present_index < 0) 
-    this->present_index = -1;
+  if (structdim != dim)
+    {
+      if (this->present_index < 0) 
+	this->present_index = -1;
+    }
+  else
+    {
+      while (this->present_index < 0) 
+	{
+					   // no -> go one level down
+	  --this->present_level;
+					   // lowest level reached?
+	  if (this->present_level == -1) 
+	    {
+					       // return with past the end pointer
+	      this->present_level = this->present_index = -1;
+	      return;
+	    }
+					   // else
+	  this->present_index = this->tria->levels[this->present_level]->cells.cells.size()-1;
+	}
+    }
 }
 
 
@@ -175,26 +262,90 @@ namespace internal
 				      */
     template <int dim>
     inline
-    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<1> > &
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<1> >*
     get_objects (internal::Triangulation::TriaFaces<dim> *faces,
 		 const internal::int2type<1>)
     {
-      return faces->lines;
+      return &faces->lines;
     }
 
 
     template <int dim>
     inline
-    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<2> > &
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<2> >*
     get_objects (internal::Triangulation::TriaFaces<dim> *faces,
 		 const internal::int2type<2>)
     {
-      return faces->quads;
+      return &faces->quads;
+    }
+    
+    inline
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<1> >*
+    get_objects (internal::Triangulation::TriaFaces<1>*,
+		 const internal::int2type<1>)
+    {
+      Assert (false, ExcInternalError());
+      return 0;
+    }
+    
+    inline
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<2> >*
+    get_objects (internal::Triangulation::TriaFaces<2>*,
+		 const internal::int2type<2>)
+    {
+      Assert (false, ExcInternalError());
+      return 0;
+    }
+    
+    inline
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<3> >*
+    get_objects (internal::Triangulation::TriaFaces<3>*,
+		 const internal::int2type<3>)
+    {
+      Assert (false, ExcInternalError());
+      return 0;
+    }
+    
+				     /**
+				      * This function should never be
+				      * used, but we need it for the
+				      * template instantiation of TriaAccessorBase<dim,dim,spacedim>::objects() const
+				      */
+    template <int dim>
+    inline
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<3> >*
+    get_objects (internal::Triangulation::TriaFaces<dim> *faces,
+		 const internal::int2type<3>)
+    {
+      Assert (false, ExcInternalError());
+      return 0;
+    }
+
+				     /**
+				      * Copy the above functions for
+				      * cell objects.
+				      */
+    template <int structdim, int dim>
+    inline
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<structdim> >*
+    get_objects (internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<dim> >*,
+		 const internal::int2type<structdim>)
+    {
+      Assert (false, ExcInternalError());
+      return 0;
+    }
+
+    template <int dim>
+    inline
+    internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<dim> >*
+    get_objects (internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<dim> >* cells,
+		 const internal::int2type<dim>)
+    {
+      return cells;
     }
   }
 }
 
-  
 
 
 template <int structdim, int dim, int spacedim>
@@ -202,182 +353,19 @@ inline
 internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<structdim> > &
 TriaAccessorBase<structdim,dim,spacedim>::objects() const
 {
-				   // get sub-objects. note that the
-				   // current class is only used for
-				   // objects that are *not* cells
-  return internal::TriaAccessorBase::get_objects (this->tria->faces,
-					      internal::int2type<structdim> ());
-}
-
-
-
-
-/*------------------------ Functions: TriaAccessorBase<dim,dim> ---------------------------*/
-
-template <int dim, int spacedim>
-inline
-TriaAccessorBase<dim,dim,spacedim>::TriaAccessorBase (const Triangulation<dim,spacedim> *parent,
-				     const int                 level,
-				     const int                 index,
-				     const AccessorData       *)
-                :
-                present_level (level),
-                present_index (index),
-                tria (parent)
-{}
-
-
-
-template <int dim, int spacedim>
-inline
-void
-TriaAccessorBase<dim,dim,spacedim>::copy_from (const TriaAccessorBase<dim,dim,spacedim> &a)
-{
-  present_level = a.present_level;
-  present_index = a.present_index;
-  tria = a.tria;
-}
-
-
-
-template <int dim, int spacedim>
-inline
-bool
-TriaAccessorBase<dim,dim,spacedim>::operator == (const TriaAccessorBase<dim,dim,spacedim> &a) const
-{
-  Assert (tria == a.tria, TriaAccessorExceptions::ExcCantCompareIterators());
-  return ((present_index == a.present_index) &&
-	  (present_level == a.present_level));
-}
-
-
-
-template <int dim, int spacedim>
-inline
-bool
-TriaAccessorBase<dim,dim,spacedim>::operator != (const TriaAccessorBase<dim,dim,spacedim> &a) const
-{
-  Assert (tria == a.tria, TriaAccessorExceptions::ExcCantCompareIterators());
-  return ((present_index != a.present_index) ||
-	  (present_level != a.present_level));
-}
-
-
-
-template <int dim, int spacedim>
-inline
-int
-TriaAccessorBase<dim,dim,spacedim>::level () const
-{
-  return present_level;
-}
-
-
-
-template <int dim, int spacedim>
-inline
-int
-TriaAccessorBase<dim,dim,spacedim>::index () const
-{
-  return present_index;
-}
-
-
-
-template <int dim, int spacedim>
-inline
-IteratorState::IteratorStates
-TriaAccessorBase<dim,dim,spacedim>::state () const
-{
-  if ((present_level>=0) && (present_index>=0))
-    return IteratorState::valid;
+  if (structdim != dim)
+				     // get sub-objects. note that the
+				     // current class is only used for
+				     // objects that are *not* cells
+    return *internal::TriaAccessorBase::get_objects (this->tria->faces,
+						     internal::int2type<structdim> ());
   else
-    if ((present_index==-1) && (present_index==-1))
-      return IteratorState::past_the_end;
-    else
-      return IteratorState::invalid;
+    return *internal::TriaAccessorBase::get_objects (&this->tria->levels[this->present_level]->cells,
+						     internal::int2type<structdim> ());
 }
 
 
 
-template <int dim, int spacedim>
-inline
-const Triangulation<dim,spacedim> &
-TriaAccessorBase<dim,dim,spacedim>::get_triangulation () const
-{
-  return *tria;
-}
-
-
-
-template <int dim, int spacedim>
-inline
-void
-TriaAccessorBase<dim,dim,spacedim>::operator ++ ()
-{
-				   // in structdim == dim, so we have
-				   // cells here. these have levels
-  ++this->present_index;
-				   // is index still in the range of
-				   // the vector?
-  while (this->present_index
-	 >=
-	 static_cast<int>(this->tria->levels[this->present_level]->cells.cells.size()))
-    {
-				       // no -> go one level up until we find
-				       // one with more than zero cells
-      ++this->present_level;
-      this->present_index = 0;
-				       // highest level reached?
-      if (this->present_level >= static_cast<int>(this->tria->levels.size()))
-	{
-					   // return with past the end pointer
-	  this->present_level = this->present_index = -1;
-	  return;
-	}
-    }
-}
-
-
-template <int dim, int spacedim>
-inline
-void
-TriaAccessorBase<dim,dim,spacedim>::operator -- ()
-{
-				   // same as operator++
-  --this->present_index;
-				   // is index still in the range of
-				   // the vector?
-  while (this->present_index < 0) 
-    {
-				       // no -> go one level down
-      --this->present_level;
-				       // lowest level reached?
-      if (this->present_level == -1) 
-	{
-					   // return with past the end pointer
-	  this->present_level = this->present_index = -1;
-	  return;
-	}
-				       // else
-      this->present_index = this->tria->levels[this->present_level]->cells.cells.size()-1;
-    }
-}
-
-
-
-template <int dim, int spacedim>
-inline
-internal::Triangulation::TriaObjects<internal::Triangulation::TriaObject<dim> > &
-TriaAccessorBase<dim,dim,spacedim>::objects() const
-{
-  return this->tria->levels[this->present_level]->cells;
-}
-
-
-
-
-
 /*------------------------ Functions: InvalidAccessor ---------------------------*/
 
 template <int structdim, int dim, int spacedim>
@@ -1862,6 +1850,19 @@ TriaAccessor<structdim, dim, spacedim>::at_boundary () const
 
 
 
+template <int structdim, int dim, int spacedim>
+const Boundary<dim,spacedim> &
+TriaAccessor<structdim, dim, spacedim>::get_boundary () const
+{
+  Assert (structdim<dim, ExcImpossibleInDim(dim));
+  Assert (this->used(), TriaAccessorExceptions::ExcCellNotUsed());
+
+  return this->tria->get_boundary(this->objects()
+				  .material_id[this->present_index]);
+}
+
+
+
 template <int structdim, int dim, int spacedim>
 double
 TriaAccessor<structdim, dim, spacedim>::diameter () const
@@ -1896,6 +1897,43 @@ TriaAccessor<structdim, dim, spacedim>::center () const
 }
 
 
+template <int structdim, int dim, int spacedim>
+bool
+TriaAccessor<structdim, dim, spacedim>::
+is_translation_of (const TriaIterator<TriaAccessor<structdim,dim,spacedim> > &o) const
+{
+				   // go through the vertices and check... The
+				   // cell is a translation of the previous
+				   // one in case the distance between the
+				   // individual vertices in the two cell is
+				   // the same for all the vertices. So do the
+				   // check by first getting the distance on
+				   // the first vertex, and then checking
+				   // whether all others have the same down to
+				   // rounding errors (we have to be careful
+				   // here because the calculation of the
+				   // displacement between one cell and the
+				   // next can already result in the loss of
+				   // one or two digits), so we choose 1e-12
+				   // times the distance between the zeroth
+				   // vertices here.
+  bool is_translation = true;
+  const Point<spacedim> dist = o->vertex(0) - this->vertex(0);
+  const double tol_square = 1e-24 * dist.norm_square();
+  for (unsigned int i=1; i<GeometryInfo<structdim>::vertices_per_cell; ++i)
+    {
+      const Point<spacedim> dist_new = (o->vertex(i) - this->vertex(i)) - dist;
+      if (dist_new.norm_square() > tol_square)
+	{
+	  is_translation = false;
+	  break;
+	}
+    }
+  return is_translation;
+}
+
+
+
 /*------------------------ Functions: CellAccessor<dim,spacedim> -----------------------*/
 
 
@@ -1979,8 +2017,7 @@ inline
 int
 CellAccessor<dim,spacedim>::neighbor_index (const unsigned int i) const 
 {
-  Assert (i<GeometryInfo<dim>::faces_per_cell,
-	  TriaAccessorExceptions::ExcInvalidNeighbor(i));
+  AssertIndexRange (i,GeometryInfo<dim>::faces_per_cell);
   return this->tria->levels[this->present_level]->
     neighbors[this->present_index*GeometryInfo<dim>::faces_per_cell+i].second;
 }
@@ -1992,8 +2029,7 @@ inline
 int
 CellAccessor<dim,spacedim>::neighbor_level (const unsigned int i) const
 {
-  Assert (i<GeometryInfo<dim>::faces_per_cell,
-	  TriaAccessorExceptions::ExcInvalidNeighbor(i));
+  AssertIndexRange (i, GeometryInfo<dim>::faces_per_cell);
   return this->tria->levels[this->present_level]->
     neighbors[this->present_index*GeometryInfo<dim>::faces_per_cell+i].first;
 }
