@@ -1,8 +1,8 @@
 //----------------------------------------------------------------------
-//    $Id: dof_handler.h 17907 2008-12-10 17:36:06Z bangerth $
+//    $Id: dof_handler.h 21036 2010-04-26 04:48:58Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -77,6 +77,9 @@ namespace hp
   {
       typedef internal::DoFHandler::Iterators<DoFHandler<dim,spacedim> > IteratorSelector;
     public:
+      typedef typename IteratorSelector::CellAccessor         cell_accessor;
+      typedef typename IteratorSelector::FaceAccessor         face_accessor;
+      
       typedef typename IteratorSelector::raw_line_iterator raw_line_iterator;
       typedef typename IteratorSelector::line_iterator line_iterator;
       typedef typename IteratorSelector::active_line_iterator active_line_iterator;
@@ -172,8 +175,8 @@ namespace hp
                                         * Go through the triangulation and
                                         * distribute the degrees of freedoms
                                         * needed for the given finite element
-                                        * according to the given distribution
-                                        * method.
+                                        * according to the current distribution
+                                        * of active fe indices.
                                         *
                                         * A pointer of the transferred
                                         * finite element is
@@ -189,6 +192,23 @@ namespace hp
                                         */
       virtual void distribute_dofs (const hp::FECollection<dim,spacedim> &fe);
 
+                                       /**
+                                        * Go through the triangulation and set
+                                        * the active FE indices of all active
+                                        * cells to the values given in @p
+                                        * active_fe_indices.
+                                        */
+      void set_active_fe_indices (const std::vector<unsigned int>& active_fe_indices);
+
+                                       /**
+                                        * Go through the triangulation and
+                                        * store the active FE indices of all
+                                        * active cells to the vector @p
+                                        * active_fe_indices. This vector is
+                                        * resized, if necessary.
+                                        */
+      void get_active_fe_indices (std::vector<unsigned int>& active_fe_indices) const;
+
                                        /**
                                         * Clear all data of this object and
                                         * especially delete the lock this object
@@ -919,7 +939,7 @@ namespace hp
                                         * Address of the triangulation to
                                         * work on.
                                         */
-      SmartPointer<const Triangulation<dim,spacedim> > tria;
+      SmartPointer<const Triangulation<dim,spacedim>,DoFHandler<dim,spacedim> > tria;
 
                                        /**
                                         * Store a pointer to the finite
@@ -936,7 +956,7 @@ namespace hp
                                         * (this clears all data of this
                                         * object as well, though).
                                         */
-      SmartPointer<const hp::FECollection<dim,spacedim> > finite_elements;
+      SmartPointer<const hp::FECollection<dim,spacedim>,hp::DoFHandler<dim,spacedim> > finite_elements;
 
     private:
 
@@ -1084,7 +1104,7 @@ namespace hp
                                         * Store the number of dofs
                                         * created last time.
                                         */
-      unsigned int              used_dofs;
+      unsigned int used_dofs;
 
                                        /**
                                         * Array to store the indices
