@@ -1,8 +1,8 @@
 //----------------------------  hp_fe_values.h  ---------------------------
-//    $Id: fe_values.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: fe_values.h 20602 2010-02-13 17:44:17Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2003, 2004, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2003, 2004, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -81,7 +81,15 @@ namespace internal
       FEValuesBase (const dealii::hp::FECollection<dim,FEValues::space_dimension> &fe_collection,
                       const dealii::hp::QCollection<q_dim> &q_collection,
                       const UpdateFlags         update_flags);
-        
+
+					 /**
+					  * Get a reference to the
+					  * collection of finite
+					  * element objects used here.
+					  */
+	const dealii::hp::FECollection<dim,FEValues::space_dimension> &
+	get_fe_collection () const;
+	
                                          /**
                                           * Return a reference to the @p
                                           * FEValues object selected by the
@@ -121,14 +129,16 @@ namespace internal
                                           * collection of finite
                                           * elements to be used.
                                           */
-        const SmartPointer<const dealii::hp::FECollection<dim,FEValues::space_dimension> > fe_collection;
+        const SmartPointer<const dealii::hp::FECollection<dim,FEValues::space_dimension>,
+			   FEValuesBase<dim,q_dim,FEValues> > fe_collection;
         
                                          /**
                                           * A pointer to the
                                           * collection of mappings to
                                           * be used.
                                           */
-        const SmartPointer<const dealii::hp::MappingCollection<dim, FEValues::space_dimension> > mapping_collection;
+        const SmartPointer<const dealii::hp::MappingCollection<dim, FEValues::space_dimension>,
+			   FEValuesBase<dim,q_dim,FEValues> > mapping_collection;
     
                                          /**
                                           * Copy of the quadrature
@@ -190,7 +200,7 @@ namespace hp
 {
   
 /**
- * An hp equivalent of the ::FEValues class. See the @ref step_27 "step-27"
+ * An hp equivalent of the ::FEValues class. See the step-27
  * tutorial program for examples of use.
  *
  * The idea of this class is as follows: when one assembled matrices in the hp
@@ -1088,6 +1098,16 @@ namespace internal
     {
       return *fe_values_table(present_fe_values_index);
     }
+
+
+
+    template <int dim, int q_dim, class FEValues>
+    inline
+    const dealii::hp::FECollection<dim,FEValues::space_dimension> &
+    FEValuesBase<dim,q_dim,FEValues>::get_fe_collection () const
+    {
+      return *fe_collection;
+    }
   }
   
 }
