@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_dof_accessor.h 17893 2008-12-08 19:18:38Z bangerth $
+//    $Id: mg_dof_accessor.h 21058 2010-05-03 22:15:00Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -22,7 +22,16 @@ DEAL_II_NAMESPACE_OPEN
 
 template <int dim, int spacedim> class MGDoFHandler;
 
-template <int celldim, int dim, int spacedim=dim> class MGDoFAccessor;
+template <int structdim, int dim, int spacedim=dim> class MGDoFAccessor;
+
+namespace internal
+{
+  namespace MGDoFHandler
+  {
+    class Implementation;
+  }
+}
+
 
 /*!@addtogroup mg */
 /*@{*/
@@ -35,25 +44,25 @@ namespace internal
 /**
  * This is a switch class which only declares a @p typedef. It is meant to
  * determine which class aMGDoFAccessor class is to be derived
- * from. By default, <tt>MGDoFAccessor<celldim,dim></tt> derives from
- * the @p typedef in the general <tt>MGDoFAccessor_Inheritance<celldim,dim></tt>
- * class, which is <tt>DoFAccessor<celldim,dim></tt>,
- * but if <tt>celldim==dim</tt>, then the specialization <tt>MGDoFAccessor_Inheritance<dim,dim></tt>
+ * from. By default, <tt>MGDoFAccessor<structdim,dim></tt> derives from
+ * the @p typedef in the general <tt>MGDoFAccessor_Inheritance<structdim,dim></tt>
+ * class, which is <tt>DoFAccessor<structdim,dim></tt>,
+ * but if <tt>structdim==dim</tt>, then the specialization <tt>MGDoFAccessor_Inheritance<dim,dim></tt>
  * is used which declares its local type to be <tt>DoFCellAccessor<dim></tt>. Therefore,
  * the inheritance is automatically chosen to be from @p DoFCellAccessor if the
  * object under consideration has full dimension, i.e. constitutes a cell.
  *
  * @author Wolfgang Bangerth, 1999
  */
-    template <int celldim, int dim, int spacedim>
-    struct Inheritance 
+    template <int structdim, int dim, int spacedim>
+    struct Inheritance
     {
 					 /**
 					  * Declaration of the @p typedef.
 					  * See the full documentation for
 					  * more information.
 					  */
-	typedef dealii::DoFAccessor<celldim, dealii::DoFHandler<dim,spacedim> > BaseClass;
+	typedef dealii::DoFAccessor<structdim, dealii::DoFHandler<dim,spacedim> > BaseClass;
     };
 
 
@@ -62,9 +71,9 @@ namespace internal
  * meant to determine which class a DoFAccessor class is to be
  * derived from. By default, DoFAccessor<structdim,dim>
  * derives from the @p typedef in the general
- * <tt>DoFAccessor_Inheritance<celldim,dim></tt> class, which is
- * <tt>TriaAccessor<celldim,dim></tt>, but if
- * <tt>celldim==dim</tt>, then the specialization
+ * <tt>DoFAccessor_Inheritance<structdim,dim></tt> class, which is
+ * <tt>TriaAccessor<structdim,dim></tt>, but if
+ * <tt>structdim==dim</tt>, then the specialization
  * <tt>TriaAccessor<dim,dim></tt> is used which declares its
  * local type to be <tt>CellAccessor<dim></tt>. Therefore, the
  * inheritance is automatically chosen to be from @p CellAccessor if
@@ -123,7 +132,7 @@ namespace internal
  * since they are more secure to changes in the class naming and
  * template interface as well as providing easier typing (much less
  * complicated names!).
- * 
+ *
  *
  * <h3>Inheritance</h3>
  *
@@ -151,7 +160,7 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
 				      * class to make accessing some
 				      * of the exception classes
 				      * simpler.
-				      */    
+				      */
     typedef
     typename internal::MGDoFAccessor::Inheritance<structdim,dim,spacedim>::BaseClass
     BaseClass;
@@ -161,7 +170,13 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
 				      * iterator classes.
 				      */
     typedef MGDoFHandler<dim,spacedim> AccessorData;
-    
+
+				     /**
+				      * The dof handler type used by
+				      * this accessor.
+				      */
+    typedef MGDoFHandler<dim, spacedim> Container;
+
 				     /**
 				      * @name Constructors
 				      */
@@ -215,7 +230,7 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
 				      * previous one.
 				      */
     template <int structdim2, int dim2, int spacedim2>
-    MGDoFAccessor (const MGDoFAccessor<structdim2,dim2,spacedim2> &);    
+    MGDoFAccessor (const MGDoFAccessor<structdim2,dim2,spacedim2> &);
 
 				     /**
 				      * @}
@@ -274,8 +289,11 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
     unsigned int mg_vertex_dof_index (const int level,
 				      const unsigned int vertex,
 				      const unsigned int i) const;
-
+//TODO: This function is broken for faces
 				     /**
+				      * @todo This function is broken
+				      * for faces.
+				      *
 				      * Return the index of the @p ith
 				      * degree of freedom of this line
 				      * on the level this line lives
@@ -319,7 +337,7 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
 				     /**
 				      * @}
 				      */
-    
+
 				     /**
 				      * Implement the copy operator needed
 				      * for the iterator classes.
@@ -336,7 +354,7 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
 				      * Store the address of the @p MGDoFHandler object
 				      * to be accessed.
 				      */
-    MGDoFHandler<dim,spacedim> *mg_dof_handler;  
+    MGDoFHandler<dim,spacedim> *mg_dof_handler;
 
 				     /**
 				      * Reset the DoF handler pointer.
@@ -368,6 +386,7 @@ class MGDoFAccessor : public internal::MGDoFAccessor::Inheritance<structdim,dim,
     MGDoFAccessor & operator = (const MGDoFAccessor &da);
 
     template <int, int> friend class MGDoFHandler;
+    friend class internal::MGDoFHandler::Implementation;
 };
 
 
@@ -395,7 +414,7 @@ class MGDoFCellAccessor :  public MGDoFAccessor<dim,dim,spacedim>
 {
   public:
     typedef MGDoFAccessor<dim,dim,spacedim> BaseClass;
-    
+
 				     /**
 				      * Type of faces.
 				      */
@@ -408,7 +427,7 @@ class MGDoFCellAccessor :  public MGDoFAccessor<dim,dim,spacedim>
 				      * iterator classes.
 				      */
     typedef typename MGDoFAccessor<dim,dim,spacedim>::AccessorData  AccessorData;
-    
+
 				     /**
 				      * @name Constructors
 				      */
