@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_dof_handler.h 17866 2008-12-05 22:27:44Z bangerth $
+//    $Id: mg_dof_handler.h 21036 2010-04-26 04:48:58Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -20,6 +20,16 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+
+namespace internal
+{
+  namespace MGDoFHandler
+  {
+    class Implementation;
+  }
+}
+
+
 /*!@addtogroup mg */
 /*@{*/
 
@@ -34,36 +44,40 @@ DEAL_II_NAMESPACE_OPEN
  * At present, multilevel algorithms are not fully functional, so this
  * documentation is still very brief.
  *
- * This class has not yet been implemented for the use in the codimension
+ * @todo This class has not yet been implemented for the use in the codimension
  * one case (<tt>spacedim != dim </tt>).
  *
-//TODO:[WB] Extend MGDoFHandler doc 
+//TODO:[WB] Extend MGDoFHandler doc
  *
  * @author Wolfgang Bangerth, 1998, 1999
  */
 template <int dim, int spacedim=dim>
 class MGDoFHandler : public DoFHandler<dim,spacedim>
 {
+    typedef internal::MGDoFHandler::Iterators<dim,spacedim> IteratorSelector;
   public:
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::raw_line_iterator raw_line_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::line_iterator line_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::active_line_iterator active_line_iterator;
+    typedef typename IteratorSelector::CellAccessor cell_accessor;
+    typedef typename IteratorSelector::FaceAccessor face_accessor;
+    
+    typedef typename IteratorSelector::raw_line_iterator raw_line_iterator;
+    typedef typename IteratorSelector::line_iterator line_iterator;
+    typedef typename IteratorSelector::active_line_iterator active_line_iterator;
 
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::raw_quad_iterator raw_quad_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::quad_iterator quad_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::active_quad_iterator active_quad_iterator;
+    typedef typename IteratorSelector::raw_quad_iterator raw_quad_iterator;
+    typedef typename IteratorSelector::quad_iterator quad_iterator;
+    typedef typename IteratorSelector::active_quad_iterator active_quad_iterator;
 
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::raw_hex_iterator raw_hex_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::hex_iterator hex_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::active_hex_iterator active_hex_iterator;
+    typedef typename IteratorSelector::raw_hex_iterator raw_hex_iterator;
+    typedef typename IteratorSelector::hex_iterator hex_iterator;
+    typedef typename IteratorSelector::active_hex_iterator active_hex_iterator;
 
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::raw_cell_iterator raw_cell_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::cell_iterator cell_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::active_cell_iterator active_cell_iterator;
+    typedef typename IteratorSelector::raw_cell_iterator raw_cell_iterator;
+    typedef typename IteratorSelector::cell_iterator cell_iterator;
+    typedef typename IteratorSelector::active_cell_iterator active_cell_iterator;
 
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::raw_face_iterator raw_face_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::face_iterator face_iterator;
-    typedef typename internal::MGDoFHandler::Iterators<dim,spacedim>::active_face_iterator active_face_iterator;
+    typedef typename IteratorSelector::raw_face_iterator raw_face_iterator;
+    typedef typename IteratorSelector::face_iterator face_iterator;
+    typedef typename IteratorSelector::active_face_iterator active_face_iterator;
 
 				     /**
 				      * Make the dimension and the space_dimension available
@@ -83,7 +97,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * Destructor
 				      */
     virtual ~MGDoFHandler ();
-    
+
 				     /**
 				      * Go through the triangulation
 				      * and distribute the degrees of
@@ -132,9 +146,9 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * name, see above.
 				      */
     void renumber_dofs (const std::vector<unsigned int> &new_numbers);
-    
+
 				     /*--------------------------------------*/
-    
+
 				     /**
 				      *  @name Cell iterator functions
 				      */
@@ -194,7 +208,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * <tt>end()</tt>.
 				      */
     cell_iterator        end (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is
 				      * the first iterator not on
@@ -334,7 +348,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * past-the-end or
 				      * before-the-beginning states.
 				      *
-				      * This is the same as 
+				      * This is the same as
 				      * <tt>end_face()</tt>.
 				      */
     raw_face_iterator    end_raw_face () const;
@@ -346,7 +360,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * past-the-end or
 				      * before-the-beginning states.
 				      *
-				      * This is the same as 
+				      * This is the same as
 				      * <tt>end_face()</tt>.
 				      */
     active_face_iterator end_active_face () const;
@@ -427,7 +441,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * <tt>end()</tt>.
 				      */
     line_iterator        end_line (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is
 				      * the first iterator not on
@@ -485,7 +499,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * @p level.
 				      */
     active_line_iterator last_active_line (const unsigned int level) const;
-				     /*@}*/	  
+				     /*@}*/
 
 				     /*---------------------------------------*/
 
@@ -531,7 +545,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * <tt>end()</tt>.
 				      */
     quad_iterator        end_quad (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is
 				      * the first iterator not on
@@ -635,7 +649,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * <tt>end()</tt>.
 				      */
     hex_iterator        end_hex (const unsigned int level) const;
-    
+
 				     /**
 				      * Return a raw iterator which is
 				      * the first iterator not on
@@ -697,7 +711,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 
 				     /*---------------------------------------*/
 
-    
+
     				     /**
 				      * Return the number of degrees
 				      * of freedom on the specified
@@ -716,7 +730,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * name, see above.
 				      */
     unsigned int n_dofs () const;
-    
+
     				     /**
 				      * Determine an estimate for the
 				      * memory consumption (in bytes)
@@ -725,7 +739,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * This function is made virtual,
 				      * since a dof handler object
 				      * might be accessed through a
-				      * pointers to thisr base class,
+				      * pointers to this base class,
 				      * although the actual object
 				      * might be a derived class.
 				      */
@@ -751,7 +765,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 		    int,
 		    << "You tried to do something on level " << arg1
 		    << ", but this level is empty.");
-    
+
   private:
 
 				     /**
@@ -782,7 +796,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
     class MGVertexDoFs
     {
       public:
-	
+
 					 /**
 					  * Constructor. This one is
 					  * empty because it is
@@ -824,7 +838,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 					  * @p init is supposed to do.
 					  */
 	MGVertexDoFs & operator = (const MGVertexDoFs &vertex);
-	
+
 					 /**
 					  * Set the index with number
 					  * @p dof_number of this
@@ -846,7 +860,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 			const unsigned int dof_number,
 			const unsigned int dofs_per_vertex,
 			const unsigned int index);
-	
+
 					 /**
 					  * Return the index with
 					  * number @p dof_number of
@@ -881,7 +895,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 					  * lives on.
 					  */
 	unsigned int get_finest_level () const;
-	
+
 					 /**
 					  * Exception.
 					  */
@@ -893,7 +907,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 			int,
 			<< "The given level number " << arg1 << " is outside "
 			<< "the range of levels this vertex lives on.");
-	
+
       private:
 					 /**
 					  * Store the coarsest level
@@ -921,29 +935,6 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
     };
 
 
-				     /**
-				      * Distribute dofs on the given
-				      * cell, with new dofs starting
-				      * with index
-				      * @p next_free_dof. Return the
-				      * next unused index number. The
-				      * finite element used is the one
-				      * given to @p distribute_dofs,
-				      * which is copied to
-				      * @p selected_fe.
-				      *
-				      * This function is excluded from
-				      * the @p distribute_dofs
-				      * function since it can not be
-				      * implemented dimension
-				      * independent.
-				      *
-				      * Note that unlike for the usual
-				      * dofs, here all cells and not
-				      * only active ones are allowed.
-				      */
-    unsigned int distribute_dofs_on_cell (cell_iterator &cell,
-					  unsigned int   next_free_dof);
 				     /**
 				      *  Return the @p i-th dof-index. This function calls
 				      *  the respective function of DoFObjects.
@@ -963,8 +954,8 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 			const unsigned int       fe_index,
 			const unsigned int       local_index,
 			const unsigned int       global_index) const;
-    
-    
+
+
 				     /**
 				      * Reserve enough space for the
 				      * MG dof indices for a given
@@ -976,7 +967,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * Free all used memory.
 				      */
     void clear_space ();
-    
+
     				     /**
 				      * Space to store the DoF numbers
 				      * for the different
@@ -988,7 +979,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * zero on each level.
 				      */
     std::vector<internal::DoFHandler::DoFLevel<dim>*>    mg_levels;
-    
+
 				     /**
 				      * Space to store the DoF numbers
 				      * for the faces.
@@ -1003,7 +994,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * and which are these levels.
 				      */
     std::vector<MGVertexDoFs>      mg_vertex_dofs;
-    
+
 				     /**
 				      * Vectors storing the number of
 				      * degrees of freedom on each
@@ -1015,6 +1006,7 @@ class MGDoFHandler : public DoFHandler<dim,spacedim>
 				      * Make accessor objects friends.
 				      */
     template <int dim1, int dim2, int dim3> friend class MGDoFAccessor;
+    friend class internal::MGDoFHandler::Implementation;
 };
 
 /*@}*/
@@ -1025,7 +1017,7 @@ template <int dim, int spacedim>
 inline
 unsigned int MGDoFHandler<dim,spacedim>::n_dofs() const
 {
-  return DoFHandler<dim>::n_dofs();
+  return DoFHandler<dim,spacedim>::n_dofs();
 }
 
 
@@ -1033,7 +1025,7 @@ template <int dim, int spacedim>
 inline
 void MGDoFHandler<dim,spacedim>::renumber_dofs (const std::vector<unsigned int> &new_numbers)
 {
-  return DoFHandler<dim>::renumber_dofs (new_numbers);
+  return DoFHandler<dim,spacedim>::renumber_dofs (new_numbers);
 }
 
 
@@ -1049,7 +1041,7 @@ void MGDoFHandler<dim,spacedim>::MGVertexDoFs::set_index  (const unsigned int le
 	  ExcInvalidLevel(level));
   Assert (dof_number < dofs_per_vertex,
 	  ExcIndexRange(dof_number, 0, dofs_per_vertex));
-  
+
   indices[(level-coarsest_level)*dofs_per_vertex + dof_number] = index;
 }
 
@@ -1064,7 +1056,7 @@ MGDoFHandler<dim,spacedim>::MGVertexDoFs::get_index  (const unsigned int level,
 	  ExcInvalidLevel(level));
   Assert (dof_number < dofs_per_vertex,
 	  ExcIndexRange (dof_number, 0, dofs_per_vertex));
-  
+
   return indices[(level-coarsest_level)*dofs_per_vertex + dof_number];
 }
 
