@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: multigrid.h 14357 2007-01-22 02:23:14Z bangerth $
+//    $Id: multigrid.h 21220 2010-06-17 04:01:18Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -17,10 +17,11 @@
 #include <base/config.h>
 #include <base/subscriptor.h>
 #include <base/smartpointer.h>
+#include <numerics/data_out.h>
 #include <lac/sparse_matrix.h>
 #include <lac/vector.h>
 #include <multigrid/mg_base.h>
-#include <multigrid/mg_level_object.h>
+#include <base/mg_level_object.h>
 #include <multigrid/mg_dof_handler.h>
 
 #include <vector>
@@ -103,6 +104,7 @@ class Multigrid : public Subscriptor
 	      const MGSmootherBase<VECTOR>& pre_smooth,
 	      const MGSmootherBase<VECTOR>& post_smooth,
 	      Cycle cycle = v_cycle);
+    
 				     /**
 				      * Experimental constructor for
 				      * cases in which no MGDoFHandler
@@ -214,6 +216,37 @@ class Multigrid : public Subscriptor
 				      */
     void Tvmult_add(VECTOR& dst, const VECTOR& src) const;
     
+				     /**
+				      * Set additional matrices to
+				      * correct residual computation
+				      * at refinement edges. Since we
+				      * only smoothen in the interior
+				      * of the refined part of the
+				      * mesh, the coupling across the
+				      * refinement edge is
+				      * missing. This coupling is
+				      * provided by these two
+				      * matrices.
+				      *
+				      * @note While
+				      * <tt>edge_out.vmult</tt> is
+				      * used, for the second argument,
+				      * we use
+				      * <tt>edge_in.Tvmult</tt>. Thus,
+				      * <tt>edge_in</tt> should be
+				      * assembled in transposed
+				      * form. This saves a second
+				      * sparsity pattern for
+				      * <tt>edge_in</tt>. In
+				      * particular, for symmetric
+				      * operators, both arguments can
+				      * refer to the same matrix,
+				      * saving assembling of one of
+				      * them.
+				      */
+    void set_edge_matrices (const MGMatrixBase<VECTOR>& edge_out,
+			    const MGMatrixBase<VECTOR>& edge_in);
+
 				     /**
 				      * Set additional matrices to
 				      * correct residual computation
@@ -221,12 +254,28 @@ class Multigrid : public Subscriptor
 				      * matrices originate from
 				      * discontinuous Galerkin methods
 				      * (see FE_DGQ etc.), where they
-				      * correspond tu the edge fluxes
+				      * correspond to the edge fluxes
 				      * at the refinement edge between
 				      * two levels.
+				      *
+				      * @note While
+				      * <tt>edge_down.vmult</tt> is
+				      * used, for the second argument,
+				      * we use
+				      * <tt>edge_up.Tvmult</tt>. Thus,
+				      * <tt>edge_up</tt> should be
+				      * assembled in transposed
+				      * form. This saves a second
+				      * sparsity pattern for
+				      * <tt>edge_up</tt>. In
+				      * particular, for symmetric
+				      * operators, both arguments can
+				      * refer to the same matrix,
+				      * saving assembling of one of
+				      * them.
 				      */
-    void set_edge_matrices (const MGMatrixBase<VECTOR>& edge_down,
-			    const MGMatrixBase<VECTOR>& edge_up);
+    void set_edge_flux_matrices (const MGMatrixBase<VECTOR>& edge_down,
+				 const MGMatrixBase<VECTOR>& edge_up);
 
 				     /**
 				      * Return the finest level for
@@ -374,37 +423,63 @@ class Multigrid : public Subscriptor
 				     /**
 				      * The matrix for each level.
 				      */
-    SmartPointer<const MGMatrixBase<VECTOR> > matrix;
+    SmartPointer<const MGMatrixBase<VECTOR>,Multigrid<VECTOR> > matrix;
     
 				     /**
 				      * The matrix for each level.
 				      */
-    SmartPointer<const MGCoarseGridBase<VECTOR> > coarse;
+    SmartPointer<const MGCoarseGridBase<VECTOR>,Multigrid<VECTOR> > coarse;
     
 				     /**
 				      * Object for grid tranfer.
 				      */
-    SmartPointer<const MGTransferBase<VECTOR> > transfer;
+    SmartPointer<const MGTransferBase<VECTOR>,Multigrid<VECTOR> > transfer;
 
 				     /**
 				      * The pre-smoothing object.
 				      */
-    SmartPointer<const MGSmootherBase<VECTOR> > pre_smooth;
+    SmartPointer<const MGSmootherBase<VECTOR>,Multigrid<VECTOR> > pre_smooth;
 
 				     /**
 				      * The post-smoothing object.
 				      */
-    SmartPointer<const MGSmootherBase<VECTOR> > post_smooth;
+    SmartPointer<const MGSmootherBase<VECTOR>,Multigrid<VECTOR> > post_smooth;
     
 				     /**
-				      * Edge matrix from fine to coarse.
+				      * Edge matrix from the interior
+				      * of the refined part to the
+				      * refinement edge.
+				      *
+				      * @note Only <tt>vmult</tt> is
+				      * used for these matrices.
 				      */
-    SmartPointer<const MGMatrixBase<VECTOR> > edge_down;
+    SmartPointer<const MGMatrixBase<VECTOR> > edge_out;
+
+				     /**
+				      * Transpose edge matrix from the
+				      * refinement edge to the
+				      * interior of the refined part.
+				      *
+				      * @note Only <tt>Tvmult</tt> is
+				      * used for these matrices.
+				      */
+    SmartPointer<const MGMatrixBase<VECTOR> > edge_in;
+
+				     /**
+				      * Edge matrix from fine to coarse.
+				      *
+				      * @note Only <tt>vmult</tt> is
+				      * used for these matrices.
+				      */
+    SmartPointer<const MGMatrixBase<VECTOR>,Multigrid<VECTOR> > edge_down;
 
 				     /**
 				      * Transpose edge matrix from coarse to fine.
+				      *
+				      * @note Only <tt>Tvmult</tt> is
+				      * used for these matrices.
 				      */
-    SmartPointer<const MGMatrixBase<VECTOR> > edge_up;
+    SmartPointer<const MGMatrixBase<VECTOR>,Multigrid<VECTOR> > edge_up;
 
 				     /**
 				      * Level for debug
@@ -494,17 +569,17 @@ class PreconditionMG : public Subscriptor
 				     /**
 				      * Associated @p MGDoFHandler.
 				      */
-    SmartPointer<const MGDoFHandler<dim> > mg_dof_handler;
+    SmartPointer<const MGDoFHandler<dim>,PreconditionMG<dim,VECTOR,TRANSFER> > mg_dof_handler;
 
 				     /**
 				      * The multigrid object.
 				      */
-    SmartPointer<Multigrid<VECTOR> > multigrid;
+    SmartPointer<Multigrid<VECTOR>,PreconditionMG<dim,VECTOR,TRANSFER> > multigrid;
     
 				     /**
 				      * Object for grid tranfer.
 				      */
-    SmartPointer<const TRANSFER> transfer;  
+    SmartPointer<const TRANSFER,PreconditionMG<dim,VECTOR,TRANSFER> > transfer;  
 };
 
 /*@}*/
@@ -589,13 +664,47 @@ PreconditionMG<dim, VECTOR, TRANSFER>::vmult (
   VECTOR2& dst,
   const VECTOR2& src) const
 {
+#ifdef BAERBEL_MG_TEST
+  const unsigned int min_l = multigrid->solution.get_minlevel();
+  const unsigned int max_l = multigrid->solution.get_maxlevel();
+  deallog << "PreconditionMG in " << src.l2_norm() << std::endl;
+#endif
   transfer->copy_to_mg(*mg_dof_handler,
 		       multigrid->defect,
 		       src);
+#ifdef BAERBEL_MG_TEST
+  for(unsigned int l=min_l; l<max_l; ++l)
+  deallog << "defect [" << l << "] " << multigrid->defect[l].l2_norm() << std::endl;
+#endif
   multigrid->cycle();
+
+#ifdef BAERBEL_MG_TEST
+  for(unsigned int l=min_l; l<max_l; ++l)
+  deallog << "solution [" << l << "] " << multigrid->solution[l].l2_norm() << std::endl;
+#endif
   transfer->copy_from_mg(*mg_dof_handler,
 			 dst,
 			 multigrid->solution);
+#ifdef BAERBEL_MG_TEST
+  deallog << "PreconditionMG out " << dst.l2_norm() << std::endl;
+
+  /*
+  DataOut<dim> data_out;
+  data_out.attach_dof_handler (*mg_dof_handler);
+  VECTOR2 tmp;
+  tmp.reinit(dst);
+  tmp = dst;
+  transfer->constraints->distribute(tmp);
+  data_out.add_data_vector (tmp, "mg");
+  data_out.build_patches ();
+
+  std::ostringstream filename;
+  filename << "mg.gpl";
+
+  std::ofstream output (filename.str().c_str());
+  data_out.write_gnuplot (output);
+  */
+#endif
 }
 
 
