@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: matrices.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: matrices.h 20602 2010-02-13 17:44:17Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
@@ -629,160 +629,6 @@ class MatrixCreator
 	active_cell_iterator first, second;
     };
     
-    
-				     /**
-				      * Version of the same function
-				      * (without suffix @p _1) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, typename number, int spacedim>
-    static
-    void create_mass_matrix_1 (const Mapping<dim, spacedim>       &mapping,
-			       const DoFHandler<dim,spacedim>    &dof,
-			       const Quadrature<dim>    &q,
-			       SparseMatrix<number>     &matrix,
-			       const Function<spacedim> * const a,
-			       const IteratorRange<DoFHandler<dim,spacedim> >  range,
-			       Threads::ThreadMutex     &mutex);
-
-				     /**
-				      * Version of the same function
-				      * (without suffix @p _2) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, typename number, int spacedim>
-    static
-    void create_mass_matrix_2 (const Mapping<dim, spacedim>       &mapping,
-			       const DoFHandler<dim,spacedim>    &dof,
-			       const Quadrature<dim>    &q,
-			       SparseMatrix<number>     &matrix,
-			       const Function<spacedim>      &rhs,
-			       Vector<double>           &rhs_vector,
-			       const Function<spacedim> * const a,
-			       const IteratorRange<DoFHandler<dim,spacedim> >  range,
-			       Threads::ThreadMutex     &mutex);
-
-				     /**
-				      * Same function as above, but for hp
-				      * objects.
-				      */
-    template <int dim, typename number, int spacedim>
-    static
-    void create_mass_matrix_1 (const hp::MappingCollection<dim,spacedim>       &mapping,
-			       const hp::DoFHandler<dim,spacedim>    &dof,
-			       const hp::QCollection<dim>    &q,
-			       SparseMatrix<number>     &matrix,
-			       const Function<spacedim> * const a,
-			       const IteratorRange<hp::DoFHandler<dim,spacedim> >  range,
-			       Threads::ThreadMutex     &mutex);
-
-    template <int dim, typename number, int spacedim>
-    static
-    void create_mass_matrix_2 (const hp::MappingCollection<dim,spacedim>       &mapping,
-			       const hp::DoFHandler<dim,spacedim>    &dof,
-			       const hp::QCollection<dim>    &q,
-			       SparseMatrix<number>     &matrix,
-			       const Function<spacedim>      &rhs,
-			       Vector<double>           &rhs_vector,
-			       const Function<spacedim> * const a,
-			       const IteratorRange<hp::DoFHandler<dim,spacedim> >  range,
-			       Threads::ThreadMutex     &mutex);
-
-
-/**
-				      * Version of the same function
-				      * (without suffix @p _1) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, int spacedim>
-    static
-    void create_laplace_matrix_1 (const Mapping<dim, spacedim>       &mapping,
-				  const DoFHandler<dim,spacedim>    &dof,
-				  const Quadrature<dim>    &q,
-				  SparseMatrix<double>     &matrix,
-				  const Function<spacedim> * const a,
-				  const IteratorRange<DoFHandler<dim,spacedim> >  range,
-				  Threads::ThreadMutex     &mutex);
-
-				     /**
-				      * Version of the same function
-				      * (without suffix @p _2) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, int spacedim>
-    static
-    void create_laplace_matrix_2 (const Mapping<dim, spacedim>       &mapping,
-				  const DoFHandler<dim,spacedim>    &dof,
-				  const Quadrature<dim>    &q,
-				  SparseMatrix<double>     &matrix,
-				  const Function<spacedim>      &rhs,
-				  Vector<double>           &rhs_vector,
-				  const Function<spacedim> * const a,
-				  const IteratorRange<DoFHandler<dim,spacedim> >  range,
-				  Threads::ThreadMutex     &mutex);
-
-				     /**
-				      * Version of the same function
-				      * (without suffix @p _1) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, int spacedim>
-    static
-    void create_laplace_matrix_1 (const hp::MappingCollection<dim,spacedim>       &mapping,
-				  const hp::DoFHandler<dim,spacedim>    &dof,
-				  const hp::QCollection<dim>    &q,
-				  SparseMatrix<double>     &matrix,
-				  const Function<spacedim> * const a,
-				  const IteratorRange<hp::DoFHandler<dim,spacedim> >  range,
-				  Threads::ThreadMutex     &mutex);
-
-				     /**
-				      * Version of the same function
-				      * (without suffix @p _2) with
-				      * the same argument list that
-				      * operates only on an interval
-				      * of iterators. Used for
-				      * parallelization. The mutex is
-				      * used to synchronise access to
-				      * the matrix.
-				      */
-    template <int dim, int spacedim>
-    static
-    void create_laplace_matrix_2 (const hp::MappingCollection<dim,spacedim>       &mapping,
-				  const hp::DoFHandler<dim,spacedim>    &dof,
-				  const hp::QCollection<dim>    &q,
-				  SparseMatrix<double>     &matrix,
-				  const Function<spacedim>      &rhs,
-				  Vector<double>           &rhs_vector,
-				  const Function<spacedim> * const a,
-				  const IteratorRange<hp::DoFHandler<dim,spacedim> >  range,
-				  Threads::ThreadMutex     &mutex);
-    
 				     /**
 				      * Version of the same function
 				      * (without suffix @p _1) with
@@ -1097,8 +943,8 @@ class MatrixTools : public MatrixCreator
  				      * an MPI system.
  				      *
  				      * This function is used in
-				      * @ref step_17 "step-17" and
-				      * @ref step_18 "step-18".
+				      * step-17 and
+				      * step-18.
  				      */
     static void
     apply_boundary_values (const std::map<unsigned int,double> &boundary_values,
