@@ -2,7 +2,7 @@
 //    $id$
 //    Version: $name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -14,18 +14,25 @@
 
 #include <base/thread_management.h>
 #include <base/utilities.h>
+#include <base/quadrature_lib.h>
+
 #include <lac/sparsity_pattern.h>
 #include <lac/sparsity_tools.h>
 #include <lac/compressed_simple_sparsity_pattern.h>
 #include <lac/constraint_matrix.h>
+
 #include <dofs/dof_accessor.h>
+#include <dofs/dof_handler.h>
+#include <dofs/dof_tools.h>
+#include <dofs/dof_renumbering.h>
+
 #include <grid/tria_iterator.h>
 #include <grid/tria.h>
-#include <dofs/dof_handler.h>
-#include <hp/dof_handler.h>
-#include <dofs/dof_tools.h>
+
 #include <fe/fe.h>
-#include <dofs/dof_renumbering.h>
+#include <hp/dof_handler.h>
+#include <hp/fe_collection.h>
+#include <hp/fe_values.h>
 
 #include <multigrid/mg_dof_handler.h>
 #include <multigrid/mg_dof_accessor.h>
@@ -112,15 +119,16 @@ namespace DoFRenumbering
 	T::operator==;
     };
   }
-  
+
   namespace boost
   {
+#ifndef DEAL_II_BOOST_GRAPH_COMPILER_BUG
     namespace types
     {
       using namespace ::boost;
       using namespace std;
 
-      typedef adjacency_list<vecS, vecS, undirectedS, 
+      typedef adjacency_list<vecS, vecS, undirectedS,
 			     property<vertex_color_t, default_color_type,
 				      property<vertex_degree_t,int> > > Graph;
       typedef graph_traits<Graph>::vertex_descriptor Vertex;
@@ -128,7 +136,7 @@ namespace DoFRenumbering
 
       typedef std::pair<size_type, size_type> Pair;
     }
-  
+
 
     namespace internal
     {
@@ -143,7 +151,7 @@ namespace DoFRenumbering
 				   // (faster than directly submitting
 				   // indices)
 	  ConstraintMatrix constraints;
-	  if (use_constraints) 
+	  if (use_constraints)
 	    DoFTools::make_hanging_node_constraints (dof_handler, constraints);
 	  constraints.close ();
 	  CompressedSimpleSparsityPattern csp (dof_handler.n_dofs(),
@@ -161,11 +169,11 @@ namespace DoFRenumbering
 	graph_degree = get(::boost::vertex_degree, graph);
 	for (::boost::tie(ui, ui_end) = vertices(graph); ui != ui_end; ++ui)
 	  graph_degree[*ui] = degree(*ui, graph);
-
-      }
+       }
     }
+#endif
+
 
-  
     template <class DH>
     void
     Cuthill_McKee (DH&              dof_handler,
@@ -191,13 +199,23 @@ namespace DoFRenumbering
 			   const bool       reversed_numbering,
 			   const bool       use_constraints)
     {
+#ifdef DEAL_II_BOOST_GRAPH_COMPILER_BUG
+      (void)new_dof_indices;
+      (void)dof_handler;
+      (void)reversed_numbering;
+      (void)use_constraints;
+      Assert (false,
+	      ExcMessage ("Due to a bug in your compiler, this function triggers an internal "
+			  "compiler error and has been disabled. If you need to use the "
+			  "function, you need to upgrade to a newer version of the compiler."));
+#else
       types::Graph
 	graph(dof_handler.n_dofs());
       types::property_map<types::Graph,types::vertex_degree_t>::type
 	graph_degree;
 
       internal::create_graph (dof_handler, use_constraints, graph, graph_degree);
-    
+
       types::property_map<types::Graph, types::vertex_index_t>::type
 	index_map = get(::boost::vertex_index, graph);
 
@@ -212,13 +230,14 @@ namespace DoFRenumbering
 	::boost::cuthill_mckee_ordering(graph, inv_perm.begin(),
 					get(::boost::vertex_color, graph),
 					make_degree_map(graph));
-    
+
       for (types::size_type c = 0; c != inv_perm.size(); ++c)
 	new_dof_indices[index_map[inv_perm[c]]] = c;
 
       Assert (std::find (new_dof_indices.begin(), new_dof_indices.end(),
 			 DH::invalid_dof_index) == new_dof_indices.end(),
 	      ExcInternalError());
+#endif
     }
 
 
@@ -248,13 +267,23 @@ namespace DoFRenumbering
 			   const bool       reversed_numbering,
 			   const bool       use_constraints)
     {
+#ifdef DEAL_II_BOOST_GRAPH_COMPILER_BUG
+      (void)new_dof_indices;
+      (void)dof_handler;
+      (void)reversed_numbering;
+      (void)use_constraints;
+      Assert (false,
+	      ExcMessage ("Due to a bug in your compiler, this function triggers an internal "
+			  "compiler error and has been disabled. If you need to use the "
+			  "function, you need to upgrade to a newer version of the compiler."));
+#else
       types::Graph
 	graph(dof_handler.n_dofs());
       types::property_map<types::Graph,types::vertex_degree_t>::type
 	graph_degree;
 
       internal::create_graph (dof_handler, use_constraints, graph, graph_degree);
-    
+
       types::property_map<types::Graph, types::vertex_index_t>::type
 	index_map = get(::boost::vertex_index, graph);
 
@@ -265,13 +294,14 @@ namespace DoFRenumbering
 	::boost::king_ordering(graph, inv_perm.rbegin());
       else
 	::boost::king_ordering(graph, inv_perm.begin());
-    
+
       for (types::size_type c = 0; c != inv_perm.size(); ++c)
 	new_dof_indices[index_map[inv_perm[c]]] = c;
 
       Assert (std::find (new_dof_indices.begin(), new_dof_indices.end(),
 			 DH::invalid_dof_index) == new_dof_indices.end(),
 	      ExcInternalError());
+#endif
     }
 
 
@@ -344,8 +374,8 @@ namespace DoFRenumbering
 		  add_edge (dofs_on_this_cell[j], dofs_on_this_cell[i], G);
 		}
 	}
-  
-  
+
+
       typedef std::vector<int> Vector;
 
 
@@ -393,7 +423,7 @@ namespace DoFRenumbering
     }
 
   }  // namespace boost
-  
+
 
 
   template <class DH>
@@ -439,18 +469,18 @@ namespace DoFRenumbering
 				     // empty, and calling condense with
 				     // it is a no-op
     ConstraintMatrix constraints;
-    if (use_constraints) 
+    if (use_constraints)
       DoFTools::make_hanging_node_constraints (dof_handler, constraints);
     constraints.close ();
-    
+
     SparsityPattern sparsity;
-    if (DH::dimension <= 2)
+    if (DH::dimension < 2)
       {
 	sparsity.reinit (dof_handler.n_dofs(),
 			 dof_handler.n_dofs(),
 			 dof_handler.max_couplings_between_dofs());
-	DoFTools::make_sparsity_pattern (dof_handler, sparsity);
-	constraints.condense (sparsity);
+	DoFTools::make_sparsity_pattern (dof_handler, sparsity, constraints);
+	sparsity.compress();
       }
     else
       {
@@ -462,7 +492,7 @@ namespace DoFRenumbering
 
 				     // constraints are not needed anymore
     constraints.clear ();
-  
+
     Assert(new_indices.size() == sparsity.n_rows(),
 	   ExcDimensionMismatch(new_indices.size(),
 				sparsity.n_rows()));
@@ -482,12 +512,12 @@ namespace DoFRenumbering
 		      const std::vector<unsigned int> &starting_indices)
   {
 //TODO: we should be doing the same here as in the other compute_CMK function to preserve some memory
-  
+
 				     // make the connection graph
     SparsityPattern sparsity (dof_handler.n_dofs(level),
 			      dof_handler.max_couplings_between_dofs());
     MGTools::make_sparsity_pattern (dof_handler, sparsity, level);
-    
+
     std::vector<unsigned int> new_indices(sparsity.n_rows());
     SparsityTools::reorder_Cuthill_McKee (sparsity, new_indices,
 					  starting_indices);
@@ -504,7 +534,6 @@ namespace DoFRenumbering
 
   template <int dim>
   void
-
   component_wise (DoFHandler<dim>                 &dof_handler,
 		  const std::vector<unsigned int> &component_order_arg)
   {
@@ -520,24 +549,56 @@ namespace DoFRenumbering
     const typename DoFHandler<dim>::cell_iterator
       end = dof_handler.end();
 
-    unsigned int result =
+    const unsigned int result =
       compute_component_wise<dim, ITERATOR,
       typename DoFHandler<dim>::cell_iterator>(renumbering,
 					       start, end,
 					       component_order_arg);
 
     if (result == 0) return;
-  
+
     Assert (result == dof_handler.n_dofs(),
 	    ExcRenumberingIncomplete());
-  
+
     dof_handler.renumber_dofs (renumbering);
   }
 
 
 
   template <int dim>
-  void 
+  void
+  component_wise (hp::DoFHandler<dim>             &dof_handler,
+		  const std::vector<unsigned int> &component_order_arg)
+  {
+    std::vector<unsigned int> renumbering (dof_handler.n_dofs(),
+					   hp::DoFHandler<dim>::invalid_dof_index);
+
+    typedef
+      internal::WrapDoFIterator<typename hp::DoFHandler<dim>::active_cell_iterator> ITERATOR;
+
+    typename hp::DoFHandler<dim>::active_cell_iterator
+      istart = dof_handler.begin_active();
+    ITERATOR start = istart;
+    const typename hp::DoFHandler<dim>::cell_iterator
+      end = dof_handler.end();
+
+    const unsigned int result =
+      compute_component_wise<dim, ITERATOR,
+      typename hp::DoFHandler<dim>::cell_iterator>(renumbering,
+						   start, end,
+						   component_order_arg);
+
+    if (result == 0) return;
+
+    Assert (result == dof_handler.n_dofs(),
+	    ExcRenumberingIncomplete());
+
+    dof_handler.renumber_dofs (renumbering);
+  }
+
+
+  template <int dim>
+  void
   component_wise (MGDoFHandler<dim> &dof_handler,
 		  const unsigned int level,
 		  const std::vector<unsigned int> &component_order_arg)
@@ -547,7 +608,7 @@ namespace DoFRenumbering
 
     typedef
       internal::WrapMGDoFIterator<typename MGDoFHandler<dim>::cell_iterator> ITERATOR;
-  
+
     typename MGDoFHandler<dim>::cell_iterator
       istart =dof_handler.begin(level);
     ITERATOR start = istart;
@@ -560,34 +621,37 @@ namespace DoFRenumbering
 	renumbering, start, end, component_order_arg);
 
     if (result == 0) return;
-  
+
     Assert (result == dof_handler.n_dofs(level),
 	    ExcRenumberingIncomplete());
-  
+
     if (renumbering.size()!=0)
       dof_handler.renumber_dofs (level, renumbering);
   }
 
 
-
   template <int dim>
-  void 
+  void
   component_wise (MGDoFHandler<dim> &dof_handler,
 		  const std::vector<unsigned int> &component_order_arg)
   {
-    Threads::ThreadGroup<> threads;
-
+				     // renumber the non-MG part of
+				     // the DoFHandler in parallel to
+				     // the MG part. Because
+				     // MGDoFHandler::renumber_dofs
+				     // uses the user flags we can't
+				     // run renumbering on individual
+				     // levels in parallel to the
+				     // other levels
     void (*non_mg_part) (DoFHandler<dim> &, const std::vector<unsigned int> &)
       = &component_wise<dim>;
-    void (*mg_part) (MGDoFHandler<dim> &, unsigned int, const std::vector<unsigned int> &)
-      = &component_wise<dim>;
-  
-    threads += Threads::spawn (non_mg_part) (static_cast<DoFHandler<dim>&> (dof_handler),
-					     component_order_arg);
-    for (unsigned int level=0; level<dof_handler.get_tria().n_levels(); ++level)
-      threads += Threads::spawn (mg_part) (dof_handler, level, component_order_arg);
+    Threads::Task<>
+      task = Threads::new_task (non_mg_part, dof_handler, component_order_arg);
 
-    threads.join_all();
+    for (unsigned int level=0; level<dof_handler.get_tria().n_levels(); ++level)
+      component_wise (dof_handler, level, component_order_arg);
+
+    task.join();
   }
 
 
@@ -600,22 +664,20 @@ namespace DoFRenumbering
 			  const ENDITERATOR& end,
 			  const std::vector<unsigned int> &component_order_arg)
   {
-//TODO: Modify for hp
-    const FiniteElement<dim>& fe = start->get_fe();
-    const unsigned int dofs_per_cell = fe.dofs_per_cell;
-
-//    Assert (new_indices.size() ==  start->dof_handler().n_dofs(),
-//  	  ExcDimensionMismatch(new_indices.size(),
-//  			       start->dof_handler.n_dofs()));
+    const hp::FECollection<dim> fe_collection (start->get_dof_handler().get_fe ());
 
 				     // do nothing if the FE has only
 				     // one component
-    if (fe.n_components() == 1)
+    if (fe_collection.n_components() == 1)
       {
 	new_indices.resize(0);
 	return 0;
       }
 
+//    Assert (new_indices.size() ==  start->dof_handler().n_dofs(),
+//  	  ExcDimensionMismatch(new_indices.size(),
+//  			       start->dof_handler.n_dofs()));
+
 				     // Copy last argument into a
 				     // writable vector.
     std::vector<unsigned int> component_order (component_order_arg);
@@ -624,19 +686,19 @@ namespace DoFRenumbering
 				     // store components in the order
 				     // found in the system.
     if (component_order.size() == 0)
-      for (unsigned int i=0; i<fe.n_components(); ++i)
+      for (unsigned int i=0; i<fe_collection.n_components(); ++i)
 	component_order.push_back (i);
 
-    Assert (component_order.size() == fe.n_components(),
-	    ExcDimensionMismatch(component_order.size(), fe.n_components()));
-  
+    Assert (component_order.size() == fe_collection.n_components(),
+	    ExcDimensionMismatch(component_order.size(), fe_collection.n_components()));
+
     for (unsigned int i=0; i< component_order.size(); ++i)
-      Assert(component_order[i] < fe.n_components(),
-	     ExcIndexRange(component_order[i], 0, fe.n_components()));
-  
+      Assert(component_order[i] < fe_collection.n_components(),
+	     ExcIndexRange(component_order[i], 0, fe_collection.n_components()));
+
 				     // vector to hold the dof indices on
 				     // the cell we visit at a time
-    std::vector<unsigned int> local_dof_indices(dofs_per_cell);
+    std::vector<unsigned int> local_dof_indices;
 
 				     // prebuilt list to which component
 				     // a given dof on a cell
@@ -648,32 +710,30 @@ namespace DoFRenumbering
 				     // belongs. in this case, assign it
 				     // to the first vector component to
 				     // which it belongs
-    std::vector<unsigned int> component_list (dofs_per_cell);
-    for (unsigned int i=0; i<component_list.size(); ++i)
-      if (fe.is_primitive(i))
-	component_list[i]
-	  = component_order[fe.system_to_component_index(i).first];
-      else
-	{
-	  const unsigned int base_element =
-	    fe.system_to_base_index(i).first.first;
-	  const unsigned int base_multiplicity =
-	    fe.system_to_base_index(i).first.second;
-					   // sum up the number of
-					   // components all the
-					   // elements before that have
-	  unsigned int c=0;
-	  for (unsigned int b=0; b<base_element; ++b)
-	    c += fe.base_element(b).n_components() *
-		 fe.element_multiplicity(b);
-	  for (unsigned int m=0; m<base_multiplicity; ++m)
-	    c += fe.base_element(base_element).n_components();
+    std::vector<std::vector<unsigned int> > component_list (fe_collection.size());
+    for (unsigned int f=0; f<fe_collection.size(); ++f)
+      {
+	const FiniteElement<dim> & fe = fe_collection[f];
+	const unsigned int dofs_per_cell = fe.dofs_per_cell;
+	component_list[f].resize(dofs_per_cell);
+	for (unsigned int i=0; i<dofs_per_cell; ++i)
+	  if (fe.is_primitive(i))
+	    component_list[f][i]
+	      = component_order[fe.system_to_component_index(i).first];
+	  else
+	    {
+	      const unsigned int comp = (std::find(fe.get_nonzero_components(i).begin(),
+						   fe.get_nonzero_components(i).end(),
+						   true) -
+					 fe.get_nonzero_components(i).begin());
+
 					   // then associate this degree
 					   // of freedom with this
 					   // component
-	  component_list[i] = component_order[c];
-	};
-  
+	      component_list[f][i] = component_order[comp];
+	    };
+      }
+
 				     // set up a map where for each
 				     // component the respective degrees
 				     // of freedom are collected.
@@ -686,17 +746,21 @@ namespace DoFRenumbering
 				     // multiply, so we will have to
 				     // take care of that
     std::vector<std::vector<unsigned int> >
-      component_to_dof_map (fe.n_components());
+      component_to_dof_map (fe_collection.n_components());
     for (;start!=end;++start)
       {
 					 // on each cell: get dof indices
 					 // and insert them into the global
 					 // list using their component
+	const unsigned int fe_index = start->active_fe_index();
+	const unsigned int dofs_per_cell =fe_collection[fe_index].dofs_per_cell;
+	local_dof_indices.resize (dofs_per_cell);
 	start.get_dof_indices (local_dof_indices);
 	for (unsigned int i=0; i<dofs_per_cell; ++i)
-	  component_to_dof_map[component_list[i]].push_back (local_dof_indices[i]);
+	  component_to_dof_map[component_list[fe_index][i]].
+	    push_back (local_dof_indices[i]);
       };
-  
+
 				     // now we've got all indices sorted
 				     // into buckets labelled with their
 				     // target component number. we've
@@ -720,7 +784,8 @@ namespace DoFRenumbering
 				     // into the first one. The same
 				     // holds if several components were
 				     // joined into a single target.
-    for (unsigned int component=0; component<fe.n_components(); ++component)
+    for (unsigned int component=0; component<fe_collection.n_components();
+	 ++component)
       {
 	std::sort (component_to_dof_map[component].begin(),
 		   component_to_dof_map[component].end());
@@ -734,14 +799,14 @@ namespace DoFRenumbering
 				     // components in the order the user
 				     // desired to see
     unsigned int next_free_index = 0;
-    for (unsigned int c=0; c<fe.n_components(); ++c)
+    for (unsigned int c=0; c<fe_collection.n_components(); ++c)
       {
 	const unsigned int component = c;
-      
+
 	const typename std::vector<unsigned int>::const_iterator
 	  begin_of_component = component_to_dof_map[component].begin(),
 	  end_of_component   = component_to_dof_map[component].end();
-            
+
 	for (typename std::vector<unsigned int>::const_iterator
 	       dof_index = begin_of_component;
 	     dof_index != end_of_component; ++dof_index)
@@ -781,11 +846,11 @@ namespace DoFRenumbering
 				     // their selection state
     Assert (new_indices.size() == n_dofs,
 	    ExcDimensionMismatch(new_indices.size(), n_dofs));
-  
+
     const unsigned int   n_selected_dofs = std::count (selected_dofs.begin(),
 						       selected_dofs.end(),
 						       false);
-  
+
     unsigned int next_unselected = 0;
     unsigned int next_selected   = n_selected_dofs;
     for (unsigned int i=0; i<n_dofs; ++i)
@@ -813,11 +878,11 @@ namespace DoFRenumbering
     std::vector<unsigned int> renumbering(dof.n_dofs());
     std::vector<unsigned int> reverse(dof.n_dofs());
     compute_cell_wise_dg(renumbering, reverse, dof, cells);
-  
+
     dof.renumber_dofs(renumbering);
   }
 
-  
+
 
   template <class DH>
   void
@@ -828,7 +893,7 @@ namespace DoFRenumbering
     std::vector<unsigned int> renumbering(dof.n_dofs());
     std::vector<unsigned int> reverse(dof.n_dofs());
     compute_cell_wise(renumbering, reverse, dof, cells);
-  
+
     dof.renumber_dofs(renumbering);
   }
 
@@ -851,19 +916,19 @@ namespace DoFRenumbering
 				     // Actually, we compute the
 				     // inverse of the reordering
 				     // vector, called reverse here.
-				     // Later, irs inverse is computed
+				     // Later, its inverse is computed
 				     // into new_indices, which is the
 				     // return argument.
-  
+
     Assert(new_indices.size() == n_global_dofs,
 	   ExcDimensionMismatch(new_indices.size(), n_global_dofs));
     Assert(reverse.size() == n_global_dofs,
 	   ExcDimensionMismatch(reverse.size(), n_global_dofs));
-  
+
     std::vector<unsigned int> cell_dofs;
 
     unsigned int global_index = 0;
-  
+
     typename std::vector<typename DH::cell_iterator>::const_iterator cell;
 
     for(cell = cells.begin(); cell != cells.end(); ++cell)
@@ -875,7 +940,7 @@ namespace DoFRenumbering
 					 // numbers.
 	unsigned int n_cell_dofs = (*cell)->get_fe().n_dofs_per_cell();
 	cell_dofs.resize(n_cell_dofs);
-      
+
 	(*cell)->get_dof_indices(cell_dofs);
 
 					 // Sort here to make sure that
@@ -909,14 +974,14 @@ namespace DoFRenumbering
 				dof.get_tria().n_active_cells()));
 
     unsigned int n_global_dofs = dof.n_dofs();
-  
+
 				     // Actually, we compute the
 				     // inverse of the reordering
 				     // vector, called reverse here.
 				     // Later, irs inverse is computed
 				     // into new_indices, which is the
 				     // return argument.
-  
+
     Assert(new_indices.size() == n_global_dofs,
 	   ExcDimensionMismatch(new_indices.size(), n_global_dofs));
     Assert(reverse.size() == n_global_dofs,
@@ -929,7 +994,7 @@ namespace DoFRenumbering
     std::vector<unsigned int> cell_dofs;
 
     unsigned int global_index = 0;
-  
+
     typename std::vector<typename DH::cell_iterator>::const_iterator cell;
 
     for(cell = cells.begin(); cell != cells.end(); ++cell)
@@ -940,7 +1005,7 @@ namespace DoFRenumbering
 					 // numbers.
 	unsigned int n_cell_dofs = (*cell)->get_fe().n_dofs_per_cell();
 	cell_dofs.resize(n_cell_dofs);
-      
+
 	(*cell)->get_dof_indices(cell_dofs);
 
 					 // Sort here to make sure that
@@ -964,7 +1029,7 @@ namespace DoFRenumbering
       new_indices[reverse[i]] = i;
   }
 
-  
+
 
   template <int dim>
   void cell_wise_dg (
@@ -974,12 +1039,12 @@ namespace DoFRenumbering
   {
     std::vector<unsigned int> renumbering(dof.n_dofs(level));
     std::vector<unsigned int> reverse(dof.n_dofs(level));
-  
+
     compute_cell_wise_dg(renumbering, reverse, dof, level, cells);
     dof.renumber_dofs(level, renumbering);
   }
 
-  
+
 
   template <int dim>
   void cell_wise (
@@ -989,12 +1054,12 @@ namespace DoFRenumbering
   {
     std::vector<unsigned int> renumbering(dof.n_dofs(level));
     std::vector<unsigned int> reverse(dof.n_dofs(level));
-  
+
     compute_cell_wise(renumbering, reverse, dof, level, cells);
     dof.renumber_dofs(level, renumbering);
   }
 
-  
+
 
   template <int dim>
   void compute_cell_wise_dg (
@@ -1019,19 +1084,19 @@ namespace DoFRenumbering
 	      Assert(dof.get_fe().n_dofs_per_vertex()==0,
 		     ExcNotDGFEM());
       }
-  
+
     Assert (new_order.size() == dof.n_dofs(level),
 	    ExcDimensionMismatch(new_order.size(), dof.n_dofs(level)));
     Assert (reverse.size() == dof.n_dofs(level),
 	    ExcDimensionMismatch(reverse.size(), dof.n_dofs(level)));
-  
+
     unsigned int n_global_dofs = dof.n_dofs(level);
     unsigned int n_cell_dofs = dof.get_fe().n_dofs_per_cell();
-  
+
     std::vector<unsigned int> cell_dofs(n_cell_dofs);
-  
+
     unsigned int global_index = 0;
-  
+
     typename std::vector<typename MGDoFHandler<dim>::cell_iterator>::const_iterator cell;
 
     for(cell = cells.begin(); cell != cells.end(); ++cell)
@@ -1049,7 +1114,7 @@ namespace DoFRenumbering
     Assert(global_index == n_global_dofs, ExcRenumberingIncomplete());
 
     for (unsigned int i=0;i<new_order.size(); ++i)
-      new_order[reverse[i]] = i;  
+      new_order[reverse[i]] = i;
   }
 
 
@@ -1069,15 +1134,15 @@ namespace DoFRenumbering
 	    ExcDimensionMismatch(new_order.size(), dof.n_dofs(level)));
     Assert (reverse.size() == dof.n_dofs(level),
 	    ExcDimensionMismatch(reverse.size(), dof.n_dofs(level)));
-  
+
     unsigned int n_global_dofs = dof.n_dofs(level);
     unsigned int n_cell_dofs = dof.get_fe().n_dofs_per_cell();
-  
+
     std::vector<bool> already_sorted(n_global_dofs, false);
     std::vector<unsigned int> cell_dofs(n_cell_dofs);
-  
+
     unsigned int global_index = 0;
-  
+
     typename std::vector<typename MGDoFHandler<dim>::cell_iterator>::const_iterator cell;
 
     for(cell = cells.begin(); cell != cells.end(); ++cell)
@@ -1086,7 +1151,7 @@ namespace DoFRenumbering
 
 	(*cell)->get_mg_dof_indices(cell_dofs);
 	std::sort(cell_dofs.begin(), cell_dofs.end());
-      
+
 	for (unsigned int i=0;i<n_cell_dofs;++i)
 	  {
 	    if (!already_sorted[cell_dofs[i]])
@@ -1099,7 +1164,7 @@ namespace DoFRenumbering
     Assert(global_index == n_global_dofs, ExcRenumberingIncomplete());
 
     for (unsigned int i=0;i<new_order.size(); ++i)
-      new_order[reverse[i]] = i;  
+      new_order[reverse[i]] = i;
   }
 
 
@@ -1118,11 +1183,13 @@ namespace DoFRenumbering
 
   template <class DH, int dim>
   void
-  downstream (DH& dof, const Point<dim>& direction)
+  downstream (DH& dof, const Point<dim>& direction,
+	      const bool dof_wise_renumbering)
   {
     std::vector<unsigned int> renumbering(dof.n_dofs());
     std::vector<unsigned int> reverse(dof.n_dofs());
-    compute_downstream(renumbering, reverse, dof, direction);
+    compute_downstream(renumbering, reverse, dof, direction,
+		       dof_wise_renumbering);
 
     dof.renumber_dofs(renumbering);
   }
@@ -1139,10 +1206,10 @@ namespace DoFRenumbering
     std::vector<typename DH::cell_iterator>
       ordered_cells(dof.get_tria().n_active_cells());
     const CompareDownstream<typename DH::cell_iterator, dim> comparator(direction);
-  
+
     typename DH::active_cell_iterator begin = dof.begin_active();
     typename DH::active_cell_iterator end = dof.end();
-  
+
     copy (begin, end, ordered_cells.begin());
     std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
 
@@ -1150,7 +1217,7 @@ namespace DoFRenumbering
     compute_cell_wise_dg(new_indices, reverse, dof, ordered_cells);
   }
 
-  
+
 
   template <class DH, int dim>
   void
@@ -1163,13 +1230,13 @@ namespace DoFRenumbering
     std::vector<typename DH::cell_iterator>
       ordered_cells(dof.get_tria().n_active_cells());
     const CompareDownstream<typename DH::cell_iterator, dim> comparator(direction);
-  
+
     typename DH::active_cell_iterator begin = dof.begin_active();
     typename DH::active_cell_iterator end = dof.end();
-  
+
     copy (begin, end, ordered_cells.begin());
     std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
-  
+
     compute_cell_wise_dg(new_indices, reverse, dof, ordered_cells);
   }
 
@@ -1180,50 +1247,115 @@ namespace DoFRenumbering
     std::vector<unsigned int>& new_indices,
     std::vector<unsigned int>& reverse,
     const DH& dof,
-    const Point<dim>& direction)
+    const Point<dim>& direction,
+    const bool dof_wise_renumbering)
   {
-    std::vector<typename DH::cell_iterator>
-      ordered_cells(dof.get_tria().n_active_cells());
-    const CompareDownstream<typename DH::cell_iterator, dim> comparator(direction);
-  
-    typename DH::active_cell_iterator begin = dof.begin_active();
-    typename DH::active_cell_iterator end = dof.end();
-  
-    copy (begin, end, ordered_cells.begin());
-    std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
-  
-    compute_cell_wise(new_indices, reverse, dof, ordered_cells);
+    if (dof_wise_renumbering == false)
+      {
+	std::vector<typename DH::cell_iterator>
+	  ordered_cells(dof.get_tria().n_active_cells());
+	const CompareDownstream<typename DH::cell_iterator, dim> comparator(direction);
+
+	typename DH::active_cell_iterator begin = dof.begin_active();
+	typename DH::active_cell_iterator end = dof.end();
+
+	copy (begin, end, ordered_cells.begin());
+	std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
+
+	compute_cell_wise(new_indices, reverse, dof, ordered_cells);
+      }
+    else
+      {
+				// similar code as for
+				// DoFTools::map_dofs_to_support_points, but
+				// need to do this for general DH classes and
+				// want to be able to sort the result
+				// (otherwise, could use something like
+				// DoFTools::map_support_points_to_dofs)
+	const unsigned int n_dofs = dof.n_dofs();
+	std::vector<std::pair<Point<dim>,unsigned int> > support_point_list
+	  (n_dofs);
+
+	const hp::FECollection<dim> fe_collection (dof.get_fe ());
+	Assert (fe_collection[0].has_support_points(),
+		DoFTools::ExcFEHasNoSupportPoints());
+	hp::QCollection<dim> quadrature_collection;
+	for (unsigned int comp=0; comp<fe_collection.size(); ++comp)
+	  {
+	    Assert (fe_collection[comp].has_support_points(),
+		    DoFTools::ExcFEHasNoSupportPoints());
+	    quadrature_collection.push_back
+	      (Quadrature<DH::dimension> (fe_collection[comp].
+					  get_unit_support_points()));
+	  }
+	hp::FEValues<DH::dimension,DH::space_dimension>
+	  hp_fe_values (fe_collection, quadrature_collection,
+			update_quadrature_points);
+
+	std::vector<bool> already_touched (n_dofs, false);
+
+	std::vector<unsigned int> local_dof_indices;
+	typename DH::active_cell_iterator begin = dof.begin_active();
+	typename DH::active_cell_iterator end = dof.end();
+	for ( ; begin != end; ++begin)
+	  {
+	    const unsigned int dofs_per_cell = begin->get_fe().dofs_per_cell;
+	    local_dof_indices.resize (dofs_per_cell);
+	    hp_fe_values.reinit (begin);
+	    const FEValues<dim> &fe_values =
+	      hp_fe_values.get_present_fe_values ();
+	    begin->get_dof_indices(local_dof_indices);
+	    const std::vector<Point<DH::space_dimension> > & points
+	      = fe_values.get_quadrature_points ();
+	    for (unsigned int i=0; i<dofs_per_cell; ++i)
+	      if (!already_touched[local_dof_indices[i]])
+		{
+		  support_point_list[local_dof_indices[i]].first = points[i];
+		  support_point_list[local_dof_indices[i]].second =
+		    local_dof_indices[i];
+		  already_touched[local_dof_indices[i]] = true;
+		}
+	  }
+
+	ComparePointwiseDownstream<dim> comparator (direction);
+	std::sort (support_point_list.begin(), support_point_list.end(),
+		   comparator);
+	for (unsigned int i=0; i<n_dofs; ++i)
+	  new_indices[support_point_list[i].second] = i;
+      }
   }
 
 
-  
+
   template <int dim>
   void downstream_dg (MGDoFHandler<dim>& dof,
 		      const unsigned int level,
-		      const Point<dim>& direction)
+		      const Point<dim>&  direction)
   {
     std::vector<unsigned int> renumbering(dof.n_dofs(level));
     std::vector<unsigned int> reverse(dof.n_dofs(level));
     compute_downstream_dg(renumbering, reverse, dof, level, direction);
-  
+
     dof.renumber_dofs(level, renumbering);
   }
 
-  
+
 
   template <int dim>
   void downstream (MGDoFHandler<dim>& dof,
 		   const unsigned int level,
-		   const Point<dim>& direction)
+		   const Point<dim>&  direction,
+		   const bool         dof_wise_renumbering)
   {
     std::vector<unsigned int> renumbering(dof.n_dofs(level));
     std::vector<unsigned int> reverse(dof.n_dofs(level));
-    compute_downstream(renumbering, reverse, dof, level, direction);
-  
+    compute_downstream(renumbering, reverse, dof, level, direction,
+		       dof_wise_renumbering);
+
     dof.renumber_dofs(level, renumbering);
   }
 
-  
+
 
   template <int dim>
   void
@@ -1236,18 +1368,19 @@ namespace DoFRenumbering
   {
     std::vector<typename MGDoFHandler<dim>::cell_iterator>
       ordered_cells(dof.get_tria().n_cells(level));
-    const CompareDownstream<typename MGDoFHandler<dim>::cell_iterator, dim> comparator(direction);
-  
+    const CompareDownstream<typename MGDoFHandler<dim>::cell_iterator, dim>
+      comparator(direction);
+
     typename MGDoFHandler<dim>::cell_iterator begin = dof.begin(level);
     typename MGDoFHandler<dim>::cell_iterator end = dof.end(level);
-  
+
     std::copy (begin, end, ordered_cells.begin());
     std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
-  
+
     compute_cell_wise_dg(new_indices, reverse, dof, level, ordered_cells);
   }
 
-  
+
 
   template <int dim>
   void
@@ -1256,19 +1389,63 @@ namespace DoFRenumbering
     std::vector<unsigned int>& reverse,
     const MGDoFHandler<dim>& dof,
     const unsigned int level,
-    const Point<dim>& direction)
+    const Point<dim>& direction,
+    const bool dof_wise_renumbering)
   {
-    std::vector<typename MGDoFHandler<dim>::cell_iterator>
-      ordered_cells(dof.get_tria().n_cells(level));
-    const CompareDownstream<typename MGDoFHandler<dim>::cell_iterator, dim> comparator(direction);
-  
-    typename MGDoFHandler<dim>::cell_iterator begin = dof.begin(level);
-    typename MGDoFHandler<dim>::cell_iterator end = dof.end(level);
-  
-    std::copy (begin, end, ordered_cells.begin());
-    std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
-  
-    compute_cell_wise(new_indices, reverse, dof, level, ordered_cells);
+    if (dof_wise_renumbering == false)
+      {
+	std::vector<typename MGDoFHandler<dim>::cell_iterator>
+	  ordered_cells(dof.get_tria().n_cells(level));
+	const CompareDownstream<typename MGDoFHandler<dim>::cell_iterator, dim> comparator(direction);
+
+	typename MGDoFHandler<dim>::cell_iterator begin = dof.begin(level);
+	typename MGDoFHandler<dim>::cell_iterator end = dof.end(level);
+
+	std::copy (begin, end, ordered_cells.begin());
+	std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
+
+	compute_cell_wise(new_indices, reverse, dof, level, ordered_cells);
+      }
+    else
+      {
+	Assert (dof.get_fe().has_support_points(),
+		DoFTools::ExcFEHasNoSupportPoints());
+	const unsigned int n_dofs = dof.n_dofs(level);
+	std::vector<std::pair<Point<dim>,unsigned int> > support_point_list
+	  (n_dofs);
+
+	Quadrature<dim>   q_dummy(dof.get_fe().get_unit_support_points());
+	FEValues<dim,dim> fe_values (dof.get_fe(), q_dummy,
+				     update_quadrature_points);
+
+	std::vector<bool> already_touched (dof.n_dofs(), false);
+
+	const unsigned int dofs_per_cell = dof.get_fe().dofs_per_cell;
+	std::vector<unsigned int> local_dof_indices (dofs_per_cell);
+	typename MGDoFHandler<dim>::cell_iterator begin = dof.begin(level);
+	typename MGDoFHandler<dim>::cell_iterator end = dof.end(level);
+	for ( ; begin != end; ++begin)
+	  {
+	    begin->get_mg_dof_indices(local_dof_indices);
+	    fe_values.reinit (begin);
+	    const std::vector<Point<dim> > & points
+	      = fe_values.get_quadrature_points ();
+	    for (unsigned int i=0; i<dofs_per_cell; ++i)
+	      if (!already_touched[local_dof_indices[i]])
+		{
+		  support_point_list[local_dof_indices[i]].first = points[i];
+		  support_point_list[local_dof_indices[i]].second =
+		    local_dof_indices[i];
+		  already_touched[local_dof_indices[i]] = true;
+		}
+	  }
+
+	ComparePointwiseDownstream<dim> comparator (direction);
+	std::sort (support_point_list.begin(), support_point_list.end(),
+		   comparator);
+	for (unsigned int i=0; i<n_dofs; ++i)
+	  new_indices[support_point_list[i].second] = i;
+      }
   }
 
 
@@ -1289,7 +1466,7 @@ namespace DoFRenumbering
 					  * Revert sorting order.
 					  */
 	bool counter;
-    
+
 					 /**
 					  * Constructor.
 					  */
@@ -1304,7 +1481,7 @@ namespace DoFRenumbering
 	bool operator () (const DHCellIterator& c1,
 			  const DHCellIterator& c2) const
 	  {
-	
+
 	    const Point<dim> v1 = c1->center() - center;
 	    const Point<dim> v2 = c2->center() - center;
 	    const double s1 = std::atan2(v1(0), v1(1));
@@ -1314,7 +1491,7 @@ namespace DoFRenumbering
     };
   }
 
-  
+
 
   template <class DH, int dim>
   void
@@ -1342,10 +1519,10 @@ namespace DoFRenumbering
     std::vector<typename DH::cell_iterator>
       ordered_cells(dof.get_tria().n_active_cells());
     internal::ClockCells<dim> comparator(center, counter);
-  
+
     typename DH::active_cell_iterator begin = dof.begin_active();
     typename DH::active_cell_iterator end = dof.end();
-  
+
     std::copy (begin, end, ordered_cells.begin());
     std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
 
@@ -1353,7 +1530,7 @@ namespace DoFRenumbering
     compute_cell_wise_dg(new_indices, reverse, dof, ordered_cells);
   }
 
-  
+
 
   template <int dim>
   void clockwise_dg (MGDoFHandler<dim>& dof,
@@ -1364,10 +1541,10 @@ namespace DoFRenumbering
     std::vector<typename MGDoFHandler<dim>::cell_iterator>
       ordered_cells(dof.get_tria().n_cells(level));
     internal::ClockCells<dim> comparator(center, counter);
-  
+
     typename MGDoFHandler<dim>::cell_iterator begin = dof.begin(level);
     typename MGDoFHandler<dim>::cell_iterator end = dof.end(level);
-  
+
     std::copy (begin, end, ordered_cells.begin());
     std::sort (ordered_cells.begin(), ordered_cells.end(), comparator);
 
@@ -1387,7 +1564,7 @@ namespace DoFRenumbering
     dof_handler.renumber_dofs(renumbering);
   }
 
-  
+
 
   template <class DH>
   void
@@ -1398,10 +1575,10 @@ namespace DoFRenumbering
     const unsigned int n_dofs = dof_handler.n_dofs();
     Assert(new_indices.size() == n_dofs,
 	   ExcDimensionMismatch(new_indices.size(), n_dofs));
-  
+
     for (unsigned i=0; i<n_dofs; ++i)
       new_indices[i] = i;
-  
+
     std::random_shuffle (new_indices.begin(), new_indices.end());
   }
 
@@ -1418,7 +1595,7 @@ namespace DoFRenumbering
     dof_handler.renumber_dofs(renumbering);
   }
 
-  
+
 
   template <class DH>
   void
@@ -1438,7 +1615,7 @@ namespace DoFRenumbering
     const unsigned int n_subdomains
       = *std::max_element (subdomain_association.begin(),
 			   subdomain_association.end()) + 1;
-  
+
 				     // then renumber the subdomains by first
 				     // looking at those belonging to subdomain
 				     // 0, then those of subdomain 1, etc. note
@@ -1500,8 +1677,36 @@ namespace DoFRenumbering
     void
     compute_minimum_degree (std::vector<unsigned int> &,
 			    const DoFHandler<deal_II_dimension> &, bool, bool);
+
+
+    template
+    void
+    Cuthill_McKee (hp::DoFHandler<deal_II_dimension> &, bool, bool);
+
+    template
+    void
+    compute_Cuthill_McKee (std::vector<unsigned int> &,
+			   const hp::DoFHandler<deal_II_dimension> &, bool, bool);
+
+    template
+    void
+    king_ordering (hp::DoFHandler<deal_II_dimension> &, bool, bool);
+
+    template
+    void
+    compute_king_ordering (std::vector<unsigned int> &,
+			   const hp::DoFHandler<deal_II_dimension> &, bool, bool);
+
+    template
+    void
+    minimum_degree (hp::DoFHandler<deal_II_dimension> &, bool, bool);
+
+    template
+    void
+    compute_minimum_degree (std::vector<unsigned int> &,
+			    const hp::DoFHandler<deal_II_dimension> &, bool, bool);
   }
-  
+
 
 // non-boost functions:
 
@@ -1538,12 +1743,32 @@ namespace DoFRenumbering
    const bool,
    const std::vector<unsigned int>&);
 
+  template
+  void Cuthill_McKee<MGDoFHandler<deal_II_dimension> >
+  (MGDoFHandler<deal_II_dimension>&,
+   const bool,
+   const bool,
+   const std::vector<unsigned int>&);
+
+  template
+  void
+  compute_Cuthill_McKee<MGDoFHandler<deal_II_dimension> >
+  (std::vector<unsigned int>&,
+   const MGDoFHandler<deal_II_dimension>&,
+   const bool,
+   const bool,
+   const std::vector<unsigned int>&);
 
   template
   void component_wise<deal_II_dimension>
   (DoFHandler<deal_II_dimension>&,
    const std::vector<unsigned int>&);
 
+  template
+  void component_wise<deal_II_dimension>
+  (hp::DoFHandler<deal_II_dimension>&,
+   const std::vector<unsigned int>&);
+
 //  template
 //  void
 //  compute_component_wise<deal_II_dimension>
@@ -1609,12 +1834,14 @@ namespace DoFRenumbering
 
   template void
   downstream<DoFHandler<deal_II_dimension> >
-  (DoFHandler<deal_II_dimension>&, const Point<deal_II_dimension>&);
+  (DoFHandler<deal_II_dimension>&, const Point<deal_II_dimension>&,
+   const bool);
 
   template void
   compute_downstream<DoFHandler<deal_II_dimension> >
   (std::vector<unsigned int>&,std::vector<unsigned int>&,
-   const DoFHandler<deal_II_dimension>&, const Point<deal_II_dimension>&);
+   const DoFHandler<deal_II_dimension>&, const Point<deal_II_dimension>&,
+   const bool);
 
   template
   void
@@ -1671,13 +1898,15 @@ namespace DoFRenumbering
   template void
   downstream<hp::DoFHandler<deal_II_dimension> >
   (hp::DoFHandler<deal_II_dimension>&,
-   const Point<deal_II_dimension>&);
+   const Point<deal_II_dimension>&,
+   const bool);
 
   template void
   compute_downstream<hp::DoFHandler<deal_II_dimension> >
   (std::vector<unsigned int>&,std::vector<unsigned int>&,
    const hp::DoFHandler<deal_II_dimension>&,
-   const Point<deal_II_dimension>&);
+   const Point<deal_II_dimension>&,
+   const bool);
 
   template
   void
@@ -1703,7 +1932,8 @@ namespace DoFRenumbering
   template
   void downstream<deal_II_dimension>
   (MGDoFHandler<deal_II_dimension>&, const unsigned int,
-   const Point<deal_II_dimension>&);
+   const Point<deal_II_dimension>&,
+   const bool);
 
   template
   void clockwise_dg<deal_II_dimension>
