@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_abf.cc 18191 2009-01-13 00:19:17Z kanschat $
+//    $Id: fe_abf.cc 20115 2009-11-16 14:00:07Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -700,7 +700,7 @@ FE_ABF<dim>::interpolate(
 
   // TODO: Check if this "correction" can be removed.
   for (unsigned int i=0; i<boundary_weights_abf.size(1); ++i)
-    if (fabs (local_dofs[start_abf_dofs+i]) < 1.0e-16)
+    if (std::fabs (local_dofs[start_abf_dofs+i]) < 1.0e-16)
       local_dofs[start_abf_dofs+i] = 0.0;
 }
 
@@ -760,7 +760,7 @@ FE_ABF<dim>::interpolate(
 
   // TODO: Check if this "correction" can be removed.
   for (unsigned int i=0; i<boundary_weights_abf.size(1); ++i)
-    if (fabs (local_dofs[start_abf_dofs+i]) < 1.0e-16)
+    if (std::fabs (local_dofs[start_abf_dofs+i]) < 1.0e-16)
       local_dofs[start_abf_dofs+i] = 0.0;
 }
 
