@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_nedelec.cc 18502 2009-03-23 16:54:38Z bangerth $
+//    $Id: fe_nedelec.cc 18933 2009-06-12 08:53:35Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -997,7 +997,7 @@ FE_Nedelec<dim,spacedim>::fill_fe_values
    typename Mapping<dim,spacedim>::InternalDataBase &mapping_data,
    typename Mapping<dim,spacedim>::InternalDataBase &fedata,
    FEValuesData<dim,spacedim>                       &data,
-   enum CellSimilarity::Similarity                  &/*cell_similarity*/) const
+   CellSimilarity::Similarity                  &/*cell_similarity*/) const
 {
  				   // convert data object to internal
  				   // data for this class. fails with
