@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_poly_tensor.cc 18473 2009-03-10 14:53:36Z kronbichler $
+//    $Id: fe_poly_tensor.cc 20358 2010-01-13 13:52:46Z leicht $
 //    Version: $Name$
 //
 //    Copyright (C) 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -64,20 +64,7 @@ namespace
 	Triangulation<dim,spacedim>::face_iterator face = cell->face (f);
 	if (!face->at_boundary ())
 	  {
-	    const unsigned int neighbor_level = cell->neighbor (f)->level ();
-	    const unsigned int cell_level = cell->level ();
-
-					     // check of neighbor is more
-					     // refined (this case should
-					     // never be encountered)
-	    Assert (neighbor_level <= cell_level,
-		    ExcMessage ("neighbor_level is larger than cell level!"));
-
-	    const unsigned int nn = (neighbor_level == cell_level
-				     ?
-				     cell->neighbor_of_neighbor (f)
-				     :
-				     cell->neighbor_of_coarser_neighbor (f).first);
+	    const unsigned int nn = cell->neighbor_face_no(f);
 	      
 	    if (nn < GeometryInfo<dim>::faces_per_cell / 2)
 	      for (unsigned int j = 0; j < dofs_per_face; ++j)
@@ -330,8 +317,12 @@ FE_PolyTensor<POLY,dim,spacedim>::get_data (
 		data->shape_values[i][k] = values[i];
 	    else
 	      for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-		for (unsigned int j=0; j<this->dofs_per_cell; ++j)
-		  data->shape_values[i][k] += inverse_node_matrix(j,i) * values[j];
+		{
+		  Tensor<1,dim> add_values;
+		  for (unsigned int j=0; j<this->dofs_per_cell; ++j)
+		    add_values += inverse_node_matrix(j,i) * values[j];
+		  data->shape_values[i][k] = add_values;
+		}
 	  }
 	
 	if (flags & update_gradients)
@@ -341,8 +332,12 @@ FE_PolyTensor<POLY,dim,spacedim>::get_data (
 		data->shape_grads[i][k] = grads[i];
 	    else
 	      for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-		for (unsigned int j=0; j<this->dofs_per_cell; ++j)
-		  data->shape_grads[i][k] += inverse_node_matrix(j,i) * grads[j];
+		{
+		  Tensor<2,dim> add_grads;
+		  for (unsigned int j=0; j<this->dofs_per_cell; ++j)
+		    add_grads += inverse_node_matrix(j,i) * grads[j];
+		  data->shape_grads[i][k] = add_grads;
+		}
 	  }
 	
       }
@@ -365,7 +360,7 @@ FE_PolyTensor<POLY,dim,spacedim>::fill_fe_values (
   typename Mapping<dim,spacedim>::InternalDataBase &mapping_data,
   typename Mapping<dim,spacedim>::InternalDataBase &fedata,
   FEValuesData<dim,spacedim>                       &data,
-  enum CellSimilarity::Similarity                  &cell_similarity) const
+  CellSimilarity::Similarity                  &cell_similarity) const
 {
 				   // convert data object to internal
 				   // data for this class. fails with
@@ -404,48 +399,44 @@ FE_PolyTensor<POLY,dim,spacedim>::fill_fe_values (
       const unsigned int first = data.shape_function_to_row_table[i];
       
       if (flags & update_values && cell_similarity != CellSimilarity::translation)
-	{
-	  if (cell_similarity == 1)
-	    std::cout << "hier";
-	  switch (mapping_type)
+	switch (mapping_type)
+	  {
+	  case mapping_none:
 	    {
-	      case mapping_none:
-	      {
-		for (unsigned int k=0; k<n_q_points; ++k)
-		  for (unsigned int d=0; d<dim; ++d)
-		    data.shape_values(first+d,k) = fe_data.shape_values[i][k][d];
-		break;
-	      }
-	      
-	      case mapping_covariant:
-	      case mapping_contravariant:
-	      {
-		std::vector<Tensor<1,dim> > shape_values (n_q_points);
-		mapping.transform(fe_data.shape_values[i],
-				  shape_values, mapping_data, mapping_type);
-		
-		for (unsigned int k=0; k<n_q_points; ++k)
-		  for (unsigned int d=0; d<dim; ++d)
-		    data.shape_values(first+d,k) = shape_values[k][d];
-		
-		break;
-	      }
-	      case mapping_piola:
-	      {
-		std::vector<Tensor<1,dim> > shape_values (n_q_points);
-		mapping.transform(fe_data.shape_values[i], shape_values,
-				  mapping_data, mapping_piola);
-		for (unsigned int k=0; k<n_q_points; ++k)
-		  for (unsigned int d=0; d<dim; ++d)
-		    data.shape_values(first+d,k)
-		      = sign_change[i] * shape_values[k][d];
-		break;
-	      }
-	      
-	      default:
-		Assert(false, ExcNotImplemented());
+	      for (unsigned int k=0; k<n_q_points; ++k)
+		for (unsigned int d=0; d<dim; ++d)
+		  data.shape_values(first+d,k) = fe_data.shape_values[i][k][d];
+	      break;
 	    }
-	}
+ 
+	  case mapping_covariant:
+	  case mapping_contravariant:
+	    {
+	      std::vector<Tensor<1,dim> > shape_values (n_q_points);
+	      mapping.transform(fe_data.shape_values[i],
+				shape_values, mapping_data, mapping_type);
+
+	      for (unsigned int k=0; k<n_q_points; ++k)
+		for (unsigned int d=0; d<dim; ++d)
+		  data.shape_values(first+d,k) = shape_values[k][d];
+
+	      break;
+	    }
+	  case mapping_piola:
+	    {
+	      std::vector<Tensor<1,dim> > shape_values (n_q_points);
+	      mapping.transform(fe_data.shape_values[i], shape_values,
+				mapping_data, mapping_piola);
+	      for (unsigned int k=0; k<n_q_points; ++k)
+		for (unsigned int d=0; d<dim; ++d)
+		  data.shape_values(first+d,k)
+		    = sign_change[i] * shape_values[k][d];
+	      break;
+	    }
+
+	  default:
+	    Assert(false, ExcNotImplemented());
+	  }
       
       if (flags & update_gradients && cell_similarity != CellSimilarity::translation)
 	{
