@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: fe_q_hierarchical.cc 16430 2008-07-08 15:25:01Z hartmann $
+//    $Id: fe_q_hierarchical.cc 21190 2010-06-10 08:25:49Z buerg $
 //    Version: $Name$
 //
 //    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
@@ -12,6 +12,7 @@
 //---------------------------------------------------------------------------
 
 #include <fe/fe_q_hierarchical.h>
+#include <fe/fe_nothing.h>
 
 #include <cmath>
 #include <sstream>
@@ -120,6 +121,49 @@ FE_Q_Hierarchical<dim>::clone() const
 }
 
 
+
+template <int dim>
+bool
+FE_Q_Hierarchical<dim>::hp_constraints_are_implemented () const
+{
+  return false;
+}
+
+
+template <int dim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_Q_Hierarchical<dim>::
+hp_vertex_dof_identities (const FiniteElement<dim> &fe_other) const
+{
+				   // we can presently only compute
+				   // these identities if both FEs are
+				   // FE_Q_Hierarchicals or if the other
+				   // one is an FE_Nothing. in the first
+				   // case, there should be exactly one
+				   // single DoF of each FE at a
+				   // vertex, and they should have
+				   // identical value
+  if (dynamic_cast<const FE_Q_Hierarchical<dim>*>(&fe_other) != 0)
+    {
+      return
+	std::vector<std::pair<unsigned int, unsigned int> >
+	(1, std::make_pair (0U, 0U));
+    }
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
+    {
+				       // the FE_Nothing has no
+				       // degrees of freedom, so there
+				       // are no equivalencies to be
+				       // recorded
+      return std::vector<std::pair<unsigned int, unsigned int> > ();
+    }
+  else
+    {
+      Assert (false, ExcNotImplemented());
+      return std::vector<std::pair<unsigned int, unsigned int> > ();
+    }
+}
+
 //---------------------------------------------------------------------------
 // Auxiliary functions
 //---------------------------------------------------------------------------
