@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_cartesian.cc 18473 2009-03-10 14:53:36Z kronbichler $
+//    $Id: mapping_cartesian.cc 18933 2009-06-12 08:53:35Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -132,7 +132,7 @@ void
 MappingCartesian<dim, spacedim>::compute_fill (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
 					       const unsigned int        face_no,
 					       const unsigned int        sub_no,
-					       const enum CellSimilarity::Similarity cell_similarity,
+					       const CellSimilarity::Similarity cell_similarity,
 					       InternalData             &data,
 					       std::vector<Point<dim> > &quadrature_points,
 					       std::vector<Point<dim> > &normal_vectors) const
@@ -310,7 +310,7 @@ fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator& cell,
 		std::vector<Tensor<3,spacedim> >& jacobian_grads,
 		std::vector<Tensor<2,spacedim> >& inverse_jacobians,
 	        std::vector<Point<spacedim> >       &,
-		enum CellSimilarity::Similarity& cell_similarity) const
+		CellSimilarity::Similarity& cell_similarity) const
 {
 				   // convert data object to internal
 				   // data for this class. fails with
