@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mapping_q.cc 18473 2009-03-10 14:53:36Z kronbichler $
+//    $Id: mapping_q.cc 20688 2010-02-24 18:12:25Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -313,8 +313,8 @@ MappingQ<dim,spacedim>::fill_fe_values (
   std::vector<Tensor<2,spacedim> >                          &jacobians,
   std::vector<Tensor<3,spacedim> >                          &jacobian_grads,
   std::vector<Tensor<2,spacedim> >                          &inverse_jacobians,
-  std::vector<Point<spacedim> >                             &cell_normal_vectors,
-  enum CellSimilarity::Similarity                           &cell_similarity) const
+  std::vector<Point<spacedim> >                             &normal_vectors,
+  CellSimilarity::Similarity                           &cell_similarity) const
 {
 				   // convert data object to internal
 				   // data for this class. fails with
@@ -351,13 +351,14 @@ MappingQ<dim,spacedim>::fill_fe_values (
   else
     {
       p_data=&data;
-      cell_similarity = CellSimilarity::invalid_next_cell;
+      if (get_degree() > 1)
+	cell_similarity = CellSimilarity::invalid_next_cell;
     }
   
   MappingQ1<dim,spacedim>::fill_fe_values(cell, q, *p_data,
 	        			  quadrature_points, JxW_values,
 				          jacobians, jacobian_grads, inverse_jacobians,
-					  cell_normal_vectors, cell_similarity);
+					  normal_vectors, cell_similarity);
 }
 
 
@@ -475,10 +476,11 @@ MappingQ<dim,spacedim>::fill_fe_subface_values (const typename Triangulation<dim
                            n_q_points,
                            QProjector<dim>::DataSetDescriptor::
                            subface (face_no, sub_no,
-                                     cell->face_orientation(face_no),
-                                     cell->face_flip(face_no),
-                                     cell->face_rotation(face_no),
-                                     n_q_points),
+				    cell->face_orientation(face_no),
+				    cell->face_flip(face_no),
+				    cell->face_rotation(face_no),
+				    n_q_points,
+				    cell->subface_case(face_no)),
                            q.get_weights(),
                            *p_data,
                            quadrature_points, JxW_values,
