@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: mg_dof_accessor.cc 18097 2009-01-06 04:49:31Z bangerth $
+//    $Id: mg_dof_accessor.cc 20340 2010-01-09 00:13:01Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -58,7 +58,7 @@ template <int structdim, int dim, int spacedim>
 void
 MGDoFAccessor<structdim, dim, spacedim>::set_mg_dof_handler (MGDoFHandler<dim,spacedim> *dh)
 {
-  typedef DoFAccessor<dim,DoFHandler<dim> > BaseClass;
+  typedef DoFAccessor<dim,DoFHandler<dim, spacedim> > BaseClass;
   Assert (dh != 0, typename BaseClass::ExcInvalidObject());
   mg_dof_handler = dh;
 }
@@ -70,7 +70,7 @@ MGDoFAccessor<structdim, dim, spacedim> &
 MGDoFAccessor<structdim, dim, spacedim>::operator = (const MGDoFAccessor &da)
 {
   BaseClass::operator= (*this);
-  
+
   set_dof_handler (da.mg_dof_handler);
   return *this;
 }
@@ -162,7 +162,7 @@ MGDoFAccessor<structdim, dim, spacedim>::child (const unsigned int i) const
 	0),
        this->child_index (i),
        this->mg_dof_handler);
-  
+
 				   // make sure that we either created
 				   // a past-the-end iterator or one
 				   // pointing to a used cell
@@ -180,21 +180,20 @@ template <int structdim, int dim, int spacedim>
 void
 MGDoFAccessor<structdim, dim, spacedim>::copy_from (const MGDoFAccessor &a)
 {
-  DoFAccessor<structdim, DoFHandler<dim> >::copy_from (a);
+  DoFAccessor<structdim, DoFHandler<dim, spacedim> >::copy_from (a);
   this->set_mg_dof_handler (a.mg_dof_handler);
 }
 
 
 
 template <int structdim, int dim, int spacedim>
-inline
 typename internal::MGDoFHandler::Iterators<dim,spacedim>::line_iterator
 MGDoFAccessor<structdim,dim,spacedim>::line (const unsigned int i) const
 {
   Assert (structdim > 1, ExcImpossibleInDim(structdim));
 				   // checking of 'i' happens in
 				   // line_index(i)
-  
+
   return typename internal::MGDoFHandler::Iterators<dim,spacedim>::line_iterator
     (
       this->tria,
@@ -206,7 +205,6 @@ MGDoFAccessor<structdim,dim,spacedim>::line (const unsigned int i) const
 
 
 template <int structdim, int dim, int spacedim>
-inline
 typename internal::MGDoFHandler::Iterators<dim,spacedim>::quad_iterator
 MGDoFAccessor<structdim,dim,spacedim>::quad (const unsigned int i) const
 {
@@ -243,7 +241,7 @@ namespace internal
 	  *next++ = accessor.mg_vertex_dof_index(level,vertex,d);
       for (unsigned int d=0; d<dofs_per_line; ++d)
 	*next++ = accessor.mg_dof_index(level,d);
-  
+
       Assert (next == dof_indices.end(),
 	      ExcInternalError());
     }
@@ -253,18 +251,18 @@ namespace internal
     void
     get_mg_dof_values (const dealii::MGDoFAccessor<1, dim, spacedim> &accessor,
 		       const int level,
-		       const Vector<number> &values,
-		       Vector<number>       &dof_values)
+		       const dealii::Vector<number> &values,
+		       dealii::Vector<number>       &dof_values)
     {
       const unsigned int dofs_per_vertex = accessor.get_dof_handler().get_fe().dofs_per_vertex,
 			 dofs_per_line   = accessor.get_dof_handler().get_fe().dofs_per_line;
-      typename Vector<number>::iterator next_dof_value=dof_values.begin();
+      typename dealii::Vector<number>::iterator next_dof_value=dof_values.begin();
       for (unsigned int vertex=0; vertex<2; ++vertex)
 	for (unsigned int d=0; d<dofs_per_vertex; ++d)
 	  *next_dof_value++ = values(accessor.mg_vertex_dof_index(level,vertex,d));
       for (unsigned int d=0; d<dofs_per_line; ++d)
 	*next_dof_value++ = values(accessor.mg_dof_index(level,d));
-  
+
       Assert (next_dof_value == dof_values.end(),
 	      ExcInternalError());
     }
@@ -290,7 +288,7 @@ namespace internal
 	  *next++ = accessor.line(line)->mg_dof_index(level,d);
       for (unsigned int d=0; d<dofs_per_quad; ++d)
 	*next++ = accessor.mg_dof_index(level,d);
-  
+
       Assert (next == dof_indices.end(),
 	      ExcInternalError());
     }
@@ -300,13 +298,13 @@ namespace internal
     void
     get_mg_dof_values (const dealii::MGDoFAccessor<2, dim, spacedim> &accessor,
 		       const int level,
-		       const Vector<number> &values,
-		       Vector<number>       &dof_values)
+		       const dealii::Vector<number> &values,
+		       dealii::Vector<number>       &dof_values)
     {
       const unsigned int dofs_per_vertex = accessor.get_dof_handler().get_fe().dofs_per_vertex,
 			 dofs_per_line   = accessor.get_dof_handler().get_fe().dofs_per_line,
 			 dofs_per_quad   = accessor.get_dof_handler().get_fe().dofs_per_quad;
-      typename Vector<number>::iterator next_dof_value=dof_values.begin();
+      typename dealii::Vector<number>::iterator next_dof_value=dof_values.begin();
       for (unsigned int vertex=0; vertex<4; ++vertex)
 	for (unsigned int d=0; d<dofs_per_vertex; ++d)
 	  *next_dof_value++ = values(accessor.mg_vertex_dof_index(level,vertex,d));
@@ -315,7 +313,7 @@ namespace internal
 	  *next_dof_value++ = values(accessor.line(line)->mg_dof_index(level,d));
       for (unsigned int d=0; d<dofs_per_quad; ++d)
 	*next_dof_value++ = values(accessor.mg_dof_index(level,d));
-  
+
       Assert (next_dof_value == dof_values.end(),
 	      ExcInternalError());
     }
@@ -343,7 +341,7 @@ namespace internal
 	  *next++ = accessor.quad(quad)->mg_dof_index(level,d);
       for (unsigned int d=0; d<dofs_per_hex; ++d)
 	*next++ = accessor.mg_dof_index(level,d);
-  
+
       Assert (next == dof_indices.end(),
 	      ExcInternalError());
     }
@@ -353,14 +351,14 @@ namespace internal
     void
     get_mg_dof_values (const dealii::MGDoFAccessor<3, dim, spacedim> &accessor,
 		       const int level,
-		       const Vector<number> &values,
-		       Vector<number>       &dof_values)
+		       const dealii::Vector<number> &values,
+		       dealii::Vector<number>       &dof_values)
     {
       const unsigned int dofs_per_vertex = accessor.get_dof_handler().get_fe().dofs_per_vertex,
 			 dofs_per_line   = accessor.get_dof_handler().get_fe().dofs_per_line,
 			 dofs_per_quad   = accessor.get_dof_handler().get_fe().dofs_per_quad,
 			 dofs_per_hex    = accessor.get_dof_handler().get_fe().dofs_per_hex;
-      typename Vector<number>::iterator next_dof_value=dof_values.begin();
+      typename dealii::Vector<number>::iterator next_dof_value=dof_values.begin();
       for (unsigned int vertex=0; vertex<8; ++vertex)
 	for (unsigned int d=0; d<dofs_per_vertex; ++d)
 	  *next_dof_value++ = values(accessor.mg_vertex_dof_index(level,vertex,d));
@@ -372,7 +370,7 @@ namespace internal
 	  *next_dof_value++ = values(accessor.quad(quad)->mg_dof_index(level,d));
       for (unsigned int d=0; d<dofs_per_hex; ++d)
 	*next_dof_value++ = values(accessor.dof_index(d));
-  
+
       Assert (next_dof_value == dof_values.end(),
 	      ExcInternalError());
     }
@@ -494,7 +492,7 @@ MGDoFCellAccessor<dim,spacedim>::neighbor (const unsigned int i) const
 					       this->neighbor_level (i),
 					       this->neighbor_index (i),
 					       this->mg_dof_handler);
-  
+
 #ifdef DEBUG
   if (q.state() != IteratorState::past_the_end)
     Assert (q->used(), TriaAccessorExceptions::ExcUnusedCellAsNeighbor());
@@ -511,7 +509,7 @@ MGDoFCellAccessor<dim,spacedim>::child (const unsigned int i) const
 					       this->present_level+1,
 					       this->child_index (i),
 					       this->mg_dof_handler);
-  
+
 #ifdef DEBUG
   if (q.state() != IteratorState::past_the_end)
     Assert (q->used(), TriaAccessorExceptions::ExcUnusedCellAsChild());
@@ -536,7 +534,7 @@ MGDoFCellAccessor<dim,spacedim>::face (const unsigned int i) const
 	    return this->quad(i);
       default:
 	    Assert (false, ExcNotImplemented());
-	    return 
+	    return
 	      typename internal::MGDoFHandler::Iterators<dim,spacedim>::face_iterator();
     }
 }
@@ -627,10 +625,13 @@ get_mg_dof_values (const Vector<float> &values,
 
 
 
+
 #if deal_II_dimension == 1
-template class MGDoFAccessor<1, 1>;
+template class MGDoFAccessor<1, 1, 1>;
+template class MGDoFAccessor<1, 1, 2>;
 
 template class MGDoFCellAccessor<1>;
+template class MGDoFCellAccessor<1,2>;
 
 template class TriaRawIterator<MGDoFCellAccessor<1> >;
 template class TriaIterator<MGDoFCellAccessor<1> >;
@@ -638,14 +639,22 @@ template class TriaActiveIterator<MGDoFCellAccessor<1> >;
 #endif
 
 #if deal_II_dimension == 2
-template class MGDoFAccessor<1, 2>;
-template class MGDoFAccessor<2, 2>;
+template class MGDoFAccessor<1, 2, 2>;
+template class MGDoFAccessor<2, 2, 2>;
+
+template class MGDoFAccessor<1, 2, 3>;
+template class MGDoFAccessor<2, 2, 3>;
 
 template class MGDoFCellAccessor<2>;
+template class MGDoFCellAccessor<2,3>;
 
-template class TriaRawIterator   <MGDoFAccessor<1, 2> >;
-template class TriaIterator      <MGDoFAccessor<1, 2> >;
-template class TriaActiveIterator<MGDoFAccessor<1, 2> >;
+template class TriaRawIterator   <MGDoFAccessor<1, 2, 2> >;
+template class TriaIterator      <MGDoFAccessor<1, 2, 2> >;
+template class TriaActiveIterator<MGDoFAccessor<1, 2, 2> >;
+
+template class TriaRawIterator   <MGDoFAccessor<1, 2, 3> >;
+template class TriaIterator      <MGDoFAccessor<1, 2, 3> >;
+template class TriaActiveIterator<MGDoFAccessor<1, 2, 3> >;
 
 template class TriaRawIterator   <MGDoFCellAccessor<2> >;
 template class TriaIterator      <MGDoFCellAccessor<2> >;
@@ -654,19 +663,19 @@ template class TriaActiveIterator<MGDoFCellAccessor<2> >;
 
 
 #if deal_II_dimension == 3
-template class MGDoFAccessor<1, 3>;
-template class MGDoFAccessor<2, 3>;
-template class MGDoFAccessor<3, 3>;
+template class MGDoFAccessor<1, 3, 3>;
+template class MGDoFAccessor<2, 3, 3>;
+template class MGDoFAccessor<3, 3, 3>;
 
 template class MGDoFCellAccessor<3>;
 
-template class TriaRawIterator   <MGDoFAccessor<1, 3> >;
-template class TriaIterator      <MGDoFAccessor<1, 3> >;
-template class TriaActiveIterator<MGDoFAccessor<1, 3> >;
+template class TriaRawIterator   <MGDoFAccessor<1, 3, 3> >;
+template class TriaIterator      <MGDoFAccessor<1, 3, 3> >;
+template class TriaActiveIterator<MGDoFAccessor<1, 3, 3> >;
 
-template class TriaRawIterator   <MGDoFAccessor<2, 3> >;
-template class TriaIterator      <MGDoFAccessor<2, 3> >;
-template class TriaActiveIterator<MGDoFAccessor<2, 3> >;
+template class TriaRawIterator   <MGDoFAccessor<2, 3, 3> >;
+template class TriaIterator      <MGDoFAccessor<2, 3, 3> >;
+template class TriaActiveIterator<MGDoFAccessor<2, 3, 3> >;
 
 template class TriaRawIterator   <MGDoFCellAccessor<3> >;
 template class TriaIterator      <MGDoFCellAccessor<3> >;
