@@ -1,10 +1,10 @@
 /**
- * @page changes_after_6.2 Changes after Version 6.2
+ * @page changes_after_6_3_1 Changes after Version 6.3.1
 
 <p>
-This is the list of changes made after the release of 
-deal.II version 6.2. It is subdivided into changes
-made to the three sub-libraries <a href="#base">base</a>, 
+This is the list of changes made after the release of
+deal.II version 6.3.1. It is subdivided into changes
+made to the three sub-libraries <a href="#base">base</a>,
 <a href="#lac">lac</a>, and <a href="#deal.II">deal.II</a>, as well as
 changes to the <a href="#general">general infrastructure,
 documentation, etc</a>.
@@ -30,10 +30,7 @@ inconvenience this causes.
 </p>
 
 <ol>
-  <li>
-  <p> 
-  </p>
-  </li>
+  <li>None so far.
 </ol>
 
 
@@ -41,10 +38,7 @@ inconvenience this causes.
 <h3>General</h3>
 
 <ol>
-  <li>
-  <p>
-  </p>
-  </li>
+  <li>None so far.
 </ol>
 
 
@@ -53,34 +47,23 @@ inconvenience this causes.
 <h3>base</h3>
 
 <ol>
-  <li>
-  <p>
-  </p>
-  </li>
+  <li>None so far.
 </ol>
 
 
-
 <a name="lac"></a>
 <h3>lac</h3>
 
 <ol>
-  <li>
-  <p>
-  </p>
-  </li>
+  <li>None so far.
 </ol>
 
 
-
 <a name="deal.II"></a>
 <h3>deal.II</h3>
 
 <ol>
-  <li>
-  <p>
-  </p>
-  </li>
+  <li>None so far.
 </ol>
 
 
