@@ -1,6 +1,6 @@
-/* $Id: step-1.cc 14941 2007-08-10 21:05:06Z kanschat $
+/* $Id: step-1.cc 19994 2009-10-25 02:47:44Z bangerth $
  *
- * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007 by the deal.II authors
+ * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009 by the deal.II authors
  *
  * This file is subject to QPL and may not be  distributed
  * without copyright and license information. Please refer
@@ -68,14 +68,14 @@ void first_grid ()
                                    // two-dimensional domain:
   Triangulation<2> triangulation;
 				   // Here and in many following
-                                   // cases, the string "@<2@>" after a
+                                   // cases, the string "<2>" after a
                                    // class name indicates that this
                                    // is an object that shall work in
                                    // two space dimensions. Likewise,
                                    // there are versions of the
                                    // triangulation class that are
-                                   // working in one ("@<1@>") and three
-                                   // ("@<3@>") space dimensions. The
+                                   // working in one ("<1>") and three
+                                   // ("<3>") space dimensions. The
                                    // way this works is through some
                                    // template magic that we will
                                    // investigate in some more detail
@@ -241,9 +241,9 @@ void second_grid ()
                                          // numbers like a 4 that
                                          // needs to be replaced by an
                                          // 8:
-        for (unsigned int vertex=0;
-             vertex < GeometryInfo<2>::vertices_per_cell;
-             ++vertex)
+        for (unsigned int v=0;
+             v < GeometryInfo<2>::vertices_per_cell;
+             ++v)
           {
                                              // If this cell is at the
                                              // inner boundary, then
@@ -266,7 +266,7 @@ void second_grid ()
                                              // vertices and move on
                                              // to the next cell.
             const double distance_from_center
-              = center.distance (cell->vertex(vertex));
+              = center.distance (cell->vertex(v));
             
             if (std::fabs(distance_from_center - inner_radius) < 1e-10)
               {
