@@ -1,9 +1,9 @@
-/* $Id: step-2.cc 16774 2008-09-10 12:16:22Z bangerth $ */
+/* $Id: step-2.cc 21275 2010-06-23 13:36:27Z bangerth $ */
 /* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
 
-/*    $Id: step-2.cc 16774 2008-09-10 12:16:22Z bangerth $       */
+/*    $Id: step-2.cc 21275 2010-06-23 13:36:27Z bangerth $       */
 /*                                                                */
-/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2006, 2008 by the deal.II authors */
+/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2006, 2008, 2009, 2010 by the deal.II authors */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -50,6 +50,12 @@
 				 // on the grid. That class can be
 				 // found here:
 #include <lac/sparse_matrix.h>
+				 // We will also need to use an
+				 // intermediate sparsity patter
+				 // structure, which is found in this
+				 // file:
+#include <lac/compressed_sparsity_pattern.h>
+
 				 // We will want to use a special
 				 // algorithm to renumber degrees of
 				 // freedom. It is declared here:
@@ -85,9 +91,9 @@ using namespace dealii;
                                  // function, which is too early. We avoid the
                                  // problem by declaring it 'static' which
                                  // makes sure that the object is initialized
-                                 // the first time control flow passes its
-                                 // point of declaration, but at the same time
-                                 // assures that it lives until the end of the
+                                 // the first time control the program passes
+                                 // this point, but at the same time assures
+                                 // that it lives until the end of the
                                  // program.
 void make_grid (Triangulation<2> &triangulation)
 {
@@ -108,12 +114,12 @@ void make_grid (Triangulation<2> &triangulation)
 	endc = triangulation.end();
 
       for (; cell!=endc; ++cell)
-	for (unsigned int vertex=0;
-	     vertex < GeometryInfo<2>::vertices_per_cell;
-	     ++vertex)
+	for (unsigned int v=0;
+	     v < GeometryInfo<2>::vertices_per_cell;
+	     ++v)
 	  {
             const double distance_from_center
-              = center.distance (cell->vertex(vertex));
+              = center.distance (cell->vertex(v));
 	    
 	    if (std::fabs(distance_from_center - inner_radius) < 1e-10)
 	      {
@@ -201,15 +207,14 @@ void distribute_dofs (DoFHandler<2> &dof_handler)
 				   // Now that we have associated a degree of
 				   // freedom with a global number to each
 				   // vertex, we wonder how to visualize this?
-				   // Unfortunately, presently no way is
-				   // implemented to directly show the DoF
-				   // number associated with each
-				   // vertex. However, such information would
-				   // hardly ever be truly important, since
-				   // the numbering itself is more or less
-				   // arbitrary. There are more important
-				   // factors, of which we will visualize one
-				   // in the following.
+				   // There is no simple way to directly
+				   // visualize the DoF number associated with
+				   // each vertex. However, such information
+				   // would hardly ever be truly important,
+				   // since the numbering itself is more or
+				   // less arbitrary. There are more important
+				   // factors, of which we will demonstrate
+				   // one in the following.
 				   //
 				   // Associated with each vertex of the
 				   // triangulation is a shape
@@ -223,7 +228,7 @@ void distribute_dofs (DoFHandler<2> &dof_handler)
 				   // they are associated to, matrix entries
 				   // will be nonzero only if the supports of
 				   // the shape functions associated to that
-				   // column and row numbers intersect. This
+				   // column and row %numbers intersect. This
 				   // is only the case for adjacent shape
 				   // functions, and therefore only for
 				   // adjacent vertices. Now, since the
@@ -234,61 +239,61 @@ void distribute_dofs (DoFHandler<2> &dof_handler)
 				   // will be somewhat ragged, and we will
 				   // take a look at it now.
 				   //
-				   // First we have to create a
-				   // structure which we use to store
-				   // the places of nonzero
-				   // elements. We have to give it the
-				   // size of the matrix, which in our
-				   // case will be square with as
-				   // many rows and columns as there
-				   // are degrees of freedom on the
+				   // First we have to create a structure
+				   // which we use to store the places of
+				   // nonzero elements. This can then later be
+				   // used by one or more sparse matrix
+				   // objects that store the values of the
+				   // entries in the locations stored by this
+				   // sparsity pattern. The class that stores
+				   // the locations is the SparsityPattern
+				   // class. As it turns out, however, this
+				   // class has some drawbacks when we try to
+				   // fill it right away: its data structures
+				   // are set up in such a way that we need to
+				   // have an estimate for the maximal number
+				   // of entries we may wish to have in each
+				   // row. In two space dimensions, reasonable
+				   // values for this estimate are available
+				   // through the
+				   // DoFHandler::max_couplings_between_dofs()
+				   // function, but in three dimensions the
+				   // function almost always severely
+				   // overestimates the true number, leading
+				   // to a lot of wasted memory, sometimes too
+				   // much for the machine used, even if the
+				   // unused memory can be released
+				   // immediately after computing the sparsity
+				   // pattern. In order to avoid this, we use
+				   // an intermediate object of type
+				   // CompressedSparsityPattern that uses a
+				   // different %internal data structure and
+				   // that we can later copy into the
+				   // SparsityPattern object without much
+				   // overhead. (Some more information on
+				   // these data structures can be found in
+				   // the @ref Sparsity module.) In order to
+				   // initialize this intermediate data
+				   // structure, we have to give it the size
+				   // of the matrix, which in our case will be
+				   // square with as many rows and columns as
+				   // there are degrees of freedom on the
 				   // grid:
-  SparsityPattern sparsity_pattern (dof_handler.n_dofs(),
-				    dof_handler.n_dofs(),
-                                    20);
-                                   // The last argument to the constructor
-                                   // indicates the maximum number of entries
-                                   // we expect per row. If this were a
-                                   // uniformly refined square, then we know
-                                   // that each vertex degree of freedom would
-                                   // couple with itself and the eight degrees
-                                   // of freedom around it. However, our mesh
-                                   // is more complicated, and it may well be
-                                   // that more couplings will occur. The
-                                   // value 20 we use here is on the safe side
-                                   // of that, though it may actually be too
-                                   // large. In the step-3 tutorial program,
-                                   // we will see a way to compute a
-                                   // reasonable upper bound to the number of
-                                   // nonzero entries, and later programs will
-                                   // show several other methods to compute
-                                   // the numbers of zeros per row.
+  CompressedSparsityPattern c_sparsity(dof_handler.n_dofs());
 
 				   // We then fill this object with the
 				   // places where nonzero elements will be
 				   // located given the present numbering of
 				   // degrees of freedom:
-  DoFTools::make_sparsity_pattern (dof_handler, sparsity_pattern);
-				   // Before further work can be done
-				   // on the object, we have to allow
-				   // for some internal
-				   // reorganization:
-  sparsity_pattern.compress ();
-                                   // What actually happens in this call is
-                                   // the following: upon creation of a
-                                   // <code>SparsityPattern</code> object, memory is
-                                   // allocated for a maximum number of
-                                   // entries per row (20 in our case). The
-                                   // call to
-                                   // <code>DoFTools::make_sparsity_pattern</code> then
-                                   // actually allocates entries as necessary,
-                                   // but the number of nonzero entries in any
-                                   // given row may be less than the 20 we
-                                   // have allocated memory for. To save
-                                   // memory and to simplify some other
-                                   // operations, one then needs to
-                                   // <code>compress</code> the sparsity pattern before
-                                   // anything else.
+  DoFTools::make_sparsity_pattern (dof_handler, c_sparsity);
+
+				   // Now we are ready to create the actual
+				   // sparsity pattern that we could later use
+				   // for our matrix. It will just contain the
+				   // data already assembled in the
+				   // CompressedSparsityPattern.
+  SparsityPattern sparsity_pattern;
+  sparsity_pattern.copy_from(c_sparsity);
 
 				   // With this, we can now write the results
 				   // to a file:
