@@ -1,9 +1,9 @@
-/* $Id: step-3.cc 16263 2008-05-28 16:41:01Z kanschat $ */
+/* $Id: step-3.cc 21235 2010-06-20 17:53:33Z kanschat $ */
 /* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
 
-/*    $Id: step-3.cc 16263 2008-05-28 16:41:01Z kanschat $       */
+/*    $Id: step-3.cc 21235 2010-06-20 17:53:33Z kanschat $       */
 /*                                                                */
-/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008 by the deal.II authors */
+/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2010 by the deal.II authors */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -80,6 +80,7 @@
 #include <lac/vector.h>
 #include <lac/full_matrix.h>
 #include <lac/sparse_matrix.h>
+#include <lac/compressed_sparsity_pattern.h>
 #include <lac/solver_cg.h>
 #include <lac/precondition.h>
 
@@ -218,8 +219,7 @@ void LaplaceProblem::make_grid_and_dofs ()
   std::cout << "Number of active cells: "
 	    << triangulation.n_active_cells()
 	    << std::endl;
-                                   // Here, by active we mean the cells on the
-				   // finest level, i.e. cells that aren't
+                                   // Here, by active we mean the cells that aren't
 				   // refined any further.  We stress the
 				   // adjective `active', since there are more
 				   // cells, namely the parent cells of the
@@ -260,25 +260,16 @@ void LaplaceProblem::make_grid_and_dofs ()
 				   // As we have seen in the previous example,
 				   // we set up a sparsity pattern for the
 				   // system matrix and tag those entries that
-				   // might be nonzero. Compared to what we
-				   // did in step-2, the only change is that
-				   // instead of giving a magically obtained
-				   // maximal number of nonzero entries per
-				   // row, we now use a function in the
-				   // <code>DoFHandler</code> class that can compute
-				   // this number for us:
-  sparsity_pattern.reinit (dof_handler.n_dofs(),
-			   dof_handler.n_dofs(),
-			   dof_handler.max_couplings_between_dofs());
-  DoFTools::make_sparsity_pattern (dof_handler, sparsity_pattern);
-  sparsity_pattern.compress();
+				   // might be nonzero.  
+  CompressedSparsityPattern c_sparsity(dof_handler.n_dofs());
+  DoFTools::make_sparsity_pattern (dof_handler, c_sparsity);
+  sparsity_pattern.copy_from(c_sparsity);
 
 				   // Now the sparsity pattern is
-				   // built and fixed (after
-				   // `compress' has been called, you
+				   // built, you
 				   // can't add nonzero entries
-				   // anymore; the sparsity pattern is
-				   // `sealed', so to say), we can
+				   // anymore. The sparsity pattern is
+				   // `sealed', so to say, and we can
 				   // initialize the matrix itself
 				   // with it. Note that the
 				   // SparsityPattern object does
@@ -597,8 +588,10 @@ void LaplaceProblem::assemble_system ()
 				       // quadrature points of the
 				       // value of the shape function
 				       // at that point times the
-				       // right hand side function
-				       // (i.e. 1) times the Jacobian
+				       // right hand side function,
+				       // here the constant function
+				       // equal to one,
+				       // times the Jacobian
 				       // determinant times the weight
 				       // of that quadrature point:
       for (unsigned int i=0; i<dofs_per_cell; ++i)
