@@ -1,9 +1,9 @@
-/* $Id: step-4.cc 16263 2008-05-28 16:41:01Z kanschat $ */
+/* $Id: step-4.cc 21235 2010-06-20 17:53:33Z kanschat $ */
 /* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
 
-/*    $Id: step-4.cc 16263 2008-05-28 16:41:01Z kanschat $       */
+/*    $Id: step-4.cc 21235 2010-06-20 17:53:33Z kanschat $       */
 /*                                                                */
-/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors */
+/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -33,6 +33,7 @@
 #include <lac/vector.h>
 #include <lac/full_matrix.h>
 #include <lac/sparse_matrix.h>
+#include <lac/compressed_sparsity_pattern.h>
 #include <lac/solver_cg.h>
 #include <lac/precondition.h>
 
@@ -344,14 +345,12 @@ void LaplaceProblem<dim>::make_grid_and_dofs ()
 	    << dof_handler.n_dofs()
 	    << std::endl;
 
-  sparsity_pattern.reinit (dof_handler.n_dofs(),
-			   dof_handler.n_dofs(),
-			   dof_handler.max_couplings_between_dofs());
-  DoFTools::make_sparsity_pattern (dof_handler, sparsity_pattern);
-  sparsity_pattern.compress();
-
+  CompressedSparsityPattern c_sparsity(dof_handler.n_dofs());
+  DoFTools::make_sparsity_pattern (dof_handler, c_sparsity);
+  sparsity_pattern.copy_from(c_sparsity);
+  
   system_matrix.reinit (sparsity_pattern);
-
+  
   solution.reinit (dof_handler.n_dofs());
   system_rhs.reinit (dof_handler.n_dofs());
 }
@@ -445,7 +444,7 @@ void LaplaceProblem<dim>::assemble_system ()
 				   // world they look alike and you will
 				   // probably never see a difference although
 				   // the classes that this typedef stands for
-				   // are in fact completelye unrelated:
+				   // are in fact completely unrelated:
   typename DoFHandler<dim>::active_cell_iterator cell = dof_handler.begin_active(),
 						 endc = dof_handler.end();
   for (; cell!=endc; ++cell)
@@ -490,11 +489,13 @@ void LaplaceProblem<dim>::assemble_system ()
 	  }
                                        // As a final remark to these loops:
                                        // when we assemble the local
-                                       // contributions, we have to multiply
-                                       // the gradients of shape functions i
-                                       // and j at point q_point and multiply
+                                       // contributions into
+				       // <code>cell_matrix(i,j)</code>,
+				       // we have to multiply
+                                       // the gradients of shape functions $i$
+                                       // and $j$ at point q_point and multiply
                                        // it with the scalar weights JxW. This
-                                       // is actually what happens:
+                                       // is what actually happens:
                                        // <code>fe_values.shape_grad(i,q_point)</code>
                                        // returns a <code>dim</code> dimensional
                                        // vector, represented by a
@@ -538,7 +539,7 @@ void LaplaceProblem<dim>::assemble_system ()
   
 				   // As the final step in this function, we
 				   // wanted to have non-homogeneous boundary
-				   // values in this example, contrary to the
+				   // values in this example, unlike the
 				   // one before. This is a simple task, we
 				   // only have to replace the
 				   // <code>ZeroFunction</code> used there by an object
