@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: block_matrix_array.h 17981 2008-12-17 21:26:46Z kanschat $
+//    $Id: block_matrix_array.h 20934 2010-04-02 13:09:20Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -91,7 +91,7 @@ template <typename> class Vector;
  * GrowingVectorMemory type, since it remembers the vector and avoids
  * reallocating.
  *
- * @ line Growing
+ * @line Growing
  *
  * Now, we are ready to build a <i>2x2</i> BlockMatrixArray.
  * @line Block
@@ -116,7 +116,8 @@ template <typename> class Vector;
  * and is described in the documentation of
  * BlockTrianglePrecondition.
  *
- * @author Guido Kanschat, 2000 - 2005
+ * @see @ref GlossBlockLA "Block (linear algebra)"
+ * @author Guido Kanschat, 2000-2005
  */
 template <typename number = double>
 class BlockMatrixArray : public Subscriptor
@@ -394,7 +395,7 @@ class BlockMatrixArray : public Subscriptor
 				      * The memory used for auxiliary
 				      * vectors.
 				      */
-    mutable SmartPointer<VectorMemory<Vector<number> > > mem;
+    mutable SmartPointer<VectorMemory<Vector<number> >,BlockMatrixArray<number> > mem;
 };
 
 /*@}*/
