@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: block_matrix_base.h 18575 2009-04-09 04:56:42Z bangerth $
+//    $Id: block_matrix_base.h 21162 2010-06-07 20:47:13Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -46,6 +46,7 @@ template <typename number>     class BlockSparseMatrixEZ;
  * want to do things differently depending on whether a template type
  * denotes a regular or a block matrix type.
  *
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, 2009
  */
 template <typename MatrixType>
@@ -96,7 +97,7 @@ struct IsBlockMatrix
 				      * indicates whether the template
 				      * argument to this class is a block
 				      * matrix (in fact whether the type is
-				      * derived from BlockMatrix<T>).
+				      * derived from BlockMatrixBase<T>).
 				      */
     static const bool value = (sizeof(check_for_block_matrix
 				      ((MatrixType*)0))
@@ -109,7 +110,7 @@ struct IsBlockMatrix
 template <typename MatrixType>
 const bool IsBlockMatrix<MatrixType>::value;
 
-    
+
 
 /*! @addtogroup Matrix1
  *@{
@@ -142,34 +143,34 @@ namespace BlockMatrixIterators
 					* default values.
 					*/
       AccessorBase ();
-      
+
 				       /**
 					* Block row of the
 					* element represented by
 					* this object.
 					*/
       unsigned int block_row() const;
-      
+
 				       /**
 					* Block column of the
 					* element represented by
 					* this object.
 					*/
       unsigned int block_column() const;
-      
+
     protected:
 				       /**
 					* Block row into which we presently
 					* point.
 					*/
       unsigned int row_block;
-      
+
 				       /**
 					* Block column into which we
 					* presently point.
 					*/
       unsigned int col_block;
-      
+
 				       /**
 					* Let the iterator class be a
 					* friend.
@@ -183,7 +184,7 @@ namespace BlockMatrixIterators
 				    */
   template <class BlockMatrix, bool ConstNess>
   class Accessor;
-  
+
 				   /**
 				    * Block matrix accessor for non
 				    * const matrices.
@@ -199,13 +200,13 @@ namespace BlockMatrixIterators
 					* this accessor.
 					*/
       typedef BlockMatrix MatrixType;
-      
+
 				       /**
 					* Typedef the value type of the
 					* matrix we point into.
 					*/
       typedef typename BlockMatrix::value_type value_type;
-      
+
 				       /**
 					* Constructor. Since we use
 					* accessors only for read
@@ -221,59 +222,59 @@ namespace BlockMatrixIterators
       Accessor (BlockMatrix* m,
 		const unsigned int row,
 		const unsigned int col);
-      
+
 				       /**
 					* Row number of the element
 					* represented by this
 					* object.
 					*/
       unsigned int row() const;
-      
+
 				       /**
 					* Column number of the
 					* element represented by
 					* this object.
 					*/
       unsigned int column() const;
-      
+
 				       /**
 					* Value of the entry at the
 					* current position.
 					*/
       value_type value() const;
-      
+
 				       /**
 					* Set new value.
 					*/
       void set_value(value_type newval) const;
-      
+
     protected:
 				       /**
 					* The matrix accessed.
 					*/
       BlockMatrix * matrix;
-      
+
 				       /**
 					* Iterator of the underlying matrix
 					* class.
 					*/
       typename BlockMatrix::BlockType::iterator base_iterator;
-      
+
 				       /**
 					* Move ahead one element.
 					*/
       void advance ();
-      
+
 				       /**
 					* Compare this accessor with another
 					* one for equality.
 					*/
       bool operator == (const Accessor &a) const;
-      
+
       template <typename> friend class MatrixIterator;
       friend class Accessor<BlockMatrix, true>;
   };
-  
+
 				   /**
 				    * Block matrix accessor for
 				    * constant matrices, implementing
@@ -290,13 +291,13 @@ namespace BlockMatrixIterators
 					* this accessor.
 					*/
       typedef const BlockMatrix MatrixType;
-      
+
 				       /**
 					* Typedef the value type of the
 					* matrix we point into.
 					*/
       typedef typename BlockMatrix::value_type value_type;
-      
+
 				       /**
 					* Constructor. Since we use
 					* accessors only for read
@@ -312,27 +313,27 @@ namespace BlockMatrixIterators
       Accessor (const BlockMatrix* m,
 		const unsigned int row,
 		const unsigned int col);
-      
+
 				       /**
 					* Initalize const accessor
 					* from non const accessor.
 					*/
       Accessor(const Accessor<BlockMatrix, false>&);
-      
+
 				       /**
 					* Row number of the element
 					* represented by this
 					* object.
 					*/
       unsigned int row() const;
-      
+
 				       /**
 					* Column number of the
 					* element represented by
 					* this object.
 					*/
       unsigned int column() const;
-      
+
 				       /**
 					* Value of the entry at the
 					* current position.
@@ -343,24 +344,24 @@ namespace BlockMatrixIterators
 					* The matrix accessed.
 					*/
       const BlockMatrix * matrix;
-      
+
 				       /**
 					* Iterator of the underlying matrix
 					* class.
 					*/
       typename BlockMatrix::BlockType::const_iterator base_iterator;
-      
+
 				       /**
 					* Move ahead one element.
 					*/
       void advance ();
-      
+
 				       /**
 					* Compare this accessor with another
 					* one for equality.
 					*/
       bool operator == (const Accessor &a) const;
-      
+
 				       /**
 					* Let the iterator class be a
 					* friend.
@@ -402,7 +403,7 @@ namespace BlockMatrixIterators
  * to view the matrix alternatively as one big matrix, or as a number of
  * individual blocks.
  *
- * 
+ *
  * <h3>Inheriting from this class</h3>
  *
  * Since this class simply forwards its calls to the subobjects (if necessary
@@ -413,7 +414,7 @@ namespace BlockMatrixIterators
  * members provided by this base class, as they are only used passively in
  * this class.
  *
- * 
+ *
  * Most of the functions take a vector or block vector argument. These
  * functions can, in general, only successfully be compiled if the
  * individual blocks of this matrix implement the respective functions
@@ -428,6 +429,7 @@ namespace BlockMatrixIterators
  * @<double@></tt>; others can be generated in application programs (see the
  * section on @ref Instantiations in the manual).
  *
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, 2000, 2004
  */
 template <typename MatrixType>
@@ -464,7 +466,7 @@ class BlockMatrixBase : public Subscriptor
                                       * Default constructor.
                                       */
     BlockMatrixBase ();
-    
+
 				     /**
 				      * Copy the given matrix to this
 				      * one.  The operation throws an
@@ -503,8 +505,8 @@ class BlockMatrixBase : public Subscriptor
     BlockType &
     block (const unsigned int row,
 	   const unsigned int column);
-    
-    
+
+
 				     /**
 				      * Access the block with the
 				      * given coordinates. Version for
@@ -512,8 +514,8 @@ class BlockMatrixBase : public Subscriptor
 				      */
     const BlockType &
     block (const unsigned int row,
-	   const unsigned int column) const;    
-    
+	   const unsigned int column) const;
+
 				     /**
 				      * Return the dimension of the
 				      * image space.  To remember: the
@@ -521,7 +523,7 @@ class BlockMatrixBase : public Subscriptor
 				      * $m \times n$.
 				      */
     unsigned int m () const;
-    
+
 				     /**
 				      * Return the dimension of the
 				      * range space.  To remember: the
@@ -530,7 +532,7 @@ class BlockMatrixBase : public Subscriptor
 				      */
     unsigned int n () const;
 
-    
+
 				     /**
 				      * Return the number of blocks in
 				      * a column. Returns zero if no
@@ -538,7 +540,7 @@ class BlockMatrixBase : public Subscriptor
 				      * associated to this matrix.
 				      */
     unsigned int n_block_rows () const;
-    
+
 				     /**
 				      * Return the number of blocks in
 				      * a row. Returns zero if no
@@ -546,7 +548,7 @@ class BlockMatrixBase : public Subscriptor
 				      * associated to this matrix.
 				      */
     unsigned int n_block_cols () const;
-    
+
 				     /**
 				      * Set the element <tt>(i,j)</tt>
 				      * to <tt>value</tt>. Throws an
@@ -822,13 +824,13 @@ class BlockMatrixBase : public Subscriptor
 				      * fixed factor.
 				      */
     BlockMatrixBase & operator *= (const value_type factor);
-    
+
 				     /**
 				      * Divide the entire matrix by a
 				      * fixed factor.
 				      */
     BlockMatrixBase & operator /= (const value_type factor);
-    
+
 				     /**
 				      * Add <tt>matrix</tt> scaled by
 				      * <tt>factor</tt> to this matrix,
@@ -848,9 +850,9 @@ class BlockMatrixBase : public Subscriptor
 				      */
     template <class BlockMatrixType>
     void add (const value_type       factor,
-	      const BlockMatrixType &matrix);    
-  
-    
+	      const BlockMatrixType &matrix);
+
+
 				     /**
 				      * Adding Matrix-vector
 				      * multiplication. Add $M*src$ on
@@ -860,7 +862,7 @@ class BlockMatrixBase : public Subscriptor
     template <class BlockVectorType>
     void vmult_add (BlockVectorType       &dst,
 		    const BlockVectorType &src) const;
-    
+
 				     /**
 				      * Adding Matrix-vector
 				      * multiplication. Add
@@ -906,12 +908,12 @@ class BlockMatrixBase : public Subscriptor
 				     /**
 				      * Compute the matrix scalar
 				      * product $\left(u,Mv\right)$.
-				      */    
+				      */
     template <class BlockVectorType>
     value_type
     matrix_scalar_product (const BlockVectorType &u,
 			   const BlockVectorType &v) const;
-    
+
 				     /**
 				      * Compute the residual
 				      * <i>r=b-Ax</i>. Write the
@@ -932,7 +934,7 @@ class BlockMatrixBase : public Subscriptor
 				      * Final iterator.
 				      */
     iterator end ();
-    
+
 				     /**
 				      * STL-like iterator with the
 				      * first entry of row <tt>r</tt>.
@@ -953,7 +955,7 @@ class BlockMatrixBase : public Subscriptor
 				      * Final iterator.
 				      */
     const_iterator end () const;
-    
+
 				     /**
 				      * STL-like iterator with the
 				      * first entry of row <tt>r</tt>.
@@ -1022,11 +1024,11 @@ class BlockMatrixBase : public Subscriptor
                                       */
     BlockIndices row_block_indices;
     BlockIndices column_block_indices;
-    
+
 				     /**
 				      * Array of sub-matrices.
 				      */
-    Table<2,SmartPointer<BlockType> > sub_objects;
+    Table<2,SmartPointer<BlockType, BlockMatrixBase<MatrixType> > > sub_objects;
 
                                      /**
                                       * This function collects the
@@ -1152,7 +1154,7 @@ class BlockMatrixBase : public Subscriptor
     template <class VectorType>
     void vmult_nonblock_nonblock (VectorType       &dst,
 		                  const VectorType &src) const;
-    
+
 				     /**
 				      * Matrix-vector multiplication:
 				      * let $dst = M^T*src$ with $M$
@@ -1175,7 +1177,7 @@ class BlockMatrixBase : public Subscriptor
     template <class BlockVectorType>
     void Tvmult_block_block (BlockVectorType       &dst,
 		             const BlockVectorType &src) const;
-  
+
 				     /**
 				      * Matrix-vector
 				      * multiplication. Just like the
@@ -1243,33 +1245,56 @@ class BlockMatrixBase : public Subscriptor
     template <class VectorType>
     void Tvmult_nonblock_nonblock (VectorType       &dst,
 		                   const VectorType &src) const;
-    
+
+
+  protected:
+
+				     /**
+				      * Some matrix types, in particular PETSc,
+				      * need to synchronize set and add
+				      * operations. This has to be done for all
+				      * matrices in the BlockMatrix.
+				      * This routine prepares adding of elements
+				      * by notifying all blocks. Called by all
+				      * internal routines before adding
+				      * elements.
+				      */
+    void prepare_add_operation();
+
+				     /**
+				      * Notifies all blocks to let them prepare
+				      * for setting elements, see
+				      * prepare_add_operation().
+				      */
+    void prepare_set_operation();
+
+
   private:
-                                       /**
-					* Temporary vector for counting the
-					* elements written into the
-					* individual blocks when doing a
-					* collective add or set.
-					*/
-    std::vector<unsigned int> counter_within_block; 
+				     /**
+				      * Temporary vector for counting the
+				      * elements written into the
+				      * individual blocks when doing a
+				      * collective add or set.
+				      */
+    std::vector<unsigned int> counter_within_block;
 
-                                       /**
-					* Temporary vector for column
-					* indices on each block when writing
-					* local to global data on each
-					* sparse matrix.
-					*/
-    std::vector<std::vector<unsigned int> > column_indices; 
+				     /**
+				      * Temporary vector for column
+				      * indices on each block when writing
+				      * local to global data on each
+				      * sparse matrix.
+				      */
+    std::vector<std::vector<unsigned int> > column_indices;
 
-                                       /**
-					* Temporary vector for storing the
-					* local values (they need to be
-					* reordered when writing local to
-					* global).
-					*/
+				     /**
+				      * Temporary vector for storing the
+				      * local values (they need to be
+				      * reordered when writing local to
+				      * global).
+				      */
     std::vector<std::vector<double> > column_values;
 
-    
+
 				     /**
 				      * Make the iterator class a
 				      * friend. We have to work around
@@ -1285,7 +1310,7 @@ class BlockMatrixBase : public Subscriptor
     typedef BlockMatrixIterators::Accessor<BlockMatrixBase, true> ConstAccessor;
     typedef BlockMatrixIterators::Accessor<BlockMatrixBase, false> Accessor;
     friend class ConstAccessor;
-    
+
     friend class const_iterator;
 #endif
 
@@ -1307,7 +1332,7 @@ namespace BlockMatrixIterators
 		  row_block(0),
 		  col_block(0)
   {}
-  
+
 
   template <class BlockMatrix>
   inline
@@ -1316,11 +1341,11 @@ namespace BlockMatrixIterators
   {
     Assert (row_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return row_block;
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   unsigned int
@@ -1328,11 +1353,11 @@ namespace BlockMatrixIterators
   {
     Assert (col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return col_block;
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   Accessor<BlockMatrix, true>::Accessor (
@@ -1344,14 +1369,14 @@ namespace BlockMatrixIterators
 		  base_iterator(matrix->block(0,0).begin())
   {
     Assert(col==0, ExcNotImplemented());
-    
+
 				     // check if this is a regular row or
 				     // the end of the matrix
     if (row < matrix->m())
       {
 	const std::pair<unsigned int,unsigned int> indices
 	  = matrix->row_block_indices.global_to_local(row);
-	
+
 					 // find the first block that does
 					 // have an entry in this row
 	for (unsigned int bc=0; bc<matrix->n_block_cols(); ++bc)
@@ -1366,7 +1391,7 @@ namespace BlockMatrixIterators
 		return;
 	      }
 	  }
-	
+
 					 // hm, there is no block that has
 					 // an entry in this column. we need
 					 // to take the next entry then,
@@ -1383,7 +1408,7 @@ namespace BlockMatrixIterators
 	this->col_block = numbers::invalid_unsigned_int;
       }
   }
-  
+
 
 //   template <class BlockMatrix>
 //   inline
@@ -1395,7 +1420,7 @@ namespace BlockMatrixIterators
 //     this->row_block = other.row_block;
 //     this->col_block = other.col_block;
 //   }
-  
+
 
   template <class BlockMatrix>
   inline
@@ -1404,10 +1429,10 @@ namespace BlockMatrixIterators
 		  matrix(other.matrix),
 		  base_iterator(other.base_iterator)
   {
-     this->row_block = other.row_block;
-     this->col_block = other.col_block;    
+    this->row_block = other.row_block;
+    this->col_block = other.col_block;
   }
-  
+
 
   template <class BlockMatrix>
   inline
@@ -1416,12 +1441,12 @@ namespace BlockMatrixIterators
   {
     Assert (this->row_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return (matrix->row_block_indices.local_to_global(this->row_block, 0) +
 	    base_iterator->row());
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   unsigned int
@@ -1429,12 +1454,12 @@ namespace BlockMatrixIterators
   {
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return (matrix->column_block_indices.local_to_global(this->col_block,0) +
 	    base_iterator->column());
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   typename Accessor<BlockMatrix, true>::value_type
@@ -1444,29 +1469,29 @@ namespace BlockMatrixIterators
 	    ExcIteratorPastEnd());
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return base_iterator->value();
   }
-  
-  
-  
+
+
+
   template <class BlockMatrix>
   inline
   void
   Accessor<BlockMatrix, true>::advance ()
-  {      
+  {
     Assert (this->row_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
 				     // Remember current row inside block
     unsigned int local_row = base_iterator->row();
-    
+
 				     // Advance one element inside the
 				     // current block
     ++base_iterator;
-    
+
 				     // while we hit the end of the row of a
 				     // block (which may happen multiple
 				     // times if rows inside a block are
@@ -1508,19 +1533,19 @@ namespace BlockMatrixIterators
 		    return;
 		  }
 	      }
-              
+
 	    base_iterator
 	      = matrix->block(this->row_block, this->col_block).begin(local_row);
 	  }
       }
   }
-    
+
 
   template <class BlockMatrix>
   inline
   bool
   Accessor<BlockMatrix, true>::operator == (const Accessor &a) const
-  {      
+  {
     if (matrix != a.matrix)
       return false;
 
@@ -1538,10 +1563,10 @@ namespace BlockMatrixIterators
 
     return false;
   }
-    
-//----------------------------------------------------------------------//      
 
-  
+//----------------------------------------------------------------------//
+
+
   template <class BlockMatrix>
   inline
   Accessor<BlockMatrix, false>::Accessor (
@@ -1559,7 +1584,7 @@ namespace BlockMatrixIterators
       {
 	const std::pair<unsigned int,unsigned int> indices
 	  = matrix->row_block_indices.global_to_local(row);
-	
+
 					 // find the first block that does
 					 // have an entry in this row
 	for (unsigned int bc=0; bc<matrix->n_block_cols(); ++bc)
@@ -1574,7 +1599,7 @@ namespace BlockMatrixIterators
 		return;
 	      }
 	  }
-	
+
 					 // hm, there is no block that has
 					 // an entry in this column. we need
 					 // to take the next entry then,
@@ -1591,8 +1616,8 @@ namespace BlockMatrixIterators
 	this->col_block = numbers::invalid_unsigned_int;
       }
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   unsigned int
@@ -1600,12 +1625,12 @@ namespace BlockMatrixIterators
   {
     Assert (this->row_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return (matrix->row_block_indices.local_to_global(this->row_block, 0) +
 	    base_iterator->row());
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   unsigned int
@@ -1613,12 +1638,12 @@ namespace BlockMatrixIterators
   {
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return (matrix->column_block_indices.local_to_global(this->col_block,0) +
 	    base_iterator->column());
   }
-  
-  
+
+
   template <class BlockMatrix>
   inline
   typename Accessor<BlockMatrix, false>::value_type
@@ -1628,12 +1653,12 @@ namespace BlockMatrixIterators
 	    ExcIteratorPastEnd());
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     return base_iterator->value();
   }
-  
-  
-  
+
+
+
   template <class BlockMatrix>
   inline
   void
@@ -1643,29 +1668,29 @@ namespace BlockMatrixIterators
 	    ExcIteratorPastEnd());
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
     base_iterator->value() = newval;
   }
-  
-  
-  
+
+
+
   template <class BlockMatrix>
   inline
   void
   Accessor<BlockMatrix, false>::advance ()
-  {      
+  {
     Assert (this->row_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
     Assert (this->col_block != numbers::invalid_unsigned_int,
 	    ExcIteratorPastEnd());
-    
+
 				     // Remember current row inside block
     unsigned int local_row = base_iterator->row();
-    
+
 				     // Advance one element inside the
 				     // current block
     ++base_iterator;
-    
+
 				     // while we hit the end of the row of a
 				     // block (which may happen multiple
 				     // times if rows inside a block are
@@ -1707,20 +1732,20 @@ namespace BlockMatrixIterators
 		    return;
 		  }
 	      }
-              
+
 	    base_iterator
 	      = matrix->block(this->row_block, this->col_block).begin(local_row);
 	  }
       }
   }
-    
-      
+
+
 
   template <class BlockMatrix>
   inline
   bool
   Accessor<BlockMatrix, false>::operator == (const Accessor &a) const
-  {      
+  {
     if (matrix != a.matrix)
       return false;
 
@@ -1761,7 +1786,7 @@ copy_from (const BlockMatrixType &source)
   for (unsigned int r=0; r<n_block_rows(); ++r)
     for (unsigned int c=0; c<n_block_cols(); ++c)
       block(r,c).copy_from (source.block(r,c));
-  
+
   return *this;
 }
 
@@ -1770,7 +1795,7 @@ copy_from (const BlockMatrixType &source)
 template <class MatrixType>
 inline
 void
-BlockMatrixBase<MatrixType>::clear () 
+BlockMatrixBase<MatrixType>::clear ()
 {
   for (unsigned int r=0; r<n_block_rows(); ++r)
     for (unsigned int c=0; c<n_block_cols(); ++c)
@@ -1797,7 +1822,7 @@ BlockMatrixBase<MatrixType>::block (const unsigned int row,
           ExcIndexRange (row, 0, n_block_rows()));
   Assert (column<n_block_cols(),
           ExcIndexRange (column, 0, n_block_cols()));
-  
+
   return *sub_objects[row][column];
 }
 
@@ -1807,13 +1832,13 @@ template <class MatrixType>
 inline
 const typename BlockMatrixBase<MatrixType>::BlockType &
 BlockMatrixBase<MatrixType>::block (const unsigned int row,
-                                   const unsigned int column) const
+				    const unsigned int column) const
 {
   Assert (row<n_block_rows(),
           ExcIndexRange (row, 0, n_block_rows()));
   Assert (column<n_block_cols(),
           ExcIndexRange (column, 0, n_block_cols()));
-  
+
   return *sub_objects[row][column];
 }
 
@@ -1858,9 +1883,9 @@ BlockMatrixBase<MatrixType>::n_block_rows () const
 
 
 
-                                   // Write the single set manually,
-                                   // since the other function has a lot
-                                   // of overhead in that case.
+				 // Write the single set manually,
+				 // since the other function has a lot
+				 // of overhead in that case.
 template <class MatrixType>
 inline
 void
@@ -1868,8 +1893,9 @@ BlockMatrixBase<MatrixType>::set (const unsigned int i,
                                   const unsigned int j,
                                   const value_type value)
 {
+  prepare_set_operation();
 
-  Assert (numbers::is_finite(value), 
+  Assert (numbers::is_finite(value),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   const std::pair<unsigned int,unsigned int>
@@ -1940,9 +1966,9 @@ BlockMatrixBase<MatrixType>::set (const unsigned int               row,
 
 
 
-				   // This is a very messy function, since
-				   // we need to calculate to each position 
-				   // the location in the global array.
+				 // This is a very messy function, since
+				 // we need to calculate to each position
+				 // the location in the global array.
 template <class MatrixType>
 template <typename number>
 inline
@@ -1953,6 +1979,8 @@ BlockMatrixBase<MatrixType>::set (const unsigned int  row,
 				  const number       *values,
 				  const bool          elide_zero_values)
 {
+  prepare_set_operation();
+
 				   // Resize scratch arrays
   if (column_indices.size() < this->n_block_cols())
     {
@@ -2024,16 +2052,16 @@ BlockMatrixBase<MatrixType>::set (const unsigned int  row,
 				   // where we should start reading out
 				   // data. Now let's write the data into
 				   // the individual blocks!
-  const std::pair<unsigned int,unsigned int> 
+  const std::pair<unsigned int,unsigned int>
     row_index = this->row_block_indices.global_to_local (row);
   for (unsigned int block_col=0; block_col<n_block_cols(); ++block_col)
     {
       if (counter_within_block[block_col] == 0)
 	continue;
 
-      block(row_index.first, block_col).set 
-	(row_index.second, 
-	 counter_within_block[block_col], 
+      block(row_index.first, block_col).set
+	(row_index.second,
+	 counter_within_block[block_col],
 	 &column_indices[block_col][0],
 	 &column_values[block_col][0],
 	 false);
@@ -2050,18 +2078,20 @@ BlockMatrixBase<MatrixType>::add (const unsigned int i,
                                   const value_type value)
 {
 
-  Assert (numbers::is_finite(value), 
+  Assert (numbers::is_finite(value),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
+  prepare_add_operation();
+
                                    // save some cycles for zero additions, but
                                    // only if it is safe for the matrix we are
                                    // working with
   typedef typename MatrixType::Traits MatrixTraits;
   if ((MatrixTraits::zero_addition_can_be_elided == true)
       &&
-      (value == 0))
+      (value == value_type()))
     return;
-  
+
   const std::pair<unsigned int,unsigned int>
     row_index = row_block_indices.global_to_local (i),
     col_index = column_block_indices.global_to_local (j);
@@ -2130,9 +2160,9 @@ BlockMatrixBase<MatrixType>::add (const unsigned int               row,
 
 
 
-				   // This is a very messy function, since
-				   // we need to calculate to each position 
-				   // the location in the global array.
+				 // This is a very messy function, since
+				 // we need to calculate to each position
+				 // the location in the global array.
 template <class MatrixType>
 template <typename number>
 inline
@@ -2144,23 +2174,25 @@ BlockMatrixBase<MatrixType>::add (const unsigned int   row,
 				  const bool           elide_zero_values,
 				  const bool           col_indices_are_sorted)
 {
+  prepare_add_operation();
+
 				   // TODO: Look over this to find out
 				   // whether we can do that more
 				   // efficiently.
   if (col_indices_are_sorted == true)
     {
 #ifdef DEBUG
-				   // check whether indices really are
-				   // sorted.
+				       // check whether indices really are
+				       // sorted.
       unsigned int before = col_indices[0];
       for (unsigned int i=1; i<n_cols; ++i)
 	if (col_indices[i] <= before)
 	  Assert (false, ExcMessage ("Flag col_indices_are_sorted is set, but "
 				     "indices appear to not be sorted."))
-	else
-	  before = col_indices[i];
+	  else
+	    before = col_indices[i];
 #endif
-      const std::pair<unsigned int,unsigned int> 
+      const std::pair<unsigned int,unsigned int>
 	row_index = this->row_block_indices.global_to_local (row);
 
       if (this->n_block_cols() > 1)
@@ -2170,8 +2202,8 @@ BlockMatrixBase<MatrixType>::add (const unsigned int   row,
 							       this->column_block_indices.block_start(1));
 
 	  const unsigned int n_zero_block_indices = first_block - col_indices;
-	  block(row_index.first, 0).add (row_index.second, 
-					 n_zero_block_indices, 
+	  block(row_index.first, 0).add (row_index.second,
+					 n_zero_block_indices,
 					 col_indices,
 					 values,
 					 elide_zero_values,
@@ -2184,14 +2216,14 @@ BlockMatrixBase<MatrixType>::add (const unsigned int   row,
 	}
       else
 	{
-	  block(row_index.first, 0). add (row_index.second, 
-					  n_cols, 
+	  block(row_index.first, 0). add (row_index.second,
+					  n_cols,
 					  col_indices,
 					  values,
 					  elide_zero_values,
 					  col_indices_are_sorted);
 	}
-						     
+
       return;
     }
 
@@ -2266,16 +2298,16 @@ BlockMatrixBase<MatrixType>::add (const unsigned int   row,
 				   // where we should start reading out
 				   // data. Now let's write the data into
 				   // the individual blocks!
-  const std::pair<unsigned int,unsigned int> 
+  const std::pair<unsigned int,unsigned int>
     row_index = this->row_block_indices.global_to_local (row);
   for (unsigned int block_col=0; block_col<n_block_cols(); ++block_col)
     {
       if (counter_within_block[block_col] == 0)
 	continue;
 
-      block(row_index.first, block_col).add 
-	(row_index.second, 
-	 counter_within_block[block_col], 
+      block(row_index.first, block_col).add
+	(row_index.second,
+	 counter_within_block[block_col],
 	 &column_indices[block_col][0],
 	 &column_values[block_col][0],
 	 false,
@@ -2380,7 +2412,7 @@ BlockMatrixBase<MatrixType>::operator /= (const value_type factor)
 
 
 template <class MatrixType>
-const BlockIndices & 
+const BlockIndices &
 BlockMatrixBase<MatrixType>::get_row_indices () const
 {
   return this->row_block_indices;
@@ -2389,7 +2421,7 @@ BlockMatrixBase<MatrixType>::get_row_indices () const
 
 
 template <class MatrixType>
-const BlockIndices & 
+const BlockIndices &
 BlockMatrixBase<MatrixType>::get_column_indices () const
 {
   return this->column_block_indices;
@@ -2515,7 +2547,7 @@ Tvmult_block_block (BlockVectorType       &dst,
 	  ExcDimensionMismatch(src.n_blocks(), n_block_rows()));
 
   dst = 0.;
-  
+
   for (unsigned int row=0; row<n_block_rows(); ++row)
     {
       for (unsigned int col=0; col<n_block_cols(); ++col)
@@ -2540,7 +2572,7 @@ Tvmult_block_nonblock (BlockVectorType  &dst,
 	  ExcDimensionMismatch(1, n_block_rows()));
 
   dst = 0.;
-  
+
   for (unsigned int col=0; col<n_block_cols(); ++col)
     block(0,col).Tvmult_add (dst.block(col), src);
 }
@@ -2561,7 +2593,7 @@ Tvmult_nonblock_block (VectorType    &dst,
 	  ExcDimensionMismatch(src.n_blocks(), n_block_rows()));
 
   block(0,0).Tvmult (dst, src.block(0));
-  
+
   for (unsigned int row=1; row<n_block_rows(); ++row)
     block(row,0).Tvmult_add (dst, src.block(row));
 }
@@ -2683,10 +2715,10 @@ residual (BlockVectorType          &dst,
       block(row,0).residual (dst.block(row),
 			     x.block(0),
 			     b.block(row));
-      
+
       for (unsigned int i=0; i<dst.block(row).size(); ++i)
 	dst.block(row)(i) = -dst.block(row)(i);
-      
+
       for (unsigned int col=1; col<n_block_cols(); ++col)
 	block(row,col).vmult_add (dst.block(row),
 				  x.block(col));
@@ -2809,8 +2841,8 @@ BlockMatrixBase<MatrixType>::collect_sizes ()
                                    // finally initialize the row
                                    // indices with this array
   this->row_block_indices.reinit (row_sizes);
-  
-  
+
+
                                    // then do the same with the columns
   for (unsigned int c=0; c<this->n_block_cols(); ++c)
     col_sizes[c] = sub_objects[0][c]->n();
@@ -2824,6 +2856,28 @@ BlockMatrixBase<MatrixType>::collect_sizes ()
   this->column_block_indices.reinit (col_sizes);
 }
 
+
+
+template <class MatrixType>
+void
+BlockMatrixBase<MatrixType>::prepare_add_operation ()
+{
+  for (unsigned int row=0; row<n_block_rows(); ++row)
+    for (unsigned int col=0; col<n_block_cols(); ++col)
+      block(row, col).prepare_add();
+}
+
+
+
+template <class MatrixType>
+void
+BlockMatrixBase<MatrixType>::prepare_set_operation ()
+{
+  for (unsigned int row=0; row<n_block_rows(); ++row)
+    for (unsigned int col=0; col<n_block_cols(); ++col)
+      block(row, col).prepare_set();
+}
+
 #endif // DOXYGEN
 
 
