@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: block_sparsity_pattern.h 18745 2009-04-26 22:08:40Z bangerth $
+//    $Id: block_sparsity_pattern.h 21358 2010-06-24 23:38:14Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -73,6 +73,7 @@ namespace TrilinosWrappers
  * You will in general not want to use this class, but one of the
  * derived classes.
  *
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, 2000, 2001
  */
 template <class SparsityPatternBase>
@@ -92,7 +93,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * SparsityPattern class.
 				      */
     static const unsigned int invalid_entry = SparsityPattern::invalid_entry;
-    
+
 				     /**
 				      * Initialize the matrix empty,
 				      * that is with no memory
@@ -115,7 +116,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      */
     BlockSparsityPatternBase (const unsigned int n_block_rows,
 			      const unsigned int n_block_columns);
-    
+
 				     /**
 				      * Copy constructor. This
 				      * constructor is only allowed to
@@ -133,7 +134,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * Destructor.
 				      */
     ~BlockSparsityPatternBase ();
-    
+
 				     /**
 				      * Resize the matrix, by setting
 				      * the number of block rows and
@@ -186,7 +187,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * the sub-objects.
 				      */
     void collect_sizes ();
-    
+
 				     /**
 				      * Access the block with the
 				      * given coordinates.
@@ -194,8 +195,8 @@ class BlockSparsityPatternBase : public Subscriptor
     SparsityPatternBase &
     block (const unsigned int row,
 	   const unsigned int column);
-    
-    
+
+
 				     /**
 				      * Access the block with the
 				      * given coordinates. Version for
@@ -203,7 +204,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      */
     const SparsityPatternBase &
     block (const unsigned int row,
-	   const unsigned int column) const;    
+	   const unsigned int column) const;
 
 				     /**
 				      * Grant access to the object
@@ -222,7 +223,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      */
     const BlockIndices &
     get_column_indices () const;
-    
+
 				     /**
 				      * This function compresses the
 				      * sparsity structures that this
@@ -237,13 +238,13 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * column.
 				      */
     unsigned int n_block_rows () const;
-    
+
 				     /**
 				      * Return the number of blocks in a
 				      * row.
 				      */
     unsigned int n_block_cols () const;
-  
+
 				     /**
 				      * Return whether the object is
 				      * empty. It is empty if no
@@ -297,7 +298,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * blocks.
 				      */
     template <typename ForwardIterator>
-    void add_entries (const unsigned int row, 
+    void add_entries (const unsigned int row,
 		      ForwardIterator    begin,
 		      ForwardIterator    end,
 		      const bool         indices_are_sorted = false);
@@ -335,7 +336,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * row.
 				      */
     unsigned int row_length (const unsigned int row) const;
-    
+
 				     /**
 				      * Return the number of nonzero
 				      * elements of this
@@ -408,7 +409,7 @@ class BlockSparsityPatternBase : public Subscriptor
 				      */
     DeclException0 (ExcInvalidConstructorCall);
 				     //@}
-    
+
   protected:
 
 				     /**
@@ -420,11 +421,11 @@ class BlockSparsityPatternBase : public Subscriptor
 				      * Number of block columns.
 				      */
     unsigned int columns;
-    
+
 				     /**
 				      * Array of sparsity patterns.
 				      */
-    Table<2,SmartPointer<SparsityPatternBase> > sub_objects;
+    Table<2,SmartPointer<SparsityPatternBase, BlockSparsityPatternBase<SparsityPatternBase> > > sub_objects;
 
 				     /**
 				      * Object storing and managing
@@ -449,7 +450,7 @@ class BlockSparsityPatternBase : public Subscriptor
 					* individual blocks when doing a
 					* collective add or set.
 					*/
-    std::vector<unsigned int> counter_within_block; 
+    std::vector<unsigned int> counter_within_block;
 
                                        /**
 					* Temporary vector for column
@@ -457,7 +458,7 @@ class BlockSparsityPatternBase : public Subscriptor
 					* local to global data on each
 					* sparse matrix.
 					*/
-    std::vector<std::vector<unsigned int> > block_column_indices; 
+    std::vector<std::vector<unsigned int> > block_column_indices;
 
 				     /**
 				      * Make the block sparse matrix a
@@ -484,7 +485,7 @@ class BlockSparsityPatternBase : public Subscriptor
 class BlockSparsityPattern : public BlockSparsityPatternBase<SparsityPattern>
 {
   public:
-    
+
 				     /**
 				      * Initialize the matrix empty,
 				      * that is with no memory
@@ -514,7 +515,7 @@ class BlockSparsityPattern : public BlockSparsityPatternBase<SparsityPattern>
 				      */
     void reinit (const unsigned int n_block_rows,
 		 const unsigned int n_block_columns);
-    
+
     				     /**
 				      * Initialize the pattern with
 				      * two BlockIndices for the block
@@ -526,7 +527,7 @@ class BlockSparsityPattern : public BlockSparsityPatternBase<SparsityPattern>
     void reinit (const BlockIndices& row_indices,
 		 const BlockIndices& col_indices,
 		 const std::vector<std::vector<unsigned int> >& row_lengths);
-    
+
 
 				     /**
 				      * Return whether the structure
@@ -534,7 +535,7 @@ class BlockSparsityPattern : public BlockSparsityPatternBase<SparsityPattern>
 				      * i.e. whether all sub-matrices
 				      * are compressed.
 				      */
-    bool is_compressed () const;    
+    bool is_compressed () const;
 
 				     /**
 				      * Determine an estimate for the
@@ -641,7 +642,7 @@ class BlockSparsityPattern : public BlockSparsityPatternBase<SparsityPattern>
 class BlockCompressedSparsityPattern : public BlockSparsityPatternBase<CompressedSparsityPattern>
 {
   public:
-    
+
 				     /**
 				      * Initialize the matrix empty,
 				      * that is with no memory
@@ -678,7 +679,16 @@ class BlockCompressedSparsityPattern : public BlockSparsityPatternBase<Compresse
 				      */
     BlockCompressedSparsityPattern (const std::vector<unsigned int>& row_block_sizes,
 				    const std::vector<unsigned int>& col_block_sizes);
-    
+
+    				     /**
+				      * Initialize the pattern with
+				      * two BlockIndices for the block
+				      * structures of matrix rows and
+				      * columns.
+				      */
+    BlockCompressedSparsityPattern (const BlockIndices& row_indices,
+				    const BlockIndices& col_indices);
+
 				     /**
 				      * Resize the matrix to a tensor
 				      * product of matrices with
@@ -697,6 +707,18 @@ class BlockCompressedSparsityPattern : public BlockSparsityPatternBase<Compresse
     void reinit (const std::vector< unsigned int > &row_block_sizes,
 		 const std::vector< unsigned int > &col_block_sizes);
 
+				     /**
+				      * Resize the matrix to a tensor
+				      * product of matrices with
+				      * dimensions defined by the
+				      * arguments. The two
+				      * BlockIndices objects must be
+				      * initialized and the sparsity
+				      * pattern will have the
+				      * same block structure afterwards.
+				      */
+    void reinit (const BlockIndices& row_indices, const BlockIndices& col_indices);
+
 				     /**
 				      * Allow the use of the reinit
 				      * functions of the base class as
@@ -737,7 +759,7 @@ typedef BlockCompressedSparsityPattern CompressedBlockSparsityPattern;
 class BlockCompressedSetSparsityPattern : public BlockSparsityPatternBase<CompressedSetSparsityPattern>
 {
   public:
-    
+
 				     /**
 				      * Initialize the matrix empty,
 				      * that is with no memory
@@ -759,7 +781,7 @@ class BlockCompressedSetSparsityPattern : public BlockSparsityPatternBase<Compre
 				      * you assign them sizes.
 				      */
     BlockCompressedSetSparsityPattern (const unsigned int n_rows,
-				    const unsigned int n_columns);
+				       const unsigned int n_columns);
 
     				     /**
 				      * Initialize the pattern with
@@ -773,8 +795,17 @@ class BlockCompressedSetSparsityPattern : public BlockSparsityPatternBase<Compre
 				      * values.
 				      */
     BlockCompressedSetSparsityPattern (const std::vector<unsigned int>& row_block_sizes,
-				    const std::vector<unsigned int>& col_block_sizes);
-    
+				       const std::vector<unsigned int>& col_block_sizes);
+
+    				     /**
+				      * Initialize the pattern with
+				      * two BlockIndices for the block
+				      * structures of matrix rows and
+				      * columns.
+				      */
+    BlockCompressedSetSparsityPattern (const BlockIndices& row_indices,
+				       const BlockIndices& col_indices);
+
 				     /**
 				      * Resize the matrix to a tensor
 				      * product of matrices with
@@ -793,6 +824,18 @@ class BlockCompressedSetSparsityPattern : public BlockSparsityPatternBase<Compre
     void reinit (const std::vector< unsigned int > &row_block_sizes,
 		 const std::vector< unsigned int > &col_block_sizes);
 
+				     /**
+				      * Resize the matrix to a tensor
+				      * product of matrices with
+				      * dimensions defined by the
+				      * arguments. The two
+				      * BlockIndices objects must be
+				      * initialized and the sparsity
+				      * pattern will have the
+				      * same block structure afterwards.
+				      */
+    void reinit (const BlockIndices& row_indices, const BlockIndices& col_indices);
+
 				     /**
 				      * Allow the use of the reinit
 				      * functions of the base class as
@@ -819,7 +862,7 @@ class BlockCompressedSetSparsityPattern : public BlockSparsityPatternBase<Compre
  * There are several, exchangeable variations of this class, see @ref Sparsity,
  * section 'Dynamic block sparsity patterns' for more information.
  *
- * This class is used in @ref step_22 "step-22" and @ref step_31 "step-31".
+ * This class is used in step-22 and step-31.
  *
  * @author Timo Heister, 2008
  */
@@ -911,13 +954,11 @@ class BlockCompressedSimpleSparsityPattern : public BlockSparsityPatternBase<Com
  * (in the sense that the method compress() needs to be called before the
  * pattern can be used).
  *
- * This class is used in the future step-32 tutorial program.
- *
- * @author Martin Kronbichler, 2008
+ * @author Martin Kronbichler, 2008, 2009
  */
 namespace TrilinosWrappers
 {
-  class BlockSparsityPattern : 
+  class BlockSparsityPattern :
     public dealii::BlockSparsityPatternBase<SparsityPattern>
   {
     public:
@@ -973,7 +1014,24 @@ namespace TrilinosWrappers
 				      * mapping vector and then entering the
 				      * index values.
 				      */
-      BlockSparsityPattern (const std::vector<Epetra_Map>& input_maps);
+      BlockSparsityPattern (const std::vector<Epetra_Map>& parallel_partitioning);
+
+    				     /**
+				      * Initialize the pattern with an array
+				      * of index sets that specifies both
+				      * rows and columns of the matrix (so
+				      * the final matrix will be a square
+				      * matrix), where the IndexSet
+				      * specifies the parallel distribution
+				      * of the degrees of freedom on the
+				      * individual block.  This function is
+				      * equivalent to calling the second
+				      * constructor with the length of the
+				      * mapping vector and then entering the
+				      * index values.
+				      */
+      BlockSparsityPattern (const std::vector<IndexSet>& parallel_partitioning,
+			    const MPI_Comm &communicator = MPI_COMM_WORLD);
 
 				     /**
 				      * Resize the matrix to a tensor
@@ -1000,7 +1058,17 @@ namespace TrilinosWrappers
 				      * specifications in the array of
 				      * Epetra_Maps.
 				      */
-      void reinit (const std::vector<Epetra_Map>& input_maps);
+      void reinit (const std::vector<Epetra_Map>& parallel_partitioning);
+
+				     /**
+				      * Resize the matrix to a square tensor
+				      * product of matrices with parallel
+				      * distribution according to the
+				      * specifications in the array of
+				      * Epetra_Maps.
+				      */
+      void reinit (const std::vector<IndexSet>& parallel_partitioning,
+		   const MPI_Comm             & communicator = MPI_COMM_WORLD);
 
 
 				     /**
@@ -1155,7 +1223,7 @@ BlockSparsityPatternBase<SparsityPatternBase>::add_entries (const unsigned int r
 				   // where we should start reading out
 				   // data. Now let's write the data into
 				   // the individual blocks!
-  const std::pair<unsigned int,unsigned int> 
+  const std::pair<unsigned int,unsigned int>
     row_index = this->row_indices.global_to_local (row);
   for (unsigned int block_col=0; block_col<n_block_cols(); ++block_col)
     {
@@ -1199,7 +1267,7 @@ row_length (const unsigned int row) const
     row_index = row_indices.global_to_local (row);
 
   unsigned int c = 0;
-  
+
   for (unsigned int b=0; b<rows; ++b)
     c += sub_objects[row_index.first][b]->row_length (row_index.second);
 
