@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: block_vector_base.h 15688 2008-01-25 20:38:43Z prill $
+//    $Id: block_vector_base.h 21162 2010-06-07 20:47:13Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -20,9 +20,11 @@
 #include <base/memory_consumption.h>
 #include <lac/exceptions.h>
 #include <lac/block_indices.h>
+#include <lac/vector.h>
 #include <vector>
 #include <iterator>
 #include <cmath>
+#include <cstddef>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -31,6 +33,69 @@ DEAL_II_NAMESPACE_OPEN
  *@{
  */
 
+template <typename> class BlockVectorBase;
+
+
+/**
+ * A class that can be used to determine whether a given type is a block
+ * vector type or not. For example,
+ * @code
+ *   IsBlockVector<Vector<double> >::value
+ * @endcode
+ * has the value false, whereas
+ * @code
+ *   IsBlockVector<BlockVector<double> >::value
+ * @endcode
+ * is true. This is sometimes useful in template contexts where we may
+ * want to do things differently depending on whether a template type
+ * denotes a regular or a block vector type.
+ *
+ * @author Wolfgang Bangerth, 2010
+ */
+template <typename VectorType>
+struct IsBlockVector
+{
+  private:
+    struct yes_type { char c[1]; };
+    struct no_type  { char c[2]; };
+
+				     /**
+				      * Overload returning true if the class
+				      * is derived from BlockVectorBase,
+				      * which is what block vectors do.
+				      */
+    template <typename T>
+    static yes_type check_for_block_vector (const BlockVectorBase<T> *);
+
+				     /**
+				      * Catch all for all other potential
+				      * vector types that are not block
+				      * matrices.
+				      */
+    static no_type check_for_block_vector (...);
+
+  public:
+				     /**
+				      * A statically computable value that
+				      * indicates whether the template
+				      * argument to this class is a block
+				      * vector (in fact whether the type is
+				      * derived from BlockVectorBase<T>).
+				      */
+    static const bool value = (sizeof(check_for_block_vector
+				      ((VectorType*)0))
+			       ==
+			       sizeof(yes_type));
+};
+
+
+// instantiation of the static member
+template <typename VectorType>
+const bool IsBlockVector<VectorType>::value;
+
+
+
+
 namespace internal
 {
 
@@ -54,7 +119,7 @@ namespace internal
     };
 
 
-  
+
                                      /**
                                       * Declaration of a specialized
                                       * template of a structure which is
@@ -102,7 +167,7 @@ namespace internal
     };
 
 
-  
+
                                      /**
                                       * Declaration of a specialized
                                       * template of a structure which is
@@ -216,12 +281,12 @@ namespace internal
                                       */
     template <class BlockVectorType, bool constness>
     class Iterator :
-#ifdef HAVE_STD_ITERATOR_CLASS  
+#ifdef HAVE_STD_ITERATOR_CLASS
         public std::iterator<std::random_access_iterator_tag,
                              typename Types<BlockVectorType,constness>::value_type>
 #else
     random_access_iterator<typename Types<BlockVectorType,constness>::value_type,int>
-#endif      
+#endif
     {
       private:
                                          /**
@@ -245,7 +310,7 @@ namespace internal
         typedef
         typename Types<BlockVectorType,constness>::value_type
         value_type;
-	
+
                                          /**
                                           * Declare some typedefs which
                                           * are standard for iterators
@@ -255,14 +320,14 @@ namespace internal
                                           * they work on.
                                           */
         typedef std::random_access_iterator_tag               iterator_type;
-        typedef ptrdiff_t                                     difference_type;
+        typedef std::ptrdiff_t                                difference_type;
         typedef typename BlockVectorType::reference           reference;
         typedef value_type                                   *pointer;
 
         typedef
         typename Types<BlockVectorType,constness>::dereference_type
         dereference_type;
-        
+
                                          /**
                                           * Typedef the type of the
                                           * block vector (which differs
@@ -290,7 +355,7 @@ namespace internal
                                           */
         Iterator (BlockVector    &parent,
                   const unsigned int  global_index);
-	
+
                                          /**
                                           * Copy constructor.
                                           */
@@ -322,9 +387,9 @@ namespace internal
                   const unsigned int  index_within_block,
                   const unsigned int  next_break_forward,
                   const unsigned int  next_break_backward);
-      
+
       public:
-      
+
                                          /**
                                           * Copy operator.
                                           */
@@ -347,7 +412,7 @@ namespace internal
                                           * presently pointed to.
                                           */
         dereference_type operator [] (const difference_type d) const;
-      
+
                                          /**
                                           * Prefix increment operator. This
                                           * operator advances the iterator to
@@ -393,7 +458,7 @@ namespace internal
                                           * exception.
                                           */
         bool operator == (const Iterator &i) const;
-      
+
                                          /**
                                           * Same, but compare with an
                                           * iterator of different
@@ -416,7 +481,7 @@ namespace internal
                                           * iterator of different
                                           * constness.
                                           */
-        bool operator != (const InverseConstnessIterator &i) const;      
+        bool operator != (const InverseConstnessIterator &i) const;
 
                                          /**
                                           * Check whether this
@@ -436,7 +501,7 @@ namespace internal
                                           * iterator of different
                                           * constness.
                                           */
-        bool operator < (const InverseConstnessIterator &i) const;      
+        bool operator < (const InverseConstnessIterator &i) const;
 
                                          /**
                                           * Comparison operator alike
@@ -449,7 +514,7 @@ namespace internal
                                           * iterator of different
                                           * constness.
                                           */
-        bool operator <= (const InverseConstnessIterator &i) const;      
+        bool operator <= (const InverseConstnessIterator &i) const;
 
                                          /**
                                           * Comparison operator alike
@@ -462,7 +527,7 @@ namespace internal
                                           * iterator of different
                                           * constness.
                                           */
-        bool operator > (const InverseConstnessIterator &i) const;      
+        bool operator > (const InverseConstnessIterator &i) const;
 
                                          /**
                                           * Comparison operator alike
@@ -496,7 +561,7 @@ namespace internal
                                           * in front of the present one.
                                           */
         Iterator operator + (const difference_type &d) const;
-      
+
                                          /**
                                           * Return an iterator which is
                                           * the given number of elements
@@ -517,10 +582,10 @@ namespace internal
                                           * and return the result.
                                           */
         Iterator & operator -= (const difference_type &d);
-      
+
 					 /** @addtogroup Exceptions
 					  * @{ */
-	
+
                                          /**
                                           * Exception.
                                           */
@@ -529,7 +594,7 @@ namespace internal
                                           * Exception.
                                           */
         DeclException0 (ExcCastingAwayConstness);
-					 //@}	
+					 //@}
       private:
                                          /**
                                           * Pointer to the block
@@ -584,7 +649,7 @@ namespace internal
                                           */
         void move_backward ();
 
-      
+
 #ifndef DEAL_II_NAMESP_TEMPL_FRIEND_BUG
                                          /**
                                           * Mark all other instances of
@@ -633,7 +698,7 @@ namespace internal
  * operators and reinit() functions of derived classes are responsible. This
  * class only handles the common part that is independent of the actual vector
  * type the block vector is built on.
- * 
+ *
  *
  * <h3>Accessing individual blocks, and resizing vectors</h3>
  *
@@ -648,6 +713,7 @@ namespace internal
  * debug version does not check consistency here for performance
  * reasons!
  *
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, Guido Kanschat, 1999, 2000, 2001, 2002, 2004
  */
 template <class VectorType>
@@ -659,7 +725,7 @@ class BlockVectorBase : public Subscriptor
                                       * vector.
                                       */
     typedef VectorType BlockType;
-    
+
 				     /*
 				      * Declare standard types used in
 				      * all containers. These types
@@ -700,7 +766,7 @@ class BlockVectorBase : public Subscriptor
 				      * norms.
 				      */
     typedef typename BlockType::real_type real_type;
-    
+
                                      /**
                                       * Default constructor.
                                       */
@@ -718,7 +784,7 @@ class BlockVectorBase : public Subscriptor
 				      * size.
 				      */
     void collect_sizes ();
-    
+
                                      /**
                                       * Call the compress() function on all
                                       * the subblocks of the matrix.
@@ -730,7 +796,7 @@ class BlockVectorBase : public Subscriptor
 				      */
     BlockType &
     block (const unsigned int i);
-    
+
 				     /**
 				      * Read-only access to a single block.
 				      */
@@ -748,13 +814,13 @@ class BlockVectorBase : public Subscriptor
 				      * versions
 				      */
     const BlockIndices &
-    get_block_indices () const;    
+    get_block_indices () const;
 
 				     /**
 				      * Number of blocks.
 				      */
     unsigned int n_blocks () const;
-  
+
   				     /**
   				      * Return dimension of the vector. This
   				      * is the sum of the dimensions of all
@@ -774,7 +840,7 @@ class BlockVectorBase : public Subscriptor
 				      * constant block vector.
 				      */
     const_iterator begin () const;
-    
+
 				     /**
 				      * Return an iterator pointing to
 				      * the element past the end.
@@ -786,13 +852,13 @@ class BlockVectorBase : public Subscriptor
 				      * the element past the end of a
 				      * constant block vector.
 				      */
-    const_iterator end () const;    
+    const_iterator end () const;
 
 				     /**
 				      * Access components, returns U(i).
 				      */
     value_type operator() (const unsigned int i) const;
-    
+
 				     /**
 				      * Access components, returns U(i)
 				      * as a writeable reference.
@@ -827,7 +893,7 @@ class BlockVectorBase : public Subscriptor
                                       */
     BlockVectorBase &
     operator = (const VectorType &v);
-    
+
                                      /**
                                       * Check for equality of two block vector
                                       * types. This operation is only allowed
@@ -837,7 +903,7 @@ class BlockVectorBase : public Subscriptor
     template <class VectorType2>
     bool
     operator == (const BlockVectorBase<VectorType2> &v) const;
-    
+
     				     /**
 				      * $U = U * V$: scalar product.
 				      */
@@ -883,7 +949,7 @@ class BlockVectorBase : public Subscriptor
 				      * some time.
 				      */
     bool all_zero () const;
-    
+
                                      /**
                                       * Return @p true if the vector has no
                                       * negative entries, i.e. all entries are
@@ -908,6 +974,44 @@ class BlockVectorBase : public Subscriptor
     BlockVectorBase &
     operator -= (const BlockVectorBase &V);
 
+
+				       /**
+                                        * A collective add operation:
+                                        * This funnction adds a whole
+                                        * set of values stored in @p
+                                        * values to the vector
+                                        * components specified by @p
+                                        * indices.
+                                        */
+    template <typename Number>
+    void add (const std::vector<unsigned int> &indices,
+	      const std::vector<Number>       &values);
+
+				       /**
+				        * This is a second collective
+				        * add operation. As a
+				        * difference, this function
+				        * takes a deal.II vector of
+				        * values.
+				        */
+    template <typename Number>
+    void add (const std::vector<unsigned int> &indices,
+	      const Vector<Number>            &values);
+
+				      /**
+				       * Take an address where
+				       * <tt>n_elements</tt> are stored
+				       * contiguously and add them into
+				       * the vector. Handles all cases
+				       * which are not covered by the
+				       * other two <tt>add()</tt>
+				       * functions above.
+				       */
+    template <typename Number>
+    void add (const unsigned int  n_elements,
+	      const unsigned int *indices,
+	      const Number       *values);
+
 				     /**
 				      * $U(0-DIM)+=s$.  Addition of <tt>s</tt>
 				      * to all components. Note that
@@ -915,39 +1019,39 @@ class BlockVectorBase : public Subscriptor
 				      * vector.
 				      */
     void add (const value_type s);
-    
+
 				     /**
 				      * U+=V.
 				      * Simple vector addition, equal to the
 				      * <tt>operator +=</tt>.
 				      */
     void add (const BlockVectorBase& V);
-    
+
 				     /**
 				      * U+=a*V.
 				      * Simple addition of a scaled vector.
 				      */
     void add (const value_type a, const BlockVectorBase& V);
-    
+
 				     /**
 				      * U+=a*V+b*W.
 				      * Multiple addition of scaled vectors.
 				      */
     void add (const value_type a, const BlockVectorBase& V,
 	      const value_type b, const BlockVectorBase& W);
-    
+
 				     /**
 				      * U=s*U+V.
 				      * Scaling and simple vector addition.
 				      */
     void sadd (const value_type s, const BlockVectorBase& V);
-    
+
 				     /**
 				      * U=s*U+a*V.
 				      * Scaling and simple addition.
 				      */
     void sadd (const value_type s, const value_type a, const BlockVectorBase& V);
-    
+
 				     /**
 				      * U=s*U+a*V+b*W.
 				      * Scaling and multiple addition.
@@ -955,16 +1059,16 @@ class BlockVectorBase : public Subscriptor
     void sadd (const value_type s, const value_type a,
 	       const BlockVectorBase& V,
 	       const value_type b, const BlockVectorBase& W);
-    
+
 				     /**
 				      * U=s*U+a*V+b*W+c*X.
 				      * Scaling and multiple addition.
 				      */
     void sadd (const value_type s, const value_type a,
 	       const BlockVectorBase& V,
-	       const value_type b, const BlockVectorBase& W, 
+	       const value_type b, const BlockVectorBase& W,
 	       const value_type c, const BlockVectorBase& X);
-    
+
 				     /**
 				      * Scale each element of the
 				      * vector by a constant
@@ -978,7 +1082,7 @@ class BlockVectorBase : public Subscriptor
 				      * given value.
 				      */
     BlockVectorBase & operator /= (const value_type factor);
-    
+
 				     /**
 				      * Multiply each element of this
 				      * vector by the corresponding
@@ -986,13 +1090,13 @@ class BlockVectorBase : public Subscriptor
 				      */
     template <class BlockVector2>
     void scale (const BlockVector2 &v);
-    
+
 				     /**
 				      *  U=a*V. Assignment.
 				      */
     template <class BlockVector2>
     void equ (const value_type a, const BlockVector2 &V);
-    
+
 				     /**
 				      * U=a*V+b*W.
 				      * Replacing by sum.
@@ -1006,7 +1110,7 @@ class BlockVectorBase : public Subscriptor
 				      * of this object.
 				      */
     unsigned int memory_consumption () const;
-      
+
   protected:
 				     /**
 				      * Pointer to the array of components.
@@ -1033,7 +1137,7 @@ class BlockVectorBase : public Subscriptor
     friend class iterator;
     friend class const_iterator;
 #endif
-    
+
     template <typename> friend class BlockVectorBase;
 };
 
@@ -1083,7 +1187,7 @@ namespace internal
                                        // constness. disallow this
       Assert (constness==true, ExcCastingAwayConstness());
     }
-  
+
 
 
     template <class BlockVectorType, bool constness>
@@ -1104,7 +1208,7 @@ namespace internal
                     next_break_backward (next_break_backward)
     {
     }
-  
+
 
 
     template <class BlockVectorType, bool constness>
@@ -1148,7 +1252,7 @@ namespace internal
       if ((global_index+d >= next_break_backward) &&
           (global_index+d <= next_break_forward))
         return parent->block(current_block)(index_within_block + d);
-    
+
                                        // if the index is not within the
                                        // block of the block vector into
                                        // which we presently point, then
@@ -1230,7 +1334,7 @@ namespace internal
 
       return (global_index == i.global_index);
     }
-  
+
 
 
     template <class BlockVectorType, bool constness>
@@ -1256,7 +1360,7 @@ namespace internal
 
       return (global_index != i.global_index);
     }
-  
+
 
 
     template <class BlockVectorType, bool constness>
@@ -1347,7 +1451,7 @@ namespace internal
 
       return (global_index >= i.global_index);
     }
-  
+
 
 
     template <class BlockVectorType, bool constness>
@@ -1516,7 +1620,7 @@ namespace internal
             indices = parent.block_indices.global_to_local(global_index);
           current_block      = indices.first;
           index_within_block = indices.second;
-	
+
           next_break_backward
             = parent.block_indices.local_to_global (current_block, 0);
           next_break_forward
@@ -1535,7 +1639,7 @@ namespace internal
         };
     }
 
-  
+
 
     template <class BlockVectorType, bool constness>
     void
@@ -1565,7 +1669,7 @@ namespace internal
                                              // away
             next_break_forward = numbers::invalid_unsigned_int;
         };
-  
+
       ++global_index;
     }
 
@@ -1584,11 +1688,11 @@ namespace internal
                                              // between blocks:
             --current_block;
             index_within_block = parent->block_indices.block_size(current_block)-1;
-	
+
                                              // break forwards is now old
                                              // break backward
             next_break_forward = next_break_backward-1;
-	
+
                                              // compute new break forward
             next_break_backward
               -= parent->block_indices.block_size (current_block);
@@ -1602,11 +1706,11 @@ namespace internal
             next_break_forward = 0;
             next_break_backward = 0;
           };
-  
+
       --global_index;
     }
 
-    
+
   } // namespace BlockVectorIterators
 
 } //namespace internal
@@ -1769,11 +1873,11 @@ operator * (const BlockVectorBase<VectorType>& v) const
 {
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   value_type sum = 0.;
   for (unsigned int i=0;i<n_blocks();++i)
     sum += components[i]*v.components[i];
-    
+
   return sum;
 }
 
@@ -1785,7 +1889,7 @@ BlockVectorBase<VectorType>::norm_sqr () const
   real_type sum = 0.;
   for (unsigned int i=0;i<n_blocks();++i)
     sum += components[i].norm_sqr();
-    
+
   return sum;
 }
 
@@ -1798,7 +1902,7 @@ BlockVectorBase<VectorType>::mean_value () const
   value_type sum = 0.;
   for (unsigned int i=0;i<n_blocks();++i)
     sum += components[i].mean_value() * components[i].size();
-  
+
   return sum/size();
 }
 
@@ -1811,7 +1915,7 @@ BlockVectorBase<VectorType>::l1_norm () const
   real_type sum = 0.;
   for (unsigned int i=0;i<n_blocks();++i)
     sum += components[i].l1_norm();
-    
+
   return sum;
 }
 
@@ -1858,7 +1962,7 @@ BlockVectorBase<VectorType>::operator -= (const BlockVectorBase<VectorType>& v)
 {
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i] -= v.components[i];
@@ -1868,11 +1972,54 @@ BlockVectorBase<VectorType>::operator -= (const BlockVectorBase<VectorType>& v)
 
 
 
+template <class VectorType>
+template <typename Number>
+inline
+void
+BlockVectorBase<VectorType>::add (const std::vector<unsigned int> &indices,
+				  const std::vector<Number>       &values)
+{
+  Assert (indices.size() == values.size(),
+	  ExcDimensionMismatch(indices.size(), values.size()));
+  add (indices.size(), &indices[0], &values[0]);
+}
+
+
+
+template <class VectorType>
+template <typename Number>
+inline
+void
+BlockVectorBase<VectorType>::add (const std::vector<unsigned int> &indices,
+				  const Vector<Number>            &values)
+{
+  Assert (indices.size() == values.size(),
+	  ExcDimensionMismatch(indices.size(), values.size()));
+  const unsigned int n_indices = indices.size();
+  for (unsigned int i=0; i<n_indices; ++i)
+    (*this)(indices[i]) += values(i);
+}
+
+
+
+template <class VectorType>
+template <typename Number>
+inline
+void
+BlockVectorBase<VectorType>::add (const unsigned int  n_indices,
+				  const unsigned int *indices,
+				  const Number       *values)
+{
+  for (unsigned int i=0; i<n_indices; ++i)
+    (*this)(indices[i]) += values[i];
+}
+
+
+
 template <class VectorType>
 void BlockVectorBase<VectorType>::add (const value_type a)
 {
-
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   for (unsigned int i=0;i<n_blocks();++i)
@@ -1888,7 +2035,7 @@ void BlockVectorBase<VectorType>::add (const BlockVectorBase<VectorType>& v)
 {
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].add(v.components[i]);
@@ -1902,12 +2049,12 @@ void BlockVectorBase<VectorType>::add (const value_type a,
                                        const BlockVectorBase<VectorType>& v)
 {
 
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].add(a, v.components[i]);
@@ -1923,17 +2070,17 @@ void BlockVectorBase<VectorType>::add (const value_type a,
                                        const BlockVectorBase<VectorType>& w)
 {
 
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
   Assert (n_blocks() == w.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), w.n_blocks()));
-  
-  
+
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].add(a, v.components[i], b, w.components[i]);
@@ -1947,12 +2094,12 @@ void BlockVectorBase<VectorType>::sadd (const value_type x,
                                         const BlockVectorBase<VectorType>& v)
 {
 
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].sadd(x, v.components[i]);
@@ -1966,14 +2113,14 @@ void BlockVectorBase<VectorType>::sadd (const value_type x, const value_type a,
                                         const BlockVectorBase<VectorType>& v)
 {
 
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].sadd(x, a, v.components[i]);
@@ -1989,18 +2136,18 @@ void BlockVectorBase<VectorType>::sadd (const value_type x, const value_type a,
                                         const BlockVectorBase<VectorType>& w)
 {
 
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
   Assert (n_blocks() == w.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), w.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].sadd(x, a, v.components[i], b, w.components[i]);
@@ -2018,22 +2165,22 @@ void BlockVectorBase<VectorType>::sadd (const value_type x, const value_type a,
                                         const BlockVectorBase<VectorType>& y)
 {
 
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(c), 
+  Assert (numbers::is_finite(c),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
-	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));  
+	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
   Assert (n_blocks() == w.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), w.n_blocks()));
   Assert (n_blocks() == y.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), y.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].sadd(x, a, v.components[i],
@@ -2062,16 +2209,16 @@ void BlockVectorBase<VectorType>::equ (const value_type a,
                                        const BlockVectorBase<VectorType>& w)
 {
 
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
   Assert (n_blocks() == w.n_blocks(),
-	  ExcDimensionMismatch(n_blocks(), w.n_blocks()));  
-  
+	  ExcDimensionMismatch(n_blocks(), w.n_blocks()));
+
   for (unsigned int i=0;i<n_blocks();++i)
     {
       components[i].equ( a, v.components[i], b, w.components[i]);
@@ -2098,12 +2245,12 @@ void BlockVectorBase<VectorType>::equ (const value_type    a,
                                        const BlockVector2 &v)
 {
 
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     components[i].equ( a, v.components[i]);
 }
@@ -2115,12 +2262,12 @@ BlockVectorBase<VectorType>&
 BlockVectorBase<VectorType>::operator = (const value_type s)
 {
 
-  Assert (numbers::is_finite(s), 
+  Assert (numbers::is_finite(s),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   for (unsigned int i=0;i<n_blocks();++i)
     components[i] = s;
-    
+
   return *this;
 }
 
@@ -2131,7 +2278,7 @@ BlockVectorBase<VectorType>::operator = (const BlockVectorBase<VectorType>& v)
 {
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     components[i] = v.components[i];
 
@@ -2146,10 +2293,10 @@ BlockVectorBase<VectorType>::operator = (const BlockVectorBase<VectorType2> &v)
 {
   Assert (n_blocks() == v.n_blocks(),
 	  ExcDimensionMismatch(n_blocks(), v.n_blocks()));
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     components[i] = v.components[i];
-  
+
   return *this;
 }
 
@@ -2181,7 +2328,7 @@ operator == (const BlockVectorBase<VectorType2> &v) const
 {
   Assert (block_indices == v.block_indices,
           ExcDifferentBlockIndices());
-  
+
   for (unsigned int i=0;i<n_blocks();++i)
     if ( ! (components[i] == v.components[i]))
       return false;
@@ -2197,7 +2344,7 @@ BlockVectorBase<VectorType> &
 BlockVectorBase<VectorType>::operator *= (const value_type factor)
 {
 
-  Assert (numbers::is_finite(factor), 
+  Assert (numbers::is_finite(factor),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   for (unsigned int i=0;i<n_blocks();++i)
@@ -2214,7 +2361,7 @@ BlockVectorBase<VectorType> &
 BlockVectorBase<VectorType>::operator /= (const value_type factor)
 {
 
-  Assert (numbers::is_finite(factor), 
+  Assert (numbers::is_finite(factor),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
   Assert (factor > 0., ExcZero() );
 
