@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: chunk_sparse_matrix.h 16496 2008-08-06 05:00:56Z bangerth $
+//    $Id: chunk_sparse_matrix.h 19894 2009-10-15 22:19:51Z kanschat $
 //    Version: $Name:  $
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -1152,7 +1152,7 @@ class ChunkSparseMatrix : public virtual Subscriptor
 				      * it using the SmartPointer
 				      * class.
 				      */
-    SmartPointer<const ChunkSparsityPattern> cols;
+    SmartPointer<const ChunkSparsityPattern,ChunkSparseMatrix<number> > cols;
     
 				     /**
 				      * Array of values for all the
