@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: chunk_sparsity_pattern.h 16516 2008-08-12 01:50:50Z bangerth $
+//    $Id: chunk_sparsity_pattern.h 20212 2009-12-08 16:02:22Z kronbichler $
 //    Version: $Name$
 //
 //    Copyright (C) 2008 by the deal.II authors
@@ -442,32 +442,18 @@ class ChunkSparsityPattern : public Subscriptor
 		    const bool optimize_diagonal = true);
 
 				     /**
-				      * Copy data from an object of
-				      * type
-				      * CompressedSparsityPattern.
-				      * Previous content of this
-				      * object is lost, and the
-				      * sparsity pattern is in
+				      * Copy data from an object of type
+				      * CompressedSparsityPattern,
+				      * CompressedSetSparsityPattern or
+				      * CompressedSimpleSparsityPattern.
+				      * Previous content of this object is
+				      * lost, and the sparsity pattern is in
 				      * compressed mode afterwards.
 				      */
-    void copy_from (const CompressedSparsityPattern &csp,
-		    const unsigned int chunk_size,
-		    const bool optimize_diagonal = true);
-
-
-				     /**
-				      * Copy data from an object of
-				      * type
-				      * CompressedSetSparsityPattern.
-				      * Previous content of this
-				      * object is lost, and the
-				      * sparsity pattern is in
-				      * compressed mode afterwards.
-				      */
-    void copy_from (const CompressedSetSparsityPattern &csp,
-		    const unsigned int chunk_size,
-		    const bool optimize_diagonal = true);
-
+    template <typename SparsityType>
+    void copy_from (const SparsityType &csp,
+		    const unsigned int  chunk_size,
+		    const bool          optimize_diagonal = true);
 
 				     /**
 				      * Take a full matrix and use its
