@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: compressed_simple_sparsity_pattern.h 18558 2009-04-06 15:47:02Z kronbichler $
+//    $Id: compressed_simple_sparsity_pattern.h 20214 2009-12-08 17:07:06Z kronbichler $
 //    Version: $Name$
 //
 //    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -17,6 +17,7 @@
 #include <base/config.h>
 #include <base/subscriptor.h>
 #include <lac/exceptions.h>
+#include <base/index_set.h>
 
 #include <vector>
 #include <algorithm>
@@ -41,8 +42,8 @@ template <typename number> class SparseMatrix;
  * times can only be achieved at the expense of either increased
  * memory or run time consumption upon use. The main purpose of this
  * class is to avoid some memory bottlenecks, so we chose to implement
- * it memory conservative, but the chosen data format is too unsuited
- * to be used for actual matrices. It is therefore necessary to first
+ * it memory conservative. The chosen data format is too unsuited
+ * to be used for actual matrices, though. It is therefore necessary to first
  * copy the data of this object over to an object of type
  * SparsityPattern before using it in actual matrices.
  *
@@ -86,6 +87,14 @@ template <typename number> class SparseMatrix;
 class CompressedSimpleSparsityPattern : public Subscriptor
 {
   public:
+				     /**
+				      * An iterator that can be used to
+				      * iterate over the elements of a single
+				      * row. The result of dereferencing such
+				      * an iterator is a column index.
+				      */
+    typedef std::vector<unsigned int>::const_iterator row_iterator;
+
 				     /**
 				      * Initialize the matrix empty,
 				      * that is with no memory
@@ -106,7 +115,7 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 				      * involuntary copies of objects for
 				      * temporaries, which can use large
 				      * amounts of computing time.  However,
-				      * copy constructors are needed if yo
+				      * copy constructors are needed if you
 				      * want to use the STL data types on
 				      * classes like this, e.g. to write such
 				      * statements like <tt>v.push_back
@@ -119,10 +128,17 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 				     /**
 				      * Initialize a rectangular
 				      * matrix with @p m rows and
-				      * @p n columns.
+				      * @p n columns. The @p rowset
+				      * restricts the storage to
+				      * elements in rows of this set.
+				      * Adding elements outside of
+				      * this set has no effect. The
+				      * default argument keeps all
+				      * entries.
 				      */
     CompressedSimpleSparsityPattern (const unsigned int m,
-			       const unsigned int n);
+				     const unsigned int n,
+				     const IndexSet & rowset = IndexSet());
 
 				     /**
 				      * Initialize a square matrix of
@@ -146,10 +162,17 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 				      * matrix with @p m rows and
 				      * @p n columns, with at most
 				      * max_entries_per_row() nonzero
-				      * entries per row.
+				      * entries per row. The @p rowset
+				      * restricts the storage to
+				      * elements in rows of this set.
+				      * Adding elements outside of
+				      * this set has no effect. The
+				      * default argument keeps all
+				      * entries.
 				      */
     void reinit (const unsigned int m,
-		 const unsigned int n);
+		 const unsigned int n,
+		 const IndexSet & rowset = IndexSet());
 
 				     /**
 				      * Since this object is kept
@@ -194,7 +217,7 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 				      * happens.
 				      */
     template <typename ForwardIterator>
-    void add_entries (const unsigned int row, 
+    void add_entries (const unsigned int row,
 		      ForwardIterator    begin,
 		      ForwardIterator    end,
 		      const bool         indices_are_unique_and_sorted = false);
@@ -270,7 +293,12 @@ class CompressedSimpleSparsityPattern : public Subscriptor
     unsigned int n_cols () const;
 
 				     /**
-				      * Number of entries in a specific row.
+				      * Number of entries in a
+				      * specific row. This function
+				      * can only be called if the
+				      * given row is a member of the
+				      * index set of rows that we want
+				      * to store.
 				      */
     unsigned int row_length (const unsigned int row) const;
 
@@ -282,6 +310,18 @@ class CompressedSimpleSparsityPattern : public Subscriptor
     unsigned int column_number (const unsigned int row,
 				const unsigned int index) const;
 
+ 				     /**
+				      * Return an iterator that can loop over
+				      * all entries in the given
+				      * row. Dereferencing the iterator yields
+				      * a column index.
+				      */
+    row_iterator row_begin (const unsigned int row) const;
+
+				     /**
+				      * Returns the end of the current row.
+				      */
+    row_iterator row_end (const unsigned int row) const; 
 				     /**
 				      * Compute the bandwidth of the matrix
 				      * represented by this structure. The
@@ -298,8 +338,17 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 				      */
     unsigned int n_nonzero_elements () const;
 
+				     /**
+				      * Return the IndexSet that sets which
+				      * rows are active on the current
+				      * processor. It corresponds to the
+				      * IndexSet given to this class in the
+				      * constructor or in the reinit function.
+				      */
+    const IndexSet & row_index_set () const;
+
                                      /**
-                                      * Return whether this object stores only
+                                      * return whether this object stores only
                                       * those entries that have been added
                                       * explicitly, or if the sparsity pattern
                                       * contains elements that have been added
@@ -316,7 +365,12 @@ class CompressedSimpleSparsityPattern : public Subscriptor
     static
     bool stores_only_added_elements ();
 
-
+				     /**
+				      * Determine an estimate for the
+				      * memory consumption (in bytes)
+				      * of this object.
+				      */
+    unsigned int memory_consumption () const;
 
   private:
 				     /**
@@ -331,6 +385,13 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 				      */
     unsigned int cols;
 
+				     /**
+				      * A set that contains the valid rows.
+				      */
+
+    IndexSet rowset;
+
+
                                      /**
                                       * Store some data for each row
                                       * describing which entries of this row
@@ -369,6 +430,11 @@ class CompressedSimpleSparsityPattern : public Subscriptor
 	void add_entries (ForwardIterator begin,
 			  ForwardIterator end,
 			  const bool indices_are_sorted);
+
+					 /**
+					  * estimates memory consumption.
+					  */
+	unsigned int memory_consumption () const;
     };
 
 
@@ -441,7 +507,12 @@ CompressedSimpleSparsityPattern::add (const unsigned int i,
   Assert (i<rows, ExcIndexRange(i, 0, rows));
   Assert (j<cols, ExcIndexRange(j, 0, cols));
 
-  lines[i].add (j);
+  if (rowset.size() > 0 && !rowset.is_element(i))
+    return;
+
+  const unsigned int rowindex = 
+    rowset.size()==0 ? i : rowset.index_within_set(i);
+  lines[rowindex].add (j);
 }
 
 
@@ -456,7 +527,12 @@ CompressedSimpleSparsityPattern::add_entries (const unsigned int row,
 {
   Assert (row < rows, ExcIndexRange (row, 0, rows));
 
-  lines[row].add_entries (begin, end, indices_are_sorted);
+  if (rowset.size() > 0 && !rowset.is_element(row))
+    return;
+
+  const unsigned int rowindex = 
+    rowset.size()==0 ? row : rowset.index_within_set(row);
+  lines[rowindex].add_entries (begin, end, indices_are_sorted);
 }
 
 
@@ -472,8 +548,12 @@ unsigned int
 CompressedSimpleSparsityPattern::row_length (const unsigned int row) const
 {
   Assert (row < n_rows(), ExcIndexRange (row, 0, n_rows()));
+  if (rowset.size() > 0 && !rowset.is_element(row))
+    return 0;
 
-  return lines[row].entries.size();
+  const unsigned int rowindex = 
+    rowset.size()==0 ? row : rowset.index_within_set(row);
+  return lines[rowindex].entries.size();
 }
 
 
@@ -484,13 +564,47 @@ CompressedSimpleSparsityPattern::column_number (const unsigned int row,
 						const unsigned int index) const
 {
   Assert (row < n_rows(), ExcIndexRange (row, 0, n_rows()));
-  Assert (index < lines[row].entries.size(),
-	  ExcIndexRange (index, 0, lines[row].entries.size()));
+  Assert( rowset.size() == 0 || rowset.is_element(row), ExcInternalError());
 
-  return lines[row].entries[index];
+  const unsigned int local_row = rowset.size() ? rowset.index_within_set(row) : row;
+  Assert (index < lines[local_row].entries.size(),
+	  ExcIndexRange (index, 0, lines[local_row].entries.size()));
+  return lines[local_row].entries[index];
 }
 
 
+
+inline
+CompressedSimpleSparsityPattern::row_iterator
+CompressedSimpleSparsityPattern::row_begin (const unsigned int row) const
+{
+  Assert (row < n_rows(), ExcIndexRange (row, 0, n_rows()));
+  const unsigned int local_row = rowset.size() ? rowset.index_within_set(row) : row;
+  return lines[local_row].entries.begin();
+}
+
+
+
+inline
+CompressedSimpleSparsityPattern::row_iterator
+CompressedSimpleSparsityPattern::row_end (const unsigned int row) const
+{
+  Assert (row < n_rows(), ExcIndexRange (row, 0, n_rows()));
+  const unsigned int local_row = rowset.size() ? rowset.index_within_set(row) : row;
+  return lines[local_row].entries.end();
+}
+
+
+
+inline
+const IndexSet &
+CompressedSimpleSparsityPattern::row_index_set () const
+{
+  return rowset;
+}
+
+
+
 inline
 bool
 CompressedSimpleSparsityPattern::stores_only_added_elements ()
