@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: filtered_matrix.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: filtered_matrix.h 21131 2010-05-15 14:05:24Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -606,11 +606,7 @@ class FilteredMatrix : public Subscriptor
 				     /**
 				      * Pointer to the sparsity
 				      * pattern used for this
-				      * matrix. In order to guarantee
-				      * that it is not deleted while
-				      * still in use, we subscribe to
-				      * it using the SmartPointer
-				      * class.
+				      * matrix.
 				      */
     std_cxx1x::shared_ptr<PointerMatrixBase<VECTOR> > matrix;
 
@@ -812,8 +808,7 @@ inline
 void
 FilteredMatrix<VECTOR>::initialize (const MATRIX &m, bool ecs)
 {
-  matrix = std_cxx1x::shared_ptr<PointerMatrixBase<VECTOR> > (
-    new_pointer_matrix_base(m, VECTOR()));
+  matrix.reset (new_pointer_matrix_base(m, VECTOR()));
   
   expect_constrained_source = ecs;
 }
