@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: full_matrix.h 17866 2008-12-05 22:27:44Z bangerth $
+//    $Id: full_matrix.h 20220 2009-12-09 18:12:05Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -678,7 +678,7 @@ class FullMatrix : public Table<2,number>
 				      * inverse of the given factor.
 				      */
     FullMatrix & operator /= (const number factor);
-    
+
                                      /**
 				      * Simple addition of a scaled
 				      * matrix, i.e. <tt>*this +=
@@ -694,7 +694,7 @@ class FullMatrix : public Table<2,number>
     template<typename number2>
     void add (const number               a,
 	      const FullMatrix<number2> &A);
-    
+
                                      /**
 				      * Multiple addition of scaled
 				      * matrices, i.e. <tt>*this +=
@@ -713,7 +713,7 @@ class FullMatrix : public Table<2,number>
 	      const FullMatrix<number2> &A,
 	      const number               b,
 	      const FullMatrix<number2> &B);
-    
+
                                      /**
 				      * Multiple addition of scaled
 				      * matrices, i.e. <tt>*this +=
@@ -734,7 +734,7 @@ class FullMatrix : public Table<2,number>
 	      const FullMatrix<number2> &B,
 	      const number               c,
 	      const FullMatrix<number2> &C);
-    
+
 				     /**
 				      * Add rectangular block.
 				      *
@@ -759,7 +759,7 @@ class FullMatrix : public Table<2,number>
 	      const unsigned int dst_offset_j = 0,
 	      const unsigned int src_offset_i = 0,
 	      const unsigned int src_offset_j = 0);
-    
+
 				     /**
 				      * Weighted addition of the
 				      * transpose of <tt>B</tt> to
@@ -770,9 +770,9 @@ class FullMatrix : public Table<2,number>
     template<typename number2>
     void Tadd (const number               s,
 	       const FullMatrix<number2> &B);
-    
+
 				     /**
-				      * Add transose of a rectangular block.
+				      * Add transpose of a rectangular block.
 				      *
 				      * A rectangular block of the
 				      * matrix <tt>src</tt> is
@@ -940,6 +940,28 @@ class FullMatrix : public Table<2,number>
     template <typename number2>
     void invert (const FullMatrix<number2> &M);
 
+				     /**
+				      * Assign the Cholesky decomposition
+				      * of the given matrix to <tt>*this</tt>. 
+				      * The given matrix must be symmetric
+				      * positive definite.
+				      *
+				      * ExcMatrixNotPositiveDefinite
+				      * will be thrown in the case that the
+				      * matrix is not positive definite.
+				      */
+    template <typename number2>
+    void cholesky (const FullMatrix<number2> &A);
+
+				     /**
+				      * <tt>*this(i,j)</tt> = $V(i) W(j)$ 
+				      * where $V,W$
+				      * are vectors of the same length.
+				      */
+    template <typename number2>
+    void outer_product (const Vector<number2> &V,
+			const Vector<number2> &W);
+    
 				     /**
 				      * Assign the left_inverse of the given matrix
 				      * to <tt>*this</tt>. The calculation being 
@@ -1008,7 +1030,7 @@ class FullMatrix : public Table<2,number>
 //@}
 ///@name Multiplications
 //@{
-    
+
 				     /**
 				      * Matrix-matrix-multiplication.
 				      *
@@ -1023,9 +1045,17 @@ class FullMatrix : public Table<2,number>
 				      * if (!adding)
 				      *  <i>C = A*B</i>
 				      *
-				      * Assumes that <tt>A</tt> and <tt>B</tt> have
-				      * compatible sizes and that <tt>C</tt>
-				      * already has the right size.
+				      * Assumes that <tt>A</tt> and
+				      * <tt>B</tt> have compatible sizes and
+				      * that <tt>C</tt> already has the
+				      * right size.
+				      *
+				      * This function uses the BLAS function
+				      * Xgemm if the calling matrix has more
+				      * than 15 rows and BLAS was detected
+				      * during configuration. Using BLAS
+				      * usually results in considerable
+				      * performance gains.
 				      */
     template<typename number2>
     void mmult (FullMatrix<number2>       &C,
@@ -1051,11 +1081,83 @@ class FullMatrix : public Table<2,number>
 				      * <tt>B</tt> have compatible
 				      * sizes and that <tt>C</tt>
 				      * already has the right size.
+				      *
+				      * This function uses the BLAS function
+				      * Xgemm if the calling matrix has more
+				      * than 15 columns and BLAS was
+				      * detected during configuration. Using
+				      * BLAS usually results in considerable
+				      * performance gains.
 				      */
     template<typename number2>
     void Tmmult (FullMatrix<number2>       &C,
 		 const FullMatrix<number2> &B,
 		 const bool                 adding=false) const;
+
+				     /**
+				      * Matrix-matrix-multiplication using
+				      * transpose of <tt>B</tt>.
+				      *
+				      * The optional parameter
+				      * <tt>adding</tt> determines, whether the
+				      * result is stored in <tt>C</tt> or added
+				      * to <tt>C</tt>.
+				      *
+				      * if (adding)
+				      *  <i>C += A*B<sup>T</sup></i>
+				      *
+				      * if (!adding)
+				      *  <i>C = A*B<sup>T</sup></i>
+				      *
+				      * Assumes that <tt>A</tt> and
+				      * <tt>B</tt> have compatible sizes and
+				      * that <tt>C</tt> already has the
+				      * right size.
+				      *
+				      * This function uses the BLAS function
+				      * Xgemm if the calling matrix has more
+				      * than 15 rows and BLAS was detected
+				      * during configuration. Using BLAS
+				      * usually results in considerable
+				      * performance gains.
+				      */
+    template<typename number2>
+    void mTmult (FullMatrix<number2>       &C,
+		 const FullMatrix<number2> &B,
+		 const bool                 adding=false) const;
+    
+				     /**
+				      * Matrix-matrix-multiplication using
+				      * transpose of <tt>this</tt> and
+				      * <tt>B</tt>.
+				      *
+				      * The optional parameter
+				      * <tt>adding</tt> determines, whether the
+				      * result is stored in <tt>C</tt> or added
+				      * to <tt>C</tt>.
+				      *
+				      * if (adding)
+				      *  <i>C += A<sup>T</sup>*B<sup>T</sup></i>
+				      *
+				      * if (!adding)
+				      *  <i>C = A<sup>T</sup>*B<sup>T</sup></i>
+				      *
+				      * Assumes that <tt>A</tt> and
+				      * <tt>B</tt> have compatible
+				      * sizes and that <tt>C</tt>
+				      * already has the right size.
+				      *
+				      * This function uses the BLAS function
+				      * Xgemm if the calling matrix has more
+				      * than 15 columns and BLAS was
+				      * detected during configuration. Using
+				      * BLAS usually results in considerable
+				      * performance gains.
+				      */
+    template<typename number2>
+    void TmTmult (FullMatrix<number2>       &C,
+		  const FullMatrix<number2> &B,
+		  const bool                 adding=false) const;
     
 				     /**
 				      * Matrix-vector-multiplication.
@@ -1219,6 +1321,10 @@ class FullMatrix : public Table<2,number>
                                       * Exception
                                       */
     DeclException0 (ExcSourceEqualsDestination);
+				     /**
+				      * Exception
+				      */
+    DeclException0 (ExcMatrixNotPositiveDefinite);
 				     //@}
     
     friend class Accessor;
@@ -1259,7 +1365,7 @@ FullMatrix<number>::operator = (const number d)
   Assert (d==number(0), ExcScalarAssignmentOnlyForZeroValue());
 
   if (this->n_elements() != 0)
-    std::fill_n (this->val, this->n_elements(), number());
+    memset (this->val, 0, this->n_elements()*sizeof(number));
 
   return *this;
 }
