@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: full_matrix.templates.h 18539 2009-03-31 15:34:35Z kronbichler $
+//    $Id: full_matrix.templates.h 20012 2009-10-30 08:11:34Z kronbichler $
 //    Version: $Name$
 //
 //    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
@@ -198,27 +198,17 @@ FullMatrix<number>::vmult (Vector<number2>& dst,
   Assert (&src != &dst, ExcSourceEqualsDestination());
 
   const number* e = this->data();
-  const unsigned int size_m = m(),
-		     size_n = n();
-  if (!adding)
+				   // get access to the data in order to
+				   // avoid copying it when using the ()
+				   // operator
+  const number2* src_ptr = &(*const_cast<Vector<number2>*>(&src))(0);
+  const unsigned int size_m = m(), size_n = n();
+  for (unsigned int i=0; i<size_m; ++i)
     {
-      for (unsigned int i=0; i<size_m; ++i)
-	{
-	  number2 s = 0.;
-	  for (unsigned int j=0; j<size_n; ++j)
-	    s += number2(src(j)) * number2(*(e++));
-	  dst(i) = s;
-	};
-    }
-  else
-    {
-      for (unsigned int i=0; i<size_m; ++i)
-	{
-	  number2 s = 0.;
-	  for (unsigned int j=0; j<size_n; ++j)
-	    s += number2(src(j)) * number2(*(e++));
-	  dst(i) += s;
-	}
+      number2 s = adding ? dst(i) : 0.;
+      for (unsigned int j=0; j<size_n; ++j)
+	s += src_ptr[j] * number2(*(e++));
+      dst(i) = s;
     }
 }
 
@@ -231,35 +221,29 @@ void FullMatrix<number>::Tvmult (Vector<number2>       &dst,
 				 const bool             adding) const
 {
   Assert (!this->empty(), ExcEmptyMatrix());
-  
+
   Assert(dst.size() == n(), ExcDimensionMismatch(dst.size(), n()));
   Assert(src.size() == m(), ExcDimensionMismatch(src.size(), m()));
 
   Assert (&src != &dst, ExcSourceEqualsDestination());
 
-  const unsigned int size_m = m(),
-		     size_n = n();
+  const number* e = this->data();
+  number2* dst_ptr = &dst(0);
+  const unsigned int size_m = m(), size_n = n();
 
+				   // zero out data if we are not adding
   if (!adding)
+    for (unsigned int j=0; j<size_n; ++j)
+      dst_ptr[j] = 0.;
+
+				   // write the loop in a way that we can
+				   // access the data contiguously
+  for (unsigned int i=0; i<size_m; ++i)
     {
-      for (unsigned int i=0; i<size_n; ++i)
-	{
-	  number s = 0.;
-	  for (unsigned int j=0; j<size_m; ++j)
-	    s += number(src(j)) * (*this)(j,i);
-	  dst(i) = s;
-	};
-    }
-  else
-    {
-      for (unsigned int i=0; i<size_n; ++i)
-	{
-	  number s = 0.;
-	  for (unsigned int j=0; j<size_m; ++j)
-	    s += number(src(j)) * (*this)(j,i);
-	  dst(i) += s;
-	}
-    }
+      const number2 d = src(i);
+      for (unsigned int j=0; j<size_n; ++j)
+	dst_ptr[j] += d * number2(*(e++));
+    };
 }
 
 
@@ -560,7 +544,7 @@ namespace internal
 				   // LAPACK.
   namespace gemm_switch
   {
-    using ::gemm;
+    using ::dealii::gemm;
 
     template <typename T, typename T2>
     void
@@ -587,10 +571,10 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
   Assert (dst.n() == src.n(), ExcDimensionMismatch(dst.n(), src.n()));
   Assert (dst.m() == m(), ExcDimensionMismatch(m(), dst.m()));
 
-				   // see if we can use Lapack algorithms
+				   // see if we can use BLAS algorithms
 				   // for this and if the type for 'number'
 				   // works for us (it is usually not
-				   // efficient to use Lapack for very small
+				   // efficient to use BLAS for very small
 				   // matrices):
 #if defined(HAVE_DGEMM_) && defined (HAVE_SGEMM_)
   if ((types_are_equal<number,double>::value
@@ -598,15 +582,15 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
        types_are_equal<number,float>::value)
       &&
       types_are_equal<number,number2>::value)
-    if (this->n_cols() > 25)
+    if (this->n()*this->m()*src.n() > 300)
     {
-				       // In case we have the LAPACK
+				       // In case we have the BLAS
 				       // function gemm detected at
-				       // configure, we use that algorithms
+				       // configure, we use that algorithm
 				       // for matrix-matrix multiplication
 				       // since it provides better
 				       // performance than the deal.II
-				       // native functions (it uses cache
+				       // native function (it uses cache
 				       // and register blocking in order to
 				       // access local data).
 				       //
@@ -616,8 +600,8 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
 				       // in the next, etc.), whereas the
 				       // FullMatrix stores them row-wise.
 				       // We ignore that difference, and
-				       // give our row-wise data to LAPACK,
-				       // let LAPACK build the product of
+				       // give our row-wise data to BLAS,
+				       // let BLAS build the product of
 				       // transpose matrices, and read the
 				       // result as if it were row-wise
 				       // again. In other words, we calculate 
@@ -631,8 +615,9 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
       const number alpha = 1.;
       const number beta = (adding == true) ? 1. : 0.;
 
-				       // Use the LAPACK function getrf for 
-				       // calculating the LU factorization.
+				       // Use the BLAS function gemm for
+				       // calculating the matrix-matrix
+				       // product.
       internal::gemm_switch::gemm(notrans, notrans, &m, &n, &k, &alpha, &src(0,0), &m,
 				  this->val, &k, &beta, &dst(0,0), &m);
 
@@ -641,24 +626,22 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
 
 #endif
 
-  if (!adding)
-    for (unsigned int i=0; i<m(); i++)
-      for (unsigned int j=0; j<src.n(); j++)
-	{
-	  number s = 0.;
-	  for (unsigned k=0; k<n(); k++)
-	    s += (*this)(i,k) * number(src(k,j));
-	  dst(i,j) = s;
-	}
-  else
-    for (unsigned int i=0; i<m(); i++)
-      for (unsigned int j=0; j<src.n(); j++)
-	{
-	  number s = 0.;
-	  for (unsigned k=0; k<n(); k++)
-	    s += (*this)(i,k) * number(src(k,j));
-	  dst(i,j) += s;
-	}
+  const unsigned int m = this->m(), n = src.n(), l = this->n();
+
+					// arrange the loops in a way that
+					// we keep write operations low,
+					// (writing is usually more costly
+					// than reading), even though we
+					// need to access the data in src
+					// not in a contiguous way.
+  for (unsigned int i=0; i<m; i++)
+    for (unsigned int j=0; j<n; j++)
+      {
+        number2 add_value = adding ? dst(i,j) : 0.;
+	for (unsigned int k=0; k<l; k++)
+	  add_value += (number2)(*this)(i,k) * (number2)(src(k,j));
+	dst(i,j) = add_value;
+      }
 }
 
 
@@ -675,10 +658,10 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
   Assert (src.n() == dst.n(), ExcDimensionMismatch(src.n(), dst.n()));
 
 
-				   // see if we can use Lapack algorithms
+				   // see if we can use BLAS algorithms
 				   // for this and if the type for 'number'
 				   // works for us (it is usually not
-				   // efficient to use Lapack for very small
+				   // efficient to use BLAS for very small
 				   // matrices):
 #if defined(HAVE_DGEMM_) && defined (HAVE_SGEMM_)
   if ((types_are_equal<number,double>::value
@@ -686,15 +669,15 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
        types_are_equal<number,float>::value)
       &&
       types_are_equal<number,number2>::value)
-    if (this->n_cols() > 25)
+    if (this->n()*this->m()*src.n() > 300)
     {
-				       // In case we have the LAPACK
+				       // In case we have the BLAS
 				       // function gemm detected at
-				       // configure, we use that algorithms
+				       // configure, we use that algorithm
 				       // for matrix-matrix multiplication
 				       // since it provides better
 				       // performance than the deal.II
-				       // native functions (it uses cache
+				       // native function (it uses cache
 				       // and register blocking in order to
 				       // access local data).
 				       //
@@ -704,12 +687,12 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
 				       // in the next, etc.), whereas the
 				       // FullMatrix stores them row-wise.
 				       // We ignore that difference, and
-				       // give our row-wise data to LAPACK,
-				       // let LAPACK build the product of
+				       // give our row-wise data to BLAS,
+				       // let BLAS build the product of
 				       // transpose matrices, and read the
 				       // result as if it were row-wise
 				       // again. In other words, we calculate 
-				       // (B^T A^T)^T, which is AB.
+				       // (B^T A)^T, which is A^T B.
 
       const int m = src.n();
       const int n = this->n();
@@ -720,8 +703,9 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
       const number alpha = 1.;
       const number beta = (adding == true) ? 1. : 0.;
 
-				       // Use the LAPACK function getrf for 
-				       // calculating the LU factorization.
+				       // Use the BLAS function gemm for
+				       // calculating the matrix-matrix
+				       // product.
       internal::gemm_switch::gemm(notrans, trans, &m, &n, &k, &alpha, &src(0,0), &m,
 				  this->val, &n, &beta, &dst(0,0), &m);
 
@@ -730,25 +714,203 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
 
 #endif
 
+  const unsigned int m = n(), n = src.n(), l = this->m();
 
-  if (!adding)
-    for (unsigned int i=0; i<n(); i++)
-      for (unsigned int j=0; j<src.n(); j++)
-	{
-	  number s = 0;
-	  for (unsigned int k=0; k<m(); k++)
-	    s += (*this)(k,i) * number(src(k,j));
-	  dst(i,j) = s;
-	}
-  else
-    for (unsigned int i=0; i<n(); i++)
-      for (unsigned int j=0; j<src.n(); j++)
-	{
-	  number s = 0;
-	  for (unsigned int k=0; k<m(); k++)
-	    s += (*this)(k,i) * number(src(k,j));
-	  dst(i,j) += s;
-	}
+					// arrange the loops in a way that
+					// we keep write operations low,
+					// (writing is usually more costly
+					// than reading), even though we
+					// need to access the data in src
+					// not in a contiguous way. However,
+					// we should usually end up in the
+					// optimized gemm operation in case
+					// the matrix is big, so this
+					// shouldn't be too bad.
+  for (unsigned int i=0; i<m; i++)
+    for (unsigned int j=0; j<n; j++)
+      {
+        number2 add_value = adding ? dst(i,j) : 0.;
+	for (unsigned int k=0; k<l; k++)
+	  add_value += (number2)(*this)(k,i) * (number2)(src(k,j));
+	dst(i,j) = add_value;
+      }
+}
+
+
+
+template <typename number>
+template <typename number2>
+void FullMatrix<number>::mTmult (FullMatrix<number2>       &dst,
+				 const FullMatrix<number2> &src,
+				 const bool                 adding) const
+{
+  Assert (!this->empty(), ExcEmptyMatrix());
+  Assert (n() == src.n(), ExcDimensionMismatch(n(), src.n()));
+  Assert (dst.n() == src.m(), ExcDimensionMismatch(dst.n(), src.m()));
+  Assert (dst.m() == m(), ExcDimensionMismatch(m(), dst.m()));
+
+				   // see if we can use BLAS algorithms
+				   // for this and if the type for 'number'
+				   // works for us (it is usually not
+				   // efficient to use BLAS for very small
+				   // matrices):
+#if defined(HAVE_DGEMM_) && defined (HAVE_SGEMM_)
+  if ((types_are_equal<number,double>::value
+       ||
+       types_are_equal<number,float>::value)
+      &&
+      types_are_equal<number,number2>::value)
+    if (this->n()*this->m()*src.m() > 300)
+    {
+				       // In case we have the BLAS
+				       // function gemm detected at
+				       // configure, we use that algorithm
+				       // for matrix-matrix multiplication
+				       // since it provides better
+				       // performance than the deal.II
+				       // native function (it uses cache
+				       // and register blocking in order to
+				       // access local data).
+				       //
+				       // Note that BLAS/LAPACK stores
+				       // matrix elements column-wise (i.e.,
+				       // all values in one column, then all
+				       // in the next, etc.), whereas the
+				       // FullMatrix stores them row-wise.
+				       // We ignore that difference, and
+				       // give our row-wise data to BLAS,
+				       // let BLAS build the product of
+				       // transpose matrices, and read the
+				       // result as if it were row-wise
+				       // again. In other words, we calculate 
+				       // (B A^T)^T, which is AB^T.
+
+      const int m = src.m();
+      const int n = this->m();
+      const int k = this->n();
+      const char *notrans = "n";
+      const char *trans = "t";
+
+      const number alpha = 1.;
+      const number beta = (adding == true) ? 1. : 0.;
+
+				       // Use the BLAS function gemm for
+				       // calculating the matrix-matrix
+				       // product.
+      internal::gemm_switch::gemm(trans, notrans, &m, &n, &k, &alpha, &src(0,0), &k,
+				  this->val, &k, &beta, &dst(0,0), &m);
+
+      return;
+    }
+
+#endif
+
+  const unsigned int m = this->m(), n = src.m(), l = this->n();
+
+					// arrange the loops in a way that
+					// we keep write operations low,
+					// (writing is usually more costly
+					// than reading).
+  for (unsigned int i=0; i<m; i++)
+    for (unsigned int j=0; j<n; j++)
+      {
+        number2 add_value = adding ? dst(i,j) : 0.;
+	for (unsigned int k=0; k<l; k++)
+	  add_value += (number2)(*this)(i,k) * (number2)(src(j,k));
+	dst(i,j) = add_value;
+      }
+}
+
+
+
+template <typename number>
+template <typename number2>
+void FullMatrix<number>::TmTmult (FullMatrix<number2>       &dst,
+				  const FullMatrix<number2> &src,
+				  const bool                 adding) const
+{
+  Assert (!this->empty(), ExcEmptyMatrix());  
+  Assert (m() == src.n(), ExcDimensionMismatch(m(), src.n()));
+  Assert (n() == dst.m(), ExcDimensionMismatch(n(), dst.m()));
+  Assert (src.m() == dst.n(), ExcDimensionMismatch(src.m(), dst.n()));
+
+
+				   // see if we can use BLAS algorithms
+				   // for this and if the type for 'number'
+				   // works for us (it is usually not
+				   // efficient to use BLAS for very small
+				   // matrices):
+#if defined(HAVE_DGEMM_) && defined (HAVE_SGEMM_)
+  if ((types_are_equal<number,double>::value
+       ||
+       types_are_equal<number,float>::value)
+      &&
+      types_are_equal<number,number2>::value)
+    if (this->n()*this->m()*src.m() > 300)
+    {
+				       // In case we have the BLAS
+				       // function gemm detected at
+				       // configure, we use that algorithm
+				       // for matrix-matrix multiplication
+				       // since it provides better
+				       // performance than the deal.II
+				       // native function (it uses cache
+				       // and register blocking in order to
+				       // access local data).
+				       //
+				       // Note that BLAS/LAPACK stores
+				       // matrix elements column-wise (i.e.,
+				       // all values in one column, then all
+				       // in the next, etc.), whereas the
+				       // FullMatrix stores them row-wise.
+				       // We ignore that difference, and
+				       // give our row-wise data to BLAS,
+				       // let BLAS build the product of
+				       // transpose matrices, and read the
+				       // result as if it were row-wise
+				       // again. In other words, we calculate 
+				       // (B A)^T, which is A^T B^T.
+
+      const int m = src.n();
+      const int n = this->n();
+      const int k = this->m();
+      const char *trans = "t";
+
+      const number alpha = 1.;
+      const number beta = (adding == true) ? 1. : 0.;
+
+				       // Use the BLAS function gemm for
+				       // calculating the matrix-matrix
+				       // product.
+      internal::gemm_switch::gemm(trans, trans, &m, &n, &k, &alpha, &src(0,0), &k,
+				  this->val, &n, &beta, &dst(0,0), &m);
+
+      return;
+    }
+
+#endif
+
+  const unsigned int m = n(), n = src.m(), l = this->m();
+
+					// arrange the loops in a way that
+					// we keep write operations low,
+					// (writing is usually more costly
+					// than reading), even though we
+					// need to access the data in the
+					// calling matrix not in a
+					// contiguous way. However, we
+					// should usually end up in the
+					// optimized gemm operation in case
+					// the matrix is big, so this
+					// shouldn't be too bad.
+  for (unsigned int i=0; i<m; i++)
+    for (unsigned int j=0; j<n; j++)
+      {
+        number2 add_value = adding ? dst(i,j) : 0.;
+	for (unsigned int k=0; k<l; k++)
+	  add_value += (number2)(*this)(k,i) * (number2)(src(j,k));
+	dst(i,j) = add_value;
+      }
 }
 
 
@@ -1292,6 +1454,7 @@ FullMatrix<number>::invert (const FullMatrix<number2> &M)
 	
 	break;
       }
+      
 
       default:
 					     // if no inversion is
@@ -1303,6 +1466,70 @@ FullMatrix<number>::invert (const FullMatrix<number2> &M)
     };    
 }
 
+
+template <typename number>
+template <typename number2>
+void
+FullMatrix<number>::cholesky (const FullMatrix<number2> &A)
+{
+  Assert (!A.empty(), ExcEmptyMatrix());
+  Assert (A.n() == A.m(),
+	  ExcNotQuadratic());
+					// Matrix must be symmetric.
+  Assert(A.relative_symmetry_norm2() < 1.0e-10, ExcMessage("A must be symmetric."));
+
+  if (PointerComparison::equal(&A, this))
+    {
+				       // avoid overwriting source
+				       // by destination matrix:
+      FullMatrix<number2> A2 = A;
+      cholesky(A2);
+    }
+  else 
+    {
+				   /* reinit *this to 0 */
+      this->reinit(A.m(), A.n());
+
+      double SLik2 = 0.0, SLikLjk = 0.0;
+      for (unsigned int i=0; i< this->n_cols(); i++){
+	SLik2 = 0.0;
+	for (unsigned int j = 0; j < i; j++){
+	  SLikLjk = 0.0;
+	  for (unsigned int k =0; k<j; k++)
+	    {
+	      SLikLjk += (*this)(i,k)*(*this)(j,k);
+	    };
+	  (*this)(i,j) = (1./(*this)(j,j))*(A(i,j) - SLikLjk);
+	  SLik2 += (*this)(i,j)*(*this)(i,j);
+	}
+	AssertThrow (A(i,i) - SLik2 >= 0,
+		     ExcMatrixNotPositiveDefinite());
+	
+	(*this)(i,i) = std::sqrt(A(i,i) - SLik2);
+      }
+    }
+  }
+
+
+template <typename number>
+template <typename number2>
+void
+FullMatrix<number>::outer_product (const Vector<number2> &V,
+				   const Vector<number2> &W)
+{
+  Assert (V.size() == W.size(), ExcMessage("Vectors V, W must be the same size."));
+  this->reinit(V.size(), V.size());
+
+  for(unsigned int i = 0; i<this->n(); i++)
+    {
+      for(unsigned int j = 0; j< this->n(); j++)
+	{
+	  (*this)(i,j) = V(i)*W(j);
+	}
+    }
+}
+
+
 template <typename number>
 template <typename number2>
 void
@@ -1504,17 +1731,17 @@ namespace internal
 				   // LAPACK.
   namespace getrf_switch
   {
-    using ::getrf;
+    using ::dealii::getrf;
 
     template <typename T>
     void
     getrf (const int*, const int*, T*, const int*, int*, int*)
     {
-      Assert (false, LAPACKSupport::ExcMissing("dgetr for this data type"));
+      Assert (false, LAPACKSupport::ExcMissing("dgetrf for this data type"));
     }
     
 
-    using ::getri;
+    using ::dealii::getri;
 
     template <typename T>
     void
