@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_block_vector.h 14783 2007-06-18 14:52:01Z bangerth $
+//    $Id: petsc_block_vector.h 20934 2010-04-02 13:09:20Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,14 +15,15 @@
 
 
 #include <base/config.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_parallel_block_vector.h>
-#include <lac/block_indices.h>
-#include <lac/block_vector_base.h>
-#include <lac/exceptions.h>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <lac/petsc_vector.h>
+#  include <lac/petsc_parallel_block_vector.h>
+#  include <lac/block_indices.h>
+#  include <lac/block_vector_base.h>
+#  include <lac/exceptions.h>
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -40,6 +41,7 @@ namespace PETScWrappers
  * that are specific to the underlying vector type.
  *
  * @ingroup Vectors
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, 2004
  */
   class BlockVector : public BlockVectorBase<Vector>
@@ -290,6 +292,14 @@ namespace PETScWrappers
                                         */
       void swap (BlockVector &v);    
 
+				     /**
+				      * Print to a stream.
+				      */
+      void print (std::ostream       &out,
+		  const unsigned int  precision = 3,
+		  const bool          scientific = true,
+		  const bool          across = true) const;
+
 				       /** @addtogroup Exceptions
 					* @{ */
 
@@ -462,6 +472,25 @@ namespace PETScWrappers
   }
 
 
+
+  inline
+  void 
+  BlockVector::print (std::ostream       &out,
+		      const unsigned int  precision,
+		      const bool          scientific,
+		      const bool          across) const
+  {
+    for (unsigned int i=0;i<this->n_blocks();++i)
+      {
+	if (across)
+	  out << 'C' << i << ':';
+	else
+	  out << "Component " << i << std::endl;
+	this->components[i].print(out, precision, scientific, across);
+      }
+  }
+
+
   
 
 /**
@@ -469,7 +498,6 @@ namespace PETScWrappers
  * of the C++ standard library which uses a temporary object. The
  * function simply exchanges the data of the two vectors.
  *
- * @ingroup PETScWrappers
  * @relates PETScWrappers::BlockVector
  * @author Wolfgang Bangerth, 2000
  */
