@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_matrix_base.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: petsc_matrix_base.h 20745 2010-03-07 12:50:06Z young $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,19 +15,22 @@
 
 
 #include <base/config.h>
-#include <base/subscriptor.h>
-#include <lac/full_matrix.h>
-#include <lac/exceptions.h>
 
 #ifdef DEAL_II_USE_PETSC
 
-#include <petscmat.h>
-#include <base/std_cxx1x/shared_ptr.h>
-#include <vector>
-#include <cmath>
+#  include <base/subscriptor.h>
+#  include <lac/full_matrix.h>
+#  include <lac/exceptions.h>
+
+#  include <petscmat.h>
+#  include <base/std_cxx1x/shared_ptr.h>
+#  include <vector>
+#  include <cmath>
 
 DEAL_II_NAMESPACE_OPEN
 
+template <typename Matrix> class BlockMatrixBase;
+
 
 namespace PETScWrappers
 {
@@ -109,7 +112,7 @@ namespace PETScWrappers
                             << " of a distributed matrix, but only rows "
                             << arg2 << " through " << arg3
                             << " are stored locally and can be accessed.");
-            
+
           private:
                                              /**
                                               * The matrix accessed.
@@ -157,7 +160,7 @@ namespace PETScWrappers
                                               * of this row.
                                               */
             std_cxx1x::shared_ptr<const std::vector<PetscScalar> > value_cache;
-            
+
                                              /**
                                               * Discard the old row caches
                                               * (they may still be used by
@@ -173,18 +176,18 @@ namespace PETScWrappers
                                               */
             friend class const_iterator;
         };
-        
+
       public:
-          
+
                                          /**
                                           * Constructor. Create an iterator
                                           * into the matrix @p matrix for the
                                           * given row and the index within it.
-                                          */ 
+                                          */
         const_iterator (const MatrixBase   *matrix,
                         const unsigned int  row,
                         const unsigned int  index);
-          
+
                                          /**
                                           * Prefix increment.
                                           */
@@ -236,7 +239,7 @@ namespace PETScWrappers
                         << "Attempt to access element " << arg2
                         << " of row " << arg1
                         << " which doesn't have that many elements.");
-        
+
       private:
                                          /**
                                           * Store an object of the
@@ -244,10 +247,10 @@ namespace PETScWrappers
                                           */
         Accessor accessor;
     };
-    
+
   }
-  
-  
+
+
 /**
  * Base class for all matrix classes that are implemented on top of the PETSc
  * matrix types. Since in PETSc all matrix types (i.e. sequential and
@@ -275,7 +278,7 @@ namespace PETScWrappers
  * no more free entries available after that any more, it is better to only
  * call SparseMatrix::compress() once at the end of the assembly stage and
  * before the matrix is actively used.
- * 
+ *
  * @ingroup PETScWrappers
  * @ingroup Matrix1
  * @author Wolfgang Bangerth, 2004
@@ -294,7 +297,7 @@ namespace PETScWrappers
                                         * the other container classes.
                                         */
       typedef PetscScalar value_type;
-      
+
                                        /**
                                         * Default constructor.
                                         */
@@ -321,7 +324,7 @@ namespace PETScWrappers
                                         * previously used.
                                         */
       MatrixBase &
-      operator = (const double d);      
+      operator = (const value_type d);
                                        /**
                                         * Release all memory and return
                                         * to a state just like after
@@ -649,7 +652,7 @@ namespace PETScWrappers
                                         */
       void clear_rows (const std::vector<unsigned int> &rows,
                        const PetscScalar                new_diag_value = 0);
-      
+
                                        /**
                                         * PETSc matrices store their own
                                         * sparsity patterns. So, in analogy to
@@ -662,9 +665,13 @@ namespace PETScWrappers
                                         * allowed. This function must
                                         * therefore be called once you have
                                         * assembled the matrix.
+					*
+					* See @ref GlossCompress "Compressing distributed objects"
+					* for more information.
+					* more information.
                                         */
       void compress ();
-      
+
                                        /**
                                         * Return the value of the entry
                                         * (<i>i,j</i>).  This may be an
@@ -714,7 +721,7 @@ namespace PETScWrappers
                                         * SparseMatrix class.
                                         */
       PetscScalar diag_element (const unsigned int i) const;
-      
+
                                        /**
                                         * Return the number of rows in this
                                         * matrix.
@@ -791,10 +798,10 @@ namespace PETScWrappers
                                         * Number of entries in a specific row.
                                         */
       unsigned int row_length (const unsigned int row) const;
-      
+
                                        /**
                                         * Return the l1-norm of the matrix, that is
-                                        * $|M|_1=max_{all columns j}\sum_{all 
+                                        * $|M|_1=max_{all columns j}\sum_{all
                                         * rows i} |M_ij|$,
                                         * (max. sum of columns).
                                         * This is the
@@ -809,7 +816,7 @@ namespace PETScWrappers
                                        /**
                                         * Return the linfty-norm of the
                                         * matrix, that is
-                                        * $|M|_infty=max_{all rows i}\sum_{all 
+                                        * $|M|_infty=max_{all rows i}\sum_{all
                                         * columns j} |M_ij|$,
                                         * (max. sum of rows).
                                         * This is the
@@ -828,13 +835,13 @@ namespace PETScWrappers
                                         * matrix.
                                         */
       PetscReal frobenius_norm () const;
-      
+
                                        /**
                                         * Multiply the entire matrix by a
                                         * fixed factor.
                                         */
       MatrixBase & operator *= (const PetscScalar factor);
-    
+
                                        /**
                                         * Divide the entire matrix by a
                                         * fixed factor.
@@ -858,7 +865,7 @@ namespace PETScWrappers
 					* well. Conversely, if the matrix is
 					* not distributed, then neither of the
 					* vectors may be.
-                                        */      
+                                        */
       void vmult (VectorBase       &dst,
                   const VectorBase &src) const;
 
@@ -1063,7 +1070,7 @@ namespace PETScWrappers
                                         * iterator for the last row of a matrix.
                                         */
       const_iterator end (const unsigned int r) const;
-      
+
                                        /**
                                         * Conversion operator to gain access
                                         * to the underlying PETSc type. If you
@@ -1077,34 +1084,42 @@ namespace PETScWrappers
                                         */
       operator const Mat () const;
 
-                                        /**  
-					 * Make an in-place transpose of a 
+                                        /**
+					 * Make an in-place transpose of a
 					 * matrix.
 					 */
       void transpose ();
 
-                                        /**  
+                                        /**
 					 * Test whether a matrix is symmetric.
-					 * Default tolerance is zero. 
+					 * Default tolerance is zero.
 					 */
       PetscTruth is_symmetric (const double tol = 0.0);
 
-                                        /** 
-					 * Test whether a matrix is Hermitian, 
-					 * i.e. it is the complex conjugate 
-					 * of its transpose. Default tolerance 
-					 * is zero. 
+#if DEAL_II_PETSC_VERSION_GTE(2,3,0)
+                                        /**
+					 * Test whether a matrix is Hermitian,
+					 * i.e. it is the complex conjugate
+					 * of its transpose. Default tolerance
+					 * is zero.
 					 */
       PetscTruth is_hermitian (const double tol = 0.0);
+#endif
 
-                                        /* 
-					 * Abstract PETSc object that helps view 
+                                        /*
+					 * Abstract PETSc object that helps view
 					 * in ASCII other PETSc objects. Currently
-					 * this function simply writes non-zero 
-					 * elements of a matrix to the terminal. 
+					 * this function simply writes non-zero
+					 * elements of a matrix to the terminal.
 					 */
       void write_ascii ();
 
+				       /**
+					*  Returns the number bytes consumed
+					*  by this matrix on this CPU.
+					*/
+      unsigned int memory_consumption() const;
+      
                                        /**
                                         * Exception
                                         */
@@ -1157,6 +1172,49 @@ namespace PETScWrappers
                                         */
       LastAction::Values last_action;
 
+				       /**
+					* Flush buffers on all CPUs when
+					* switching between inserting and
+					* adding to elements, no-op otherwise.
+					* Should be called from all internal
+					* functions accessing matrix elements.
+					*/
+      void prepare_action(const LastAction::Values new_action);
+
+				       /**
+				        * For some matrix storage
+                                        * formats, in particular for the
+                                        * PETSc distributed blockmatrices,
+                                        * set and add operations on
+                                        * individual elements can not be
+                                        * freely mixed. Rather, one has
+                                        * to synchronize operations when
+                                        * one wants to switch from
+                                        * setting elements to adding to
+                                        * elements.
+                                        * BlockMatrixBase automatically
+                                        * synchronizes the access by
+                                        * calling this helper function
+                                        * for each block.
+                                        * This function ensures that the
+                                        * matrix is in a state that
+                                        * allows adding elements; if it
+                                        * previously already was in this
+                                        * state, the function does
+                                        * nothing.
+                                        */
+      void prepare_add();
+				       /**
+                                        * Same as prepare_add() but
+                                        * prepare the matrix for setting
+                                        * elements if the representation
+                                        * of elements in this class
+                                        * requires such an operation.
+                                        */
+      void prepare_set();
+
+
+
     private:
 				       /**
 					* An internal array of integer
@@ -1165,7 +1223,11 @@ namespace PETScWrappers
 					* adding/inserting local data into
 					* the (large) sparse matrix.
 					*/
-      std::vector<unsigned int> column_indices;
+#ifdef PETSC_USE_64BIT_INDICES
+      std::vector<PetscInt> column_indices;
+#else
+      std::vector<int> column_indices;
+#endif
 
 				       /**
 					* An internal array of double values
@@ -1176,6 +1238,13 @@ namespace PETScWrappers
 					*/
       std::vector<PetscScalar> column_values;
 
+
+				       /**
+				        *  To allow calling protected
+				        *  prepare_add() and
+				        *  prepare_set().
+				        */
+      template <class> friend class dealii::BlockMatrixBase;
   };
 
 
@@ -1263,7 +1332,7 @@ namespace PETScWrappers
         {
           accessor.a_index = 0;
           ++accessor.a_row;
-      
+
           while ((accessor.a_row < accessor.matrix->m())
                  &&
                  (accessor.matrix->row_length(accessor.a_row) == 0))
@@ -1329,17 +1398,17 @@ namespace PETScWrappers
               (accessor.row() == other.accessor.row() &&
                accessor.index() < other.accessor.index()));
     }
-    
+
   }
 
 
 
 				        // Inline the set() and add()
-				        // functions, since they will be 
-                                        // called frequently, and the 
-				        // compiler can optimize away 
+				        // functions, since they will be
+                                        // called frequently, and the
+				        // compiler can optimize away
 				        // some unnecessary loops when
-					// the sizes are given at 
+					// the sizes are given at
 				        // compile time.
   inline
   void
@@ -1417,26 +1486,21 @@ namespace PETScWrappers
 		   const PetscScalar  *values,
 		   const bool          elide_zero_values)
   {
+    prepare_action(LastAction::insert);
 
-    if (last_action != LastAction::insert)
-      {
-        int ierr;
-        ierr = MatAssemblyBegin(matrix,MAT_FLUSH_ASSEMBLY);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-        ierr = MatAssemblyEnd(matrix,MAT_FLUSH_ASSEMBLY);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-	last_action = LastAction::insert;
-      }
-    
-    const signed int petsc_i = row;
+#ifdef PETSC_USE_64BIT_INDICES
+    const PetscInt petsc_i = row;
+    PetscInt * col_index_ptr;
+#else
+    const int petsc_i = row;
     int * col_index_ptr;
+#endif
     PetscScalar const* col_value_ptr;
     int n_columns;
 
 				   // If we don't elide zeros, the pointers
 				   // are already available...
+#ifndef PETSC_USE_64BIT_INDICES
     if (elide_zero_values == false)
       {
 	col_index_ptr = (int*)col_indices;
@@ -1444,6 +1508,7 @@ namespace PETScWrappers
 	n_columns = n_cols;
       }
     else
+#endif
       {
 				   // Otherwise, extract nonzero values in
 				   // each row and get the respective index.
@@ -1456,11 +1521,11 @@ namespace PETScWrappers
 	n_columns = 0;
 	for (unsigned int j=0; j<n_cols; ++j)
 	  {
-	    const double value = values[j];
+	    const PetscScalar value = values[j];
 	    Assert (numbers::is_finite(value),
 		    ExcMessage("The given value is not finite but either "
 			       "infinite or Not A Number (NaN)"));
-	    if (value != 0)
+	    if (value != PetscScalar())
 	      {
 		column_indices[n_columns] = col_indices[j];
 		column_values[n_columns] = value;
@@ -1469,10 +1534,10 @@ namespace PETScWrappers
 	  }
 	Assert(n_columns <= (int)n_cols, ExcInternalError());
 
-	col_index_ptr = (int*)&column_indices[0];
+	col_index_ptr = &column_indices[0];
 	col_value_ptr = &column_values[0];
       }
-          
+
     const int ierr
       = MatSetValues (matrix, 1, &petsc_i, n_columns, col_index_ptr,
 		      col_value_ptr, INSERT_VALUES);
@@ -1488,11 +1553,11 @@ namespace PETScWrappers
 		   const PetscScalar  value)
   {
 
-    Assert (numbers::is_finite(value), 
+    Assert (numbers::is_finite(value),
 	    ExcMessage("The given value is not finite but either "
 		       "infinite or Not A Number (NaN)"));
 
-    if (value == 0)
+    if (value == PetscScalar())
       {
 				  // we have to do checkings on Insert/Add
 				  // in any case
@@ -1504,17 +1569,7 @@ namespace PETScWrappers
 				  // zero. However, these actions are done
 				  // in case we pass on to the other
 				  // function.
-	if (last_action != LastAction::add)
-	  {
-	    int ierr;
-	    ierr = MatAssemblyBegin(matrix,MAT_FLUSH_ASSEMBLY);
-	    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-	    ierr = MatAssemblyEnd(matrix,MAT_FLUSH_ASSEMBLY);
-	    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-	    last_action = LastAction::add;
-	  }
+	prepare_action(LastAction::add);
 
 	return;
       }
@@ -1585,26 +1640,21 @@ namespace PETScWrappers
 		   const bool          elide_zero_values,
 		   const bool          /*col_indices_are_sorted*/)
   {
-    if (last_action != LastAction::add)
-      {
-        int ierr;
-        ierr = MatAssemblyBegin(matrix,MAT_FLUSH_ASSEMBLY);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
+    prepare_action(LastAction::add);
 
-        ierr = MatAssemblyEnd(matrix,MAT_FLUSH_ASSEMBLY);
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-        last_action = LastAction::add;
-      }
-    
-    
-    const signed int petsc_i = row;
+#ifdef PETSC_USE_64BIT_INDICES
+    const PetscInt petsc_i = row;
+    PetscInt * col_index_ptr;
+#else
+    const int petsc_i = row;
     int * col_index_ptr;
+#endif
     PetscScalar const* col_value_ptr;
     int n_columns;
 
 				   // If we don't elide zeros, the pointers
 				   // are already available...
+#ifndef PETSC_USE_64BIT_INDICES
     if (elide_zero_values == false)
       {
 	col_index_ptr = (int*)col_indices;
@@ -1612,6 +1662,7 @@ namespace PETScWrappers
 	n_columns = n_cols;
       }
     else
+#endif
       {
 				   // Otherwise, extract nonzero values in
 				   // each row and get the respective index.
@@ -1624,11 +1675,11 @@ namespace PETScWrappers
 	n_columns = 0;
 	for (unsigned int j=0; j<n_cols; ++j)
 	  {
-	    const double value = values[j];
+	    const PetscScalar value = values[j];
 	    Assert (numbers::is_finite(value),
 		    ExcMessage("The given value is not finite but either "
 			       "infinite or Not A Number (NaN)"));
-	    if (value != 0)
+	    if (value != PetscScalar())
 	      {
 		column_indices[n_columns] = col_indices[j];
 		column_values[n_columns] = value;
@@ -1637,14 +1688,14 @@ namespace PETScWrappers
 	  }
 	Assert(n_columns <= (int)n_cols, ExcInternalError());
 
-	col_index_ptr = (int*)&column_indices[0];
+	col_index_ptr = &column_indices[0];
 	col_value_ptr = &column_values[0];
       }
-          
+
     const int ierr
       = MatSetValues (matrix, 1, &petsc_i, n_columns, col_index_ptr,
 		      col_value_ptr, ADD_VALUES);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    Assert (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -1660,7 +1711,7 @@ namespace PETScWrappers
     return el(i,j);
   }
 
-  
+
 
   inline
   MatrixBase::const_iterator
@@ -1702,28 +1753,70 @@ namespace PETScWrappers
     for (unsigned int i=r+1; i<m(); ++i)
       if (row_length(i) > 0)
         return const_iterator(this, i, 0);
-    
+
                                      // if there is no such line, then take the
                                      // end iterator of the matrix
     return end();
   }
 
 
-  
+
   inline
   bool
   MatrixBase::in_local_range (const unsigned int index) const
   {
+#ifdef PETSC_USE_64BIT_INDICES
+    PetscInt begin, end;
+#else
     int begin, end;
+#endif
     const int ierr = MatGetOwnershipRange (static_cast<const Mat &>(matrix),
 					   &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-    
+
     return ((index >= static_cast<unsigned int>(begin)) &&
             (index < static_cast<unsigned int>(end)));
   }
 
-#endif // DOXYGEN      
+
+
+  inline
+  void
+  MatrixBase::prepare_action(const LastAction::Values new_action)
+  {
+				     // flush PETSc buffers when switching
+				     // actions, otherwise just return.
+    if (last_action == new_action) return;
+
+    int ierr;
+    ierr = MatAssemblyBegin(matrix, MAT_FLUSH_ASSEMBLY);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+    ierr = MatAssemblyEnd(matrix, MAT_FLUSH_ASSEMBLY);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+    last_action = new_action;
+  }
+
+
+
+  inline
+  void
+  MatrixBase::prepare_add()
+  {
+    prepare_action(LastAction::add);
+  }
+
+
+
+  inline
+  void
+  MatrixBase::prepare_set()
+  {
+    prepare_action(LastAction::insert);
+  }
+
+#endif // DOXYGEN
 }
 
 
