@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_parallel_block_vector.h 14783 2007-06-18 14:52:01Z bangerth $
+//    $Id: petsc_parallel_block_vector.h 20934 2010-04-02 13:09:20Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,13 +15,14 @@
 
 
 #include <base/config.h>
-#include <lac/petsc_parallel_vector.h>
-#include <lac/block_indices.h>
-#include <lac/block_vector_base.h>
-#include <lac/exceptions.h>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <lac/petsc_parallel_vector.h>
+#  include <lac/block_indices.h>
+#  include <lac/block_vector_base.h>
+#  include <lac/exceptions.h>
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -51,6 +52,7 @@ namespace PETScWrappers
  * of these blocks to be stored on the local process.
  *
  * @ingroup Vectors
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, 2004
  */
     class BlockVector : public BlockVectorBase<Vector>
@@ -302,6 +304,14 @@ namespace PETScWrappers
                                           * to standard functions.
                                           */
         void swap (BlockVector &v);    
+
+					 /**
+					  * Print to a stream.
+					  */
+	void print (std::ostream       &out,
+		    const unsigned int  precision = 3,
+		    const bool          scientific = true,
+		    const bool          across = true) const;
     
                                          /**
                                           * Exception
@@ -451,7 +461,26 @@ namespace PETScWrappers
         this->components[i].swap (v.components[i]);
       ::dealii::swap (this->block_indices, v.block_indices);
     }
-  
+
+
+
+    inline
+    void 
+    BlockVector::print (std::ostream       &out,
+			const unsigned int  precision,
+			const bool          scientific,
+			const bool          across) const
+    {
+      for (unsigned int i=0;i<this->n_blocks();++i)
+	{
+	  if (across)
+	    out << 'C' << i << ':';
+	  else
+	    out << "Component " << i << std::endl;
+	  this->components[i].print(out, precision, scientific, across);
+	}
+    }
+    
 
 
 /**
@@ -459,7 +488,6 @@ namespace PETScWrappers
  * of the C++ standard library which uses a temporary object. The
  * function simply exchanges the data of the two vectors.
  *
- * @ingroup PETScWrappers
  * @relates PETScWrappers::MPI::BlockVector
  * @author Wolfgang Bangerth, 2000
  */
