@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_parallel_sparse_matrix.h 17486 2008-11-06 12:41:33Z kronbichler $
+//    $Id: petsc_parallel_sparse_matrix.h 20745 2010-03-07 12:50:06Z young $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,25 +13,26 @@
 #ifndef __deal2__petsc_parallel_sparse_matrix_h
 #define __deal2__petsc_parallel_sparse_matrix_h
 
-
 #include <base/config.h>
-#include <lac/exceptions.h>
-
-#include <lac/petsc_matrix_base.h>
-
-#include <vector>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <lac/exceptions.h>
+#  include <lac/petsc_matrix_base.h>
+#  include <vector>
+
 DEAL_II_NAMESPACE_OPEN
 
 
+                                 // forward declaration
+template <typename Matrix> class BlockMatrixBase;
+
 
 namespace PETScWrappers
 {
   namespace MPI
   {
-    
+
 /**
  * Implementation of a parallel sparse matrix class based on PETSC, with rows
  * of the matrix distributed across an MPI network. All the functionality is
@@ -101,7 +102,7 @@ namespace PETScWrappers
  * which. Thus, it is not sufficient to just count the number of degrees of
  * freedom that belong to a particular process, but you have to have the
  * numbers for all processes available at all processes.
- * 
+ *
  * @ingroup PETScWrappers
  * @ingroup Matrix1
  * @author Wolfgang Bangerth, 2004
@@ -144,7 +145,7 @@ namespace PETScWrappers
                                           * empty matrix.
                                           */
         SparseMatrix ();
-      
+
                                          /**
                                           * Create a sparse matrix of
                                           * dimensions @p m times @p n, with
@@ -161,7 +162,7 @@ namespace PETScWrappers
                                           * local_row and @p local_columns
                                           * parameters, see the class
                                           * documentation.
-                                          * 
+                                          *
                                           * The @p is_symmetric flag determines
                                           * whether we should tell PETSc that
                                           * the matrix is going to be symmetric
@@ -194,7 +195,7 @@ namespace PETScWrappers
                                           * local_row and @p local_columns
                                           * parameters, see the class
                                           * documentation.
-                                          * 
+                                          *
                                           * Just as for the other
                                           * constructors: PETSc is able to
                                           * cope with the situation that more
@@ -223,7 +224,7 @@ namespace PETScWrappers
                       const unsigned int               local_columns,
                       const std::vector<unsigned int> &row_lengths,
                       const bool                       is_symmetric = false);
-        
+
                                          /**
                                           * Initialize using the given
                                           * sparsity pattern with
@@ -235,7 +236,7 @@ namespace PETScWrappers
                                           * local_columns_per_process
                                           * parameters, see the class
                                           * documentation.
-                                          * 
+                                          *
                                           * Note that PETSc can be very slow
                                           * if you do not provide it with a
                                           * good estimate of the lengths of
@@ -288,7 +289,7 @@ namespace PETScWrappers
                                           * but keep the sparsity pattern
                                           * previously used.
                                           */
-        SparseMatrix & operator = (const double d);
+        SparseMatrix & operator = (const value_type d);
 
                                          /**
                                           * Throw away the present matrix and
@@ -363,14 +364,14 @@ namespace PETScWrappers
                      const std::vector<unsigned int> &local_columns_per_process,
                      const unsigned int               this_process,
                      const bool                       preset_nonzero_locations = true);
-        
+
                                          /**
                                           * Return a reference to the MPI
                                           * communicator object in use with
                                           * this matrix.
                                           */
         virtual const MPI_Comm & get_mpi_communicator () const;
-        
+
 				     /** @addtogroup Exceptions
 				      * @{ */
 					 /**
@@ -380,7 +381,7 @@ namespace PETScWrappers
 			int, int,
 			<< "The number of local rows " << arg1
 			<< " must be larger than the total number of rows " << arg2);
-					 //@}        
+					 //@}
       private:
 
                                          /**
@@ -423,6 +424,13 @@ namespace PETScWrappers
                         const std::vector<unsigned int> &local_columns_per_process,
                         const unsigned int               this_process,
                         const bool                       preset_nonzero_locations);
+
+				       /**
+				        *  To allow calling protected
+				        *  prepare_add() and
+				        *  prepare_set().
+				        */
+	friend class BlockMatrixBase<SparseMatrix>;
     };
 
 
@@ -430,7 +438,7 @@ namespace PETScWrappers
 // -------- template and inline functions ----------
 
     inline
-    const MPI_Comm &    
+    const MPI_Comm &
     SparseMatrix::get_mpi_communicator () const
     {
       return communicator;
