@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_parallel_vector.h 14783 2007-06-18 14:52:01Z bangerth $
+//    $Id: petsc_parallel_vector.h 21358 2010-06-24 23:38:14Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,21 +13,24 @@
 #ifndef __deal2__petsc_parallel_vector_h
 #define __deal2__petsc_parallel_vector_h
 
-#include <base/config.h>
-#include <base/subscriptor.h>
-#include <lac/exceptions.h>
-#include <lac/vector.h>
-#include <lac/petsc_vector_base.h>
 
+#include <base/config.h>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <base/subscriptor.h>
+#  include <lac/exceptions.h>
+#  include <lac/vector.h>
+#  include <lac/petsc_vector_base.h>
+#  include <base/index_set.h>
+
 DEAL_II_NAMESPACE_OPEN
 
 
 
 // forward declaration
 template <typename> class Vector;
+class IndexSet;
 
 
 /*! @addtogroup PETScWrappers
@@ -44,7 +47,7 @@ namespace PETScWrappers
  */
   namespace MPI
   {
-    
+
 /**
  * Implementation of a parallel vector class based on PETSC and using MPI
  * communication to synchronise distributed operations. All the functionality
@@ -156,7 +159,7 @@ namespace PETScWrappers
                                           * vector as empty.
                                           */
         Vector ();
-      
+
                                          /**
                                           * Constructor. Set dimension to
                                           * @p n and initialize all
@@ -170,7 +173,7 @@ namespace PETScWrappers
                                           * communicator over which the
                                           * different parts of the vector
                                           * shall communicate
-                                          * 
+                                          *
                                           * The constructor is made explicit
                                           * to avoid accidents like this:
                                           * <tt>v=0;</tt>. Presumably, the user
@@ -184,13 +187,14 @@ namespace PETScWrappers
         explicit Vector (const MPI_Comm     &communicator,
                          const unsigned int  n,
                          const unsigned int  local_size);
-    
+
+
                                          /**
                                           * Copy-constructor from deal.II
                                           * vectors. Sets the dimension to that
                                           * of the given vector, and copies all
                                           * elements.
-                                          * 
+                                          *
                                           * @arg local_size denotes the size
                                           * of the chunk that shall be stored
                                           * on the present process.
@@ -205,11 +209,12 @@ namespace PETScWrappers
                          const dealii::Vector<Number> &v,
                          const unsigned int      local_size);
 
+
                                          /**
                                           * Copy-constructor the
                                           * values from a PETSc wrapper vector
                                           * class.
-                                          * 
+                                          *
                                           * @arg local_size denotes the size
                                           * of the chunk that shall be stored
                                           * on the present process.
@@ -223,6 +228,30 @@ namespace PETScWrappers
                          const VectorBase   &v,
                          const unsigned int  local_size);
 
+
+                                         /**
+					  * Constructs a new parallel PETSc
+					  * vector from an Indexset. Note that
+					  * @p local must be contiguous and
+					  * the global size of the vector is
+					  * determined by local.size(). The
+					  * global indices in @p ghost are
+					  * supplied as ghost indices that can
+					  * also be read locally.
+					  *
+					  * Note that the @p ghost IndexSet
+					  * may be empty and that any indices
+					  * already contained in @p local are
+					  * ignored during construction. That
+					  * way, the ghost parameter can equal
+					  * the set of locally relevant
+					  * degrees of freedom.
+					  */
+	explicit Vector (const MPI_Comm     &communicator,
+                         const IndexSet &  local,
+                         const IndexSet & ghost = IndexSet(0));
+
+
                                          /**
                                           * Copy the given vector. Resize the
                                           * present vector if necessary. Also
@@ -231,6 +260,7 @@ namespace PETScWrappers
                                           */
         Vector & operator = (const Vector &v);
 
+
                                          /**
 					  * Copy the given sequential
 					  * (non-distributed) vector
@@ -317,17 +347,17 @@ namespace PETScWrappers
                                           * @p communicator denotes the MPI
                                           * communicator henceforth to be used
                                           * for this vector.
-                                          * 
+                                          *
                                           * If @p fast is false, the vector
                                           * is filled by zeros. Otherwise, the
                                           * elements are left an unspecified
                                           * state.
-                                          */ 
+                                          */
         void reinit (const MPI_Comm     &communicator,
                      const unsigned int  N,
                      const unsigned int  local_size,
                      const bool          fast = false);
-    
+
                                          /**
                                           * Change the dimension to that of
                                           * the vector @p v, and also take
@@ -345,6 +375,16 @@ namespace PETScWrappers
         void reinit (const Vector &v,
                      const bool    fast = false);
 
+                                         /**
+					  * Reinit as a ghosted vector. See
+					  * constructor with same signature
+					  * for more details.
+					  */
+	void reinit (const MPI_Comm     &communicator,
+                     const IndexSet &  local,
+                     const IndexSet & ghost = IndexSet(0));
+
+
                                          /**
                                           * Return a reference to the MPI
                                           * communicator object in use with
@@ -365,6 +405,21 @@ namespace PETScWrappers
         virtual void create_vector (const unsigned int n,
                                     const unsigned int local_size);
 
+
+
+					 /**
+					  * Create a vector of global length
+					  * @p n, local size @p local_size and
+					  * with the specified ghost
+					  * indices. Note that you need to
+					  * call update_ghost_values() before
+					  * accessing those.
+					  */
+	virtual void create_vector (const unsigned int  n,
+				    const unsigned int  local_size,
+				    const IndexSet & ghostnodes);
+
+
       private:
                                          /**
                                           * Copy of the communicator object to
@@ -382,7 +437,6 @@ namespace PETScWrappers
  * of the C++ standard library which uses a temporary object. The
  * function simply exchanges the data of the two vectors.
  *
- * @ingroup PETScWrappers
  * @relates PETScWrappers::MPI::Vector
  * @author Wolfgang Bangerth, 2004
  */
@@ -407,8 +461,8 @@ namespace PETScWrappers
       *this = v;
     }
 
-  
-  
+
+
     inline
     Vector &
     Vector::operator = (const PetscScalar s)
@@ -417,7 +471,7 @@ namespace PETScWrappers
 
       return *this;
     }
-  
+
 
 
     inline
@@ -428,10 +482,10 @@ namespace PETScWrappers
                                        // then first resize the present one
       if (size() != v.size())
         reinit (v.communicator, v.size(), v.local_size(), true);
-    
+
       const int ierr = VecCopy (v.vector, vector);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
-    
+
       return *this;
     }
 
@@ -440,7 +494,7 @@ namespace PETScWrappers
     template <typename number>
     inline
     Vector &
-    Vector::operator = (const dealii::Vector<number> &v) 
+    Vector::operator = (const dealii::Vector<number> &v)
     {
       Assert (size() == v.size(),
               ExcDimensionMismatch (size(), v.size()));
