@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_sparse_matrix.h 17486 2008-11-06 12:41:33Z kronbichler $
+//    $Id: petsc_sparse_matrix.h 20107 2009-11-13 15:40:05Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,14 +15,16 @@
 
 
 #include <base/config.h>
-#include <lac/exceptions.h>
-#include <lac/petsc_matrix_base.h>
-
-#include <vector>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <lac/exceptions.h>
+#  include <lac/petsc_matrix_base.h>
+#  include <vector>
+
 DEAL_II_NAMESPACE_OPEN
+                                 // forward declaration
+template <typename MatrixType> class BlockMatrixBase;
 
 
 namespace PETScWrappers
@@ -164,26 +166,10 @@ namespace PETScWrappers
                                         * significantly more efficient to
                                         * get memory allocation right from
                                         * the start.
-                                        *
-                                        * Despite the fact that it would
-                                        * seem to be an obvious win, setting
-                                        * the @p preset_nonzero_locations
-                                        * flag to @p true doesn't seem to
-                                        * accelerate program. Rather on the
-                                        * contrary, it seems to be able to
-                                        * slow down entire programs
-                                        * somewhat. This is suprising, since
-                                        * we can use efficient function
-                                        * calls into PETSc that allow to
-                                        * create multiple entries at once;
-                                        * nevertheless, given the fact that
-                                        * it is inefficient, the respective
-                                        * flag has a default value equal to
-                                        * @p false.
                                         */
       template <typename SparsityType>
       SparseMatrix (const SparsityType &sparsity_pattern,
-                    const bool          preset_nonzero_locations = false);
+                    const bool          preset_nonzero_locations = true);
 
                                        /**
                                         * This operator assigns a scalar to
@@ -279,7 +265,7 @@ namespace PETScWrappers
                                         */
       template <typename SparsityType>
       void reinit (const SparsityType &sparsity_pattern,
-                   const bool          preset_nonzero_locations = false);
+                   const bool          preset_nonzero_locations = true);
 
                                        /**
                                         * Return a reference to the MPI
@@ -318,6 +304,13 @@ namespace PETScWrappers
       template <typename SparsityType>
       void do_reinit (const SparsityType &sparsity_pattern,
                       const bool          preset_nonzero_locations);
+                      
+				       /**
+				        *  To allow calling protected
+				        *  prepare_add() and
+				        *  prepare_set().
+				        */                       
+      friend class BlockMatrixBase<SparseMatrix>;
   };
 }
 
