@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_vector_base.h 18534 2009-03-31 13:13:45Z young $
+//    $Id: petsc_vector_base.h 21162 2010-06-07 20:47:13Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,15 +15,17 @@
 
 
 #include <base/config.h>
-#include <base/subscriptor.h>
-#include <lac/exceptions.h>
-
-#include <vector>
-#include <utility>
 
 #ifdef DEAL_II_USE_PETSC
 
-#include <petscvec.h>
+#  include <base/subscriptor.h>
+#  include <lac/exceptions.h>
+
+#  include <vector>
+#  include <utility>
+
+#  include <petscvec.h>
+#  include <base/index_set.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -47,7 +49,7 @@ namespace PETScWrappers
 				   /**
 				    * @cond internal
 				    */
-  
+
 /**
  * A namespace for internal implementation details of the PETScWrapper
  * members.
@@ -84,7 +86,7 @@ namespace PETScWrappers
                                           */
         VectorReference (const VectorBase  &vector,
                          const unsigned int index);
-        
+
       public:
                                          /**
                                           * This looks like a copy operator,
@@ -107,7 +109,17 @@ namespace PETScWrappers
                                           * the assignment operator const.
                                           */
         const VectorReference & operator = (const VectorReference &r) const;
-        
+
+					 /**
+					  * The same function as above, but
+					  * for non-const reference
+					  * objects. The function is needed
+					  * since the compiler might otherwise
+					  * automatically generate a copy
+					  * operator for non-const objects.
+					  */
+        VectorReference & operator = (const VectorReference &r);
+
                                          /**
                                           * Set the referenced element of the
                                           * vector to <tt>s</tt>.
@@ -162,7 +174,7 @@ namespace PETScWrappers
                         << " of a distributed vector, but only elements "
                         << arg2 << " through " << arg3
                         << " are stored locally and can be accessed.");
-        
+
       private:
                                          /**
                                           * Point to the vector we are
@@ -187,7 +199,7 @@ namespace PETScWrappers
                                    /**
                                     * @endcond
                                     */
-  
+
 
 /**
  * Base class for all vector classes that are implemented on top of the PETSc
@@ -213,7 +225,7 @@ namespace PETScWrappers
  * @ingroup PETScWrappers
  * @author Wolfgang Bangerth, 2004
  */
-  class VectorBase
+  class VectorBase : public Subscriptor
   {
     public:
                                        /**
@@ -224,7 +236,7 @@ namespace PETScWrappers
                                         * class.
                                         */
       typedef PetscScalar               value_type;
-      typedef PetscScalar               real_type;
+      typedef PetscReal                 real_type;
       typedef size_t                    size_type;
       typedef internal::VectorReference reference;
       typedef const internal::VectorReference const_reference;
@@ -235,7 +247,7 @@ namespace PETScWrappers
                                         * to initialize the data.
                                         */
       VectorBase ();
-      
+
                                        /**
                                         * Copy constructor. Sets the dimension
                                         * to that of the given vector, and
@@ -256,6 +268,10 @@ namespace PETScWrappers
                                         * is necessary after writing into a
                                         * vector element-by-element and before
                                         * anything else can be done on it.
+					*
+					* See @ref GlossCompress "Compressing distributed objects"
+					* for more information.
+					* more information.
                                         */
       void compress ();
 
@@ -279,8 +295,8 @@ namespace PETScWrappers
                                         * other values is deprecated and may
                                         * be disallowed in the future.
                                         */
-      VectorBase & operator = (const PetscScalar s);    
-      
+      VectorBase & operator = (const PetscScalar s);
+
                                        /**
                                         * Test for equality. This function
                                         * assumes that the present vector and
@@ -290,7 +306,7 @@ namespace PETScWrappers
                                         * sense anyway.
                                         */
       bool operator == (const VectorBase &v) const;
-    
+
                                        /**
                                         * Test for inequality. This function
                                         * assumes that the present vector and
@@ -383,7 +399,30 @@ namespace PETScWrappers
                                         */
       void add (const std::vector<unsigned int> &indices,
                 const std::vector<PetscScalar>  &values);
-      
+
+				       /**
+				        * This is a second collective
+				        * add operation. As a
+				        * difference, this function
+				        * takes a deal.II vector of
+				        * values.
+				        */
+      void add (const std::vector<unsigned int>     &indices,
+		const ::dealii::Vector<PetscScalar> &values);
+
+				      /**
+				       * Take an address where
+				       * <tt>n_elements</tt> are stored
+				       * contiguously and add them into
+				       * the vector. Handles all cases
+				       * which are not covered by the
+				       * other two <tt>add()</tt>
+				       * functions above.
+				       */
+      void add (const unsigned int  n_elements,
+		const unsigned int *indices,
+		const PetscScalar  *values);
+
                                        /**
                                         * Return the scalar product of two
                                         * vectors. The vectors must have the
@@ -421,7 +460,7 @@ namespace PETScWrappers
                                         * powers of the absolute values
                                         * of the elements.
                                         */
-      real_type lp_norm (const PetscScalar p) const;
+      real_type lp_norm (const real_type p) const;
 
                                        /**
                                         * Maximum absolute value of the
@@ -437,12 +476,19 @@ namespace PETScWrappers
                                         */
       real_type normalize () const;
 
+                                       /**
+                                        * Return vector component with
+                                        * the minimal magnitude.
+                                        */
+      real_type min () const;
+
                                        /**
                                         * Return vector component with
                                         * the maximal magnitude.
                                         */
       real_type max () const;
 
+
                                        /**
                                         * Replace every element in a
                                         * vector with its absolute
@@ -461,7 +507,7 @@ namespace PETScWrappers
 					* The model for this function
 					* should be similer to add ().
                                         */
-      VectorBase & mult (); 
+      VectorBase & mult ();
 
                                        /**
                                         * Return whether the vector contains
@@ -482,13 +528,13 @@ namespace PETScWrappers
                                         * really all positive (or zero).
                                         */
       bool is_non_negative () const;
-      
+
                                        /**
                                         * Multiply the entire vector by a
                                         * fixed factor.
                                         */
       VectorBase & operator *= (const PetscScalar factor);
-    
+
                                        /**
                                         * Divide the entire vector by a
                                         * fixed factor.
@@ -513,26 +559,26 @@ namespace PETScWrappers
                                         * scalar and not a vector.
                                         */
       void add (const PetscScalar s);
-    
+
                                        /**
                                         * Simple vector addition, equal to the
                                         * <tt>operator +=</tt>.
                                         */
       void add (const VectorBase &V);
-    
+
                                        /**
                                         * Simple addition of a multiple of a
                                         * vector, i.e. <tt>*this += a*V</tt>.
                                         */
       void add (const PetscScalar a, const VectorBase &V);
-    
+
                                        /**
                                         * Multiple addition of scaled vectors,
                                         * i.e. <tt>*this += a*V+b*W</tt>.
                                         */
       void add (const PetscScalar a, const VectorBase &V,
                 const PetscScalar b, const VectorBase &W);
-    
+
                                        /**
                                         * Scaling and simple vector addition,
                                         * i.e.
@@ -540,7 +586,7 @@ namespace PETScWrappers
                                         */
       void sadd (const PetscScalar s,
                  const VectorBase     &V);
-    
+
                                        /**
                                         * Scaling and simple addition, i.e.
                                         * <tt>*this = s*(*this)+a*V</tt>.
@@ -548,7 +594,7 @@ namespace PETScWrappers
       void sadd (const PetscScalar s,
                  const PetscScalar a,
                  const VectorBase     &V);
-    
+
                                        /**
                                         * Scaling and multiple addition.
                                         */
@@ -557,7 +603,7 @@ namespace PETScWrappers
                  const VectorBase     &V,
                  const PetscScalar b,
                  const VectorBase     &W);
-    
+
                                        /**
                                         * Scaling and multiple addition.
                                         * <tt>*this = s*(*this)+a*V + b*W + c*X</tt>.
@@ -566,10 +612,10 @@ namespace PETScWrappers
                  const PetscScalar a,
                  const VectorBase     &V,
                  const PetscScalar b,
-                 const VectorBase     &W, 
+                 const VectorBase     &W,
                  const PetscScalar c,
                  const VectorBase     &X);
-    
+
                                        /**
                                         * Scale each element of this
                                         * vector by the corresponding
@@ -580,12 +626,12 @@ namespace PETScWrappers
                                         * diagonal scaling matrix.
                                         */
       void scale (const VectorBase &scaling_factors);
-    
+
                                        /**
                                         * Assignment <tt>*this = a*V</tt>.
                                         */
       void equ (const PetscScalar a, const VectorBase &V);
-    
+
                                        /**
                                         * Assignment <tt>*this = a*V + b*W</tt>.
                                         */
@@ -611,6 +657,14 @@ namespace PETScWrappers
       void ratio (const VectorBase &a,
                   const VectorBase &b);
 
+                                       /**
+					* Updates the ghost values of this
+					* vector. This is necessary after any
+					* modification before reading ghost
+					* values.
+					*/
+      void update_ghost_values() const;
+
                                        /**
                                         * Print to a
                                         * stream. @p precision denotes
@@ -654,7 +708,7 @@ namespace PETScWrappers
                                         * to standard functions.
                                         */
       void swap (VectorBase &v);
-      
+
                                        /**
                                         * Conversion operator to gain access
                                         * to the underlying PETSc type. If you
@@ -674,7 +728,7 @@ namespace PETScWrappers
 					* for this class).
 					*/
       unsigned int memory_consumption () const;
-      
+
     protected:
                                        /**
                                         * A generic vector object in
@@ -683,6 +737,23 @@ namespace PETScWrappers
                                         */
       Vec vector;
 
+				       /**
+					* Denotes if this vector has ghost
+					* indices associated with it. This
+					* means that at least one of the
+					* processes in a parallel programm has
+					* at least one ghost index.
+					*/
+      bool ghosted;
+
+                                       /**
+					* This vector contains the global
+					* indices of the ghost values. The
+					* location in this vector denotes the
+					* local numbering, which is used in
+					* PETSc.
+					*/
+      IndexSet ghost_indices;
 
                                        /**
                                         * PETSc doesn't allow to mix additions
@@ -734,21 +805,23 @@ namespace PETScWrappers
                                         * @p add_values flag set to the
                                         * corresponding value.
                                         */
-      void do_set_add_operation (const std::vector<unsigned int> &indices,
-				 const std::vector<PetscScalar>  &values,
+      void do_set_add_operation (const unsigned int  n_elements,
+				 const unsigned int *indices,
+				 const PetscScalar  *values,
 				 const bool add_values);
+
+
   };
 
 
 
-// ------------------- inline and template functions --------------  
+// ------------------- inline and template functions --------------
 
 /**
  * Global function @p swap which overloads the default implementation
  * of the C++ standard library which uses a temporary object. The
  * function simply exchanges the data of the two vectors.
  *
- * @ingroup PETScWrappers
  * @relates PETScWrappers::VectorBase
  * @author Wolfgang Bangerth, 2004
  */
@@ -784,7 +857,22 @@ namespace PETScWrappers
     }
 
 
-    
+
+    inline
+    VectorReference &
+    VectorReference::operator = (const VectorReference &r)
+    {
+                                       // as explained in the class
+                                       // documentation, this is not the copy
+                                       // operator. so simply pass on to the
+                                       // "correct" assignment operator
+      *this = static_cast<PetscScalar> (r);
+
+      return *this;
+    }
+
+
+
     inline
     const VectorReference &
     VectorReference::operator = (const PetscScalar &value) const
@@ -798,15 +886,19 @@ namespace PETScWrappers
           ierr = VecAssemblyEnd (vector);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
         }
-      
+
+#ifdef PETSC_USE_64BIT_INDICES
+      const PetscInt petsc_i = index;
+#else
       const signed int petsc_i = index;
-      
+#endif
+
       const int ierr
         = VecSetValues (vector, 1, &petsc_i, &value, INSERT_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       vector.last_action = VectorBase::LastAction::insert;
-      
+
       return *this;
     }
 
@@ -835,16 +927,20 @@ namespace PETScWrappers
                                        // PETScWrappers::MPI::Vector), but we
                                        // can save some work if the addend is
                                        // zero
-      if (value == 0)
+      if (value == PetscScalar())
         return *this;
-      
+
                                        // use the PETSc function to add something
+#ifdef PETSC_USE_64BIT_INDICES
+      const PetscInt petsc_i = index;
+#else
       const signed int petsc_i = index;
+#endif
       const int ierr
         = VecSetValues (vector, 1, &petsc_i, &value, ADD_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-      
+
       return *this;
     }
 
@@ -865,7 +961,7 @@ namespace PETScWrappers
 
           vector.last_action = VectorBase::LastAction::add;
         }
-      
+
                                        // we have to do above actions in any
                                        // case to be consistent with the MPI
                                        // communication model (see the
@@ -873,16 +969,20 @@ namespace PETScWrappers
                                        // PETScWrappers::MPI::Vector), but we
                                        // can save some work if the addend is
                                        // zero
-      if (value == 0)
+      if (value == PetscScalar())
         return *this;
 
                                        // use the PETSc function to add something
+#ifdef PETSC_USE_64BIT_INDICES
+      const PetscInt petsc_i = index;
+#else
       const signed int petsc_i = index;
+#endif
       const PetscScalar subtractand = -value;
       const int ierr
         = VecSetValues (vector, 1, &petsc_i, &subtractand, ADD_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
       return *this;
     }
 
@@ -903,7 +1003,7 @@ namespace PETScWrappers
 
           vector.last_action = VectorBase::LastAction::insert;
         }
-      
+
                                        // we have to do above actions in any
                                        // case to be consistent with the MPI
                                        // communication model (see the
@@ -914,15 +1014,19 @@ namespace PETScWrappers
       if (value == 1.)
         return *this;
 
+#ifdef PETSC_USE_64BIT_INDICES
+      const PetscInt petsc_i = index;
+#else
       const signed int petsc_i = index;
+#endif
 
       const PetscScalar new_value
         = static_cast<PetscScalar>(*this) * value;
-      
+
       const int ierr
         = VecSetValues (vector, 1, &petsc_i, &new_value, INSERT_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
       return *this;
     }
 
@@ -943,7 +1047,7 @@ namespace PETScWrappers
 
           vector.last_action = VectorBase::LastAction::insert;
         }
-      
+
                                        // we have to do above actions in any
                                        // case to be consistent with the MPI
                                        // communication model (see the
@@ -954,30 +1058,39 @@ namespace PETScWrappers
       if (value == 1.)
         return *this;
 
+ #ifdef PETSC_USE_64BIT_INDICES
+      const PetscInt petsc_i = index;
+#else
       const signed int petsc_i = index;
+#endif
 
       const PetscScalar new_value
         = static_cast<PetscScalar>(*this) / value;
-      
+
       const int ierr
         = VecSetValues (vector, 1, &petsc_i, &new_value, INSERT_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
       return *this;
     }
   }
-  
+
 
 
   inline
   bool
   VectorBase::in_local_range (const unsigned int index) const
   {
+#ifdef PETSC_USE_64BIT_INDICES
+    PetscInt begin, end;
+#else
     int begin, end;
+#endif
+
     const int ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
 					   &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-    
+
     return ((index >= static_cast<unsigned int>(begin)) &&
             (index < static_cast<unsigned int>(end)));
   }
@@ -999,7 +1112,7 @@ namespace PETScWrappers
   {
     return static_cast<PetscScalar>(internal::VectorReference (*this, index));
   }
-  
+
 
 #endif // DOXYGEN
 }
