@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: pointer_matrix.h 15739 2008-02-13 21:39:00Z kanschat $
+//    $Id: pointer_matrix.h 19894 2009-10-15 22:19:51Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -236,7 +236,7 @@ class PointerMatrix : public PointerMatrixBase<VECTOR>
 				     /**
 				      * The pointer to the actual matrix.
 				      */
-    SmartPointer<const MATRIX> m;
+    SmartPointer<const MATRIX,PointerMatrix<MATRIX,VECTOR> > m;
 };
 
 
@@ -364,12 +364,12 @@ class PointerMatrixAux : public PointerMatrixBase<VECTOR>
 				      * Object for getting the
 				      * auxiliary vector.
 				      */
-    mutable SmartPointer<VectorMemory<VECTOR> > mem;
+    mutable SmartPointer<VectorMemory<VECTOR>,PointerMatrixAux<MATRIX,VECTOR> > mem;
     
 				     /**
 				      * The pointer to the actual matrix.
 				      */
-    SmartPointer<const MATRIX> m;
+    SmartPointer<const MATRIX,PointerMatrixAux<MATRIX,VECTOR> > m;
 };
 
 
@@ -504,7 +504,7 @@ class PointerMatrixVector : public PointerMatrixBase<Vector<number> >
 				     /**
 				      * The pointer to the actual matrix.
 				      */
-    SmartPointer<const Vector<number> > m;
+    SmartPointer<const Vector<number>,PointerMatrixVector<number> > m;
 };
 
 
