@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: precondition.h 18727 2009-04-24 00:01:42Z bangerth $
+//    $Id: precondition.h 20819 2010-03-12 23:15:56Z allmaras $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -16,9 +16,10 @@
 // This file contains simple preconditioners.
 
 #include <base/config.h>
-#include <lac/vector_memory.h>
 #include <base/smartpointer.h>
 #include <base/template_constraints.h>
+#include <lac/tridiagonal_matrix.h>
+#include <lac/vector_memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -43,7 +44,7 @@ template <typename number> class SparseMatrix;
 	    PreconditionIdentity());
  @endverbatim
  *
- * See the @ref step_3 "step-3" tutorial program for an example and
+ * See the step-3 tutorial program for an example and
  * additional explanations.
  *
  * Alternatively, the IdentityMatrix class can be used to precondition
@@ -54,6 +55,33 @@ template <typename number> class SparseMatrix;
 class PreconditionIdentity : public Subscriptor
 {
   public:
+
+				     /**
+				      * This function is only 
+                                      * present to 
+                                      * provide the interface of
+                                      * a precondtioner to be 
+                                      * handed to a smoother.
+                                      * This does nothing.
+				      */
+  struct AdditionalData
+  {
+                                       /**
+					* Constructor.
+					*/
+    AdditionalData (){}
+  };
+
+				     /**
+				      * The matrix
+				      * argument is ignored and here
+				      * just for compatibility with
+				      * more complex preconditioners.
+				      */
+  template <class MATRIX>
+  void initialize (const MATRIX         &matrix,
+		   const AdditionalData &additional_data = AdditionalData());
+
 				     /**
 				      * Apply preconditioner.
 				      */
@@ -85,6 +113,16 @@ class PreconditionIdentity : public Subscriptor
 				      */
     template<class VECTOR>
     void Tvmult_add (VECTOR&, const VECTOR&) const;
+
+				     /**
+				      * This function is only 
+                                      * present to 
+                                      * provide the interface of
+                                      * a precondtioner to be 
+                                      * handed to a smoother.
+                                      * This does nothing.
+				      */
+    void clear (){}
 };
 
 
@@ -96,7 +134,7 @@ class PreconditionIdentity : public Subscriptor
  *
  * In Krylov-space methods, this preconditioner should not have any
  * effect. Using SolverRichardson, the two relaxation parameters will
- * be just multiplied. Still, this class is usefull in multigrid
+ * be just multiplied. Still, this class is useful in multigrid
  * smoother objects (MGSmootherRelaxation).
  *
  * @author Guido Kanschat, 2005
@@ -135,7 +173,7 @@ class PreconditionRichardson : public Subscriptor
 				     /**
 				      * Change the relaxaton parameter.
 				      */
-    void initialize (const AdditionalData parameters);
+    void initialize (const AdditionalData &parameters);
 
 				     /**
 				      * Change the relaxaton parameter
@@ -147,7 +185,7 @@ class PreconditionRichardson : public Subscriptor
 				      */
     template <class MATRIX>
     void initialize (const MATRIX&,
-		     const AdditionalData parameters);
+		     const AdditionalData &parameters);
 
 				     /**
 				      * Apply preconditioner.
@@ -179,7 +217,17 @@ class PreconditionRichardson : public Subscriptor
 				      */
     template<class VECTOR>
     void Tvmult_add (VECTOR&, const VECTOR&) const;
-    
+
+				     /**
+				      * This function is only 
+				      * present to 
+				      * provide the interface of
+				      * a precondtioner to be 
+				      * handed to a smoother.
+				      * This does nothing.
+				      */
+    void clear (){}
+
   private:
 				     /**
 				      * The relaxation parameter
@@ -240,7 +288,7 @@ class PreconditionUseMatrix : public Subscriptor
 				      * function of the matrix.
 				      */
     typedef void ( MATRIX::* function_ptr)(VECTOR&, const VECTOR&) const;
-    
+
 				     /**
 				      * Constructor.
 				      * This constructor stores a
@@ -252,7 +300,7 @@ class PreconditionUseMatrix : public Subscriptor
 				      */
     PreconditionUseMatrix(const MATRIX      &M,
 			  const function_ptr method);
-    
+
 				     /**
 				      * Execute preconditioning. Calls the
 				      * function passed to the constructor
@@ -267,7 +315,7 @@ class PreconditionUseMatrix : public Subscriptor
 				      * Pointer to the matrix in use.
 				      */
     const MATRIX &matrix;
-    
+
 				     /**
 				      * Pointer to the preconditioning
 				      * function.
@@ -302,9 +350,9 @@ class PreconditionRelaxation : public Subscriptor
 					 /**
 					  * Relaxation parameter.
 					  */
-	double relaxation;	
+	double relaxation;
     };
-    
+
 				     /**
 				      * Initialize matrix and
 				      * relaxation parameter. The
@@ -316,19 +364,19 @@ class PreconditionRelaxation : public Subscriptor
 				      * reasons. It defaults to 1.
 				      */
     void initialize (const MATRIX &A,
-		     AdditionalData parameters = AdditionalData());
-    
+		     const AdditionalData & parameters = AdditionalData());
+
 				     /**
 				      * Release the matrix and reset
 				      * its pointer.
 				      */
     void clear();
-    
+
   protected:
 				     /**
 				      * Pointer to the matrix object.
 				      */
-    SmartPointer<const MATRIX> A;
+    SmartPointer<const MATRIX, PreconditionRelaxation<MATRIX> > A;
 
 				     /**
 				      * Relaxation parameter.
@@ -382,11 +430,47 @@ class PreconditionJacobi : public PreconditionRelaxation<MATRIX>
 				      */
     template<class VECTOR>
     void Tvmult (VECTOR&, const VECTOR&) const;
+
+				     /**
+				      * Perform one step of the
+				      * preconditioned Richardson
+				      * iteration.
+				      */
+    template<class VECTOR>
+    void step (VECTOR& x, const VECTOR& rhs) const;
+
+				     /**
+				      * Perform one transposed step of
+				      * the preconditioned Richardson
+				      * iteration.
+				      */
+    template<class VECTOR>
+    void Tstep (VECTOR& x, const VECTOR& rhs) const;
 };
 
 
 /**
- * SOR preconditioner using matrix built-in function.  The MATRIX
+ * SOR preconditioner using matrix built-in function.
+ *
+ * Assuming the matrix <i>A = D + L + U</i> is split into its diagonal
+ * <i>D</i> as well as the strict lower and upper triangles <i>L</i>
+ * and <i>U</i>, then the SOR preconditioner with relaxation parameter
+ * <i>r</i> is
+ * @f[
+ *  P^{-1} = r (D+rL)^{-1}.
+ * @f]
+ * It is this operator <i>P<sup>-1</sup></i>, which is implemented by
+ * vmult() through forward substitution. Analogously, Tvmult()
+ * implements the operation of <i>r(D+rU)<sup>-1</sup></i>.
+ *
+ * The SOR iteration itself can be directly written as
+ * @f[
+ *  x^{k+1} = x^k - r D^{-1} \bigl(L x^{k+1} + U x^k - b\bigr).
+ * @f]
+ * Using the right hand side <i>b</i> and the previous iterate
+ * <i>x</i>, this is the operation implemented by step().
+ *
+ * The MATRIX
  * class used is required to have functions
  * <tt>precondition_SOR(VECTOR&, const VECTOR&, double)</tt> and
  * <tt>precondition_TSOR(VECTOR&, const VECTOR&, double)</tt>.
@@ -427,6 +511,22 @@ class PreconditionSOR : public PreconditionRelaxation<MATRIX>
 				      */
     template<class VECTOR>
     void Tvmult (VECTOR&, const VECTOR&) const;
+
+				     /**
+				      * Perform one step of the
+				      * preconditioned Richardson
+				      * iteration.
+				      */
+    template<class VECTOR>
+    void step (VECTOR& x, const VECTOR& rhs) const;
+
+				     /**
+				      * Perform one transposed step of
+				      * the preconditioned Richardson
+				      * iteration.
+				      */
+    template<class VECTOR>
+    void Tstep (VECTOR& x, const VECTOR& rhs) const;
 };
 
 
@@ -466,7 +566,7 @@ class PreconditionSSOR : public PreconditionRelaxation<MATRIX>
 				      */
     typedef PreconditionRelaxation<MATRIX> BaseClass;
 
-	
+
 				     /**
 				      * Initialize matrix and
 				      * relaxation parameter. The
@@ -478,7 +578,7 @@ class PreconditionSSOR : public PreconditionRelaxation<MATRIX>
 				      * reasons. It defaults to 1.
 				      */
     void initialize (const MATRIX &A,
-		     typename BaseClass::AdditionalData parameters = typename BaseClass::AdditionalData());
+		     const typename BaseClass::AdditionalData &parameters = typename BaseClass::AdditionalData());
 
 				     /**
 				      * Apply preconditioner.
@@ -496,6 +596,23 @@ class PreconditionSSOR : public PreconditionRelaxation<MATRIX>
     template<class VECTOR>
     void Tvmult (VECTOR&, const VECTOR&) const;
 
+
+				     /**
+				      * Perform one step of the
+				      * preconditioned Richardson
+				      * iteration
+				      */
+    template<class VECTOR>
+    void step (VECTOR& x, const VECTOR& rhs) const;
+
+				     /**
+				      * Perform one transposed step of
+				      * the preconditioned Richardson
+				      * iteration.
+				      */
+    template<class VECTOR>
+    void Tstep (VECTOR& x, const VECTOR& rhs) const;
+
   private:
 				     /**
 				      * An array that stores for each matrix
@@ -563,9 +680,9 @@ class PreconditionPSOR : public PreconditionRelaxation<MATRIX>
     void initialize (const MATRIX &A,
 		     const std::vector<unsigned int> &permutation,
 		     const std::vector<unsigned int> &inverse_permutation,
-		     typename PreconditionRelaxation<MATRIX>::AdditionalData
+		     const typename PreconditionRelaxation<MATRIX>::AdditionalData &
 		     parameters = typename PreconditionRelaxation<MATRIX>::AdditionalData());
-    
+
 				     /**
 				      * Apply preconditioner.
 				      */
@@ -659,7 +776,7 @@ class PreconditionLACSolver : public Subscriptor
     void initialize (SOLVER&,
 		     const MATRIX&,
 		     const PRECONDITION&);
-    
+
 				     /**
 				      * Execute preconditioning.
 				      */
@@ -670,17 +787,17 @@ class PreconditionLACSolver : public Subscriptor
 				     /**
 				      * The solver object to use.
 				      */
-    SmartPointer<SOLVER> solver;
+    SmartPointer<SOLVER,PreconditionLACSolver<SOLVER,MATRIX,PRECONDITION> > solver;
 
 				     /**
 				      * The matrix in use.
 				      */
-    SmartPointer<const MATRIX> matrix;
-    
+    SmartPointer<const MATRIX,PreconditionLACSolver<SOLVER,MATRIX,PRECONDITION> > matrix;
+
 				     /**
 				      * The preconditioner to use.
 				      */
-    SmartPointer<const PRECONDITION> precondition;
+    SmartPointer<const PRECONDITION,PreconditionLACSolver<SOLVER,MATRIX,PRECONDITION> > precondition;
 };
 
 
@@ -745,12 +862,226 @@ class PreconditionedMatrix : public Subscriptor
 };
 
 
-    
+
+/**
+ * Preconditioning with a Chebyshev polynomial for symmetric positive
+ * definite matrices. This preconditioner is similar to a Jacobi
+ * preconditioner if the degree variable is set to one, otherwise some
+ * higher order polynomial corrections are used. This preconditioner needs
+ * access to the diagonal of the matrix its acts on and needs a respective
+ * <tt>vmult</tt> implemention. However, it does not need to explicitly know
+ * the matrix entries.
+ *
+ * This class is useful e.g. in multigrid smoother objects, since it is
+ * trivially %parallel (assuming that matrix-vector products are %parallel).
+ *
+ * @author Martin Kronbichler, 2009
+ */
+template <class MATRIX=SparseMatrix<double>, class VECTOR=Vector<double> >
+class PreconditionChebyshev : public Subscriptor
+{
+  public:
+				     /**
+				      * Standardized data struct to
+				      * pipe additional parameters
+				      * to the preconditioner.
+				      */
+    struct AdditionalData
+    {
+					 /**
+					  * Constructor.
+					  */
+	AdditionalData (const unsigned int degree              = 0,
+			const double       smoothing_range     = 0.,
+			const bool         nonzero_starting    = false,
+			const unsigned int eig_cg_n_iterations = 8,
+			const double       eig_cg_residual     = 1e-2);
+
+					 /**
+					  * This determines the degree of the
+					  * Chebyshev polynomial. The degree
+					  * of the polynomial gives the number
+					  * of matrix-vector products to be
+					  * performed for one application of
+					  * the vmult() operation. Degree zero
+					  * corresponds to a damped Jacobi
+					  * method.
+					  */
+	unsigned int degree;
+
+					 /**
+					  * This sets the range between the
+					  * largest eigenvalue in the matrix
+					  * and the smallest eigenvalue to be
+					  * treated. If the parameter is zero,
+					  * an estimate for the largest and
+					  * for the smallest eigenvalue will
+					  * be calculated
+					  * internally. Otherwise, the
+					  * Chebyshev polynomial will act in
+					  * the interval
+					  * $[\lambda_\mathrm{max}/
+					  * \tt{smoothing\_range},
+					  * \lambda_\mathrm{max}]$, where
+					  * $\lambda_\mathrm{max}$ is an
+					  * estimate of the maximum eigenvalue
+					  * of the matrix. A choice of
+					  * <tt>smoothing_range</tt> between 5
+					  * and 20 is useful in case the
+					  * preconditioner is used as a
+					  * smoother in multigrid.
+					  */
+	double smoothing_range;
+
+					 /**
+					  * When this flag is set to
+					  * <tt>true</tt>, it enables the
+					  * method <tt>vmult(dst, src)</tt> to
+					  * use non-zero data in the vector
+					  * <tt>dst</tt>, appending to it the
+					  * Chebyshev corrections. This can be
+					  * useful in some situations
+					  * (e.g. when used for high-frequency
+					  * error smoothing in a multigrid
+					  * algorithm), but not the way the
+					  * solver classes expect a
+					  * preconditioner to work (where one
+					  * ignores the content in
+					  * <tt>dst</tt> for the
+					  * preconditioner application).
+					  */
+	bool nonzero_starting;
+
+					 /**
+					  * Maximum number of CG iterations
+					  * performed for finding the maximum
+					  * eigenvalue.
+					  */
+	unsigned int eig_cg_n_iterations;
+
+					 /**
+					  * Tolerance for CG iterations
+					  * performed for finding the maximum
+					  * eigenvalue.
+					  */
+	double eig_cg_residual;
+
+					 /**
+					  * Stores the inverse of the diagonal
+					  * of the underlying matrix.
+					  */
+	VECTOR matrix_diagonal_inverse;
+    };
+
+    PreconditionChebyshev ();
+
+				     /**
+				      * Initialize function. Takes the
+				      * matrix which is used to form the
+				      * preconditioner, and additional
+				      * flags if there are any. This
+				      * function works only if the input
+				      * matrix has an operator
+				      * <tt>el(i,i)</tt> for accessing all
+				      * the elements in the
+				      * diagonal. Alternatively, the
+				      * diagonal can be supplied with the
+				      * help of the AdditionalData field.
+				      *
+				      * This function calculates an
+				      * estimate of the eigenvalue range
+				      * of the matrix weighted by its
+				      * diagonal using a modified CG
+				      * iteration.
+				      */
+    void initialize (const MATRIX         &matrix,
+		     const AdditionalData &additional_data = AdditionalData());
+
+				     /**
+				      * Computes the action of the
+				      * preconditioner on <tt>src</tt>,
+				      * storing the result in
+				      * <tt>dst</tt>.
+				      */
+    void vmult (VECTOR       &dst,
+		const VECTOR &src) const;
+
+				     /**
+				      * Computes the action of the
+				      * transposed preconditioner on
+				      * <tt>src</tt>, storing the result
+				      * in <tt>dst</tt>.
+				      */
+    void Tvmult (VECTOR       &dst,
+		 const VECTOR &src) const;
+
+				     /**
+				      * Resets the preconditioner.
+				      */
+    void clear ();
+
+  private:
+
+				     /**
+				      * A pointer to the underlying
+				      * matrix.
+				      */
+    SmartPointer<const MATRIX,PreconditionChebyshev<MATRIX,VECTOR> > matrix_ptr;
+
+				     /**
+				      * Internal vector used for the
+				      * <tt>vmult</tt> operation.
+				      */
+    mutable VECTOR update1;
+
+				     /**
+				      * Internal vector used for the
+				      * <tt>vmult</tt> operation.
+				      */
+    mutable VECTOR update2;
+
+				     /**
+				      * Stores the additional data
+				      * provided to the initialize
+				      * function.
+				      */
+    AdditionalData data;
+
+				     /**
+				      * Average of the largest and
+				      * smallest eigenvalue under
+				      * consideration.
+				      */
+    double theta;
+
+				     /**
+				      * Half the interval length between
+				      * the largest and smallest
+				      * eigenvalue under consideration.
+				      */
+    double delta;
+
+				     /**
+				      * Stores whether the preconditioner
+				      * has been set up.
+				      */
+    bool is_initialized;
+};
+
+
+
 /*@}*/
 /* ---------------------------------- Inline functions ------------------- */
 
 #ifndef DOXYGEN
 
+template <class MATRIX>
+inline void
+PreconditionIdentity::initialize (
+  const MATRIX&,
+  const PreconditionIdentity::AdditionalData&)
+{}
+
 
 template<class VECTOR>
 inline void
@@ -807,7 +1138,7 @@ PreconditionRichardson::PreconditionRichardson ()
 
 inline void
 PreconditionRichardson::initialize (
-  const PreconditionRichardson::AdditionalData parameters)
+  const PreconditionRichardson::AdditionalData &parameters)
 {
   relaxation = parameters.relaxation;
 }
@@ -818,7 +1149,7 @@ template <class MATRIX>
 inline void
 PreconditionRichardson::initialize (
   const MATRIX&,
-  const PreconditionRichardson::AdditionalData parameters)
+  const PreconditionRichardson::AdditionalData &parameters)
 {
   relaxation = parameters.relaxation;
 }
@@ -862,7 +1193,7 @@ PreconditionRichardson::Tvmult_add (VECTOR &dst, const VECTOR &src) const
 template <class MATRIX>
 inline void
 PreconditionRelaxation<MATRIX>::initialize (const MATRIX &rA,
-					    AdditionalData parameters)
+					    const AdditionalData &parameters)
 {
   A = &rA;
   relaxation = parameters.relaxation;
@@ -900,6 +1231,28 @@ PreconditionJacobi<MATRIX>::Tvmult (VECTOR &dst, const VECTOR &src) const
 }
 
 
+
+template <class MATRIX>
+template<class VECTOR>
+inline void
+PreconditionJacobi<MATRIX>::step (VECTOR &dst, const VECTOR &src) const
+{
+  Assert (this->A!=0, ExcNotInitialized());
+  this->A->Jacobi_step (dst, src, this->relaxation);
+}
+
+
+
+template <class MATRIX>
+template<class VECTOR>
+inline void
+PreconditionJacobi<MATRIX>::Tstep (VECTOR &dst, const VECTOR &src) const
+{
+  step (dst, src);
+}
+
+
+
 //---------------------------------------------------------------------------
 
 template <class MATRIX>
@@ -923,28 +1276,51 @@ PreconditionSOR<MATRIX>::Tvmult (VECTOR &dst, const VECTOR &src) const
 }
 
 
+
+template <class MATRIX>
+template<class VECTOR>
+inline void
+PreconditionSOR<MATRIX>::step (VECTOR &dst, const VECTOR &src) const
+{
+  Assert (this->A!=0, ExcNotInitialized());
+  this->A->SOR_step (dst, src, this->relaxation);
+}
+
+
+
+template <class MATRIX>
+template<class VECTOR>
+inline void
+PreconditionSOR<MATRIX>::Tstep (VECTOR &dst, const VECTOR &src) const
+{
+  Assert (this->A!=0, ExcNotInitialized());
+  this->A->TSOR_step (dst, src, this->relaxation);
+}
+
+
+
 //---------------------------------------------------------------------------
 
 template <class MATRIX>
 inline void
 PreconditionSSOR<MATRIX>::initialize (const MATRIX &rA,
-				      typename BaseClass::AdditionalData parameters)
+				      const typename BaseClass::AdditionalData &parameters)
 {
   this->PreconditionRelaxation<MATRIX>::initialize (rA, parameters);
 
 				   // in case we have a SparseMatrix class,
 				   // we can extract information about the
 				   // diagonal.
-  const SparseMatrix<typename MATRIX::value_type> * mat = 
-    dynamic_cast<const SparseMatrix<typename MATRIX::value_type> *>(&*this->A);   
+  const SparseMatrix<typename MATRIX::value_type> * mat =
+    dynamic_cast<const SparseMatrix<typename MATRIX::value_type> *>(&*this->A);
 
 				   // calculate the positions first after
 				   // the diagonal.
   if (mat != 0)
     {
-      const std::size_t  * rowstart_ptr = 
+      const std::size_t  * rowstart_ptr =
 	mat->get_sparsity_pattern().get_rowstart_indices();
-      const unsigned int * const colnums = 
+      const unsigned int * const colnums =
 	mat->get_sparsity_pattern().get_column_numbers();
       const unsigned int n = this->A->n();
       pos_right_of_diagonal.resize(n);
@@ -957,7 +1333,7 @@ PreconditionSSOR<MATRIX>::initialize (const MATRIX &rA,
 				       // note: the first entry in each
 				       // line denotes the diagonal element,
 				       // which we need not check.
-	  pos_right_of_diagonal[row] = 
+	  pos_right_of_diagonal[row] =
 	    std::lower_bound (&colnums[*rowstart_ptr+1],
 			      &colnums[*(rowstart_ptr+1)],
 			      row)
@@ -988,6 +1364,28 @@ PreconditionSSOR<MATRIX>::Tvmult (VECTOR &dst, const VECTOR &src) const
 }
 
 
+
+template <class MATRIX>
+template<class VECTOR>
+inline void
+PreconditionSSOR<MATRIX>::step (VECTOR &dst, const VECTOR &src) const
+{
+  Assert (this->A!=0, ExcNotInitialized());
+  this->A->SSOR_step (dst, src, this->relaxation);
+}
+
+
+
+template <class MATRIX>
+template<class VECTOR>
+inline void
+PreconditionSSOR<MATRIX>::Tstep (VECTOR &dst, const VECTOR &src) const
+{
+  step (dst, src);
+}
+
+
+
 //---------------------------------------------------------------------------
 
 template <class MATRIX>
@@ -996,7 +1394,7 @@ PreconditionPSOR<MATRIX>::initialize (
   const MATRIX &rA,
   const std::vector<unsigned int> &p,
   const std::vector<unsigned int> &ip,
-  typename PreconditionRelaxation<MATRIX>::AdditionalData parameters)
+  const typename PreconditionRelaxation<MATRIX>::AdditionalData &parameters)
 {
   permutation = &p;
   inverse_permutation = &ip;
@@ -1090,7 +1488,7 @@ PreconditionLACSolver<SOLVER,MATRIX,PRECONDITION>::vmult (VECTOR &dst,
 {
   Assert (solver !=0 && matrix != 0 && precondition != 0,
 	  ExcNotInitialized());
-  
+
   solver->solve(*matrix, dst, src, *precondition);
 }
 
@@ -1153,6 +1551,223 @@ PreconditionedMatrix<MATRIX, PRECOND, VECTOR>
   return dst.l2_norm ();
 }
 
+
+
+//---------------------------------------------------------------------------
+
+template <class MATRIX, class VECTOR>
+inline
+PreconditionChebyshev<MATRIX,VECTOR>::AdditionalData::
+AdditionalData (const unsigned int degree,
+		const double       smoothing_range,
+		const bool         nonzero_starting,
+		const unsigned int eig_cg_n_iterations,
+		const double       eig_cg_residual)
+                :
+                degree  (degree),
+		smoothing_range (smoothing_range),
+		nonzero_starting (nonzero_starting),
+		eig_cg_n_iterations (eig_cg_n_iterations),
+		eig_cg_residual (eig_cg_residual)
+{}
+
+
+
+template <class MATRIX, class VECTOR>
+inline
+PreconditionChebyshev<MATRIX,VECTOR>::PreconditionChebyshev ()
+  :
+    is_initialized (false)
+{}
+
+
+
+template <class MATRIX, class VECTOR>
+inline
+void
+PreconditionChebyshev<MATRIX,VECTOR>::initialize (const MATRIX &matrix,
+						  const AdditionalData &additional_data)
+{
+  Assert (matrix.m() == matrix.n(), ExcMessage("Matrix not quadratic."));
+  matrix_ptr = &matrix;
+  data = additional_data;
+  if (data.matrix_diagonal_inverse.size() != matrix.m())
+    {
+      data.matrix_diagonal_inverse.reinit(matrix.m());
+      for (unsigned int i=0; i<matrix.m(); ++i)
+	data.matrix_diagonal_inverse(i) = 1./matrix.el(i,i);
+    }
+  update1.reinit (data.matrix_diagonal_inverse, true);
+  update2.reinit (data.matrix_diagonal_inverse, true);
+
+
+				 // calculate largest eigenvalue using a
+				 // hand-tuned CG iteration on the matrix
+				 // weighted by its diagonal. we start
+				 // with a vector that consists of ones
+				 // only, weighted by the length.
+				 //
+				 // TODO: can we obtain this with the
+				 // regular CG implementation? we would need
+				 // to read the logfile in that case, which
+				 // does not seem feasible.
+  double max_eigenvalue, min_eigenvalue;
+  {
+    double eigen_beta_alpha = 0;
+
+    std::vector<double> diagonal;
+    std::vector<double> offdiagonal;
+
+    VECTOR rhs, g;
+    rhs.reinit(data.matrix_diagonal_inverse, true);
+    rhs = 1./std::sqrt(static_cast<double>(matrix.m()));
+    g.reinit(data.matrix_diagonal_inverse, true);
+
+    unsigned int it=0;
+    double res,gh,alpha,beta;
+
+    g.equ(-1.,rhs);
+    res = g.l2_norm();
+    update2.equ (-1., g);
+    gh = res*res;
+
+    while (true)
+      {
+	it++;
+	matrix.vmult (update1, update2);
+	update1.scale (data.matrix_diagonal_inverse);
+	alpha = update2 * update1;
+	alpha = gh/alpha;
+	g.add (alpha, update1);
+	res = g.l2_norm();
+
+	if (it > data.eig_cg_n_iterations || res < data.eig_cg_residual)
+	  break;
+
+	beta = gh;
+	gh = res*res;
+	beta = gh/beta;
+	update2.sadd (beta, -1., g);
+
+	diagonal.push_back (1./alpha + eigen_beta_alpha);
+	eigen_beta_alpha = beta/alpha;
+	offdiagonal.push_back(std::sqrt(beta)/alpha);
+      }
+
+    TridiagonalMatrix<double> T(diagonal.size(), true);
+    for (unsigned int i=0;i<diagonal.size();++i)
+      {
+	T(i,i) = diagonal[i];
+	if (i< diagonal.size()-1)
+	  T(i,i+1) = offdiagonal[i];
+      }
+    T.compute_eigenvalues();
+    min_eigenvalue = T.eigenvalue(0);
+    max_eigenvalue = T.eigenvalue(T.n()-1);
+  }
+
+				 // include a safety factor since the CG
+				 // method will in general not be converged
+  const double beta = 1.2 * max_eigenvalue;
+  const double alpha = (data.smoothing_range > 0 ?
+			max_eigenvalue / data.smoothing_range :
+			max_eigenvalue / min_eigenvalue);
+  delta = (beta-alpha)*0.5;
+  theta = (beta+alpha)*0.5;
+  is_initialized = true;
+}
+
+
+
+template <class MATRIX, class VECTOR>
+inline
+void
+PreconditionChebyshev<MATRIX,VECTOR>::vmult (VECTOR &dst,
+					     const VECTOR &src) const
+{
+  Assert (is_initialized, ExcMessage("Preconditioner not initialized"));
+  double rhok  = delta / theta,  sigma = theta / delta;
+  if (data.nonzero_starting && !dst.all_zero())
+    {
+      matrix_ptr->vmult (update1, dst);
+      update1 -= src;
+      update1 /= theta;
+      update1.scale (data.matrix_diagonal_inverse);
+      dst -= update1;
+    }
+  else
+    {
+      dst.equ (1./theta, src);
+      dst.scale (data.matrix_diagonal_inverse);
+      update1.equ(-1.,dst);
+    }
+
+  for (unsigned int k=0; k<data.degree; ++k)
+    {
+      matrix_ptr->vmult (update2, dst);
+      update2 -= src;
+      update2.scale (data.matrix_diagonal_inverse);
+      const double rhokp = 1./(2.*sigma-rhok);
+      const double factor1 = rhokp * rhok, factor2 = 2.*rhokp/delta;
+      rhok = rhokp;
+      update1.sadd (factor1, factor2, update2);
+      dst -= update1;
+    }
+}
+
+
+
+template <class MATRIX, class VECTOR>
+inline
+void
+PreconditionChebyshev<MATRIX,VECTOR>::Tvmult (VECTOR &dst,
+					      const VECTOR &src) const
+{
+  Assert (is_initialized, ExcMessage("Preconditioner not initialized"));
+  double rhok  = delta / theta,  sigma = theta / delta;
+  if (data.nonzero_starting && !dst.all_zero())
+    {
+      matrix_ptr->Tvmult (update1, dst);
+      update1 -= src;
+      update1 /= theta;
+      update1.scale (data.matrix_diagonal_inverse);
+      dst -= update1;
+    }
+  else
+    {
+      dst.equ (1./theta, src);
+      dst.scale (data.matrix_diagonal_inverse);
+      update1.equ(-1.,dst);
+    }
+
+  for (unsigned int k=0; k<data.degree-1; ++k)
+    {
+      matrix_ptr->Tvmult (update2, dst);
+      update2 -= src;
+      update2.scale (data.matrix_diagonal_inverse);
+      const double rhokp = 1./(2.*sigma-rhok);
+      const double factor1 = rhokp * rhok, factor2 = 2.*rhokp/delta;
+      rhok = rhokp;
+      update1.sadd (factor1, factor2, update2);
+      dst -= update1;
+    }
+}
+
+
+
+template <class MATRIX, class VECTOR>
+inline
+void PreconditionChebyshev<MATRIX,VECTOR>::clear ()
+{
+  is_initialized = false;
+  matrix_ptr = 0;
+  data.matrix_diagonal_inverse.reinit(0);
+  update1.reinit(0);
+  update2.reinit(0);
+}
+
+
+
 #endif // DOXYGEN
 
 DEAL_II_NAMESPACE_CLOSE
