@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: precondition_block.h 18728 2009-04-24 00:02:10Z bangerth $
+//    $Id: precondition_block.h 21491 2010-07-13 13:55:49Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -56,7 +56,7 @@ class PreconditionBlockJacobi;
  * only above the diagonal blocks (e.g. the matrices one gets by the
  * DG method with downstream numbering) @p BlockSOR is a direct
  * solver.
- * 
+ *
  * This first implementation of the @p PreconditionBlock assumes the
  * matrix has blocks each of the same block size. Varying block sizes
  * within the matrix must still be implemented if needed.
@@ -71,6 +71,7 @@ class PreconditionBlockJacobi;
  * example, <tt>number==double, inverse_type=float</tt> might be a viable
  * choice.
  *
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000
  */
 template<class MATRIX, typename inverse_type = typename MATRIX::value_type>
@@ -86,7 +87,7 @@ class PreconditionBlock : public virtual Subscriptor
 				      * Value type for inverse matrices.
 				      */
     typedef inverse_type value_type;
-    
+
   public:
 				     /**
 				      * Parameters for block preconditioners.
@@ -109,7 +110,7 @@ class PreconditionBlock : public virtual Subscriptor
 					  * Relaxation parameter.
 					  */
 	double relaxation;
-	
+
 					 /**
 					  * Block size.
 					  */
@@ -126,13 +127,13 @@ class PreconditionBlock : public virtual Subscriptor
 					  */
 	bool same_diagonal;
     };
-    
-    
+
+
 				     /**
 				      * Constructor.
 				      */
     PreconditionBlock();
-    
+
 				     /**
 				      * Destructor.
 				      */
@@ -181,11 +182,21 @@ class PreconditionBlock : public virtual Subscriptor
 		     const std::vector<unsigned int>& inverse_permutation,
 		     const AdditionalData parameters);
 
+				     /**
+				      * Set the permutation and its
+				      * inverse. These vectors are
+				      * copied into private data, so
+				      * they can be reused or deleted
+				      * after a call to this function.
+				      */
+    void set_permutation(const std::vector<unsigned int>& permutation,
+			 const std::vector<unsigned int>& inverse_permutation);
+
 				     /**
 				      * Replacement of
 				      * invert_diagblocks() for
 				      * permuted preconditioning.
-				      */ 
+				      */
     void invert_permuted_diagblocks(
       const std::vector<unsigned int>& permutation,
       const std::vector<unsigned int>& inverse_permutation);
@@ -213,7 +224,7 @@ class PreconditionBlock : public virtual Subscriptor
 				      */
     value_type el(unsigned int i,
 		  unsigned int j) const;
-    
+
 				     /**
 				      * Use only the inverse of the
 				      * first diagonal block to save
@@ -234,7 +245,7 @@ class PreconditionBlock : public virtual Subscriptor
 				      * diagonal block?
 				      */
     bool same_diagonal () const;
-    
+
     				     /**
 				      * Stores the inverse of the
 				      * diagonal blocks in
@@ -264,6 +275,55 @@ class PreconditionBlock : public virtual Subscriptor
 				      */
     void invert_diagblocks();
 
+				     /**
+				      * Perform one block relaxation
+				      * step in forward numbering.
+				      *
+				      * Depending on the arguments @p
+				      * dst and @p pref, this performs
+				      * an SOR step (both reference
+				      * the same vector) of a Jacobi
+				      * step (botha different
+				      * vectors). For the Jacobi step,
+				      * the calling function must copy
+				      * @p dst to @p pref after this.
+				      *
+				      * @note If a permutation is set,
+				      * it is automatically honored by
+				      * this function.
+				      */
+    template <typename number2>
+    void forward_step (
+      Vector<number2>       &dst,
+      const Vector<number2> &prev,
+      const Vector<number2> &src,
+      const bool transpose_diagonal) const;
+
+				     /**
+				      * Perform one block relaxation
+				      * step in backward numbering.
+				      *
+				      * Depending on the arguments @p
+				      * dst and @p pref, this performs
+				      * an SOR step (both reference
+				      * the same vector) of a Jacobi
+				      * step (botha different
+				      * vectors). For the Jacobi step,
+				      * the calling function must copy
+				      * @p dst to @p pref after this.
+				      *
+				      * @note If a permutation is set,
+				      * it is automatically honored by
+				      * this function.
+				      */
+    template <typename number2>
+    void backward_step (
+      Vector<number2>       &dst,
+      const Vector<number2> &prev,
+      const Vector<number2> &src,
+      const bool transpose_diagonal) const;
+
+
 				     /**
 				      * Return the size of the blocks.
 				      */
@@ -274,7 +334,7 @@ class PreconditionBlock : public virtual Subscriptor
 				      * matrix.
 				      */
     unsigned int n_blocks() const;
-    
+
 				     /**
 				      * Determine an estimate for the
 				      * memory consumption (in bytes)
@@ -320,19 +380,19 @@ class PreconditionBlock : public virtual Subscriptor
     DeclException0 (ExcDiagonalsNotStored);
 
 				     //@}
-    
+
 				     /**
 				      * Access to the inverse diagonal
 				      * blocks.
 				      */
     const FullMatrix<inverse_type>& inverse (unsigned int i) const;
-    
+
 				     /**
 				      * Access to the diagonal
 				      * blocks.
 				      */
     const FullMatrix<inverse_type>& diagonal (unsigned int i) const;
-    
+
   protected:
 				     /**
 				      * Size of the blocks. Each
@@ -353,7 +413,7 @@ class PreconditionBlock : public virtual Subscriptor
 				      * @p vmult function of the
 				      * derived classes.
 				      */
-    SmartPointer<const MATRIX> A;
+    SmartPointer<const MATRIX,PreconditionBlock<MATRIX,inverse_type> > A;
 				     /**
 				      * Relaxation parameter to be
 				      * used by derived classes.
@@ -371,8 +431,23 @@ class PreconditionBlock : public virtual Subscriptor
 				      * Number of blocks.
 				      */
     unsigned int nblocks;
+
+				     /**
+				      * The permutation vector
+				      */
+    std::vector<unsigned int> permutation;
+
+				     /**
+				      * The inverse permutation vector
+				      */
+    std::vector<unsigned int> inverse_permutation;
+
+				     /**
+				      * Flag for diagonal compression.
+				      * @ref set_same_diagonal()
+				      */
   private:
-    
+
 				     /**
 				      * Storage of the inverse
 				      * matrices of the diagonal
@@ -393,11 +468,7 @@ class PreconditionBlock : public virtual Subscriptor
 				      * Used by the blocked SSOR method.
 				      */
     std::vector<FullMatrix<inverse_type> > var_diagonal;
-				      
-				     /**
-				      * Flag for diagonal compression.
-				      * @ref set_same_diagonal()
-				      */
+
     bool var_same_diagonal;
 };
 
@@ -410,7 +481,7 @@ class PreconditionBlock : public virtual Subscriptor
  * @note Instantiations for this template are provided for <tt>@<float@> and
  * @<double@></tt>; others can be generated in application programs (see the
  * section on @ref Instantiations in the manual).
- * 
+ *
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000, 2003
  */
 template<class MATRIX, typename inverse_type = typename MATRIX::value_type>
@@ -422,7 +493,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 				      * Define number type of matrix.
 				      */
     typedef typename MATRIX::value_type number;
-    
+
   public:
 				     /**
 				      * STL conforming iterator.
@@ -463,7 +534,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
                                               * Value of this matrix entry.
                                               */
             inverse_type value() const;
-	
+
           protected:
                                              /**
                                               * The matrix accessed.
@@ -475,7 +546,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 					      * for further reference.
 					      */
 	    unsigned int bs;
-	    
+
 					     /**
 					      * Current block number.
 					      */
@@ -490,21 +561,21 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 					      * End of current block.
 					      */
 	    typename FullMatrix<inverse_type>::const_iterator b_end;
-	    
+
                                              /**
                                               * Make enclosing class a
                                               * friend.
                                               */
             friend class const_iterator;
         };
-        
+
       public:
                                          /**
                                           * Constructor.
-                                          */ 
+                                          */
 	const_iterator(const PreconditionBlockJacobi<MATRIX, inverse_type>* matrix,
 		       const unsigned int row);
-	  
+
                                          /**
                                           * Prefix increment.
                                           */
@@ -561,7 +632,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 				      * publicly available.
 				      */
     PreconditionBlock<MATRIX, inverse_type>::initialize;
-    
+
 				     /**
 				      * Make function of base class public again.
 				      */
@@ -586,12 +657,12 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 				      * Make function public.
 				      */
     PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
-    
+
 				     /**
 				      * Make function public.
 				      */
     PreconditionBlock<MATRIX, inverse_type>::block_size;
-    
+
 				     /**
 				      * Make function public.
 				      */
@@ -600,7 +671,11 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 				      * Make function accessible to iterator.
 				      */
     PreconditionBlock<MATRIX, inverse_type>::inverse;
-    
+				     /**
+				      * Make function accessible.
+				      */
+    PreconditionBlock<MATRIX, inverse_type>::set_permutation;
+
 				     /**
 				      * Execute block Jacobi
 				      * preconditioning.
@@ -637,11 +712,26 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
     void vmult_add (Vector<number2>&, const Vector<number2>&) const;
 
 				     /**
-				      * Same as @p vmult_add, since Jacobi is symmetric.
+				      * Same as @p vmult_add, since
+				      * Jacobi is symmetric.
 				      */
     template <typename number2>
     void Tvmult_add (Vector<number2>&, const Vector<number2>&) const;
 
+				     /**
+				      * Perform one step of the Jacobi
+				      * iteration.
+				      */
+    template <typename number2>
+    void step (Vector<number2>& dst, const Vector<number2>& rhs) const;
+
+				     /**
+				      * Perform one step of the Jacobi
+				      * iteration.
+				      */
+    template <typename number2>
+    void Tstep (Vector<number2>& dst, const Vector<number2>& rhs) const;
+
     				     /**
 				      * STL-like iterator with the
 				      * first entry.
@@ -652,7 +742,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 				      * Final iterator.
 				      */
     const_iterator end () const;
-    
+
 				     /**
 				      * STL-like iterator with the
 				      * first entry of row @p r.
@@ -663,7 +753,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
 				      * Final iterator of row @p r.
 				      */
     const_iterator end (const unsigned int r) const;
-    
+
 
   private:
 				   /**
@@ -694,7 +784,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
  * arbitrarily.
  *
  * See PreconditionBlock for requirements on the matrix.
- * 
+ *
  * Optionally, the entries of the source vector can be treated in the
  * order of the indices in the permutation vector set by
  * #set_permutation (or the opposite order for Tvmult()). The inverse
@@ -710,7 +800,7 @@ class PreconditionBlockJacobi : public virtual Subscriptor,
  * @note Instantiations for this template are provided for <tt>@<float@> and
  * @<double@></tt>; others can be generated in application programs (see the
  * section on @ref Instantiations in the manual).
- * 
+ *
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000, 2001, 2002, 2003
  */
 template<class MATRIX, typename inverse_type = typename MATRIX::value_type>
@@ -718,50 +808,50 @@ class PreconditionBlockSOR : public virtual Subscriptor,
 			     protected PreconditionBlock<MATRIX, inverse_type>
 {
   public:
-				     /**
-				      * Set the permutation and its
-				      * inverse. These vectors are
-				      * copied into private data, so
-				      * they can be reused or deleted
-				      * after a call to this function.
-				      */
-    void set_permutation(const std::vector<unsigned int>& permutation,
-			 const std::vector<unsigned int>& inverse_permutation);
 				     /**
 				      * Define number type of matrix.
 				      */
     typedef typename MATRIX::value_type number;
-    
+
+				     /**
+				      * Make type publicly available.
+				      */
+    using PreconditionBlock<MATRIX,inverse_type>::AdditionalData;
+
 				     /**
 				      * Make initialization function
 				      * publicly available.
 				      */
-    PreconditionBlock<MATRIX, inverse_type>::initialize;
-    
-				     /**
-				      * Make function of base class public again.
-				      */
-    PreconditionBlock<MATRIX, inverse_type>::clear;
+    using PreconditionBlock<MATRIX, inverse_type>::initialize;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlock<MATRIX, inverse_type>::empty;
-    
-				     /**
-				      * Make function of base class public again.
-				      */
-    PreconditionBlock<MATRIX, inverse_type>::el;
+    using PreconditionBlock<MATRIX, inverse_type>::clear;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlock<MATRIX, inverse_type>::set_same_diagonal;
+    using PreconditionBlock<MATRIX, inverse_type>::empty;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
+    using PreconditionBlock<MATRIX, inverse_type>::el;
+
+				     /**
+				      * Make function of base class public again.
+				      */
+    using PreconditionBlock<MATRIX, inverse_type>::set_same_diagonal;
+
+				     /**
+				      * Make function of base class public again.
+				      */
+    using PreconditionBlock<MATRIX, inverse_type>::invert_diagblocks;
+				     /**
+				      * Make function accessible.
+				      */
+    using PreconditionBlock<MATRIX, inverse_type>::set_permutation;
 
 				     /**
 				      * Execute block SOR
@@ -839,6 +929,20 @@ class PreconditionBlockSOR : public virtual Subscriptor,
     template <typename number2>
     void Tvmult_add (Vector<number2>&, const Vector<number2>&) const;
 
+				     /**
+				      * Perform one step of the SOR
+				      * iteration.
+				      */
+    template <typename number2>
+    void step (Vector<number2>& dst, const Vector<number2>& rhs) const;
+
+				     /**
+				      * Perform one step of the
+				      * transposed SOR iteration.
+				      */
+    template <typename number2>
+    void Tstep (Vector<number2>& dst, const Vector<number2>& rhs) const;
+
   protected:
 				     /**
 				      * Implementation of the forward
@@ -877,18 +981,6 @@ class PreconditionBlockSOR : public virtual Subscriptor,
 		   const Vector<number2>&,
 		   const bool transpose_diagonal,
 		   const bool adding) const;
-
-  private:
-				     /**
-				      * The permutation vector
-				      */
-    std::vector<unsigned int> permutation;
-    
-				     /**
-				      * The inverse permutation vector
-				      */
-    std::vector<unsigned int> inverse_permutation;
-    
 };
 
 
@@ -900,11 +992,11 @@ class PreconditionBlockSOR : public virtual Subscriptor,
  * class requires storage of the diagonal blocks and their inverses.
  *
  * See PreconditionBlock for requirements on the matrix.
- * 
+ *
  * @note Instantiations for this template are provided for <tt>@<float@> and
  * @<double@></tt>; others can be generated in application programs (see the
  * section on @ref Instantiations in the manual).
- * 
+ *
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000
  */
 template<class MATRIX, typename inverse_type = typename MATRIX::value_type>
@@ -921,41 +1013,47 @@ class PreconditionBlockSSOR : public virtual Subscriptor,
 				      * Constructor.
 				      */
     PreconditionBlockSSOR ();
+
+				     /**
+				      * Make type publicly available.
+				      */
+    using PreconditionBlockSOR<MATRIX,inverse_type>::AdditionalData;
+
 				     /**
 				      * Make initialization function
 				      * publicly available.
 				      */
-    PreconditionBlockSOR<MATRIX,inverse_type>::initialize;
-    
-				     /**
-				      * Make function of base class public again.
-				      */
-    PreconditionBlockSOR<MATRIX,inverse_type>::clear;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::initialize;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlockSOR<MATRIX,inverse_type>::set_permutation;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::clear;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlockSOR<MATRIX, inverse_type>::empty;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::set_permutation;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlockSOR<MATRIX, inverse_type>::el;
+    using PreconditionBlockSOR<MATRIX, inverse_type>::empty;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlockSOR<MATRIX,inverse_type>::set_same_diagonal;
+    using PreconditionBlockSOR<MATRIX, inverse_type>::el;
 
 				     /**
 				      * Make function of base class public again.
 				      */
-    PreconditionBlockSOR<MATRIX,inverse_type>::invert_diagblocks;
+    using PreconditionBlockSOR<MATRIX,inverse_type>::set_same_diagonal;
+
+				     /**
+				      * Make function of base class public again.
+				      */
+    using PreconditionBlockSOR<MATRIX,inverse_type>::invert_diagblocks;
 
 				     /**
 				      * Execute block SSOR
@@ -977,6 +1075,20 @@ class PreconditionBlockSSOR : public virtual Subscriptor,
 				      */
     template <typename number2>
     void Tvmult (Vector<number2>&, const Vector<number2>&) const;
+
+				     /**
+				      * Perform one step of the SOR
+				      * iteration.
+				      */
+    template <typename number2>
+    void step (Vector<number2>& dst, const Vector<number2>& rhs) const;
+
+				     /**
+				      * Perform one step of the
+				      * transposed SOR iteration.
+				      */
+    template <typename number2>
+    void Tstep (Vector<number2>& dst, const Vector<number2>& rhs) const;
 };
 
 /*@}*/
@@ -1025,7 +1137,7 @@ PreconditionBlock<MATRIX, inverse_type>::el (
 {
   const unsigned int bs = blocksize;
   const unsigned int nb = i/bs;
-  
+
   const FullMatrix<inverse_type>& B = inverse(nb);
 
   const unsigned int ib = i % bs;
@@ -1035,7 +1147,7 @@ PreconditionBlock<MATRIX, inverse_type>::el (
     {
       return 0.;
     }
-  
+
   return B(ib, jb);
 }
 
@@ -1076,7 +1188,7 @@ PreconditionBlockJacobi<MATRIX, inverse_type>::const_iterator::Accessor::row() c
 {
   Assert (a_block < matrix->n_blocks(),
 	  ExcIteratorPastEnd());
-  
+
   return bs * a_block + b_iterator->row();
 }
 
@@ -1088,7 +1200,7 @@ PreconditionBlockJacobi<MATRIX, inverse_type>::const_iterator::Accessor::column(
 {
   Assert (a_block < matrix->n_blocks(),
 	  ExcIteratorPastEnd());
-  
+
   return bs * a_block + b_iterator->column();
 }
 
@@ -1100,7 +1212,7 @@ PreconditionBlockJacobi<MATRIX, inverse_type>::const_iterator::Accessor::value()
 {
   Assert (a_block < matrix->n_blocks(),
 	  ExcIteratorPastEnd());
-  
+
   return b_iterator->value();
 }
 
@@ -1121,12 +1233,12 @@ typename PreconditionBlockJacobi<MATRIX, inverse_type>::const_iterator &
 PreconditionBlockJacobi<MATRIX, inverse_type>::const_iterator::operator++ ()
 {
   Assert (*this != accessor.matrix->end(), ExcIteratorPastEnd());
-  
+
   ++accessor.b_iterator;
   if (accessor.b_iterator == accessor.b_end)
     {
       ++accessor.a_block;
-      
+
       if (accessor.a_block < accessor.matrix->n_blocks())
 	{
 	  accessor.b_iterator = accessor.matrix->inverse(accessor.a_block).begin();
@@ -1167,7 +1279,7 @@ operator == (const const_iterator& other) const
 
   if (accessor.a_block != other.accessor.a_block)
     return false;
-  
+
   return (accessor.row() == other.accessor.row() &&
           accessor.column() == other.accessor.column());
 }
