@@ -1,8 +1,8 @@
 //----------------------------  solver_gmres.h  ---------------------------
-//    $Id: solver_gmres.h 15606 2007-12-17 20:16:08Z kanschat $
+//    $Id: solver_gmres.h 21255 2010-06-21 17:17:23Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -51,7 +51,7 @@ namespace internal
 				      * vectors automatically,
 				      * avoiding restart.
 				      */
-    
+
     template <class VECTOR>
     class TmpVectors
     {
@@ -76,7 +76,7 @@ namespace internal
 					  * occurs.
 					  */
 	VECTOR& operator[] (const unsigned int i) const;
-	
+
 					 /**
 					  * Get vector number
 					  * @p i. Allocate it if
@@ -89,20 +89,20 @@ namespace internal
 					  */
 	VECTOR& operator() (const unsigned int i,
 			    const VECTOR      &temp);
-	
+
       private:
 					 /**
 					  * Pool were vectors are
 					  * obtained from.
 					  */
 	VectorMemory<VECTOR> &mem;
-        
+
 					 /**
 					  * Field for storing the
 					  * vectors.
 					  */
 	std::vector<VECTOR*> data;
-        
+
 					 /**
 					  * Offset of the first
 					  * vector. This is for later
@@ -110,7 +110,7 @@ namespace internal
 					  * be implemented.
 					  */
 	unsigned int offset;
-    };    
+    };
   }
 }
 
@@ -119,58 +119,46 @@ namespace internal
  * Minimal Residual Method. The stopping criterion is the norm of the
  * residual.
  *
- * You have to give the maximum number of temporary vectors to the
- * constructor which are to be used to do the orthogonalization. If
- * the number of iterations needed to solve the problem to the given
- * criterion, an intermediate solution is computed and a restart is
- * performed. If you don't want to use the restarted method, you can
- * limit the number of iterations (stated in the @p SolverControl
- * object given to the constructor) to be below the number of
- * temporary vectors minus three. Note the subtraction, which is due
- * to the fact that three vectors are used for other purposes, so the
- * number of iterations before a restart occurs is less by three than
- * the total number of temporary vectors. If the size of the matrix is
- * smaller than the maximum number of temporary vectors, then fewer
- * vectors are allocated since GMRES can then be used as a direct
- * solver.
+ * The AdditionalData structure contains the number of temporary
+ * vectors used. The size of the Arnoldi basis is this number minus
+ * three. Addinitonally, it allows you to choose bet right or left
+ * preconditioning. The default is left preconditioning. Finally it
+ * includes a flag indicating whether or not the default residual is
+ * used as stopping criterion.
+
+ * <h3>Left versus right preconditioning</h3>
  *
- * Note that restarts don't compensate temporary vectors very well,
- * i.e.  giving too few temporary vectors will increase the necessary
- * iteration steps greatly; it is not uncommon that you will need more
- * iterations than there are degrees of freedom in your solution
- * vector, if the number of temporary vectors is lower than the size
- * of the vector, even though GMRES is an exact solver if a sufficient
- * number of temporary vectors is given. You should therefore always
- * give as many temporary vectors as you can, unless you are limited
- * by the available memory; only then should you start to trade
- * computational speed against memory. One of the few other
- * possibilities is to use a good preconditioner.
+ * @p AdditionalData allows you to choose between left and right
+ * preconditioning. As expected, this switches between solving for the
+ * systems <i>P<sup>-1</sup>A</i> and <i>AP<sup>-1</sup></i>,
+ * respectively.
  *
- * Like all other solver classes, this class has a local structure
- * called @p AdditionalData which is used to pass additional
- * parameters to the solver, like the number of temporary vectors for
- * example. We use this additional structure instead of passing these
- * values directly to the constructor because this makes the use of
- * the @p SolverSelector and other classes much easier and guarantees
- * that these will continue to work even if number or type of the
- * additional parameters for a certain solver changes.
+ * A second consequence is the type of residual which is used to
+ * measure convergence. With left preconditioning, this is the
+ * <b>preconditioned</b> residual, while with right preconditioning,
+ * it is the residual of the unpreconditioned system.
  *
- * For the GMRes method, the @p AdditionalData structure contains the number
- * of temporary vectors as commented upon above. By default, the number of
- * these vectors is set to 30. The @p AdditionalData also containes a flag
- * indicating the use of right or left preconditioning. The default is left
- * preconditioning. Finally it includes a flag indicating whether or not the
- * default residual is used as stopping criterion. By default, the left
- * preconditioned GMRes uses the preconditioned residual and the right
- * preconditioned GMRes uses the normal, i.e. unpreconditioned, residual as
- * stopping criterion. If the @p use_default_residual flag is @p false, the
- * left preconditioned GMRes uses as stopping criterion the unpreconditioned
- * residual and the right preconditioned GMRes the preconditioned
- * residual. But be aware that the non-default residuals are not automatically
- * computed by the GMRes method but need to be computed in addition. This
- * (especially for the left preconditioned GMRes) might lead to a significant
- * loss in the solver performance. Therefore, the user should set
- * <tt>use_default_residual=false</tt> only for debugging/testing purposes.
+ * Optionally, this behavior can be overridden by using the flag
+ * AdditionalData::use_default_residual. A <tt>true</tt> value refers
+ * to the behavior described in the previous paragraph, while
+ * <tt>false</tt> reverts it. Be aware though that additional
+ * residuals have to be computed in this case, impeding the overall
+ * performance of the solver.
+ *
+ * <h3>The size of the Arnoldi basis</h3>
+ *
+ * The maximal basis size is controlled by
+ * AdditionalData::max_n_tmp_vectors, and it is this number minus 2.
+ * If the number of iteration steps exceeds this number, all basis
+ * vectors are discarded and the iteration starts anew from the
+ * approximation obtained so far.
+ *
+ * Note that the minimizing property of GMRes only pertains to the
+ * Krylov space spanned by the Arnoldi basis. Therefore, restarted
+ * GMRes is <b>not</b> minimizing anymore. The choice of the basis
+ * length is a trade-off between memory consumption and convergence
+ * speed, since a longer basis means minimization over a larger
+ * space.
  *
  * For the requirements on matrices and vectors in order to work with
  * this class, see the documentation of the Solver base class.
@@ -186,42 +174,54 @@ class SolverGMRES : public Solver<VECTOR>
 				      * pipe additional data to the
 				      * solver.
 				      */
-    struct AdditionalData 
+    struct AdditionalData
     {
 					 /**
 					  * Constructor. By default, set the
 					  * number of temporary vectors to 30,
 					  * i.e. do a restart every
-					  * approximately 30 iterations. Also
+					  * 28 iterations. Also
 					  * set preconditioning from left and
 					  * the residual of the stopping
-					  * criterion to the default residual
-					  * (cf. class documentation).
+					  * criterion to the default residual.
 					  */
 	AdditionalData (const unsigned int max_n_tmp_vectors = 30,
                         const bool right_preconditioning = false,
                         const bool use_default_residual = true);
-	
+
 					 /**
 					  * Maximum number of
-					  * tmp vectors.
+					  * temporary vectors. This
+					  * parameter controls the
+					  * size of the Arnoldi basis,
+					  * which for historical
+					  * reasons is
+					  * #max_n_tmp_vectors-2.
 					  */
 	unsigned int    max_n_tmp_vectors;
 
 					 /**
 					  * Flag for right
 					  * preconditioning.
+					  *
+					  * @note Change between left
+					  * and right preconditioning
+					  * will also change the way
+					  * residuals are
+					  * evaluated. See the
+					  * corresponding section in
+					  * the SolverGMRES.
 					  */
 	bool right_preconditioning;
 
 					 /**
 					  * Flag for the default
-					  * residual that is used as
-					  * stopping criterion.
+					  * residual that is used to
+					  * measure convergence.
 					  */
 	bool use_default_residual;
     };
-    
+
 				     /**
 				      * Constructor.
 				      */
@@ -253,20 +253,20 @@ class SolverGMRES : public Solver<VECTOR>
 		    << "The number of temporary vectors you gave ("
 		    << arg1 << ") is too small. It should be at least 10 for "
 		    << "any results, and much more for reasonable ones.");
-    
+
   protected:
 				     /**
 				      * Includes the maximum number of
 				      * tmp vectors.
 				      */
     AdditionalData additional_data;
-    
+
 				     /**
 				      * Implementation of the computation of
 				      * the norm of the residual.
 				      */
     virtual double criterion();
-    
+
 				     /**
 				      * Transformation of an upper
 				      * Hessenberg matrix into
@@ -284,7 +284,7 @@ class SolverGMRES : public Solver<VECTOR>
 				      * Auxiliary matrix for inverting @p H
 				      */
     FullMatrix<double> H1;
-    
+
   private:
     				     /**
 				      * No copy constructor.
@@ -319,7 +319,7 @@ class SolverFGMRES : public Solver<VECTOR>
 				      * pipe additional data to the
 				      * solver.
 				      */
-    struct AdditionalData 
+    struct AdditionalData
     {
 					 /**
 					  * Constructor. By default,
@@ -336,14 +336,14 @@ class SolverFGMRES : public Solver<VECTOR>
 			:
 			max_basis_size(max_basis_size)
 	  {}
-	
+
 					 /**
 					  * Maximum number of
 					  * tmp vectors.
 					  */
 	unsigned int    max_basis_size;
     };
-    
+
 				     /**
 				      * Constructor.
 				      */
@@ -423,7 +423,7 @@ namespace internal
     {
       Assert (i+offset<data.size(),
 	      ExcIndexRange(i, -offset, data.size()-offset));
-  
+
       Assert (data[i-offset] != 0, ExcNotInitialized());
       return *data[i-offset];
     }
@@ -487,7 +487,7 @@ void
 SolverGMRES<VECTOR>::givens_rotation (Vector<double> &h,
 				      Vector<double> &b,
 				      Vector<double> &ci,
-				      Vector<double> &si, 
+				      Vector<double> &si,
 				      int     col) const
 {
   for (int i=0 ; i<col ; i++)
@@ -498,7 +498,7 @@ SolverGMRES<VECTOR>::givens_rotation (Vector<double> &h,
       h(i)   =  c*dummy + s*h(i+1);
       h(i+1) = -s*dummy + c*h(i+1);
     };
-  
+
   const double r = 1./std::sqrt(h(col)*h(col) + h(col+1)*h(col+1));
   si(col) = h(col+1) *r;
   ci(col) = h(col)   *r;
@@ -523,22 +523,22 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 				   // don't make any guarantees to its
 				   // optimality or that it even works
 				   // as expected...
-  
+
 //TODO:[?] Check, why there are two different start residuals.
 //TODO:[GK] Make sure the parameter in the constructor means maximum basis size
-  
+
   deallog.push("GMRES");
   const unsigned int n_tmp_vectors = additional_data.max_n_tmp_vectors;
 
 				   // Generate an object where basis
 				   // vectors are stored.
   internal::SolverGMRES::TmpVectors<VECTOR> tmp_vectors (n_tmp_vectors, this->memory);
-  
+
 				   // number of the present iteration; this
 				   // number is not reset to zero upon a
 				   // restart
   unsigned int accumulated_iterations = 0;
-  
+
 				   // matrix used for the orthogonalization
 				   // process later
   H.reinit(n_tmp_vectors, n_tmp_vectors-1);
@@ -550,12 +550,12 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
     ci   (n_tmp_vectors-1),
     si   (n_tmp_vectors-1),
     h    (n_tmp_vectors-1);
-  
+
 
   unsigned int dim = 0;
 
   SolverControl::State iteration_state = SolverControl::iterate;
-  
+
 				   // switch to determine whether we want a
 				   // left or a right preconditioner. at
 				   // present, left is default, but both
@@ -599,8 +599,8 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
     {
 				       // reset this vector to the
 				       // right size
-      h.reinit (n_tmp_vectors-1);      
-      
+      h.reinit (n_tmp_vectors-1);
+
       if (left_precondition)
 	{
 	  A.vmult(p,x);
@@ -627,7 +627,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 	{
 	  iteration_state = this->control().check (
 	    accumulated_iterations, rho);
-	  
+
 	  if (iteration_state != SolverControl::iterate)
 	    break;
 	}
@@ -642,7 +642,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 	    }
 	  else
 	    precondition.vmult(*r,v);
-	      
+
 	  double res = r->l2_norm();
 	  iteration_state = this->control().check (
 	    accumulated_iterations, res);
@@ -656,9 +656,9 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 	      break;
 	    }
 	}
-      
+
       gamma(0) = rho;
-      
+
       v *= 1./rho;
 
 				       // inner iteration doing at
@@ -677,7 +677,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 	  ++accumulated_iterations;
 	  // yet another alias
 	  VECTOR& vv = tmp_vectors(inner_iteration+1, x);
-	  
+
 	  if (left_precondition)
 	    {
 	      A.vmult(p, tmp_vectors[inner_iteration]);
@@ -686,40 +686,38 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 	      precondition.vmult(p, tmp_vectors[inner_iteration]);
 	      A.vmult(vv,p);
 	    };
-	  
+
 	  dim = inner_iteration+1;
-	  
+
 					   /* Orthogonalization */
 	  for (unsigned int i=0 ; i<dim ; ++i)
 	    {
 	      h(i) = vv * tmp_vectors[i];
 	      vv.add(-h(i), tmp_vectors[i]);
 	    };
-      
-	  double s = vv.l2_norm();
-	  h(inner_iteration+1) = s;
-    
-					   /* Re-orthogonalization */
-	  for (unsigned i=0; i<dim; ++i)
+
+	  				   /* Re-orthogonalization */
+	  for (unsigned int i=0 ; i<dim ; ++i)
 	    {
 	      double htmp = vv * tmp_vectors[i];
 	      h(i) += htmp;
 	      vv.add(-htmp, tmp_vectors[i]);
-	    };
-	  
-	  s = vv.l2_norm();
+	    }
+
+	  const double s = vv.l2_norm();
 	  h(inner_iteration+1) = s;
+//TODO: s=0 is a lucky breakdown. Handle this somehow decently
 	  
 	  vv *= 1./s;
-	  
+
 					   /*  Transformation into
 					       triagonal structure  */
 	  givens_rotation(h,gamma,ci,si,inner_iteration);
-	  
+
 					   /*  append vector on matrix  */
 	  for (unsigned int i=0; i<dim; ++i)
 	    H(i,inner_iteration) = h(i);
-	  
+
 					   /*  default residual  */
 	  rho = std::fabs(gamma(dim));
 
@@ -729,18 +727,18 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
   	  else
   	    {
 	      deallog << "default_res=" << rho << std::endl;
-	      
+
 	      dealii::Vector<double> h_(dim);
 	      *x_=x;
 	      *gamma_=gamma;
 	      H1.reinit(dim+1,dim);
-	      
+
 	      for (unsigned int i=0; i<dim+1; ++i)
 		for (unsigned int j=0; j<dim; ++j)
 		  H1(i,j) = H(i,j);
-	      
+
 	      H1.backward(h_,*gamma_);
-	      
+
 	      if (left_precondition)
 		for (unsigned int i=0 ; i<dim; ++i)
 		  x_->add(h_(i), tmp_vectors[i]);
@@ -760,7 +758,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 	      if (left_precondition)
 		{
 		  const double res=r->l2_norm();
-		  
+
 		  iteration_state = this->control().check (
 		    accumulated_iterations, res);
 		}
@@ -768,7 +766,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 		{
 		  precondition.vmult(*x_, *r);
 		  const double preconditioned_res=x_->l2_norm();
-		  
+
 		  iteration_state = this->control().check (
 		    accumulated_iterations, preconditioned_res);
 		}
@@ -810,7 +808,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 
       delete gamma_;
     }
-  
+
   deallog.pop();
 				   // in case of failure: throw
 				   // exception
@@ -824,7 +822,7 @@ SolverGMRES<VECTOR>::solve (const MATRIX         &A,
 
 template<class VECTOR>
 double
-SolverGMRES<VECTOR>::criterion () 
+SolverGMRES<VECTOR>::criterion ()
 {
 				   // dummy implementation. this function is
 				   // not needed for the present implementation
@@ -867,8 +865,8 @@ SolverFGMRES<VECTOR>::solve (
   const PRECONDITIONER& precondition)
 {
   deallog.push("FGMRES");
-  
-  SolverControl::State iteration_state = SolverControl::iterate;  
+
+  SolverControl::State iteration_state = SolverControl::iterate;
 
   const unsigned int basis_size = additional_data.max_basis_size;
 
@@ -876,12 +874,12 @@ SolverFGMRES<VECTOR>::solve (
 				   // vectors are stored.
   typename internal::SolverGMRES::TmpVectors<VECTOR> v (basis_size, this->memory);
   typename internal::SolverGMRES::TmpVectors<VECTOR> z (basis_size, this->memory);
-  
+
 				   // number of the present iteration; this
 				   // number is not reset to zero upon a
 				   // restart
   unsigned int accumulated_iterations = 0;
-  
+
 				   // matrix used for the orthogonalization
 				   // process later
   H.reinit(basis_size+1, basis_size);
@@ -898,22 +896,22 @@ SolverFGMRES<VECTOR>::solve (
       aux->reinit(x);
       A.vmult(*aux, x);
       aux->sadd(-1., 1., b);
-      
+
       double beta = aux->l2_norm();
       if (this->control().check(accumulated_iterations,beta)
 	  == SolverControl::success)
 	break;
-      
+
       H.reinit(basis_size+1, basis_size);
       double a = beta;
-      
+
       for (unsigned int j=0;j<basis_size;++j)
 	{
 	  v(j,x).equ(1./a, *aux);
-	  
+
 	  precondition.vmult(z(j,x), v[j]);
 	  A.vmult(*aux, z[j]);
-	  
+
 					   // Gram-Schmidt
 	  for (unsigned int i=0;i<=j;++i)
 	    {
@@ -921,14 +919,14 @@ SolverFGMRES<VECTOR>::solve (
 	      aux->add(-H(i,j), v[i]);
 	    }
 	  H(j+1,j) = a = aux->l2_norm();
-	  
+
 					   // Compute projected solution
-	  
+
 	  if (j>0)
 	    {
 	      H1.reinit(j+1,j);
 	      projected_rhs.reinit(j+1);
-	      y.reinit(j);	  
+	      y.reinit(j);
 	      projected_rhs(0) = beta;
 	      H1.fill(H);
 	      Householder<double> house(H1);
@@ -941,10 +939,10 @@ SolverFGMRES<VECTOR>::solve (
 				       // Update solution vector
       for (unsigned int j=0;j<y.size();++j)
 	x.add(y(j), z[j]);
-      
+
       this->memory.free(aux);
     } while (iteration_state == SolverControl::iterate);
-  
+
   deallog.pop();
 				   // in case of failure: throw
 				   // exception
