@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: sparse_vanka.h 14038 2006-10-23 02:46:34Z bangerth $
+//    $Id: sparse_vanka.h 19894 2009-10-15 22:19:51Z kanschat $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -45,7 +45,7 @@ template <typename number> class SparseBlockVanka;
  * some diagonal elements are zero in the rows of the Lagrange multiplier.
  * The approach of Vanka is to solve a small (usually indefinite) system
  * of equations for each Langrange multiplie variable (we will also call
- * the pressure in Stoke's equation a Langrange multiplier since it
+ * the pressure in Stokes' equation a Langrange multiplier since it
  * can be interpreted as such).
  *
  * Objects of this class are constructed by passing a vector of indices
@@ -264,7 +264,7 @@ class SparseVanka
 				     /**
 				      * Pointer to the matrix.
 				      */
-    SmartPointer<const SparseMatrix<number> > matrix;
+    SmartPointer<const SparseMatrix<number>,SparseVanka<number> > matrix;
     
 				     /**
 				      * Conserve memory flag.
@@ -291,7 +291,7 @@ class SparseVanka
 				      * Only those elements will be used
 				      * that are tagged in @p selected.
 				      */
-    mutable std::vector<SmartPointer<FullMatrix<float> > > inverses;
+    mutable std::vector<SmartPointer<FullMatrix<float>,SparseVanka<number> > > inverses;
 
 				     /**
 				      * Compute the inverses of all
