@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: sparsity_pattern.h 18512 2009-03-25 16:55:27Z kronbichler $
+//    $Id: sparsity_pattern.h 20820 2010-03-14 00:52:27Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -77,7 +77,7 @@ namespace internals
 
       if (len==0)
 	return first;
-  
+
       while (true)
 	{
 					   // if length equals 8 or less,
@@ -130,12 +130,12 @@ namespace internals
 			Assert (false, ExcInternalError());
 		}
 	    }
-      
-      
-      
+
+
+
 	  const unsigned int         half   = len >> 1;
 	  const unsigned int * const middle = first + half;
-      
+
 					   // if the value is larger than
 					   // that pointed to by the
 					   // middle pointer, then the
@@ -162,7 +162,7 @@ namespace internals
 				      */
     unsigned int
     get_column_index_from_iterator (const unsigned int i);
-    
+
 				     /**
 				      * Helper function to get the
 				      * column index from a
@@ -186,232 +186,228 @@ namespace internals
     template <typename value>
     unsigned int
     get_column_index_from_iterator (const std::pair<const unsigned int, value> &i);
-    
-  }
-  
 
-    
-  namespace SparsityPatternIterators
-  {
-                                     // forward declaration
-    class Iterator;
-    
-                                     /**
-                                      * Accessor class for iterators into
-                                      * sparsity patterns. This class is also
-                                      * the base class for both const and
-                                      * non-const accessor classes into sparse
-                                      * matrices.
-                                      *
-                                      * Note that this class only allow read
-                                      * access to elements, providing their
-                                      * row and column number. It does not
-                                      * allow to modify the sparsity pattern
-                                      * itself.
-                                      */
-    class Accessor
-    {
-      public:
-                                         /**
-                                          * Constructor.
-                                          */
-        Accessor (const SparsityPattern *matrix,
-                  const unsigned int     row,
-                  const unsigned int     index);
-
-                                         /**
-                                          * Constructor. Construct the end
-                                          * accessor for the given sparsity
-                                          * pattern.
-                                          */
-        Accessor (const SparsityPattern *matrix);
-
-                                         /**
-                                          * Row number of the element
-                                          * represented by this object. This
-                                          * function can only be called for
-                                          * entries for which is_valid_entry()
-                                          * is true.
-                                          */
-        unsigned int row () const;
-
-                                         /**
-                                          * Index in row of the element
-                                          * represented by this object. This
-                                          * function can only be called for
-                                          * entries for which is_valid_entry()
-                                          * is true.
-                                          */
-        unsigned int index () const;
-
-                                         /**
-                                          * Column number of the element
-                                          * represented by this object. This
-                                          * function can only be called for
-                                          * entries for which is_valid_entry() is
-                                          * true.
-                                          */
-        unsigned int column () const;
-
-                                         /**
-                                          * Return whether the sparsity
-                                          * pattern entry pointed to by this
-                                          * iterator is valid or not. Note
-                                          * that after compressing the
-                                          * sparsity pattern, all entries are
-                                          * valid. However, before
-                                          * compression, the sparsity pattern
-                                          * allocated some memory to be used
-                                          * while still adding new nonzero
-                                          * entries; if you create iterators
-                                          * in this phase of the sparsity
-                                          * pattern's lifetime, you will
-                                          * iterate over elements that are not
-                                          * valid. If this is so, then this
-                                          * function will return false.
-                                          */
-        inline bool is_valid_entry () const;
-
-
-                                         /**
-                                          * Comparison. True, if
-                                          * both iterators point to
-                                          * the same matrix
-                                          * position.
-                                          */
-	bool operator == (const Accessor &) const;
-
-
-                                         /**
-                                          * Comparison
-                                          * operator. Result is true
-                                          * if either the first row
-                                          * number is smaller or if
-                                          * the row numbers are
-                                          * equal and the first
-                                          * index is smaller.
-                                          *
-                                          * This function is only valid if
-                                          * both iterators point into the same
-                                          * sparsity pattern.
-                                          */
-	bool operator < (const Accessor &) const;
-
-					 /** @addtogroup Exceptions
-					  * @{ */
-	
-                                         /**
-                                          * Exception
-                                          */
-        DeclException0 (ExcInvalidIterator);
-					 //@}        
-      protected:
-                                         /**
-                                          * The sparsity pattern we operate on
-                                          * accessed.
-                                          */
-        const SparsityPattern * sparsity_pattern;
-
-                                         /**
-                                          * Current row number.
-                                          */
-        unsigned int a_row;
-
-                                         /**
-                                          * Current index in row.
-                                          */
-        unsigned int a_index;
-
-                                         /**
-                                          * Move the accessor to the next
-                                          * nonzero entry in the matrix.
-                                          */
-        void advance ();
-	
-                                         /**
-                                          * Grant access to iterator class.
-                                          */
-        friend class Iterator;
-    };
-
-
-
-                                     /**
-				      * STL conforming iterator walking over
-				      * the elements of a sparsity pattern.
-				      */
-    class Iterator
-    {
-      public:
-                                         /**
-                                          * Constructor. Create an iterator
-                                          * into the sparsity pattern @p sp for the
-                                          * given row and the index within it.
-                                          */ 
-	Iterator (const SparsityPattern *sp,
-                  const unsigned int     row,
-                  const unsigned int     index);
-        
-                                         /**
-                                          * Prefix increment.
-                                          */
-	Iterator& operator++ ();
-
-                                         /**
-                                          * Postfix increment.
-                                          */
-	Iterator operator++ (int);
-
-                                         /**
-                                          * Dereferencing operator.
-                                          */
-	const Accessor & operator* () const;
-
-                                         /**
-                                          * Dereferencing operator.
-                                          */
-	const Accessor * operator-> () const;
-
-                                         /**
-                                          * Comparison. True, if
-                                          * both iterators point to
-                                          * the same matrix
-                                          * position.
-                                          */
-	bool operator == (const Iterator&) const;
-
-                                         /**
-                                          * Inverse of <tt>==</tt>.
-                                          */
-	bool operator != (const Iterator&) const;
-
-                                         /**
-                                          * Comparison
-                                          * operator. Result is true
-                                          * if either the first row
-                                          * number is smaller or if
-                                          * the row numbers are
-                                          * equal and the first
-                                          * index is smaller.
-                                          *
-                                          * This function is only valid if
-                                          * both iterators point into the same
-                                          * matrix.
-                                          */
-	bool operator < (const Iterator&) const;
-
-      private:
-                                         /**
-                                          * Store an object of the
-                                          * accessor class.
-                                          */
-        Accessor accessor;
-    };
-    
   }
 }
 
 
+
+namespace SparsityPatternIterators
+{
+				   // forward declaration
+  class Iterator;
+
+				   /**
+				    * Accessor class for iterators into
+				    * sparsity patterns. This class is also
+				    * the base class for both const and
+				    * non-const accessor classes into sparse
+				    * matrices.
+				    *
+				    * Note that this class only allow read
+				    * access to elements, providing their
+				    * row and column number. It does not
+				    * allow to modify the sparsity pattern
+				    * itself.
+				    */
+  class Accessor
+  {
+    public:
+				       /**
+					* Constructor.
+					*/
+      Accessor (const SparsityPattern *matrix,
+		const unsigned int     row,
+		const unsigned int     index);
+
+				       /**
+					* Constructor. Construct the end
+					* accessor for the given sparsity
+					* pattern.
+					*/
+      Accessor (const SparsityPattern *matrix);
+
+				       /**
+					* Row number of the element
+					* represented by this object. This
+					* function can only be called for
+					* entries for which is_valid_entry()
+					* is true.
+					*/
+      unsigned int row () const;
+
+				       /**
+					* Index in row of the element
+					* represented by this object. This
+					* function can only be called for
+					* entries for which is_valid_entry()
+					* is true.
+					*/
+      unsigned int index () const;
+
+				       /**
+					* Column number of the element
+					* represented by this object. This
+					* function can only be called for
+					* entries for which is_valid_entry() is
+					* true.
+					*/
+      unsigned int column () const;
+
+				       /**
+					* Return whether the sparsity
+					* pattern entry pointed to by this
+					* iterator is valid or not. Note
+					* that after compressing the
+					* sparsity pattern, all entries are
+					* valid. However, before
+					* compression, the sparsity pattern
+					* allocated some memory to be used
+					* while still adding new nonzero
+					* entries; if you create iterators
+					* in this phase of the sparsity
+					* pattern's lifetime, you will
+					* iterate over elements that are not
+					* valid. If this is so, then this
+					* function will return false.
+					*/
+      inline bool is_valid_entry () const;
+
+
+				       /**
+					* Comparison. True, if
+					* both iterators point to
+					* the same matrix
+					* position.
+					*/
+      bool operator == (const Accessor &) const;
+
+
+				       /**
+					* Comparison
+					* operator. Result is true
+					* if either the first row
+					* number is smaller or if
+					* the row numbers are
+					* equal and the first
+					* index is smaller.
+					*
+					* This function is only valid if
+					* both iterators point into the same
+					* sparsity pattern.
+					*/
+      bool operator < (const Accessor &) const;
+
+				       /** @addtogroup Exceptions
+					* @{ */
+
+				       //@}
+    protected:
+				       /**
+					* The sparsity pattern we operate on
+					* accessed.
+					*/
+      const SparsityPattern * sparsity_pattern;
+
+				       /**
+					* Current row number.
+					*/
+      unsigned int a_row;
+
+				       /**
+					* Current index in row.
+					*/
+      unsigned int a_index;
+
+				       /**
+					* Move the accessor to the next
+					* nonzero entry in the matrix.
+					*/
+      void advance ();
+
+				       /**
+					* Grant access to iterator class.
+					*/
+      friend class Iterator;
+  };
+
+
+
+				   /**
+				    * STL conforming iterator walking over
+				    * the elements of a sparsity pattern.
+				    */
+  class Iterator
+  {
+    public:
+				       /**
+					* Constructor. Create an iterator
+					* into the sparsity pattern @p sp for the
+					* given row and the index within it.
+					*/
+      Iterator (const SparsityPattern *sp,
+		const unsigned int     row,
+		const unsigned int     index);
+
+				       /**
+					* Prefix increment.
+					*/
+      Iterator& operator++ ();
+
+				       /**
+					* Postfix increment.
+					*/
+      Iterator operator++ (int);
+
+				       /**
+					* Dereferencing operator.
+					*/
+      const Accessor & operator* () const;
+
+				       /**
+					* Dereferencing operator.
+					*/
+      const Accessor * operator-> () const;
+
+				       /**
+					* Comparison. True, if
+					* both iterators point to
+					* the same matrix
+					* position.
+					*/
+      bool operator == (const Iterator&) const;
+
+				       /**
+					* Inverse of <tt>==</tt>.
+					*/
+      bool operator != (const Iterator&) const;
+
+				       /**
+					* Comparison
+					* operator. Result is true
+					* if either the first row
+					* number is smaller or if
+					* the row numbers are
+					* equal and the first
+					* index is smaller.
+					*
+					* This function is only valid if
+					* both iterators point into the same
+					* matrix.
+					*/
+      bool operator < (const Iterator&) const;
+
+    private:
+				       /**
+					* Store an object of the
+					* accessor class.
+					*/
+      Accessor accessor;
+  };
+}
+
+
+
 /**
  * Structure representing the sparsity pattern of a sparse matrix.
  *
@@ -430,9 +426,17 @@ class SparsityPattern : public Subscriptor
                                       * sparsity pattern.
                                       */
     typedef
-    internals::SparsityPatternIterators::Iterator
+    SparsityPatternIterators::Iterator
     const_iterator;
-    
+
+                                     /**
+                                      * Typedef an iterator class that allows
+                                      * to walk over the nonzero elements of a
+                                      * row of a sparsity pattern.
+                                      */
+    typedef
+    const unsigned int * row_iterator;
+
                                      /**
                                       * Typedef an iterator class that allows
                                       * to walk over all nonzero elements of a
@@ -444,10 +448,10 @@ class SparsityPattern : public Subscriptor
                                       * const_iterator.
                                       */
     typedef
-    internals::SparsityPatternIterators::Iterator
+    SparsityPatternIterators::Iterator
     iterator;
-    
-    
+
+
 				     /**
 				      * Define a value which is used
 				      * to indicate that a certain
@@ -473,7 +477,7 @@ class SparsityPattern : public Subscriptor
 				      * variable may change over time.
 				      */
     static const unsigned int invalid_entry = numbers::invalid_unsigned_int;
-    
+
 				     /**
 				      * Initialize the matrix empty,
 				      * that is with no memory
@@ -485,7 +489,7 @@ class SparsityPattern : public Subscriptor
 				      * the reinit() function.
 				      */
     SparsityPattern ();
-    
+
 				     /**
 				      * Copy constructor. This
 				      * constructor is only allowed to
@@ -561,7 +565,7 @@ class SparsityPattern : public Subscriptor
 		     const unsigned int               n,
 		     const std::vector<unsigned int>& row_lengths,
 		     const bool optimize_diagonal = true);
-    
+
 				     /**
 				      * Initialize a quadratic matrix
 				      * of dimension <tt>n</tt> with
@@ -641,7 +645,7 @@ class SparsityPattern : public Subscriptor
     SparsityPattern (const SparsityPattern  &original,
 		     const unsigned int      max_per_row,
 		     const unsigned int      extra_off_diagonals);
-    
+
 				     /**
 				      * Destructor.
 				      */
@@ -656,7 +660,7 @@ class SparsityPattern : public Subscriptor
 				      * objects.
 				      */
     SparsityPattern & operator = (const SparsityPattern &);
-    
+
 				     /**
 				      * Reallocate memory and set up data
 				      * structures for a new matrix with
@@ -711,7 +715,7 @@ class SparsityPattern : public Subscriptor
 		 const unsigned int               n,
 		 const VectorSlice<const std::vector<unsigned int> > &row_lengths,
 		 const bool optimize_diagonal = true);
-    
+
 				     /**
 				      * This function compresses the sparsity
 				      * structure that this object represents.
@@ -773,7 +777,34 @@ class SparsityPattern : public Subscriptor
 				      * iterator for the last row of a matrix.
 				      */
     inline iterator end (const unsigned int r) const;
-    
+
+				     /**
+				      * STL-like iterator with the first entry
+				      * of row <tt>r</tt>.
+				      *
+				      * Note that if the given row is empty,
+				      * i.e. does not contain any nonzero
+				      * entries, then the iterator returned by
+				      * this function equals
+				      * <tt>end(r)</tt>. Note also that the
+				      * iterator may not be dereferencable in
+				      * that case.
+				      */
+    inline row_iterator row_begin (const unsigned int r) const;
+
+				     /**
+				      * Final iterator of row <tt>r</tt>. It
+				      * points to the first element past the
+				      * end of line @p r, or past the end of
+				      * the entire sparsity pattern.
+				      *
+				      * Note that the end iterator is not
+				      * necessarily dereferencable. This is in
+				      * particular the case if it is the end
+				      * iterator for the last row of a matrix.
+				      */
+    inline row_iterator row_end (const unsigned int r) const;
+
     				     /**
 				      * This function can be used as a
 				      * replacement for reinit(),
@@ -920,43 +951,18 @@ class SparsityPattern : public Subscriptor
 		    const bool optimize_diagonal = true);
 
 				     /**
-				      * Copy data from an object of
-				      * type
-				      * CompressedSparsityPattern.
-				      * Previous content of this
-				      * object is lost, and the
-				      * sparsity pattern is in
-				      * compressed mode afterwards.
-				      */
-    void copy_from (const CompressedSparsityPattern &csp,
-		    const bool optimize_diagonal = true);
-
-
-                                     /**
-                                      * Copy data from an object of
-                                      * type
-                                      * CompressedSetSparsityPattern.
-                                      * Previous content of this
-                                      * object is lost, and the
-                                      * sparsity pattern is in
-                                      * compressed mode afterwards.
-                                      */
-    void copy_from (const CompressedSetSparsityPattern &csp,
-                    const bool optimize_diagonal = true);
-
-				     /**
-				      * Copy data from an object of
-				      * type
+				      * Copy data from an object of type
+				      * CompressedSparsityPattern,
+				      * CompressedSetSparsityPattern or
 				      * CompressedSimpleSparsityPattern.
-				      * Previous content of this
-				      * object is lost, and the
-				      * sparsity pattern is in
+				      * Previous content of this object is
+				      * lost, and the sparsity pattern is in
 				      * compressed mode afterwards.
 				      */
-    void copy_from (const CompressedSimpleSparsityPattern &csp,
+    template <typename CompressedSparsityType>
+    void copy_from (const CompressedSparsityType &csp,
 		    const bool optimize_diagonal = true);
 
-
 				     /**
 				      * Take a full matrix and use its
 				      * nonzero entries to generate a
@@ -971,7 +977,7 @@ class SparsityPattern : public Subscriptor
     template <typename number>
     void copy_from (const FullMatrix<number> &matrix,
 		    const bool optimize_diagonal = true);
-    
+
 				     /**
 				      * Return whether the object is empty. It
 				      * is empty if no memory is allocated,
@@ -1018,7 +1024,7 @@ class SparsityPattern : public Subscriptor
 				      * @deprecated Use
 				      * SparseMatrix::const_iterator
 				      */
-    unsigned int operator() (const unsigned int i, 
+    unsigned int operator() (const unsigned int i,
 			     const unsigned int j) const;
 
 				     /**
@@ -1043,7 +1049,7 @@ class SparsityPattern : public Subscriptor
 				      */
     std::pair<unsigned int, unsigned int>
     matrix_position (const unsigned int global_index) const;
-    
+
 				     /**
 				      * Add a nonzero entry to the matrix.
 				      * This function may only be called
@@ -1052,7 +1058,7 @@ class SparsityPattern : public Subscriptor
 				      * If the entry already exists, nothing
 				      * bad happens.
 				      */
-    void add (const unsigned int i, 
+    void add (const unsigned int i,
 	      const unsigned int j);
 
 				     /**
@@ -1065,11 +1071,11 @@ class SparsityPattern : public Subscriptor
 				      * exist, nothing bad happens.
 				      */
     template <typename ForwardIterator>
-    void add_entries (const unsigned int row, 
+    void add_entries (const unsigned int row,
 		      ForwardIterator    begin,
 		      ForwardIterator    end,
 		      const bool         indices_are_sorted = false);
-    
+
 				     /**
 				      * Make the sparsity pattern
 				      * symmetric by adding the
@@ -1082,7 +1088,7 @@ class SparsityPattern : public Subscriptor
 				      * quadratic matrix.
 				      */
     void symmetrize ();
-    
+
 				     /**
 				      * Return number of rows of this
 				      * matrix, which equals the dimension
@@ -1107,7 +1113,7 @@ class SparsityPattern : public Subscriptor
 				     /**
 				      * Number of entries in a specific row.
 				      */
-    inline unsigned int row_length (const unsigned int row) const;
+    unsigned int row_length (const unsigned int row) const;
 
 				     /**
 				      * Access to column number field.
@@ -1155,10 +1161,10 @@ class SparsityPattern : public Subscriptor
 				      * matrix struct is compressed. It does not
 				      * make too much sense otherwise anyway.
 				      */
-    unsigned int n_nonzero_elements () const;
+    std::size_t n_nonzero_elements () const;
 
 				     /**
-				      * Return whether the structure is 
+				      * Return whether the structure is
 				      * compressed or not.
 				      */
     bool is_compressed () const;
@@ -1218,7 +1224,7 @@ class SparsityPattern : public Subscriptor
 				      * template arguments.
 				      */
     bool stores_only_added_elements () const;
-    
+
                                      /**
 				      * @deprecated
 				      *
@@ -1282,7 +1288,7 @@ class SparsityPattern : public Subscriptor
                                       */
     void partition (const unsigned int         n_partitions,
                     std::vector<unsigned int> &partition_indices) const;
-    
+
 				     /**
 				      * Write the data of this object
 				      * en bloc to a file. This is
@@ -1329,7 +1335,7 @@ class SparsityPattern : public Subscriptor
 				      * more.
 				      */
     void block_read (std::istream &in);
-    
+
 				     /**
 				      * Print the sparsity of the
 				      * matrix. The output consists of
@@ -1373,10 +1379,10 @@ class SparsityPattern : public Subscriptor
 				      * of this object. See
 				      * MemoryConsumption.
 				      */
-    unsigned int memory_consumption () const;
+    std::size_t memory_consumption () const;
 
 				     /**
-				      * This is kind of an expert mode. Get 
+				      * This is kind of an expert mode. Get
 				      * access to the rowstart array, but
 				      * read-only.
 				      *
@@ -1435,20 +1441,8 @@ class SparsityPattern : public Subscriptor
 				     /** @addtogroup Exceptions
 				      * @{ */
 				     /**
-				      * Exception
-				      */
-    DeclException1 (ExcInvalidNumber,
-		    int,
-		    << "The provided number is invalid here: " << arg1);
-    				     /**
-				      * Exception
-				      */
-    DeclException2 (ExcInvalidIndex,
-		    int, int,
-		    << "The given index " << arg1
-		    << " should be less than " << arg2 << ".");
-				     /**
-				      * Exception
+				      * You tried to add an element to
+				      * a row, but there was no space left.
 				      */
     DeclException2 (ExcNotEnoughSpace,
 		    int, int,
@@ -1457,20 +1451,23 @@ class SparsityPattern : public Subscriptor
 		    << "(Maximum number of entries for this row: "
 		    << arg2 << "; maybe the matrix is already compressed?)");
 				     /**
-				      * Exception
+				      * The operation is only allowed
+				      * after the SparsityPattern has
+				      * been set up and compress() was
+				      * called.
 				      */
     DeclException0 (ExcNotCompressed);
 				     /**
-				      * Exception
+				      * This operation changes the
+				      * structure of the
+				      * SparsityPattern and is not
+				      * possible after compress() has
+				      * been called.
 				      */
     DeclException0 (ExcMatrixIsCompressed);
 				     /**
 				      * Exception
 				      */
-    DeclException0 (ExcEmptyObject);
-				     /**
-				      * Exception
-				      */
     DeclException0 (ExcInvalidConstructorCall);
 				     /**
 				      * This exception is thrown if
@@ -1496,13 +1493,6 @@ class SparsityPattern : public Subscriptor
                     int,
                     << "The number of partitions you gave is " << arg1
                     << ", but must be greater than zero.");
-                                     /**
-                                      * Exception
-                                      */
-    DeclException2 (ExcInvalidArraySize,
-                    int, int,
-                    << "The array has size " << arg1 << " but should have size "
-                    << arg2);
 				     //@}
   private:
 				     /**
@@ -1644,8 +1634,8 @@ class SparsityPattern : public Subscriptor
 				      * Is special treatment of
 				      * diagonals enabled?
 				      */
-    bool diagonal_optimized;    
-    
+    bool diagonal_optimized;
+
 				     /**
 				      * Make all sparse matrices
 				      * friends of this class.
@@ -1663,202 +1653,198 @@ class SparsityPattern : public Subscriptor
 #ifndef DOXYGEN
 
 
-namespace internals
+namespace SparsityPatternIterators
 {
-  namespace SparsityPatternIterators
-  {    
-    inline
-    Accessor::
-    Accessor (const SparsityPattern *sparsity_pattern,
-              const unsigned int     r,
-              const unsigned int     i)
-                    :
-                    sparsity_pattern(sparsity_pattern),
-                    a_row(r),
-                    a_index(i)
-    {}
+  inline
+  Accessor::
+  Accessor (const SparsityPattern *sparsity_pattern,
+	    const unsigned int     r,
+	    const unsigned int     i)
+		  :
+		  sparsity_pattern(sparsity_pattern),
+		  a_row(r),
+		  a_index(i)
+  {}
 
 
-    inline
-    Accessor::
-    Accessor (const SparsityPattern *sparsity_pattern)
-                    :
-                    sparsity_pattern(sparsity_pattern),
-                    a_row(sparsity_pattern->n_rows()),
-                    a_index(0)
-    {}
+  inline
+  Accessor::
+  Accessor (const SparsityPattern *sparsity_pattern)
+		  :
+		  sparsity_pattern(sparsity_pattern),
+		  a_row(sparsity_pattern->n_rows()),
+		  a_index(0)
+  {}
 
 
-    inline
-    unsigned int
-    Accessor::row() const
-    {
-      Assert (is_valid_entry() == true, ExcInvalidIterator());
-
-      return a_row;
-    }
-
-
-    inline
-    unsigned int
-    Accessor::column() const
-    {
-      Assert (is_valid_entry() == true, ExcInvalidIterator());
-      
-      return (sparsity_pattern
-              ->get_column_numbers()[sparsity_pattern
-                                     ->get_rowstart_indices()[a_row]+a_index]);
-    }
-
-
-    inline
-    unsigned int
-    Accessor::index() const
-    {
-      Assert (is_valid_entry() == true, ExcInvalidIterator());
-
-      return a_index;
-    }
-
-
-
-    inline
-    bool
-    Accessor::is_valid_entry () const
-    {
-      return (sparsity_pattern
-              ->get_column_numbers()[sparsity_pattern
-                                     ->get_rowstart_indices()[a_row]+a_index]
-              != SparsityPattern::invalid_entry);
-    }
-
-
-
-    inline
-    bool
-    Accessor::operator == (const Accessor& other) const
-    {      
-      return (sparsity_pattern  == other.sparsity_pattern &&
-              a_row   == other.a_row &&
-              a_index == other.a_index);
-    }
-
-
-
-    inline
-    bool
-    Accessor::operator < (const Accessor& other) const
-    {      
-      Assert (sparsity_pattern == other.sparsity_pattern,
-              ExcInternalError());
-      
-      return (a_row < other.a_row ||
-              (a_row == other.a_row &&
-               a_index < other.a_index));
-    }
-    
-
-    inline
-    void
-    Accessor::advance ()
-    {      
-      Assert (a_row < sparsity_pattern->n_rows(), ExcIteratorPastEnd());
-  
-      ++a_index;
-
-                                       // if at end of line: cycle until we
-                                       // find a row with a nonzero number of
-                                       // entries
-      while (a_index >= sparsity_pattern->row_length(a_row))
-        {
-          a_index = 0;
-          ++a_row;
-
-                                           // if we happened to find the end
-                                           // of the matrix, then stop here
-          if (a_row == sparsity_pattern->n_rows())
-            break;
-        }
-    }
-    
-
-
-    inline
-    Iterator::Iterator (const SparsityPattern *sparsity_pattern,
-                        const unsigned int     r,
-                        const unsigned int     i)
-                    :
-                    accessor(sparsity_pattern, r, i)
-    {}
-
-
-
-    inline
-    Iterator &
-    Iterator::operator++ ()
-    {
-      accessor.advance ();
-      return *this;
-    }
-
-
-
-    inline
-    Iterator
-    Iterator::operator++ (int)
-    {
-      const Iterator iter = *this;
-      accessor.advance ();
-      return iter;
-    }
-
-
-
-    inline
-    const Accessor &
-    Iterator::operator* () const
-    {
-      return accessor;
-    }
-
-
-
-    inline
-    const Accessor *
-    Iterator::operator-> () const
-    {
-      return &accessor;
-    }
-
-
-    inline
-    bool
-    Iterator::operator == (const Iterator& other) const
-    {
-      return (accessor == other.accessor);
-    }
-
-
-
-    inline
-    bool
-    Iterator::operator != (const Iterator& other) const
-    {
-      return ! (*this == other);
-    }
-
-
-    inline
-    bool
-    Iterator::operator < (const Iterator& other) const
-    {
-      return accessor < other.accessor;
-    }
-    
+  inline
+  bool
+  Accessor::is_valid_entry () const
+  {
+    return (sparsity_pattern
+	    ->get_column_numbers()[sparsity_pattern
+				   ->get_rowstart_indices()[a_row]+a_index]
+	    != SparsityPattern::invalid_entry);
   }
-}
 
 
+  inline
+  unsigned int
+  Accessor::row() const
+  {
+    Assert (is_valid_entry() == true, ExcInvalidIterator());
+
+    return a_row;
+  }
+
+
+  inline
+  unsigned int
+  Accessor::column() const
+  {
+    Assert (is_valid_entry() == true, ExcInvalidIterator());
+
+    return (sparsity_pattern
+	    ->get_column_numbers()[sparsity_pattern
+				   ->get_rowstart_indices()[a_row]+a_index]);
+  }
+
+
+  inline
+  unsigned int
+  Accessor::index() const
+  {
+    Assert (is_valid_entry() == true, ExcInvalidIterator());
+
+    return a_index;
+  }
+
+
+
+
+  inline
+  bool
+  Accessor::operator == (const Accessor& other) const
+  {
+    return (sparsity_pattern  == other.sparsity_pattern &&
+	    a_row   == other.a_row &&
+	    a_index == other.a_index);
+  }
+
+
+
+  inline
+  bool
+  Accessor::operator < (const Accessor& other) const
+  {
+    Assert (sparsity_pattern == other.sparsity_pattern,
+	    ExcInternalError());
+
+    return (a_row < other.a_row ||
+	    (a_row == other.a_row &&
+	     a_index < other.a_index));
+  }
+
+
+  inline
+  void
+  Accessor::advance ()
+  {
+    Assert (a_row < sparsity_pattern->n_rows(), ExcIteratorPastEnd());
+
+    ++a_index;
+
+				     // if at end of line: cycle until we
+				     // find a row with a nonzero number of
+				     // entries
+    while (a_index >= sparsity_pattern->row_length(a_row))
+      {
+	a_index = 0;
+	++a_row;
+
+					 // if we happened to find the end
+					 // of the matrix, then stop here
+	if (a_row == sparsity_pattern->n_rows())
+	  break;
+      }
+  }
+
+
+
+  inline
+  Iterator::Iterator (const SparsityPattern *sparsity_pattern,
+		      const unsigned int     r,
+		      const unsigned int     i)
+		  :
+		  accessor(sparsity_pattern, r, i)
+  {}
+
+
+
+  inline
+  Iterator &
+  Iterator::operator++ ()
+  {
+    accessor.advance ();
+    return *this;
+  }
+
+
+
+  inline
+  Iterator
+  Iterator::operator++ (int)
+  {
+    const Iterator iter = *this;
+    accessor.advance ();
+    return iter;
+  }
+
+
+
+  inline
+  const Accessor &
+  Iterator::operator* () const
+  {
+    return accessor;
+  }
+
+
+
+  inline
+  const Accessor *
+  Iterator::operator-> () const
+  {
+    return &accessor;
+  }
+
+
+  inline
+  bool
+  Iterator::operator == (const Iterator& other) const
+  {
+    return (accessor == other.accessor);
+  }
+
+
+
+  inline
+  bool
+  Iterator::operator != (const Iterator& other) const
+  {
+    return ! (*this == other);
+  }
+
+
+  inline
+  bool
+  Iterator::operator < (const Iterator& other) const
+  {
+    return accessor < other.accessor;
+  }
+
+}
+
 
 
 inline
@@ -1921,6 +1907,26 @@ SparsityPattern::end (const unsigned int r) const
 
 
 
+inline
+SparsityPattern::row_iterator
+SparsityPattern::row_begin (const unsigned int r) const
+{
+  Assert (r<n_rows(), ExcIndexRange(r,0,n_rows()));
+  return &colnums[rowstart[r]];
+}
+
+
+
+inline
+SparsityPattern::row_iterator
+SparsityPattern::row_end (const unsigned int r) const
+{
+  Assert (r<n_rows(), ExcIndexRange(r,0,n_rows()));
+  return &colnums[rowstart[r+1]];
+}
+
+
+
 inline
 unsigned int
 SparsityPattern::n_rows () const
@@ -1982,6 +1988,7 @@ SparsityPattern::get_column_numbers () const
 }
 
 
+
 inline
 unsigned int
 SparsityPattern::row_length (const unsigned int row) const
@@ -1991,6 +1998,7 @@ SparsityPattern::row_length (const unsigned int row) const
 }
 
 
+
 inline
 unsigned int
 SparsityPattern::column_number (const unsigned int row,
@@ -2004,10 +2012,10 @@ SparsityPattern::column_number (const unsigned int row,
 
 
 inline
-unsigned int
+std::size_t
 SparsityPattern::n_nonzero_elements () const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());  
+  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
   Assert (compressed, ExcNotCompressed());
   return rowstart[rows]-rowstart[0];
 }
@@ -2059,7 +2067,7 @@ SparsityPattern::copy_from (const unsigned int    n_rows,
 {
   Assert (static_cast<unsigned int>(std::distance (begin, end)) == n_rows,
 	  ExcIteratorRange (std::distance (begin, end), n_rows));
-  
+
 				   // first determine row lengths for
 				   // each row. if the matrix is
 				   // quadratic, then we might have to
@@ -2098,8 +2106,8 @@ SparsityPattern::copy_from (const unsigned int    n_rows,
 	{
 	  const unsigned int col
 	    = internal::SparsityPatternTools::get_column_index_from_iterator(*j);
-	  Assert (col < n_cols, ExcInvalidIndex(col,n_cols));
-	  
+	  Assert (col < n_cols, ExcIndexRange(col,0,n_cols));
+
 	  if ((col!=row) || !is_square)
 	    *cols++ = col;
 	};
