@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_block_sparse_matrix.h 18547 2009-04-03 13:04:41Z kronbichler $
+//    $Id: trilinos_block_sparse_matrix.h 20934 2010-04-02 13:09:20Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,18 +15,18 @@
 
 
 #include <base/config.h>
-#include <base/table.h>
-#include <lac/block_matrix_base.h>
-#include <lac/trilinos_sparse_matrix.h>
-#include <lac/trilinos_block_vector.h>
-#include <lac/full_matrix.h>
-#include <lac/exceptions.h>
-
-#include <cmath>
-
 
 #ifdef DEAL_II_USE_TRILINOS
 
+#  include <base/table.h>
+#  include <lac/block_matrix_base.h>
+#  include <lac/trilinos_sparse_matrix.h>
+#  include <lac/trilinos_block_vector.h>
+#  include <lac/full_matrix.h>
+#  include <lac/exceptions.h>
+
+#  include <cmath>
+
 #  define TrilinosScalar double
 
 DEAL_II_NAMESPACE_OPEN
@@ -41,7 +41,7 @@ template <typename number> class BlockSparseMatrix;
 
 namespace TrilinosWrappers
 {
-  
+
 /*! @addtogroup TrilinosWrappers
  *@{
  */
@@ -65,6 +65,7 @@ namespace TrilinosWrappers
  * the same reason as is documented with the BlockSparsityPattern class.
  *
  * @ingroup Matrix1
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Martin Kronbichler, Wolfgang Bangerth, 2008
  */
   class BlockSparseMatrix : public BlockMatrixBase<SparseMatrix>
@@ -75,7 +76,7 @@ namespace TrilinosWrappers
                                         * access to its own typedefs.
                                         */
       typedef BlockMatrixBase<SparseMatrix> BaseClass;
-    
+
                                        /**
                                         * Typedef the type of the underlying
                                         * matrix.
@@ -181,7 +182,7 @@ namespace TrilinosWrappers
                                        /**
                                         * Resize the matrix, by using an
 				        * array of Epetra maps to determine
-				        * the distribution of the 
+				        * the %parallel distribution of the
 				        * individual matrices. This function
 				        * assumes that a quadratic block
 				        * matrix is generated.
@@ -190,6 +191,19 @@ namespace TrilinosWrappers
       void reinit (const std::vector<Epetra_Map> &input_maps,
 		   const BlockSparsityType       &block_sparsity_pattern);
 
+                                       /**
+                                        * Resize the matrix, by using an
+				        * array of index sets to determine
+				        * the %parallel distribution of the
+				        * individual matrices. This function
+				        * assumes that a quadratic block
+				        * matrix is generated.
+                                        */
+      template <typename BlockSparsityType>
+      void reinit (const std::vector<IndexSet> &input_maps,
+		   const BlockSparsityType     &block_sparsity_pattern,
+		   const MPI_Comm              &communicator = MPI_COMM_WORLD);
+
                                        /**
                                         * Resize the matrix and initialize it
                                         * by the given sparsity pattern. Since
@@ -204,10 +218,10 @@ namespace TrilinosWrappers
                                         * This function initializes the
 				        * Trilinos matrix using the deal.II
 				        * sparse matrix and the entries stored
-				        * therein. It uses a threshold 
-				        * to copy only elements whose 
-				        * modulus is larger than the 
-				        * threshold (so zeros in the 
+				        * therein. It uses a threshold
+				        * to copy only elements whose
+				        * modulus is larger than the
+				        * threshold (so zeros in the
 				        * deal.II matrix can be filtered
 				        * away).
                                         */
@@ -235,14 +249,14 @@ namespace TrilinosWrappers
 
 				       /**
 					* This function calls the compress()
-				        * command of all matrices after 
-				        * the assembly is 
+				        * command of all matrices after
+				        * the assembly is
 				        * completed. Note that all MPI
-				        * processes need to call this 
-				        * command (whereas the individual 
+				        * processes need to call this
+				        * command (whereas the individual
 				        * assembly routines will most probably
 				        * only be called on each processor
-				        * individually) before any 
+				        * individually) before any
 				        * can complete it.
 				        */
       void compress ();
@@ -288,7 +302,7 @@ namespace TrilinosWrappers
                                        /**
                                         * Return the number of nonzero
                                         * elements of this
-                                        * matrix. 
+                                        * matrix.
                                         */
       unsigned int n_nonzero_elements () const;
 
@@ -603,11 +617,11 @@ namespace TrilinosWrappers
                                         * protected.
                                         */
       using BlockMatrixBase<SparseMatrix>::clear;
-      
+
 				       /** @addtogroup Exceptions
 					* @{
 					*/
-      
+
                                        /**
                                         * Exception
                                         */
@@ -632,6 +646,23 @@ namespace TrilinosWrappers
 
 // ------------- inline and template functions -----------------
 
+
+
+  inline
+  BlockSparseMatrix &
+  BlockSparseMatrix::operator = (const double d)
+  {
+    Assert (d==0, ExcScalarAssignmentOnlyForZeroValue());
+
+    for (unsigned int r=0; r<this->n_block_rows(); ++r)
+      for (unsigned int c=0; c<this->n_block_cols(); ++c)
+	this->block(r,c) = d;
+
+    return *this;
+  }
+
+
+
   inline
   bool
   BlockSparseMatrix::is_compressed () const
@@ -657,7 +688,7 @@ namespace TrilinosWrappers
   {
     BaseClass::vmult_block_block (dst, src);
   }
-  
+
 
 
   inline
@@ -677,7 +708,7 @@ namespace TrilinosWrappers
   {
     BaseClass::vmult_block_nonblock (dst, src);
   }
-  
+
 
 
   inline
@@ -697,7 +728,7 @@ namespace TrilinosWrappers
   {
     BaseClass::vmult_nonblock_block (dst, src);
   }
-  
+
 
 
   inline
