@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_precondition.h 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: trilinos_precondition.h 21411 2010-06-30 11:53:57Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,22 +15,24 @@
 
 
 #include <base/config.h>
-#include <base/subscriptor.h>
-
-#include <base/std_cxx1x/shared_ptr.h>
-
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
-#  include <Epetra_MpiComm.h>
-#else
-#  include <Epetra_SerialComm.h>
-#endif
-#include <Epetra_Map.h>
+#  include <base/subscriptor.h>
+#  include <base/std_cxx1x/shared_ptr.h>
 
-#include <Teuchos_RCP.hpp>
-#include <Epetra_Operator.h>
+#  include <lac/trilinos_vector_base.h>
+
+#  ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+#    include <Epetra_MpiComm.h>
+#  else
+#    include <Epetra_SerialComm.h>
+#  endif
+#  include <Epetra_Map.h>
+
+#  include <Teuchos_ParameterList.hpp>
+#  include <Epetra_Operator.h>
+#  include <Epetra_Vector.h>
 
 // forward declarations
 class Ifpack_Preconditioner;
@@ -46,6 +48,7 @@ DEAL_II_NAMESPACE_OPEN
 // forward declarations
 template <typename number> class SparseMatrix;
 template <typename number> class Vector;
+class SparsityPattern;
 
 /*! @addtogroup TrilinosWrappers
  *@{
@@ -53,8 +56,7 @@ template <typename number> class Vector;
 
 namespace TrilinosWrappers
 {
-
-  class VectorBase;
+  // forward declarations
   class SparseMatrix;
   class BlockSparseMatrix;
   class SolverBase;
@@ -75,7 +77,7 @@ namespace TrilinosWrappers
                                         * Standardized data struct to
                                         * pipe additional flags to the
                                         * preconditioner.
-                                        */      
+                                        */
       struct AdditionalData
       {};
 
@@ -99,7 +101,14 @@ namespace TrilinosWrappers
                                         * Destructor.
                                         */
       ~PreconditionBase ();
-      
+
+				       /**
+					* Destroys the preconditioner, leaving
+					* an object like just after having
+					* called the constructor.
+					*/
+      void clear ();
+
 				       /**
 					* Apply the preconditioner.
 					*/
@@ -136,7 +145,7 @@ namespace TrilinosWrappers
 					* is used when applying the
 					* preconditioner.
 					*/
-      Teuchos::RCP<const Epetra_Operator> preconditioner;
+      std_cxx1x::shared_ptr<Epetra_Operator> preconditioner;
 
                                        /**
 					* Internal communication
@@ -155,10 +164,10 @@ namespace TrilinosWrappers
 					* case the matrix needs to be
 					* copied from deal.II format.
 					*/
-      std::auto_ptr<Epetra_Map>   map;
+      std_cxx1x::shared_ptr<Epetra_Map>   vector_distributor;
   };
 
-  
+
 /**
  * A wrapper class for a (pointwise) Jacobi preconditioner for
  * Trilinos matrices. This preconditioner works both in serial and in
@@ -172,8 +181,8 @@ namespace TrilinosWrappers
  * settings are 1 for the damping parameter and zero for the diagonal
  * augmentation.
  *
- * @ingroup TrilinosWrappers 
- * @ingroup Preconditioners 
+ * @ingroup TrilinosWrappers
+ * @ingroup Preconditioners
  * @author Martin Kronbichler, 2008
  */
   class PreconditionJacobi : public PreconditionBase
@@ -204,7 +213,7 @@ namespace TrilinosWrappers
                                         * is not too far away from the
                                         * one we want to
                                         * treat.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -215,7 +224,7 @@ namespace TrilinosWrappers
 					*/
 	AdditionalData (const double       omega = 1,
 			const double       min_diagonal = 0);
-	
+
                                        /**
 					* This specifies the
 					* relaxation parameter in the
@@ -247,19 +256,11 @@ namespace TrilinosWrappers
 					*/
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
   };
-  
 
-  
-  
+
+
+
 /**
  * A wrapper class for a (pointwise) SSOR preconditioner for Trilinos
  * matrices. This preconditioner works both in serial and in parallel,
@@ -291,7 +292,7 @@ namespace TrilinosWrappers
   class PreconditionSSOR : public PreconditionBase
   {
     public:
- 
+
                                        /**
                                         * Standardized data struct to
                                         * pipe additional flags to the
@@ -322,7 +323,7 @@ namespace TrilinosWrappers
                                         * in parallel, forming a
                                         * so-called additive Schwarz
                                         * preconditioner.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -339,7 +340,7 @@ namespace TrilinosWrappers
 	AdditionalData (const double       omega = 1,
 			const double       min_diagonal = 0,
 			const unsigned int overlap = 0);
-	
+
                                        /**
 					* This specifies the (over-)
 					* relaxation parameter in the
@@ -369,7 +370,7 @@ namespace TrilinosWrappers
 					* processor in a parallel
 					* application should be.
 					*/
-	unsigned int overlap; 
+	unsigned int overlap;
       };
 
                                        /**
@@ -383,19 +384,11 @@ namespace TrilinosWrappers
 					*/
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
   };
-  
 
-  
-  
+
+
+
 /**
  * A wrapper class for a (pointwise) SOR preconditioner for Trilinos
  * matrices. This preconditioner works both in serial and in parallel,
@@ -410,7 +403,7 @@ namespace TrilinosWrappers
  * if and how much overlap there should be between the matrix
  * partitions on the various MPI processes. The default settings are 1
  * for the relaxation parameter, 0 for the diagonal augmentation and 0
- * for the overlap. 
+ * for the overlap.
  *
  * Note that a parallel applicatoin of the SOR preconditioner is
  * actually a block-Jacobi preconditioner with block size equal to the
@@ -458,7 +451,7 @@ namespace TrilinosWrappers
                                         * in parallel, forming a
                                         * so-called additive Schwarz
                                         * preconditioner.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -505,7 +498,7 @@ namespace TrilinosWrappers
 					* processor in a parallel
 					* application should be.
 					*/
-	unsigned int overlap; 
+	unsigned int overlap;
       };
 
                                        /**
@@ -518,15 +511,7 @@ namespace TrilinosWrappers
                                         * are any.
 					*/
       void initialize (const SparseMatrix   &matrix,
-		       const AdditionalData &additional_data = AdditionalData()); 
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
+		       const AdditionalData &additional_data = AdditionalData());
   };
 
 
@@ -610,7 +595,7 @@ namespace TrilinosWrappers
                                         * partitions when the
                                         * preconditioner runs in
                                         * parallel.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -684,7 +669,7 @@ namespace TrilinosWrappers
 					* processor in a parallel
 					* application should be.
 					*/
-	unsigned int overlap; 
+	unsigned int overlap;
       };
 
                                        /**
@@ -696,14 +681,6 @@ namespace TrilinosWrappers
                                         */
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
   };
 
 
@@ -787,7 +764,7 @@ namespace TrilinosWrappers
                                         * partitions when the
                                         * preconditioner runs in
                                         * parallel.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -860,7 +837,7 @@ namespace TrilinosWrappers
 					* processor in a parallel
 					* application should be.
 					*/
-	unsigned int overlap; 
+	unsigned int overlap;
       };
 
                                        /**
@@ -872,14 +849,6 @@ namespace TrilinosWrappers
                                         */
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
   };
 
 
@@ -961,7 +930,7 @@ namespace TrilinosWrappers
                                         * parameter specifies the overlap of
                                         * the partitions when the
                                         * preconditioner runs in parallel.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -1042,7 +1011,7 @@ namespace TrilinosWrappers
 					* processor in a parallel
 					* application should be.
 					*/
-	unsigned int overlap; 
+	unsigned int overlap;
       };
 
                                        /**
@@ -1054,14 +1023,6 @@ namespace TrilinosWrappers
                                         */
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
   };
 
 
@@ -1093,14 +1054,14 @@ namespace TrilinosWrappers
                                         * Standardized data struct to
                                         * pipe additional parameters
                                         * to the preconditioner.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
 					* Constructor.
 					*/
 	AdditionalData (const unsigned int overlap  = 0);
-	
+
 
                                        /**
 					* This determines how large
@@ -1121,16 +1082,8 @@ namespace TrilinosWrappers
                                         */
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Preconditioner> ifpack;
   };
-  
+
 
 
 
@@ -1153,7 +1106,7 @@ namespace TrilinosWrappers
                                         * Standardized data struct to
                                         * pipe additional parameters
                                         * to the preconditioner.
-                                        */      
+                                        */
       struct AdditionalData
       {
                                        /**
@@ -1165,7 +1118,7 @@ namespace TrilinosWrappers
 			const double       min_eigenvalue   = 1.,
 			const double       min_diagonal     = 1e-12,
 			const bool         nonzero_starting = false);
-	
+
                                        /**
 					* This determines the degree of the
 					* Chebyshev polynomial. The degree
@@ -1238,16 +1191,8 @@ namespace TrilinosWrappers
                                         */
       void initialize (const SparseMatrix   &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
-
-    private:
-				       /**
-					* This is a pointer to the
-					* Ifpack data contained in
-					* this preconditioner.
-					*/
-      Teuchos::RCP<Ifpack_Chebyshev> ifpack;
   };
-  
+
 
 
 /**
@@ -1259,7 +1204,7 @@ namespace TrilinosWrappers
  * preconditioner to be based on. A call of the respective
  * <code>vmult</code> function does call the respective operation in the
  * Trilinos package, where it is called <code>ApplyInverse</code>. Use of
- * this class is explained in the @ref step_31 "step-31" tutorial program.
+ * this class is explained in the step-31 tutorial program.
  *
  * Since the Trilinos objects we want to use are heavily dependent on Epetra
  * objects, we recommend using this class in conjunction with Trilinos
@@ -1306,9 +1251,11 @@ namespace TrilinosWrappers
 					*/
 	AdditionalData (const bool                             elliptic = true,
 			const bool                             higher_order_elements = false,
+			const unsigned int                     n_cycles = 1,
+			const bool                             w_cyle = false,
 			const double                           aggregation_threshold = 1e-4,
 			const std::vector<std::vector<bool> > &constant_modes = std::vector<std::vector<bool> > (1),
-			const unsigned int                     smoother_sweeps = 3,
+			const unsigned int                     smoother_sweeps = 2,
 			const unsigned int                     smoother_overlap = 0,
 			const bool                             output_details = false);
 
@@ -1333,6 +1280,20 @@ namespace TrilinosWrappers
 					*/
 	bool higher_order_elements;
 
+				       /**
+					* Defines how many multigrid cycles
+					* should be performed by the
+					* preconditioner.
+					*/
+	unsigned int n_cycles;
+
+				       /**
+					* Defines whether a w-cycle should be
+					* used instead of the standard setting
+					* of a v-cycle.
+					*/
+	bool w_cycle;
+
 				       /**
 					* This threshold tells the AMG setup
 					* how the coarsening should be
@@ -1411,6 +1372,29 @@ namespace TrilinosWrappers
       void initialize (const SparseMatrix                    &matrix,
 		       const AdditionalData &additional_data = AdditionalData());
 
+				       /**
+					* Let Trilinos compute a multilevel
+					* hierarchy for the solution of a
+					* linear system with the given
+					* matrix. The function uses the
+					* matrix format specified in
+					* TrilinosWrappers::SparseMatrix.
+					*
+					* This function is similar to the one
+					* above, but allows the user to set
+					* all the options of the Trilinos ML
+					* preconditioner. In order to find out
+					* about all the options for ML, we
+					* refer to the <a
+					* href=http://trilinos.sandia.gov/packages/ml/mlguide5.pdf>ML
+					* user's guide</a>. In particular,
+					* users need to follow the ML
+					* instructions in case a vector-valued
+					* problem ought to be solved.
+					*/
+      void initialize (const SparseMatrix           &matrix,
+		       const Teuchos::ParameterList &ml_parameters);
+
 				       /**
 					* Let Trilinos compute a multilevel
 					* hierarchy for the solution of a
@@ -1421,9 +1405,11 @@ namespace TrilinosWrappers
 					* the function can be considered
 					* rather inefficient.
 					*/
-      void initialize (const ::dealii::SparseMatrix<double> &deal_ii_sparse_matrix,
+      template <typename number>
+      void initialize (const ::dealii::SparseMatrix<number> &deal_ii_sparse_matrix,
 		       const AdditionalData                 &additional_data = AdditionalData(),
-		       const double                          drop_tolerance = 1e-13);
+		       const double                          drop_tolerance = 1e-13,
+		       const ::dealii::SparsityPattern      *use_this_sparsity = 0);
 
 				       /**
 					* This function can be used for a
@@ -1450,23 +1436,92 @@ namespace TrilinosWrappers
 					*/
       void reinit ();
 
-    private:
+				       /**
+					* Destroys the preconditioner, leaving
+					* an object like just after having
+					* called the constructor.
+					*/
+      void clear ();
 
 				       /**
-					* A pointer to the preconditioner
-					* object.
+					* Prints an estimate of the memory
+					* consumption of this class.
 					*/
-      Teuchos::RCP<ML_Epetra::MultiLevelPreconditioner> multilevel_operator;
+      unsigned int memory_consumption () const;
 
+    private:
 				       /**
 					* A copy of the deal.II matrix into
 					* Trilinos format.
 					*/
-      std_cxx1x::shared_ptr<SparseMatrix> Matrix;
+      std_cxx1x::shared_ptr<SparseMatrix> trilinos_matrix;
   };
 
+
+
+// -------------------------- inline and template functions ----------------------
+
+
+#ifndef DOXYGEN
+
+  inline
+  void
+  PreconditionBase::vmult (VectorBase       &dst,
+			   const VectorBase &src) const
+  {
+    Assert (dst.vector_partitioner().SameAs(preconditioner->OperatorRangeMap()),
+	    ExcNonMatchingMaps("dst"));
+    Assert (src.vector_partitioner().SameAs(preconditioner->OperatorDomainMap()),
+	    ExcNonMatchingMaps("src"));
+
+    const int ierr = preconditioner->ApplyInverse (src.trilinos_vector(),
+						   dst.trilinos_vector());
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+				        // For the implementation of
+				        // the <code>vmult</code>
+				        // function with deal.II data
+				        // structures we note that
+				        // invoking a call of the
+				        // Trilinos preconditioner
+				        // requires us to use Epetra
+				        // vectors as well. We do this
+				        // by providing a view, i.e.,
+				        // feed Trilinos with a
+				        // pointer to the data, so we
+				        // avoid copying the content
+				        // of the vectors during the
+				        // iteration (this function is
+				        // only useful when used in
+				        // serial anyway). In the
+				        // declaration of the right
+				        // hand side, we need to cast
+				        // the source vector (that is
+				        // <code>const</code> in all
+				        // deal.II calls) to
+				        // non-constant value, as this
+				        // is the way Trilinos wants
+				        // to have them.
+  inline
+  void PreconditionBase::vmult (dealii::Vector<double>       &dst,
+				const dealii::Vector<double> &src) const
+  {
+    Epetra_Vector LHS (View, preconditioner->OperatorDomainMap(),
+		       dst.begin());
+    Epetra_Vector RHS (View, preconditioner->OperatorRangeMap(),
+		       const_cast<double*>(src.begin()));
+
+    const int ierr = preconditioner->ApplyInverse (RHS, LHS);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+#endif
+
 }
 
+
 /*@}*/
 
 
