@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_solver.h 18195 2009-01-13 03:45:30Z bangerth $
+//    $Id: trilinos_solver.h 21131 2010-05-15 14:05:24Z kronbichler $
 //    Version: $Name$
 //
 //    Copyright (C) 2008, 2009 by the deal.II authors
@@ -13,17 +13,20 @@
 #ifndef __deal2__trilinos_solver_h
 #define __deal2__trilinos_solver_h
 
+
 #include <base/config.h>
-#include <lac/exceptions.h>
-#include <lac/solver_control.h>
-#include <lac/vector.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#include <Epetra_LinearProblem.h>
-#include <AztecOO.h>
-#include <Epetra_Operator.h>
-#include <Amesos.h>
+#  include <base/std_cxx1x/shared_ptr.h>
+#  include <lac/exceptions.h>
+#  include <lac/solver_control.h>
+#  include <lac/vector.h>
+
+#  include <Epetra_LinearProblem.h>
+#  include <AztecOO.h>
+#  include <Epetra_Operator.h>
+#  include <Amesos.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -217,7 +220,7 @@ namespace TrilinosWrappers
 					* which is passed down to the
 					* Trilinos solver.
 					*/
-      std::auto_ptr<Epetra_LinearProblem> linear_problem;
+      std_cxx1x::shared_ptr<Epetra_LinearProblem> linear_problem;
 
                                        /**
                                         * A structure that contains
@@ -635,14 +638,14 @@ namespace TrilinosWrappers
 					* which is passed down to the
 					* Trilinos solver.
 					*/
-      std::auto_ptr<Epetra_LinearProblem> linear_problem;
+      std_cxx1x::shared_ptr<Epetra_LinearProblem> linear_problem;
 
                                        /**
                                         * A structure that contains
                                         * the Trilinos solver and
                                         * preconditioner objects.
                                         */
-      std::auto_ptr<Amesos_BaseSolver> solver;
+      std_cxx1x::shared_ptr<Amesos_BaseSolver> solver;
 
                                        /**
                                         * Store a copy of the flags for this
