@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_vector.h 18302 2009-01-27 07:57:25Z kronbichler $
+//    $Id: trilinos_vector.h 21133 2010-05-17 11:11:22Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,15 +13,19 @@
 #ifndef __deal2__trilinos_vector_h
 #define __deal2__trilinos_vector_h
 
+
 #include <base/config.h>
-#include <base/subscriptor.h>
-#include <lac/exceptions.h>
-#include <lac/vector.h>
-#include <lac/trilinos_vector_base.h>
-#include <lac/trilinos_sparse_matrix.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
+#  include <base/std_cxx1x/shared_ptr.h>
+#  include <base/subscriptor.h>
+#  include <base/index_set.h>
+#  include <base/utilities.h>
+#  include <lac/exceptions.h>
+#  include <lac/vector.h>
+#  include <lac/trilinos_vector_base.h>
+
 #  include "Epetra_Map.h"
 #  include "Epetra_LocalMap.h"
 
@@ -37,6 +41,7 @@ template <typename> class Vector;
  */
 namespace TrilinosWrappers
 {
+  class SparseMatrix;
 /**
  * Namespace for Trilinos vector classes that work in parallel over
  * MPI. This namespace is restricted to vectors only, whereas matrices
@@ -47,8 +52,9 @@ namespace TrilinosWrappers
  */
   namespace MPI
   {
+    class BlockVector;
 
-/** 
+/**
  * This class implements a wrapper to use the Trilinos distributed
  * vector class Epetra_FEVector. This class is derived from the
  * TrilinosWrappers::VectorBase class and provides all functionality
@@ -118,11 +124,11 @@ namespace TrilinosWrappers
  * writes. If it encounters an operation of the opposite kind, it calls
  * compress() and flips the state. This can sometimes lead to very
  * confusing behavior, in code that may for example look like this:
- * 
- * @verbatim 
+ *
+ * @verbatim
  * TrilinosWrappers::Vector vector;
- * // do some write operations on the vector 
- * for (unsigned int i=0; i<vector->size(); ++i) 
+ * // do some write operations on the vector
+ * for (unsigned int i=0; i<vector->size(); ++i)
  *   vector(i) = i;
  *
  *                   // do some additions to vector elements, but
@@ -157,11 +163,15 @@ namespace TrilinosWrappers
  * @ingroup TrilinosWrappers
  * @ingroup Vectors
  * @see @ref SoftwareTrilinos
- * @author Martin Kronbichler, Wolfgang Bangerth, 2008
+ * @author Martin Kronbichler, Wolfgang Bangerth, 2008, 2009
  */
     class Vector : public VectorBase
     {
       public:
+/**
+ * @name Basic constructors and initalization.
+ */
+//@{
                                        /**
                                         * Default constructor that
                                         * generates an empty (zero size)
@@ -174,58 +184,16 @@ namespace TrilinosWrappers
                                         */
 	Vector ();
 
-                                       /**
-				        * This constructor takes an
-				        * Epetra_Map that already knows
-				        * how to distribute the
-				        * individual components among
-				        * the MPI processors. Since it
-				        * also includes information
-				        * about the size of the vector,
-				        * this is all we need to
-				        * generate a parallel vector.
-                                        */
-	Vector (const Epetra_Map &InputMap);
-
 	                               /**
 					* Copy constructor using the
 					* given vector.
 					*/
 	Vector (const Vector &V);
 
-				       /**
-					* Copy constructor from the
-					* TrilinosWrappers vector
-					* class. Since a vector of this
-					* class does not necessarily
-					* need to be distributed among
-					* processes, the user needs to
-					* supply us with an Epetra_Map
-					* that sets the partitioning
-					* details.
-					*/
-	explicit Vector (const Epetra_Map &InputMap,
-			 const VectorBase &v);
-
-                                         /**
-                                          * Copy-constructor from deal.II
-                                          * vectors. Sets the dimension to that
-                                          * of the given vector, and copies all
-                                          * elements.
-                                          */
-        template <typename Number>
-        explicit Vector (const Epetra_Map             &InputMap,
-                         const dealii::Vector<Number> &v);
-
-				       /**
-				        * Reinit functionality. This
-				        * function destroys the old
-				        * vector content and generates a
-				        * new one based on the input
-				        * map.
-				        */
-	void reinit (const Epetra_Map &input_map,
-		     const bool        fast = false);
+					 /**
+					  * Destructor.
+					  */
+	~Vector ();
 
 				       /**
 				        * Reinit functionality. This
@@ -240,9 +208,7 @@ namespace TrilinosWrappers
 				        * be left unchanged and the user has
 				        * to set all elements.
 					*
-				        * Reinit functionality. This class
-					* also has a third possible
-					* argument,
+				        * This function has a third  argument,
 					* <tt>allow_different_maps</tt>,
 					* that allows for an exchange of
 					* data between two equal-sized
@@ -252,7 +218,7 @@ namespace TrilinosWrappers
 					* of this function is to generate a
 					* replication of a whole vector on
 					* each machine, when the calling
-					* vector is build according to the
+					* vector is built according to the
 					* localized vector class
 					* TrilinosWrappers::Vector, and
 					* <tt>v</tt> is a distributed
@@ -268,6 +234,9 @@ namespace TrilinosWrappers
 		     const bool        fast = false,
 	             const bool        allow_different_maps = false);
 
+	void reinit (const BlockVector &v,
+	             const bool         import_data = false);
+
                                        /**
 					* Set all components of the
 					* vector to the given number @p
@@ -305,7 +274,7 @@ namespace TrilinosWrappers
 					* exception will be thrown.
 					*/
 	Vector &
-	  operator = (const ::dealii::TrilinosWrappers::Vector &V);
+	operator = (const ::dealii::TrilinosWrappers::Vector &V);
 
                                        /**
 					* Another copy function. This
@@ -325,7 +294,7 @@ namespace TrilinosWrappers
 					* &input_map) function.
 					*/
 	template <typename Number>
-	Vector & 
+	Vector &
 	operator = (const ::dealii::Vector<Number> &v);
 
 				       /**
@@ -368,22 +337,123 @@ namespace TrilinosWrappers
 					* for example done during the
 					* solution of linear systems.
 					*/
-	void import_nonlocal_data_for_fe 
+	void import_nonlocal_data_for_fe
 	  (const dealii::TrilinosWrappers::SparseMatrix &matrix,
 	   const Vector                                 &vector);
-
-      private:
+//@}
+/**
+ * @name Initialization with an Epetra_Map
+ */
+//@{
                                        /**
-                                        * The Epetra map is used to map
-                                        * (or rather, partition) vector
-                                        * data accross multiple
-                                        * processes. This is the
-                                        * communicator and data
-                                        * distribution object common to
-                                        * all Trilinos objects used by
-                                        * deal.II.
+				        * This constructor takes an
+				        * Epetra_Map that already knows
+				        * how to distribute the
+				        * individual components among
+				        * the MPI processors. Since it
+				        * also includes information
+				        * about the size of the vector,
+				        * this is all we need to
+				        * generate a parallel vector.
                                         */
-	Epetra_Map map;
+	Vector (const Epetra_Map &parallel_partitioning);
+
+				       /**
+					* Copy constructor from the
+					* TrilinosWrappers vector
+					* class. Since a vector of this
+					* class does not necessarily
+					* need to be distributed among
+					* processes, the user needs to
+					* supply us with an Epetra_Map
+					* that sets the partitioning
+					* details.
+					*/
+	explicit Vector (const Epetra_Map &parallel_partitioning,
+			 const VectorBase &v);
+
+				       /**
+					* Reinitialize from a deal.II
+					* vector. The Epetra_Map specifies the
+					* %parallel partitioning.
+					*/
+	template <typename number>
+	void reinit (const Epetra_Map             &parallel_partitioner,
+		     const dealii::Vector<number> &v);
+
+				       /**
+				        * Reinit functionality. This
+				        * function destroys the old
+				        * vector content and generates a
+				        * new one based on the input
+				        * map.
+				        */
+	void reinit (const Epetra_Map &parallel_partitioning,
+		     const bool        fast = false);
+
+                                         /**
+                                          * Copy-constructor from deal.II
+                                          * vectors. Sets the dimension to that
+                                          * of the given vector, and copies all
+                                          * elements.
+                                          */
+        template <typename Number>
+        explicit Vector (const Epetra_Map             &parallel_partitioning,
+                         const dealii::Vector<Number> &v);
+//@}
+/**
+ * @name Initialization with an IndexSet
+ */
+//@{
+                                       /**
+				        * This constructor takes an IndexSet
+				        * that defines how to distribute the
+				        * individual components among the
+				        * MPI processors. Since it also
+				        * includes information about the
+				        * size of the vector, this is all we
+				        * need to generate a %parallel
+				        * vector.
+                                        */
+	Vector (const IndexSet &parallel_partitioning,
+		const MPI_Comm &communicator = MPI_COMM_WORLD);
+
+				       /**
+					* Copy constructor from the
+					* TrilinosWrappers vector
+					* class. Since a vector of this
+					* class does not necessarily need to
+					* be distributed among processes,
+					* the user needs to supply us with
+					* an IndexSet and an MPI
+					* communicator that set the
+					* partitioning details.
+					*/
+	explicit Vector (const IndexSet   &parallel_partitioning,
+			 const VectorBase &v,
+			 const MPI_Comm   &communicator = MPI_COMM_WORLD);
+
+                                         /**
+                                          * Copy-constructor from deal.II
+                                          * vectors. Sets the dimension to
+                                          * that of the given vector, and
+                                          * copies all the elements.
+                                          */
+        template <typename Number>
+        explicit Vector (const IndexSet               &parallel_partitioning,
+			 const dealii::Vector<Number> &v,
+                         const MPI_Comm               &communicator = MPI_COMM_WORLD);
+
+				       /**
+				        * Reinit functionality. This
+				        * function destroys the old vector
+				        * content and generates a new one
+				        * based on the input partitioning.
+				        */
+	void reinit (const IndexSet &parallel_partitioning,
+		     const MPI_Comm &communicator = MPI_COMM_WORLD,
+		     const bool      fast = false);
+//@}
     };
 
 
@@ -397,7 +467,6 @@ namespace TrilinosWrappers
  * of the C++ standard library which uses a temporary object. The
  * function simply exchanges the data of the two vectors.
  *
- * @ingroup TrilinosWrappers
  * @relates TrilinosWrappers::MPI::Vector
  * @author Martin Kronbichler, Wolfgang Bangerth, 2008
  */
@@ -406,43 +475,48 @@ namespace TrilinosWrappers
     {
       u.swap (v);
     }
-  
+
 
 #ifndef DOXYGEN
 
     template <typename number>
-    Vector::Vector (const Epetra_Map             &InputMap,
+    Vector::Vector (const Epetra_Map             &input_map,
                     const dealii::Vector<number> &v)
-                    :
-                    map (InputMap)
     {
-      vector = std::auto_ptr<Epetra_FEVector> (new Epetra_FEVector(map));
-      
-      const int min_my_id = map.MinMyGID();
-      const int size = map.NumMyElements();
-
-      Assert (map.MaxLID() == size-1,
-	      ExcDimensionMismatch(map.MaxLID(), size-1));
-
-				   // Need to copy out values, since the
-				   // deal.II might not use doubles, so 
-				   // that a direct access is not possible.
-      std::vector<int> indices (size);
-      std::vector<double> values (size);
-      for (int i=0; i<size; ++i)
-	{
-	  indices[i] = map.GID(i);
-	  values[i] = v(i);
-	}
-
-      const int ierr = vector->ReplaceGlobalValues (size, &indices[0], 
-						    &values[0]);
-
-      AssertThrow (ierr == 0, VectorBase::ExcTrilinosError());
+      reinit (input_map, v);
     }
 
-  
-  
+
+
+    template <typename number>
+    Vector::Vector (const IndexSet               &parallel_partitioner,
+                    const dealii::Vector<number> &v,
+		    const MPI_Comm               &communicator)
+    {
+      *this = Vector(parallel_partitioner.make_trilinos_map (communicator, true),
+		     v);
+    }
+
+
+
+
+    template <typename number>
+    void Vector::reinit (const Epetra_Map             &parallel_partitioner,
+			 const dealii::Vector<number> &v)
+    {
+      if (vector.get() != 0 && vector->Map().SameAs(parallel_partitioner))
+	vector.reset (new Epetra_FEVector(parallel_partitioner));
+
+      const int size = parallel_partitioner.NumMyElements();
+
+				   // Need to copy out values, since the
+				   // deal.II might not use doubles, so
+				   // that a direct access is not possible.
+      for (int i=0; i<size; ++i)
+	(*vector)[0][i] = v(parallel_partitioner.GID(i));
+    }
+
+
     inline
     Vector &
     Vector::operator = (const TrilinosScalar s)
@@ -454,29 +528,32 @@ namespace TrilinosWrappers
 
 
     template <typename Number>
-    Vector & 
+    Vector &
     Vector::operator = (const ::dealii::Vector<Number> &v)
     {
       if (size() != v.size())
 	{
+	  vector.reset (new Epetra_FEVector(Epetra_Map
+					    (v.size(), 0,
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
-	  map = Epetra_Map (v.size(), 0, Epetra_MpiComm(MPI_COMM_SELF));
+					     Epetra_MpiComm(MPI_COMM_SELF)
 #else
-	  map = Epetra_Map (v.size(), 0, Epetra_SerialComm());
+					     Epetra_SerialComm()
 #endif
+					     )));
 	}
-      
-      *this = Vector(map, v);
 
+      reinit (vector_partitioner(), v);
       return *this;
     }
-    
-    
+
+
 #endif
 
   } /* end of namespace MPI */
 
 
+
 /**
  * This class is a specialization of a Trilinos vector to a localized
  * version. The purpose of this class is to provide a copy interface
@@ -521,7 +598,21 @@ namespace TrilinosWrappers
 				        * map will be generated
 				        * internally.
                                         */
-      Vector (const Epetra_Map &InputMap);
+      Vector (const Epetra_Map &partitioning);
+
+                                       /**
+				        * This constructor takes as input
+				        * the number of elements in the
+				        * vector. If the index set is not
+				        * localized, i.e., if there are some
+				        * elements that are not present on
+				        * all processes, only the global
+				        * size of the index set will be
+				        * taken and a localized version will
+				        * be generated internally.
+                                        */
+      Vector (const IndexSet &partitioning,
+	      const MPI_Comm &communicator = MPI_COMM_WORLD);
 
                                        /**
 				        * This constructor takes a
@@ -551,18 +642,41 @@ namespace TrilinosWrappers
 
                                        /**
 					* Initialization with an
-					* Epetra_Map. Similar to the
-					* call in the other class
-					* MPI::Vector, with the only
-					* difference that now a copy on
-					* all processes is
-					* generated. The variable
-					* <tt>fast</tt> determines
-					* whether the vector should be
-					* filled with zero or left
-					* untouched.
+					* Epetra_Map. Similar to the call in
+					* the other class MPI::Vector, with
+					* the difference that now a copy on
+					* all processes is generated. This
+					* initialization function is
+					* appropriate when the data in the
+					* localized vector should be
+					* imported from a distributed vector
+					* that has been initialized with the
+					* same communicator. The variable
+					* <tt>fast</tt> determines whether
+					* the vector should be filled with
+					* zero or left untouched.
 					*/
       void reinit (const Epetra_Map &input_map,
+		   const bool        fast = false);
+
+                                       /**
+					* Initialization with an
+					* IndexSet. Similar to the call in
+					* the other class MPI::Vector, with
+					* the difference that now a copy on
+					* all processes is generated. This
+					* initialization function is
+					* appropriate in case the data in
+					* the localized vector should be
+					* imported from a distributed vector
+					* that has been initialized with the
+					* same communicator. The variable
+					* <tt>fast</tt> determines whether
+					* the vector should be filled with
+					* zero or left untouched.
+					*/
+      void reinit (const IndexSet   &input_map,
+		   const MPI_Comm   &communicator = MPI_COMM_WORLD,
 		   const bool        fast = false);
 
                                        /**
@@ -610,14 +724,7 @@ namespace TrilinosWrappers
 					* the right hand argument.
 					*/
       Vector &
-	operator = (const Vector &V); 
-
-    private:
-				       /**
-					* A map indicating the size of the
-					* vector.
-					*/
-      Epetra_LocalMap map;
+	operator = (const Vector &V);
   };
 
 
@@ -630,7 +737,6 @@ namespace TrilinosWrappers
  * of the C++ standard library which uses a temporary object. The
  * function simply exchanges the data of the two vectors.
  *
- * @ingroup TrilinosWrappers
  * @relates TrilinosWrappers::Vector
  * @author Martin Kronbichler, Wolfgang Bangerth, 2008
  */
@@ -645,19 +751,14 @@ namespace TrilinosWrappers
 
   template <typename number>
   Vector::Vector (const dealii::Vector<number> &v)
-                 :
-#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
-                 map (v.size(), 0, Epetra_MpiComm(MPI_COMM_SELF))
-#else
-		 map (v.size(), 0, Epetra_SerialComm())
-#endif
   {
-    vector = std::auto_ptr<Epetra_FEVector> (new Epetra_FEVector(map));
+    Epetra_LocalMap map ((int)v.size(), 0, Utilities::Trilinos::comm_self());
+    vector.reset (new Epetra_FEVector(map));
     *this = v;
   }
 
-  
-  
+
+
   inline
   Vector &
   Vector::operator = (const TrilinosScalar s)
@@ -667,46 +768,36 @@ namespace TrilinosWrappers
     return *this;
   }
 
+
+
   template <typename Number>
-  Vector & 
+  Vector &
   Vector::operator = (const ::dealii::Vector<Number> &v)
   {
     if (size() != v.size())
       {
-	vector.release();
-#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
-	map = Epetra_LocalMap (v.size(), 0, Epetra_MpiComm(MPI_COMM_SELF));
-#else
-	map = Epetra_LocalMap (v.size(), 0, Epetra_SerialComm());
-#endif
+	vector.reset();
 
-	vector = std::auto_ptr<Epetra_FEVector> (new Epetra_FEVector(map));
+	Epetra_LocalMap map ((int)v.size(), 0,
+			     Utilities::Trilinos::comm_self());
+	vector.reset (new Epetra_FEVector(map));
       }
 
-    const int min_my_id = map.MinMyGID();
+    const Epetra_Map & map = vector_partitioner();
     const int size = map.NumMyElements();
 
     Assert (map.MaxLID() == size-1,
 	    ExcDimensionMismatch(map.MaxLID(), size-1));
 
 				   // Need to copy out values, since the
-				   // deal.II might not use doubles, so 
+				   // deal.II might not use doubles, so
 				   // that a direct access is not possible.
-    std::vector<int> indices (size);
-    std::vector<double> values (size);
     for (int i=0; i<size; ++i)
-      {
-	indices[i] = map.GID(i);
-	values[i] = v(i);
-      }
-
-    const int ierr = vector->ReplaceGlobalValues (size, &indices[0], 
-						  &values[0]);
-    AssertThrow (ierr == 0, VectorBase::ExcTrilinosError(ierr));
+      (*vector)[0][i] = v(i);
 
     return *this;
   }
-  
+
 #endif
 
 
