@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_vector_base.h 18491 2009-03-17 16:26:34Z kronbichler $
+//    $Id: trilinos_vector_base.h 21133 2010-05-17 11:11:22Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -15,16 +15,18 @@
 
 
 #include <base/config.h>
-#include <base/subscriptor.h>
-#include <lac/exceptions.h>
-#include <lac/vector.h>
-
-#include <vector>
-#include <utility>
-#include <memory>
 
 #ifdef DEAL_II_USE_TRILINOS
 
+#  include <base/std_cxx1x/shared_ptr.h>
+#  include <base/subscriptor.h>
+#  include <lac/exceptions.h>
+#  include <lac/vector.h>
+
+#  include <vector>
+#  include <utility>
+#  include <memory>
+
 #  define TrilinosScalar double
 #  include "Epetra_ConfigDefs.h"
 #  ifdef DEAL_II_COMPILER_SUPPORTS_MPI // only if MPI is installed
@@ -58,7 +60,7 @@ namespace TrilinosWrappers
 /**
  * A namespace for internal implementation details of the
  * TrilinosWrapper members.
- *  
+ *
  * @ingroup TrilinosWrappers
  */
   namespace internal
@@ -116,14 +118,14 @@ namespace TrilinosWrappers
 					* this allows us to make the
 					* assignment operator const.
 					*/
-	const VectorReference & 
+	const VectorReference &
 	  operator = (const VectorReference &r) const;
 
                                        /**
 					* Set the referenced element of the
 					* vector to <tt>s</tt>.
 					*/
-	const VectorReference & 
+	const VectorReference &
 	  operator = (const TrilinosScalar &s) const;
 
                                        /**
@@ -131,7 +133,7 @@ namespace TrilinosWrappers
 					* referenced element of the
 					* vector->
 					*/
-	const VectorReference & 
+	const VectorReference &
 	  operator += (const TrilinosScalar &s) const;
 
                                        /**
@@ -139,7 +141,7 @@ namespace TrilinosWrappers
 					* referenced element of the
 					* vector->
 					*/
-	const VectorReference & 
+	const VectorReference &
 	  operator -= (const TrilinosScalar &s) const;
 
 	                               /**
@@ -147,7 +149,7 @@ namespace TrilinosWrappers
 					* element of the vector by
 					* <tt>s</tt>.
 					*/
-	const VectorReference & 
+	const VectorReference &
 	  operator *= (const TrilinosScalar &s) const;
 
                                        /**
@@ -155,7 +157,7 @@ namespace TrilinosWrappers
 					* element of the vector by
 					* <tt>s</tt>.
 					*/
-	const VectorReference & 
+	const VectorReference &
 	  operator /= (const TrilinosScalar &s) const;
 
                                        /**
@@ -262,7 +264,7 @@ namespace TrilinosWrappers
       typedef const internal::VectorReference const_reference;
 
 				       /**
-					* @name 1: Basic Object-handling 
+					* @name 1: Basic Object-handling
 					*/
 				       //@{
 
@@ -319,8 +321,26 @@ namespace TrilinosWrappers
                                         * vector element-by-element and
                                         * before anything else can be
                                         * done on it.
+					*
+					* The (defaulted) argument can
+					* be used to specify the
+					* compress mode
+					* (<code>Add</code> or
+					* <code>Insert</code>) in case
+					* the vector has not been
+					* written to since the last
+					* time this function was
+					* called. The argument is
+					* ignored if the vector has
+					* been added or written to
+					* since the last time
+					* compress() was called.
+					*
+					* See @ref GlossCompress "Compressing distributed objects"
+					* for more information.
+					* more information.
                                         */
-      void compress ();
+      void compress (const Epetra_CombineMode last_action = Zero);
 
 				       /**
 					* Returns the state of the
@@ -365,7 +385,7 @@ namespace TrilinosWrappers
 					* distribution as the calling
 					* vector.
 					*/
-      VectorBase & 
+      VectorBase &
 	operator = (const VectorBase &v);
 
                                        /**
@@ -386,7 +406,7 @@ namespace TrilinosWrappers
 					* &input_map) function.
 					*/
       template <typename Number>
-      VectorBase & 
+      VectorBase &
 	operator = (const ::dealii::Vector<Number> &v);
 
                                        /**
@@ -677,7 +697,7 @@ namespace TrilinosWrappers
                                        /**
                                         * Simple vector addition, equal
                                         * to the <tt>operator
-                                        * +=</tt>. 
+                                        * +=</tt>.
 					*
 					* Though, if the second argument
                                         * <tt>allow_different_maps</tt>
@@ -692,7 +712,7 @@ namespace TrilinosWrappers
                                         * of a vector, i.e. <tt>*this =
                                         * a*V</tt>.
                                         */
-      void add (const TrilinosScalar  a, 
+      void add (const TrilinosScalar  a,
 		const VectorBase     &V);
 
                                        /**
@@ -700,9 +720,9 @@ namespace TrilinosWrappers
                                         * vectors, i.e. <tt>*this = a*V
                                         * + b*W</tt>.
                                         */
-      void add (const TrilinosScalar  a, 
+      void add (const TrilinosScalar  a,
 		const VectorBase     &V,
-		const TrilinosScalar  b, 
+		const TrilinosScalar  b,
 		const VectorBase     &W);
 
                                        /**
@@ -759,16 +779,16 @@ namespace TrilinosWrappers
                                         * Assignment <tt>*this =
                                         * a*V</tt>.
                                         */
-      void equ (const TrilinosScalar  a, 
+      void equ (const TrilinosScalar  a,
 		const VectorBase     &V);
 
                                        /**
                                         * Assignment <tt>*this = a*V +
                                         * b*W</tt>.
                                         */
-      void equ (const TrilinosScalar  a, 
+      void equ (const TrilinosScalar  a,
 		const VectorBase     &V,
-		const TrilinosScalar  b, 
+		const TrilinosScalar  b,
 		const VectorBase     &W);
 
                                        /**
@@ -941,7 +961,7 @@ namespace TrilinosWrappers
                                         * type. Requires an existing
                                         * Epetra_Map for storing data.
                                         */
-      std::auto_ptr<Epetra_FEVector> vector;
+      std_cxx1x::shared_ptr<Epetra_FEVector> vector;
 
 
                                        /**
@@ -963,7 +983,6 @@ namespace TrilinosWrappers
  * the C standard library which uses a temporary object. The function
  * simply exchanges the data of the two vectors.
  *
- * @ingroup TrilinosWrappers
  * @relates TrilinosWrappers::VectorBase
  * @author Martin Kronbichler, Wolfgang Bangerth, 2008
  */
@@ -1016,27 +1035,7 @@ namespace TrilinosWrappers
     const VectorReference &
     VectorReference::operator += (const TrilinosScalar &value) const
     {
-      const int local_row = vector.vector->Map().LID(index);
-      if (local_row == -1)
-	{
-				   // write the code explicitly here to make
-				   // it faster.
-	  if (vector.last_action != Add)
-	    {
-	      if (vector.last_action == Insert)
-		vector.vector->GlobalAssemble(Insert);
-	      vector.last_action = Add;
-	    }
-
-	  const int ierr = vector.vector->SumIntoGlobalValues (1, 
-							       (const int*)(&index), 
-							       &value);
-	  AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-	  vector.compressed = false;
-	}
-      else
-	(*vector.vector)[0][local_row] += value;
-
+      vector.add (1, &index, &value);
       return *this;
     }
 
@@ -1105,6 +1104,55 @@ namespace TrilinosWrappers
 
 
 
+  inline
+  void
+  VectorBase::reinit (const VectorBase &v,
+		      const bool        fast)
+  {
+    Assert (vector.get() != 0,
+	    ExcMessage("Vector has not been constructed properly."));
+
+    if (fast == false || local_range() != v.local_range())
+      vector.reset (new Epetra_FEVector(*v.vector));
+  }
+
+
+
+  inline
+  void
+  VectorBase::compress (const Epetra_CombineMode given_last_action)
+  {
+				 // Now pass over the information about
+				 // what we did last to the vector.
+    const int ierr = vector->GlobalAssemble(last_action != Zero ?
+					    last_action :
+					    given_last_action);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+    last_action = Zero;
+
+    compressed = true;
+  }
+
+
+
+  inline
+  VectorBase &
+  VectorBase::operator = (const TrilinosScalar s)
+  {
+
+    Assert (numbers::is_finite(s),
+	    ExcMessage("The given value is not finite but either "
+		       "infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->PutScalar(s);
+
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return *this;
+  }
+
+
+
   inline
   void
   VectorBase::set (const std::vector<unsigned int>    &indices,
@@ -1149,8 +1197,8 @@ namespace TrilinosWrappers
 	const int local_row = vector->Map().LID(indices[i]);
 	if (local_row == -1)
 	  {
-	    const int ierr = vector->ReplaceGlobalValues (1, 
-							  (const int*)(&row), 
+	    const int ierr = vector->ReplaceGlobalValues (1,
+							  (const int*)(&row),
 							  &values[i]);
 	    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 	    compressed = false;
@@ -1207,8 +1255,8 @@ namespace TrilinosWrappers
 	const int local_row = vector->Map().LID(row);
 	if (local_row == -1)
 	  {
-	    const int ierr = vector->SumIntoGlobalValues (1, 
-							  (const int*)(&row), 
+	    const int ierr = vector->SumIntoGlobalValues (1,
+							  (const int*)(&row),
 							  &values[i]);
 	    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 	    compressed = false;
@@ -1220,6 +1268,24 @@ namespace TrilinosWrappers
 
 
 
+  inline
+  unsigned int
+  VectorBase::size () const
+  {
+    return (unsigned int) vector->Map().MaxAllGID() + 1 - vector->Map().MinAllGID();
+  }
+
+
+
+  inline
+  unsigned int
+  VectorBase::local_size () const
+  {
+    return (unsigned int) vector->Map().NumMyElements();
+  }
+
+
+
   inline
   std::pair<unsigned int, unsigned int>
   VectorBase::local_range () const
@@ -1232,6 +1298,488 @@ namespace TrilinosWrappers
 
 
 
+  inline
+  TrilinosScalar
+  VectorBase::operator * (const VectorBase &vec) const
+  {
+    Assert (local_range() == vec.local_range(),
+	    ExcDimensionMismatch(size(), vec.size()));
+
+    TrilinosScalar result;
+
+    const int ierr = vector->Dot(*(vec.vector), &result);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return result;
+  }
+
+
+
+  inline
+  VectorBase::real_type
+  VectorBase::norm_sqr () const
+  {
+    const TrilinosScalar d = l2_norm();
+    return d*d;
+  }
+
+
+
+  inline
+  TrilinosScalar
+  VectorBase::mean_value () const
+  {
+    TrilinosScalar mean;
+
+    const int ierr = vector->MeanValue (&mean);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return mean;
+  }
+
+
+
+  inline
+  VectorBase::real_type
+  VectorBase::l1_norm () const
+  {
+    TrilinosScalar d;
+
+    const int ierr = vector->Norm1 (&d);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return d;
+  }
+
+
+
+  inline
+  VectorBase::real_type
+  VectorBase::l2_norm () const
+  {
+    TrilinosScalar d;
+
+    const int ierr = vector->Norm2 (&d);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return d;
+  }
+
+
+
+  inline
+  VectorBase::real_type
+  VectorBase::lp_norm (const TrilinosScalar p) const
+  {
+                                        // get a representation of the
+                                        // vector and loop over all
+                                        // the elements
+    TrilinosScalar *start_ptr;
+    int leading_dimension;
+    int ierr = vector->ExtractView (&start_ptr, &leading_dimension);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    TrilinosScalar norm = 0;
+    TrilinosScalar sum=0;
+
+    const TrilinosScalar * ptr  = start_ptr;
+
+                                       // add up elements
+				       // TODO: This
+				       // won't work in parallel like
+				       // this. Find out a better way to
+				       // this in that case.
+    while (ptr != start_ptr+size())
+      sum += std::pow(std::fabs(*ptr++), p);
+
+    norm = std::pow(sum, static_cast<TrilinosScalar>(1./p));
+
+    return norm;
+  }
+
+
+
+  inline
+  VectorBase::real_type
+  VectorBase::linfty_norm () const
+  {
+    TrilinosScalar d;
+
+    const int ierr = vector->NormInf (&d);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return d;
+  }
+
+
+
+				   // inline also scalar products, vector
+				   // additions etc. since they are all
+				   // representable by a single Trilinos
+				   // call. This reduces the overhead of the
+				   // wrapper class.
+  inline
+  VectorBase &
+  VectorBase::operator *= (const TrilinosScalar a)
+  {
+
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Scale(a);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return *this;
+  }
+
+
+
+  inline
+  VectorBase &
+  VectorBase::operator /= (const TrilinosScalar a)
+  {
+
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const TrilinosScalar factor = 1./a;
+
+    Assert (numbers::is_finite(factor),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Scale(factor);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return *this;
+  }
+
+
+
+  inline
+  VectorBase &
+  VectorBase::operator += (const VectorBase &v)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+
+    const int ierr = vector->Update (1.0, *(v.vector), 1.0);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return *this;
+  }
+
+
+
+  inline
+  VectorBase &
+  VectorBase::operator -= (const VectorBase &v)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+
+    const int ierr = vector->Update (-1.0, *(v.vector), 1.0);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    return *this;
+  }
+
+
+
+  inline
+  void
+  VectorBase::add (const TrilinosScalar s)
+  {
+
+    Assert (numbers::is_finite(s),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    unsigned int n_local = local_size();
+    for (unsigned int i=0; i<n_local; i++)
+      (*vector)[0][i] += s;
+  }
+
+
+
+  inline
+  void
+  VectorBase::add (const TrilinosScalar  a,
+		   const VectorBase     &v)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Update(a, *(v.vector), 1.);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::add (const TrilinosScalar  a,
+		   const VectorBase     &v,
+		   const TrilinosScalar  b,
+		   const VectorBase     &w)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+    Assert (size() == w.size(),
+	    ExcDimensionMismatch(size(), w.size()));
+
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(b),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Update(a, *(v.vector), b, *(w.vector), 1.);
+
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::sadd (const TrilinosScalar  s,
+		    const VectorBase     &v)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+
+    Assert (numbers::is_finite(s),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Update(1., *(v.vector), s);
+
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::sadd (const TrilinosScalar  s,
+		    const TrilinosScalar  a,
+		    const VectorBase     &v)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+
+    Assert (numbers::is_finite(s),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Update(a, *(v.vector), s);
+
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::sadd (const TrilinosScalar  s,
+		    const TrilinosScalar  a,
+		    const VectorBase     &v,
+		    const TrilinosScalar  b,
+		    const VectorBase     &w)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+    Assert (size() == w.size(),
+	    ExcDimensionMismatch(size(), w.size()));
+
+
+    Assert (numbers::is_finite(s),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(b),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+    const int ierr = vector->Update(a, *(v.vector), b, *(w.vector), s);
+
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::sadd (const TrilinosScalar  s,
+		    const TrilinosScalar  a,
+		    const VectorBase     &v,
+		    const TrilinosScalar  b,
+		    const VectorBase     &w,
+		    const TrilinosScalar  c,
+		    const VectorBase     &x)
+  {
+    Assert (size() == v.size(),
+	    ExcDimensionMismatch(size(), v.size()));
+    Assert (size() == w.size(),
+	    ExcDimensionMismatch(size(), w.size()));
+    Assert (size() == x.size(),
+	    ExcDimensionMismatch(size(), x.size()));
+
+    Assert (numbers::is_finite(s),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(b),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(c),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+                                        // Update member can only
+				        // input two other vectors so
+				        // do it in two steps
+    const int ierr = vector->Update(a, *(v.vector), b, *(w.vector), s);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    const int jerr = vector->Update(c, *(x.vector), 1.);
+    Assert (jerr == 0, ExcTrilinosError(jerr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::scale (const VectorBase &factors)
+  {
+    Assert (size() == factors.size(),
+	    ExcDimensionMismatch(size(), factors.size()));
+
+    const int ierr = vector->Multiply (1.0, *(factors.vector), *vector, 0.0);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
+  inline
+  void
+  VectorBase::equ (const TrilinosScalar  a,
+		   const VectorBase     &v)
+  {
+
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+				   // If we don't have the same map, copy.
+    if (local_range() != v.local_range())
+      {
+	vector.reset();
+	last_action = Zero;
+	*vector = *v.vector;
+	*this *= a;
+      }
+    else
+      {
+				   // Otherwise, just update
+	Assert (vector->Map().SameAs(v.vector->Map()) == true,
+		ExcMessage ("The Epetra maps in the assignment operator ="
+			    " do not match, even though the local_range "
+			    " seems to be the same. Check vector setup!"));
+	int ierr;
+	ierr = vector->GlobalAssemble(last_action);
+	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+	ierr = vector->Update(a, *v.vector, 0.0);
+	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+	last_action = Zero;
+      }
+
+  }
+
+
+
+  inline
+  void
+  VectorBase::equ (const TrilinosScalar  a,
+		   const VectorBase     &v,
+		   const TrilinosScalar  b,
+		   const VectorBase     &w)
+  {
+
+    Assert (v.size() == w.size(),
+	    ExcDimensionMismatch (v.size(), w.size()));
+
+    Assert (numbers::is_finite(a),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+    Assert (numbers::is_finite(b),
+	    ExcMessage("The given value is not finite but "
+		       "either infinite or Not A Number (NaN)"));
+
+				   // If we don't have the same map, copy.
+    if (local_range() != v.local_range())
+      {
+	vector.reset();
+	last_action = Zero;
+	*vector = *v.vector;
+	sadd (a, b, w);
+      }
+    else
+      {
+				   // Otherwise, just update
+	Assert (vector->Map().SameAs(v.vector->Map()) == true,
+		ExcMessage ("The Epetra maps in the assignment operator ="
+			    " do not match, even though the local_range "
+			    " seems to be the same. Check vector setup!"));
+	int ierr;
+	ierr = vector->GlobalAssemble(last_action);
+	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+	ierr = vector->Update(a, *v.vector, b, *w.vector, 0.0);
+	AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+	last_action = Zero;
+      }
+  }
+
+
+
+  inline
+  void
+  VectorBase::ratio (const VectorBase &v,
+		     const VectorBase &w)
+  {
+    Assert (v.size() == w.size(),
+	    ExcDimensionMismatch (v.size(), w.size()));
+
+    Assert (size() == w.size(),
+	    ExcDimensionMismatch (size(), w.size()));
+
+    const int ierr = vector->ReciprocalMultiply(1.0, *(w.vector),
+						*(v.vector), 0.0);
+
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
+
+
   inline
   const Epetra_MultiVector &
   VectorBase::trilinos_vector () const
@@ -1257,6 +1805,7 @@ namespace TrilinosWrappers
     return static_cast<const Epetra_Map&>(vector->Map());
   }
 
+
 #endif // DOXYGEN
 
 }
