@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: vector.h 18598 2009-04-12 09:00:53Z heltai $
+//    $Id: vector.h 20939 2010-04-06 03:53:19Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -16,9 +16,13 @@
 
 #include <base/config.h>
 #include <base/exceptions.h>
+#include <base/parallel.h>
 #include <base/subscriptor.h>
+#include <boost/lambda/lambda.hpp>
 
 #include <cstdio>
+#include <cstring>
+#include <vector>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -53,6 +57,7 @@ template <typename> class VectorView;
 
 
 
+
 /*! @addtogroup Vectors
  *@{
  */
@@ -74,7 +79,7 @@ template <typename> class VectorView;
  * @<std::complex@<long double@>@></tt>; others can be generated in
  * application programs (see the section on @ref Instantiations in the
  * manual).
- * 
+ *
  * @author Guido Kanschat, Franz-Theo Suttmeier, Wolfgang Bangerth
  */
 template <typename Number>
@@ -117,8 +122,10 @@ class Vector : public Subscriptor
 				      */
     typedef typename numbers::NumberTraits<Number>::real_type real_type;
 
+  public:
+
 				     /**
-				      * @name 1: Basic Object-handling 
+				      * @name 1: Basic Object-handling
 				      */
 				     //@{
 				     /**
@@ -126,7 +133,7 @@ class Vector : public Subscriptor
 				      *  dimension zero.
 				      */
     Vector ();
-    
+
 				     /**
 				      * Copy-constructor. Sets the dimension
                                       * to that of the given vector, and
@@ -139,7 +146,7 @@ class Vector : public Subscriptor
     Vector (const Vector<Number> &v);
 
 
-#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG    
+#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
 				     /**
 				      * Copy constructor taking a vector of
 				      * another data type. This will fail if
@@ -164,7 +171,7 @@ class Vector : public Subscriptor
     explicit
     Vector (const Vector<OtherNumber> &v);
 #endif
-    
+
 #ifdef DEAL_II_USE_PETSC
                                      /**
                                       * Another copy constructor: copy the
@@ -229,7 +236,7 @@ class Vector : public Subscriptor
                                       */
     Vector (const TrilinosWrappers::Vector &v);
 #endif
-    
+
 				     /**
 				      * Constructor. Set dimension to
 				      * @p n and initialize all
@@ -259,7 +266,7 @@ class Vector : public Subscriptor
     template <typename InputIterator>
     Vector (const InputIterator first,
             const InputIterator last);
-    
+
 				     /**
 				      * Destructor, deallocates
 				      * memory. Made virtual to allow
@@ -272,7 +279,7 @@ class Vector : public Subscriptor
                                       * This function does nothing but is
                                       * there for compatibility with the
                                       * @p PETScWrappers::Vector class.
-                                      * 
+                                      *
                                       * For the PETSc vector wrapper class,
                                       * thios function compresses the
                                       * underlying representation of the PETSc
@@ -313,10 +320,10 @@ class Vector : public Subscriptor
 				      * order to allow for derived
 				      * classes to handle memory
 				      * separately.
-				      */ 
+				      */
     virtual void reinit (const unsigned int N,
 			 const bool         fast=false);
-    
+
 				     /**
 				      * Change the dimension to that of the
 				      * vector @p V. The same applies as for
@@ -357,9 +364,9 @@ class Vector : public Subscriptor
 				      * order to allow for derived
 				      * classes to handle memory
 				      * separately.
-				      */ 
+				      */
     virtual void swap (Vector<Number> &v);
-    
+
 				     /**
                                       * Set all components of the vector to
                                       * the given number @p s. Simply pass
@@ -380,13 +387,13 @@ class Vector : public Subscriptor
                                       * be disallowed in the future.
 				      */
     Vector<Number> & operator = (const Number s);
-    
+
 				     /**
 				      * Copy the given vector. Resize the
 				      * present vector if necessary.
 				      */
     Vector<Number> & operator= (const Vector<Number> &c);
-    
+
 				     /**
 				      * Copy the given vector. Resize the
 				      * present vector if necessary.
@@ -400,7 +407,7 @@ class Vector : public Subscriptor
                                       * vector.
                                       */
     Vector<Number> & operator= (const BlockVector<Number> &v);
-    
+
 #ifdef DEAL_II_USE_PETSC
                                      /**
                                       * Another copy operator: copy the
@@ -484,7 +491,7 @@ class Vector : public Subscriptor
                                       */
     template <typename Number2>
     bool operator == (const Vector<Number2> &v) const;
-    
+
                                      /**
                                       * Test for inequality. This function
                                       * assumes that the present vector and
@@ -495,7 +502,7 @@ class Vector : public Subscriptor
                                       */
     template <typename Number2>
     bool operator != (const Vector<Number2> &v) const;
-    
+
 				     /**
 				      * Return the scalar product of
 				      * two vectors.  The return type
@@ -583,7 +590,7 @@ class Vector : public Subscriptor
 				      * thrown.
                                       */
     bool is_non_negative () const;
-    
+
 				     /**
 				      * Make the @p Vector class a bit like
 				      * the <tt>vector<></tt> class of the C++
@@ -609,7 +616,7 @@ class Vector : public Subscriptor
 				      * Return a constant iterator pointing to
 				      * the element past the end of the array.
 				      */
-    const_iterator end () const;  
+    const_iterator end () const;
 				     //@}
 
 
@@ -622,7 +629,7 @@ class Vector : public Subscriptor
 				      * component.
 				      */
     Number operator() (const unsigned int i) const;
-    
+
 				     /**
 				      * Access the @p ith component
 				      * as a writeable reference.
@@ -635,7 +642,7 @@ class Vector : public Subscriptor
 				      * @name 3: Modification of vectors
 				      */
 				     //@{
-    
+
 				     /**
 				      * Add the given vector to the present
 				      * one.
@@ -648,32 +655,69 @@ class Vector : public Subscriptor
 				      */
     Vector<Number> & operator -= (const Vector<Number> &V);
 
+				       /**
+                                        * A collective add operation:
+                                        * This funnction adds a whole
+                                        * set of values stored in @p
+                                        * values to the vector
+                                        * components specified by @p
+                                        * indices.
+                                        */
+    template <typename OtherNumber>
+    void add (const std::vector<unsigned int> &indices,
+	      const std::vector<OtherNumber>  &values);
+
+				       /**
+				        * This is a second collective
+				        * add operation. As a
+				        * difference, this function
+				        * takes a deal.II vector of
+				        * values.
+				        */
+    template <typename OtherNumber>
+    void add (const std::vector<unsigned int> &indices,
+	      const Vector<OtherNumber>       &values);
+
+				      /**
+				       * Take an address where
+				       * <tt>n_elements</tt> are stored
+				       * contiguously and add them into
+				       * the vector. Handles all cases
+				       * which are not covered by the
+				       * other two <tt>add()</tt>
+				       * functions above.
+				       */
+    template <typename OtherNumber>
+    void add (const unsigned int  n_elements,
+	      const unsigned int *indices,
+	      const OtherNumber  *values);
+
 				     /**
 				      * Addition of @p s to all
 				      * components. Note that @p s is a
 				      * scalar and not a vector.
 				      */
     void add (const Number s);
-    
+
 				     /**
 				      * Simple vector addition, equal to the
 				      * <tt>operator +=</tt>.
 				      */
     void add (const Vector<Number> &V);
-    
+
 				     /**
 				      * Simple addition of a multiple of a
 				      * vector, i.e. <tt>*this += a*V</tt>.
 				      */
     void add (const Number a, const Vector<Number> &V);
-    
+
 				     /**
 				      * Multiple addition of scaled vectors,
 				      * i.e. <tt>*this += a*V+b*W</tt>.
 				      */
     void add (const Number a, const Vector<Number> &V,
 	      const Number b, const Vector<Number> &W);
-    
+
 				     /**
 				      * Scaling and simple vector addition,
 				      * i.e.
@@ -681,7 +725,7 @@ class Vector : public Subscriptor
 				      */
     void sadd (const Number          s,
                const Vector<Number> &V);
-    
+
 				     /**
 				      * Scaling and simple addition, i.e.
 				      * <tt>*this = s*(*this)+a*V</tt>.
@@ -689,7 +733,7 @@ class Vector : public Subscriptor
     void sadd (const Number          s,
                const Number          a,
                const Vector<Number> &V);
-    
+
 				     /**
 				      * Scaling and multiple addition.
 				      */
@@ -698,7 +742,7 @@ class Vector : public Subscriptor
 	       const Vector<Number> &V,
                const Number          b,
                const Vector<Number> &W);
-    
+
 				     /**
 				      * Scaling and multiple addition.
 				      * <tt>*this = s*(*this)+a*V + b*W + c*X</tt>.
@@ -707,10 +751,10 @@ class Vector : public Subscriptor
                const Number          a,
 	       const Vector<Number> &V,
                const Number          b,
-               const Vector<Number> &W, 
+               const Vector<Number> &W,
 	       const Number          c,
                const Vector<Number> &X);
-    
+
 				     /**
 				      * Scale each element of the
 				      * vector by the given factor.
@@ -723,7 +767,7 @@ class Vector : public Subscriptor
 				      */
     void scale (const Number factor);
 
-    
+
 				     /**
 				      * Scale each element of the
 				      * vector by a constant
@@ -737,7 +781,18 @@ class Vector : public Subscriptor
 				      * given value.
 				      */
     Vector<Number> & operator /= (const Number factor);
-    
+
+				     /**
+				      * Scale each element of this
+				      * vector by the corresponding
+				      * element in the argument. This
+				      * function is mostly meant to
+				      * simulate multiplication (and
+				      * immediate re-assignment) by a
+				      * diagonal scaling matrix.
+				      */
+    void scale (const Vector<Number> &scaling_factors);
+
 				     /**
 				      * Scale each element of this
 				      * vector by the corresponding
@@ -749,19 +804,24 @@ class Vector : public Subscriptor
 				      */
     template <typename Number2>
     void scale (const Vector<Number2> &scaling_factors);
-    
+
+				     /**
+				      * Assignment <tt>*this = a*u</tt>.
+				      */
+    void equ (const Number a, const Vector<Number>& u);
+
 				     /**
 				      * Assignment <tt>*this = a*u</tt>.
 				      */
     template <typename Number2>
     void equ (const Number a, const Vector<Number2>& u);
-    
+
 				     /**
 				      * Assignment <tt>*this = a*u + b*v</tt>.
 				      */
     void equ (const Number a, const Vector<Number>& u,
 	      const Number b, const Vector<Number>& v);
-    
+
 				     /**
 				      * Assignment <tt>*this = a*u + b*v + b*w</tt>.
 				      */
@@ -913,6 +973,8 @@ class Vector : public Subscriptor
 /*----------------------- Inline functions ----------------------------------*/
 
 
+#ifndef DOXYGEN
+
 
 template <typename Number>
 inline
@@ -972,14 +1034,14 @@ template <typename Number>
 inline
 void Vector<Number>::reinit (const unsigned int n, const bool fast)
 {
-  if (n==0) 
+  if (n==0)
     {
       if (val) delete[] val;
       val = 0;
       max_vec_size = vec_size = 0;
       return;
     };
-  
+
   if (n>max_vec_size)
     {
       if (val) delete[] val;
@@ -994,17 +1056,134 @@ void Vector<Number>::reinit (const unsigned int n, const bool fast)
 
 
 
+namespace internal
+{
+  namespace Vector
+  {
+    template<typename T>
+    void set_subrange (const T            s,
+		       const unsigned int begin,
+		       const unsigned int end,
+		       dealii::Vector<T> &dst)
+    {
+      if (s == T())
+	std::memset ((dst.begin()+begin),0,(end-begin)*sizeof(T));
+      else
+	std::fill (&*(dst.begin()+begin), &*(dst.begin()+end), s);
+    }
+
+    template<typename T>
+    void copy_subrange (const dealii::Vector<T>&src,
+			const unsigned int      begin,
+			const unsigned int      end,
+			dealii::Vector<T>      &dst)
+    {
+      memcpy(&*(dst.begin()+begin), &*(src.begin()+begin),
+	     (end-begin)*sizeof(T));
+    }
+
+    template<typename T, typename U>
+    void copy_subrange_ext (const dealii::Vector<T>&src,
+			    const unsigned int      begin,
+			    const unsigned int      end,
+			    dealii::Vector<U>      &dst)
+    {
+      const T* q = src.begin()+begin;
+      const T* const end_q = src.begin()+end;
+      U* p = dst.begin()+begin;
+      for (; q!=end_q; ++q, ++p)
+	*p = *q;
+    }
+  }
+}
+
+
+
 template <typename Number>
 inline
 Vector<Number> & Vector<Number>::operator = (const Number s)
 {
-  Assert (numbers::is_finite(s), 
-          ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  
+  Assert (numbers::is_finite(s),
+          ExcMessage("The given value is not finite but either infinite or "
+		     "Not A Number (NaN)"));
+
   if (s != Number())
     Assert (vec_size!=0, ExcEmptyObject());
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::apply_to_subranges (0U, vec_size,
+				  std_cxx1x::bind(&internal::Vector::template
+						  set_subrange<Number>,
+						  s, _1, _2, std_cxx1x::ref(*this)),
+				  internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    internal::Vector::set_subrange<Number>(s, 0U, vec_size, *this);
+
+  return *this;
+}
+
+
+
+#ifdef DEAL_II_BOOST_BIND_COMPILER_BUG
+template <>
+inline
+Vector<std::complex<float> > &
+Vector<std::complex<float> >::operator = (const std::complex<float> s)
+{
+  Assert (numbers::is_finite(s),
+          ExcMessage("The given value is not finite but either infinite or "
+		     "Not A Number (NaN)"));
+
+  if (s != std::complex<float>())
+    Assert (vec_size!=0, ExcEmptyObject());
   if (vec_size!=0)
     std::fill (begin(), end(), s);
+
+  return *this;
+}
+#endif
+
+
+
+template <typename Number>
+inline
+Vector<Number> &
+Vector<Number>::operator = (const Vector<Number>& v)
+{
+  if (v.vec_size != vec_size)
+    reinit (v.vec_size, true);
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::apply_to_subranges (0U, vec_size,
+				  std_cxx1x::bind(&internal::Vector::template
+						  copy_subrange<Number>,
+						  std_cxx1x::cref(v), _1, _2,
+						  std_cxx1x::ref(*this)),
+				  internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    internal::Vector::copy_subrange<Number>(v, 0U, vec_size, *this);
+
+  return *this;
+}
+
+
+
+template <typename Number>
+template <typename Number2>
+inline
+Vector<Number> &
+Vector<Number>::operator = (const Vector<Number2>& v)
+{
+  if (v.vec_size != vec_size)
+    reinit (v.vec_size, true);
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::apply_to_subranges (0U, vec_size,
+				  std_cxx1x::bind(&internal::Vector::template
+						  copy_subrange_ext<Number2,Number>,
+						  std_cxx1x::cref(v), _1, _2,
+						  std_cxx1x::ref(*this)),
+				  internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    internal::Vector::copy_subrange_ext<Number2,Number>(v, 0U, vec_size, *this);
+
   return *this;
 }
 
@@ -1021,8 +1200,8 @@ unsigned int Vector<Number>::size () const
 
 template <typename Number>
 inline
-typename Vector<Number>::iterator 
-Vector<Number>::begin () 
+typename Vector<Number>::iterator
+Vector<Number>::begin ()
 {
   return &val[0];
 }
@@ -1031,8 +1210,8 @@ Vector<Number>::begin ()
 
 template <typename Number>
 inline
-typename Vector<Number>::const_iterator 
-Vector<Number>::begin () const 
+typename Vector<Number>::const_iterator
+Vector<Number>::begin () const
 {
   return &val[0];
 }
@@ -1042,7 +1221,7 @@ Vector<Number>::begin () const
 template <typename Number>
 inline
 typename Vector<Number>::iterator
-Vector<Number>::end () 
+Vector<Number>::end ()
 {
   return &val[vec_size];
 }
@@ -1084,7 +1263,7 @@ inline
 Vector<Number> & Vector<Number>::operator *= (const Number factor)
 {
 
-  Assert (numbers::is_finite(factor), 
+  Assert (numbers::is_finite(factor),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   scale (factor);
@@ -1098,7 +1277,7 @@ inline
 Vector<Number> &
 Vector<Number>::operator /= (const Number factor)
 {
-  Assert (numbers::is_finite(factor), 
+  Assert (numbers::is_finite(factor),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
   Assert (factor != Number(0.), ExcZero() );
 
@@ -1113,15 +1292,64 @@ inline
 void
 Vector<Number>::scale (const Number factor)
 {
-  Assert (numbers::is_finite(factor), 
+  Assert (numbers::is_finite(factor),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
 
-  iterator             ptr  = begin();
-  const const_iterator eptr = end();
-  while (ptr!=eptr)
-    *ptr++ *= factor;
+  parallel::transform (val,
+		       val+vec_size,
+		       val,
+		       (factor*boost::lambda::_1),
+		       internal::Vector::minimum_parallel_grain_size);
+}
+
+
+
+template <typename Number>
+template <typename OtherNumber>
+inline
+void
+Vector<Number>::add (const std::vector<unsigned int> &indices,
+		     const std::vector<OtherNumber>  &values)
+{
+  Assert (indices.size() == values.size(),
+	  ExcDimensionMismatch(indices.size(), values.size()));
+  add (indices.size(), &indices[0], &values[0]);
+}
+
+
+
+template <typename Number>
+template <typename OtherNumber>
+inline
+void
+Vector<Number>::add (const std::vector<unsigned int> &indices,
+		     const Vector<OtherNumber>       &values)
+{
+  Assert (indices.size() == values.size(),
+	  ExcDimensionMismatch(indices.size(), values.size()));
+  add (indices.size(), &indices[0], values.val);
+}
+
+
+
+template <typename Number>
+template <typename OtherNumber>
+inline
+void
+Vector<Number>::add (const unsigned int  n_indices,
+		     const unsigned int *indices,
+		     const OtherNumber  *values)
+{
+  for (unsigned int i=0; i<n_indices; ++i)
+    {
+      Assert (indices[i] < vec_size, ExcIndexRange(indices[i],0,vec_size));
+      Assert (numbers::is_finite(values[i]),
+	      ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
+
+      val[indices[i]] += values[i];
+    }
 }
 
 
@@ -1132,17 +1360,18 @@ void
 Vector<Number>::add (const Number a,
 		     const Vector<Number>& v)
 {
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  iterator i_ptr = begin(),
-	   i_end = end();
-  const_iterator v_ptr = v.begin();
-  while (i_ptr!=i_end)
-    *i_ptr++ += a * *v_ptr++;
+  parallel::transform (val,
+		       val+vec_size,
+		       v.val,
+		       val,
+		       (boost::lambda::_1 + a*boost::lambda::_2),
+		       internal::Vector::minimum_parallel_grain_size);
 }
 
 
@@ -1154,18 +1383,20 @@ Vector<Number>::sadd (const Number x,
 		      const Number a,
 		      const Vector<Number>& v)
 {
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
-  iterator i_ptr = begin(),
-	   i_end = end();
-  const_iterator v_ptr = v.begin();
-  for (; i_ptr!=i_end; ++i_ptr)
-    *i_ptr = x * *i_ptr  +  a * *v_ptr++;
+
+  parallel::transform (val,
+		       val+vec_size,
+		       v.val,
+		       val,
+		       (x*boost::lambda::_1 + a*boost::lambda::_2),
+		       internal::Vector::minimum_parallel_grain_size);
 }
 
 
@@ -1204,6 +1435,8 @@ Vector<Number>::swap (Vector<Number> &v)
 
 
 
+#endif
+
 
 /*! @addtogroup Vectors
  *@{
