@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: vector.templates.h 18598 2009-04-12 09:00:53Z heltai $
+//    $Id: vector.templates.h 21329 2010-06-24 18:10:33Z kronbichler $
 //    Version: $Name$
 //
-//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -35,6 +35,8 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+#define BLOCK_LEVEL 6
+
 namespace internal
 {
   template <typename T>
@@ -49,7 +51,7 @@ namespace internal
   {
     Assert (false,
 	    ExcMessage ("Complex numbers do not have an ordering."));
-    
+
     return false;
   }
 
@@ -63,7 +65,7 @@ namespace internal
     else
       std::printf (" %5.2f", double(t));
   }
-  
+
 
 
   template <typename T>
@@ -75,10 +77,41 @@ namespace internal
     else
       std::printf (" %5.2f+%5.2fi",
 		   double(t.real()), double(t.imag()));
+  }
+
+				   // call std::copy, except for in
+				   // the case where we want to copy
+				   // from std::complex to a
+				   // non-complex type
+  template <typename T, typename U>
+  void copy (const T *begin,
+	     const T *end,
+	     U       *dest)
+  {
+    std::copy (begin, end, dest);
+  }
+
+  template <typename T, typename U>
+  void copy (const std::complex<T> *begin,
+	     const std::complex<T> *end,
+	     std::complex<U>       *dest)
+  {
+    std::copy (begin, end, dest);
+  }
+
+  template <typename T, typename U>
+  void copy (const std::complex<T> *,
+	     const std::complex<T> *,
+	     U                     *)
+  {
+    Assert (false, ExcMessage ("Can't convert a vector of complex numbers "
+			       "into a vector of reals/doubles"));
   }
 }
 
 
+
+
 template <typename Number>
 Vector<Number>::Vector (const Vector<Number>& v)
                 :
@@ -119,6 +152,7 @@ Vector<Number>::Vector (const Vector<OtherNumber>& v)
 
 #ifdef DEAL_II_USE_PETSC
 
+
 template <typename Number>
 Vector<Number>::Vector (const PETScWrappers::Vector &v)
                 :
@@ -137,8 +171,8 @@ Vector<Number>::Vector (const PETScWrappers::Vector &v)
       PetscScalar *start_ptr;
       int ierr = VecGetArray (static_cast<const Vec&>(v), &start_ptr);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
-      std::copy (start_ptr, start_ptr+vec_size, begin());
+
+      internal::copy (start_ptr, start_ptr+vec_size, begin());
 
                                        // restore the representation of the
                                        // vector
@@ -198,7 +232,7 @@ Vector<Number>::Vector (const TrilinosWrappers::MPI::Vector &v)
 
       int ierr = localized_vector.trilinos_vector().ExtractView (&start_ptr);
       AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-      
+
       std::copy (start_ptr[0], start_ptr[0]+vec_size, begin());
     }
 }
@@ -224,7 +258,7 @@ Vector<Number>::Vector (const TrilinosWrappers::Vector &v)
 
       int ierr = v.trilinos_vector().ExtractView (&start_ptr);
       AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-      
+
       std::copy (start_ptr[0], start_ptr[0]+vec_size, begin());
     }
 }
@@ -272,7 +306,7 @@ bool
 Vector<Number>::is_non_negative () const
 {
   Assert (vec_size!=0, ExcEmptyObject());
-  
+
   for (unsigned int i=0; i<vec_size; ++i)
     if ( ! internal::is_non_negative (val[i]))
       return false;
@@ -287,24 +321,78 @@ template <typename Number2>
 Number Vector<Number>::operator * (const Vector<Number2>& v) const
 {
   Assert (vec_size!=0, ExcEmptyObject());
-  
+
   if (PointerComparison::equal (this, &v))
     return norm_sqr();
-  
+
   Assert (vec_size == v.size(),
 	  ExcDimensionMismatch(vec_size, v.size()));
-  
-  Number sum = 0;
+
+  const unsigned int blocking = 1<<BLOCK_LEVEL;
+  Number sum1, sum2, sum3, sum = Number();
+  const Number * X = val, *X_end = X + vec_size,
+    *X_end3 = X + ((vec_size>>(BLOCK_LEVEL))<<(BLOCK_LEVEL)),
+    *X_end2 = X + ((vec_size>>(2*BLOCK_LEVEL))<<(2*BLOCK_LEVEL)),
+    *X_end1 = X + ((vec_size>>(3*BLOCK_LEVEL))<<(3*BLOCK_LEVEL));
+  const Number2  * Y = v.val;
 
 				   // multiply the two vectors. we have to
 				   // convert the elements of u to the type of
 				   // the result vector. this is necessary
 				   // because
 				   // operator*(complex<float>,complex<double>)
-				   // is not defined by default
-  for (unsigned int i=0; i<vec_size; ++i)
-    sum += val[i] * Number(numbers::NumberTraits<Number2>::conjugate(v.val[i]));
-    
+				   // is not defined by default. do the
+				   // operations block-wise with post-update.
+				   // use three nested loops, which should make
+				   // the roundoff error very small up to
+				   // about 20m entries. in the
+				   // end do extra loops with the remainders.
+				   // this blocked algorithm has been proposed
+				   // by Castaldo, Whaley and Chronopoulos
+				   // (SIAM J. Sci. Comput. 31, 1156-1174,
+				   // 2008)
+  while (X != X_end1)
+    {
+      sum1 = Number();
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum2 = Number();
+	  for (unsigned int k=0; k<blocking; ++k)
+	    {
+	      sum3 = Number();
+	      for (unsigned int i=0; i<blocking; ++i)
+		sum3 += *X++ * Number(numbers::NumberTraits<Number2>::conjugate(*Y++));
+	      sum2 += sum3;
+	    }
+	  sum1 += sum2;
+	}
+      sum += sum1;
+    }
+  while (X != X_end2)
+    {
+      sum2 = Number();
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum3 = Number();
+	  for (unsigned int i=0; i<blocking; ++i)
+	    sum3 += *X++ * Number(numbers::NumberTraits<Number2>::conjugate(*Y++));
+	  sum2 += sum3;
+	}
+      sum += sum2;
+    }
+  while (X != X_end3)
+    {
+      sum3 = Number();
+      for (unsigned int i=0; i<blocking; ++i)
+	sum3 += *X++ * Number(numbers::NumberTraits<Number2>::conjugate(*Y++));
+      sum += sum3;
+    }
+
+  sum3 = Number();
+  while (X != X_end)
+    sum3 += *X++ * Number(numbers::NumberTraits<Number2>::conjugate(*Y++));
+  sum += sum3;
+
   return sum;
 }
 
@@ -315,11 +403,55 @@ Vector<Number>::norm_sqr () const
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  real_type sum = 0;
+  const unsigned int blocking = 1<<BLOCK_LEVEL;
+  real_type sum1, sum2, sum3, sum = 0.;
+  const Number * X = val, *X_end = X + vec_size,
+    *X_end3 = X + ((vec_size>>(BLOCK_LEVEL))<<(BLOCK_LEVEL)),
+    *X_end2 = X + ((vec_size>>(2*BLOCK_LEVEL))<<(2*BLOCK_LEVEL)),
+    *X_end1 = X + ((vec_size>>(3*BLOCK_LEVEL))<<(3*BLOCK_LEVEL));
+
+  while (X != X_end1)
+    {
+      sum1 = 0.;
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum2 = 0.;
+	  for (unsigned int k=0; k<blocking; ++k)
+	    {
+	      sum3 = 0.;
+	      for (unsigned int i=0; i<blocking; ++i)
+		sum3 += numbers::NumberTraits<Number>::abs_square(*X++);
+	      sum2 += sum3;
+	    }
+	  sum1 += sum2;
+	}
+      sum += sum1;
+    }
+  while (X != X_end2)
+    {
+      sum2 = 0.;
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum3 = 0.;
+	  for (unsigned int i=0; i<blocking; ++i)
+	    sum3 += numbers::NumberTraits<Number>::abs_square(*X++);
+	  sum2 += sum3;
+	}
+      sum += sum2;
+    }
+  while (X != X_end3)
+    {
+      sum3 = 0.;
+      for (unsigned int i=0; i<blocking; ++i)
+	sum3 += numbers::NumberTraits<Number>::abs_square(*X++);
+      sum += sum3;
+    }
+
+  sum3 = 0.;
+  while (X != X_end)
+    sum3 += numbers::NumberTraits<Number>::abs_square(*X++);
+  sum += sum3;
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    sum += numbers::NumberTraits<Number>::abs_square(val[i]);
-  
   return sum;
 }
 
@@ -329,11 +461,55 @@ Number Vector<Number>::mean_value () const
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  Number sum = 0;
+  const unsigned int blocking = 1<<BLOCK_LEVEL;
+  Number sum1, sum2, sum3, sum = Number();
+  const Number * X = val, *X_end = X + vec_size,
+    *X_end3 = X + ((vec_size>>(BLOCK_LEVEL))<<(BLOCK_LEVEL)),
+    *X_end2 = X + ((vec_size>>(2*BLOCK_LEVEL))<<(2*BLOCK_LEVEL)),
+    *X_end1 = X + ((vec_size>>(3*BLOCK_LEVEL))<<(3*BLOCK_LEVEL));
+
+  while (X != X_end1)
+    {
+      sum1 = Number();
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum2 = Number();
+	  for (unsigned int k=0; k<blocking; ++k)
+	    {
+	      sum3 = Number();
+	      for (unsigned int i=0; i<blocking; ++i)
+		sum3 += *X++;
+	      sum2 += sum3;
+	    }
+	  sum1 += sum2;
+	}
+      sum += sum1;
+    }
+  while (X != X_end2)
+    {
+      sum2 = Number();
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum3 = Number();
+	  for (unsigned int i=0; i<blocking; ++i)
+	    sum3 += *X++;
+	  sum2 += sum3;
+	}
+      sum += sum2;
+    }
+  while (X != X_end3)
+    {
+      sum3 = Number();
+      for (unsigned int i=0; i<blocking; ++i)
+	sum3 += *X++;
+      sum += sum3;
+    }
+
+  sum3 = Number();
+  while (X != X_end)
+    sum3 += *X++;
+  sum += sum3;
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    sum += val[i];
-  
   return sum / real_type(size());
 }
 
@@ -345,10 +521,54 @@ Vector<Number>::l1_norm () const
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  real_type sum = 0;
+  const unsigned int blocking = 1<<BLOCK_LEVEL;
+  real_type sum1, sum2, sum3, sum = 0.;
+  const Number * X = val, *X_end = X + vec_size,
+    *X_end3 = X + ((vec_size>>(BLOCK_LEVEL))<<(BLOCK_LEVEL)),
+    *X_end2 = X + ((vec_size>>(2*BLOCK_LEVEL))<<(2*BLOCK_LEVEL)),
+    *X_end1 = X + ((vec_size>>(3*BLOCK_LEVEL))<<(3*BLOCK_LEVEL));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    sum += numbers::NumberTraits<Number>::abs(val[i]);
+  while (X != X_end1)
+    {
+      sum1 = 0.;
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum2 = 0.;
+	  for (unsigned int k=0; k<blocking; ++k)
+	    {
+	      sum3 = 0.;
+	      for (unsigned int i=0; i<blocking; ++i)
+		sum3 += numbers::NumberTraits<Number>::abs(*X++);
+	      sum2 += sum3;
+	    }
+	  sum1 += sum2;
+	}
+      sum += sum1;
+    }
+  while (X != X_end2)
+    {
+      sum2 = 0.;
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum3 = 0.;
+	  for (unsigned int i=0; i<blocking; ++i)
+	    sum3 += numbers::NumberTraits<Number>::abs(*X++);
+	  sum2 += sum3;
+	}
+      sum += sum2;
+    }
+  while (X != X_end3)
+    {
+      sum3 = 0.;
+      for (unsigned int i=0; i<blocking; ++i)
+	sum3 += numbers::NumberTraits<Number>::abs(*X++);
+      sum += sum3;
+    }
+
+  sum3 = 0.;
+  while (X != X_end)
+    sum3 += numbers::NumberTraits<Number>::abs(*X++);
+  sum += sum3;
 
   return sum;
 }
@@ -370,11 +590,60 @@ Vector<Number>::lp_norm (const real_type p) const
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  real_type sum = 0;
+  if (p == 1.)
+    return l1_norm();
+  else if (p == 2.)
+    return std::sqrt(norm_sqr());
+
+  const unsigned int blocking = 1<<BLOCK_LEVEL;
+  real_type sum1, sum2, sum3, sum = 0.;
+  const Number * X = val, *X_end = X + vec_size,
+    *X_end3 = X + ((vec_size>>(BLOCK_LEVEL))<<(BLOCK_LEVEL)),
+    *X_end2 = X + ((vec_size>>(2*BLOCK_LEVEL))<<(2*BLOCK_LEVEL)),
+    *X_end1 = X + ((vec_size>>(3*BLOCK_LEVEL))<<(3*BLOCK_LEVEL));
+
+  while (X != X_end1)
+    {
+      sum1 = 0.;
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum2 = 0.;
+	  for (unsigned int k=0; k<blocking; ++k)
+	    {
+	      sum3 = 0.;
+	      for (unsigned int i=0; i<blocking; ++i)
+		sum3 += std::pow(numbers::NumberTraits<Number>::abs(*X++),p);
+	      sum2 += sum3;
+	    }
+	  sum1 += sum2;
+	}
+      sum += sum1;
+    }
+  while (X != X_end2)
+    {
+      sum2 = 0.;
+      for (unsigned int j=0; j<blocking; ++j)
+	{
+	  sum3 = 0.;
+	  for (unsigned int i=0; i<blocking; ++i)
+	    sum3 += std::pow(numbers::NumberTraits<Number>::abs(*X++),p);
+	  sum2 += sum3;
+	}
+      sum += sum2;
+    }
+  while (X != X_end3)
+    {
+      sum3 = 0.;
+      for (unsigned int i=0; i<blocking; ++i)
+	sum3 += std::pow(numbers::NumberTraits<Number>::abs(*X++),p);
+      sum += sum3;
+    }
+
+  sum3 = 0.;
+  while (X != X_end)
+    sum3 += std::pow(numbers::NumberTraits<Number>::abs(*X++),p);
+  sum += sum3;
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    sum += std::pow (numbers::NumberTraits<Number>::abs(val[i]), p);
-  
   return std::pow(sum, static_cast<real_type>(1./p));
 }
 
@@ -386,7 +655,7 @@ Vector<Number>::linfty_norm () const
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  real_type max = 0;
+  real_type max = 0.;
 
   for (unsigned int i=0; i<vec_size; ++i)
     max = std::max (numbers::NumberTraits<Number>::abs(val[i]), max);
@@ -411,8 +680,16 @@ Vector<Number>& Vector<Number>::operator -= (const Vector<Number>& v)
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] -= v.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (val,
+			 val+vec_size,
+			 v.val,
+			 val,
+			 (boost::lambda::_1 - boost::lambda::_2),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] -= v.val[i];
 
   return *this;
 }
@@ -423,8 +700,15 @@ void Vector<Number>::add (const Number v)
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] += v;
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (val,
+			 val+vec_size,
+			 val,
+			 (boost::lambda::_1 + v),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] += v;
 }
 
 
@@ -434,8 +718,16 @@ void Vector<Number>::add (const Vector<Number>& v)
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] += v.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (val,
+			 val+vec_size,
+			 v.val,
+			 val,
+			 (boost::lambda::_1 + boost::lambda::_2),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] += v.val[i];
 }
 
 
@@ -443,17 +735,26 @@ template <typename Number>
 void Vector<Number>::add (const Number a, const Vector<Number>& v,
 			  const Number b, const Vector<Number>& w)
 {
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
   Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] += a*v.val[i] + b*w.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (val,
+			 val+vec_size,
+			 v.val,
+			 w.val,
+			 val,
+			 (boost::lambda::_1 + a*boost::lambda::_2 + b*boost::lambda::_3),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] += a * v.val[i] + b * w.val[i];
 }
 
 
@@ -461,14 +762,22 @@ template <typename Number>
 void Vector<Number>::sadd (const Number x,
 			   const Vector<Number>& v)
 {
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] = x*val[i] + v.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (val,
+			 val+vec_size,
+			 v.val,
+			 val,
+			 (x*boost::lambda::_1 + boost::lambda::_2),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] = x * val[i] + v.val[i];
 }
 
 
@@ -478,19 +787,28 @@ void Vector<Number>::sadd (const Number x, const Number a,
 			   const Vector<Number>& v, const Number b,
                            const Vector<Number>& w)
 {
-  Assert (numbers::is_finite(x), 
+  Assert (numbers::is_finite(x),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
   Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] = x*val[i] + a*v.val[i] + b*w.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+  parallel::transform (val,
+		       val+vec_size,
+		       v.val,
+		       w.val,
+		       val,
+		       (x*boost::lambda::_1 + a*boost::lambda::_2 + b*boost::lambda::_3),
+		       internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] = x*val[i] + a * v.val[i] + b * w.val[i];
 }
 
 
@@ -500,22 +818,28 @@ void Vector<Number>::sadd (const Number x, const Number a,
 			   const Vector<Number>& w, const Number c,
                            const Vector<Number>& y)
 {
-  Assert (numbers::is_finite(x), 
-          ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(a), 
-          ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
-          ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(c), 
-          ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
+  sadd (x, a, v, b, w);
+  add (c, y);
+}
 
+
+
+template <typename Number>
+void Vector<Number>::scale (const Vector<Number> &s)
+{
   Assert (vec_size!=0, ExcEmptyObject());
-  Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
-  Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
-  Assert (vec_size == y.vec_size, ExcDimensionMismatch(vec_size, y.vec_size));
+  Assert (vec_size == s.vec_size, ExcDimensionMismatch(vec_size, s.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] = x*val[i] + a*v.val[i] + b*w.val[i] +c*y.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (val,
+			 val+vec_size,
+			 s.val,
+			 val,
+			 (boost::lambda::_1*boost::lambda::_2),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] *= s.val[i];
 }
 
 
@@ -526,18 +850,42 @@ void Vector<Number>::scale (const Vector<Number2> &s)
 {
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == s.vec_size, ExcDimensionMismatch(vec_size, s.vec_size));
-  
+
   for (unsigned int i=0; i<vec_size; ++i)
     val[i] *= s.val[i];
 }
 
 
+
+template <typename Number>
+void Vector<Number>::equ (const Number a,
+			  const Vector<Number>& u)
+{
+  Assert (numbers::is_finite(a),
+	  ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
+
+  Assert (vec_size!=0, ExcEmptyObject());
+  Assert (vec_size == u.vec_size, ExcDimensionMismatch(vec_size, u.vec_size));
+
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (u.val,
+			 u.val+u.vec_size,
+			 val,
+			 (a*boost::lambda::_1),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] = a * u.val[i];
+}
+
+
+
 template <typename Number>
 template <typename Number2>
 void Vector<Number>::equ (const Number a,
 			  const Vector<Number2>& u)
 {
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
 	  ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
@@ -554,21 +902,30 @@ void Vector<Number>::equ (const Number a,
 }
 
 
+
 template <typename Number>
 void Vector<Number>::equ (const Number a, const Vector<Number>& u,
 			  const Number b, const Vector<Number>& v)
 {
-  Assert (numbers::is_finite(a), 
+  Assert (numbers::is_finite(a),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-  Assert (numbers::is_finite(b), 
+  Assert (numbers::is_finite(b),
           ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == u.vec_size, ExcDimensionMismatch(vec_size, u.vec_size));
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] = a * u.val[i] + b * v.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (u.val,
+			 u.val+u.vec_size,
+			 v.val,
+			 val,
+			 (a*boost::lambda::_1 + b*boost::lambda::_2),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] = a * u.val[i] + b * v.val[i];
 }
 
 
@@ -582,8 +939,17 @@ void Vector<Number>::equ (const Number a, const Vector<Number>& u,
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
   Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] = a * u.val[i] + b * v.val[i] + c * w.val[i];
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (u.val,
+			 u.val+u.vec_size,
+			 v.val,
+			 w.val,
+			 val,
+			 (a*boost::lambda::_1 + b*boost::lambda::_2 + c*boost::lambda::_3),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] = a * u.val[i] + b * v.val[i] + c * w.val[i];
 }
 
 
@@ -599,37 +965,16 @@ void Vector<Number>::ratio (const Vector<Number> &a,
 				   // we overwrite them anyway
   reinit (a.size(), true);
 
-  for (unsigned int i=0; i<vec_size; ++i)
-    val[i] = a.val[i] / b.val[i];
-}
-
-
-
-template <typename Number>
-Vector<Number> &
-Vector<Number>::operator = (const Vector<Number>& v)
-{
-  if (v.vec_size != vec_size)
-    reinit (v.vec_size, true);
-  if (vec_size!=0)
-    std::copy (v.begin(), v.end(), begin());
-  
-  return *this;
-}
-
-
-
-template <typename Number>
-template <typename Number2>
-Vector<Number> &
-Vector<Number>::operator = (const Vector<Number2>& v)
-{
-  if (v.size() != vec_size)
-    reinit (v.size(), true);
-  if (vec_size!=0)
-    std::copy (v.begin(), v.end(), begin());
-  
-  return *this;
+  if (vec_size>internal::Vector::minimum_parallel_grain_size)
+    parallel::transform (a.val,
+			 a.val+a.vec_size,
+			 b.val,
+			 val,
+			 (boost::lambda::_1 / boost::lambda::_2),
+			 internal::Vector::minimum_parallel_grain_size);
+  else if (vec_size > 0)
+    for (unsigned int i=0; i<vec_size; ++i)
+      val[i] = a.val[i]/b.val[i];
 }
 
 
@@ -645,7 +990,7 @@ Vector<Number>::operator = (const BlockVector<Number>& v)
   for (unsigned int b=0; b<v.n_blocks(); ++b)
     for (unsigned int i=0; i<v.block(b).size(); ++i, ++this_index)
       val[this_index] = v.block(b)(i);
-  
+
   return *this;
 }
 
@@ -666,8 +1011,8 @@ Vector<Number>::operator = (const PETScWrappers::Vector &v)
       PetscScalar *start_ptr;
       int ierr = VecGetArray (static_cast<const Vec&>(v), &start_ptr);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
-      std::copy (start_ptr, start_ptr+vec_size, begin());
+
+      internal::copy (start_ptr, start_ptr+vec_size, begin());
 
                                        // restore the representation of the
                                        // vector
@@ -726,7 +1071,7 @@ Vector<Number>::operator = (const TrilinosWrappers::Vector &v)
       TrilinosScalar **start_ptr;
       int ierr = v.trilinos_vector().ExtractView (&start_ptr);
       AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-      
+
       std::copy (start_ptr[0], start_ptr[0]+vec_size, begin());
     }
 
@@ -781,7 +1126,7 @@ void Vector<Number>::print (std::ostream      &out,
 
   std::ios::fmtflags old_flags = out.flags();
   unsigned int old_precision = out.precision (precision);
-  
+
   out.precision (precision);
   if (scientific)
     out.setf (std::ios::scientific, std::ios::floatfield);
@@ -795,7 +1140,7 @@ void Vector<Number>::print (std::ostream      &out,
     for (unsigned int i=0; i<size(); ++i)
       out << val[i] << std::endl;
   out << std::endl;
-  
+
   AssertThrow (out, ExcIO());
                                    // reset output format
   out.flags (old_flags);
@@ -817,19 +1162,19 @@ void Vector<Number>::block_write (std::ostream &out) const
 				   // environment
   const unsigned int sz = size();
   char buf[16];
-  
+
   std::sprintf(buf, "%d", sz);
   std::strcat(buf, "\n[");
-  
+
   out.write(buf, std::strlen(buf));
   out.write (reinterpret_cast<const char*>(begin()),
 	     reinterpret_cast<const char*>(end())
 	     - reinterpret_cast<const char*>(begin()));
-  
+
 				   // out << ']';
   const char outro = ']';
   out.write (&outro, 1);
-  
+
   AssertThrow (out, ExcIO());
 }
 
@@ -843,24 +1188,24 @@ void Vector<Number>::block_read (std::istream &in)
   unsigned int sz;
 
   char buf[16];
-  
+
 
   in.getline(buf,16,'\n');
   sz=std::atoi(buf);
-  
+
 				   // fast initialization, since the
 				   // data elements are overwritten anyway
-  reinit (sz, true);     
+  reinit (sz, true);
 
   char c;
 				   //  in >> c;
   in.read (&c, 1);
   AssertThrow (c=='[', ExcIO());
-  
+
   in.read (reinterpret_cast<char*>(begin()),
 	   reinterpret_cast<const char*>(end())
 	   - reinterpret_cast<const char*>(begin()));
-  
+
 				   //  in >> c;
   in.read (&c, 1);
   AssertThrow (c==']', ExcIO());
