@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_parallel_sparse_matrix.cc 18532 2009-03-31 13:12:22Z young $
+//    $Id: petsc_parallel_sparse_matrix.cc 21162 2010-06-07 20:47:13Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2008 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,21 +13,21 @@
 
 
 #include <lac/petsc_parallel_sparse_matrix.h>
-#include <lac/petsc_vector.h>
-
-#include <lac/sparsity_pattern.h>
-#include <lac/compressed_sparsity_pattern.h>
-#include <lac/compressed_simple_sparsity_pattern.h>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <lac/petsc_vector.h>
+#  include <lac/sparsity_pattern.h>
+#  include <lac/compressed_sparsity_pattern.h>
+#  include <lac/compressed_simple_sparsity_pattern.h>
+
 DEAL_II_NAMESPACE_OPEN
 
 namespace PETScWrappers
 {
   namespace MPI
   {
-    
+
     SparseMatrix::SparseMatrix ()
     {
                                        // just like for vectors: since we
@@ -89,17 +89,17 @@ namespace PETScWrappers
                  local_columns_per_process, this_process,
                  preset_nonzero_locations);
     }
-    
+
 
 
     SparseMatrix &
-    SparseMatrix::operator = (const double d)
+    SparseMatrix::operator = (const value_type d)
     {
       MatrixBase::operator = (d);
       return *this;
     }
 
-    
+
 
     void
     SparseMatrix::reinit (const MPI_Comm    &communicator,
@@ -115,8 +115,8 @@ namespace PETScWrappers
                                        // get rid of old matrix and generate a
                                        // new one
       const int ierr = MatDestroy (matrix);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));    
-    
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
+
       do_reinit (m, n, local_rows, local_columns,
                  n_nonzero_per_row, is_symmetric);
     }
@@ -133,14 +133,14 @@ namespace PETScWrappers
                           const bool         is_symmetric)
     {
       this->communicator = communicator;
-      
+
                                        // get rid of old matrix and generate a
                                        // new one
       const int ierr = MatDestroy (matrix);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));    
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       do_reinit (m, n, local_rows, local_columns, row_lengths, is_symmetric);
-    }  
+    }
 
 
 
@@ -155,18 +155,18 @@ namespace PETScWrappers
             const bool                       preset_nonzero_locations)
     {
       this->communicator = communicator;
-      
+
                                        // get rid of old matrix and generate a
                                        // new one
       const int ierr = MatDestroy (matrix);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));    
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       do_reinit (sparsity_pattern, local_rows_per_process,
                  local_columns_per_process, this_process,
                  preset_nonzero_locations);
     }
 
-    
+
 
     void
     SparseMatrix::do_reinit (const unsigned int m,
@@ -177,7 +177,7 @@ namespace PETScWrappers
                              const bool         is_symmetric)
     {
       Assert (local_rows <= m, ExcLocalRowsTooLarge (local_rows, m));
-      
+
                                        // use the call sequence indicating only
                                        // a maximal number of elements per row
                                        // for all rows globally
@@ -192,7 +192,7 @@ namespace PETScWrappers
                                        // set symmetric flag, if so requested
       if (is_symmetric == true)
         {
-#if (PETSC_VERSION_MAJOR <= 2) 
+#if DEAL_II_PETSC_VERSION_LT(3,0,0)
           const int ierr
             = MatSetOption (matrix, MAT_SYMMETRIC);
 #else
@@ -228,7 +228,7 @@ namespace PETScWrappers
       for (unsigned int i=0; i<row_lengths.size(); ++i)
 	Assert(row_lengths[i]<=local_columns,
 	       ExcIndexRange(row_lengths[i], 1, local_columns+1));
-    
+
                                        // use the call sequence indicating a
                                        // maximal number of elements for each
                                        // row individually. annoyingly, we
@@ -237,10 +237,16 @@ namespace PETScWrappers
                                        // signed integers. so we have to
                                        // convert, unless we want to play dirty
                                        // tricks with conversions of pointers
+#ifdef PETSC_USE_64BIT_INDICES
+      const std::vector<PetscInt> int_row_lengths (row_lengths.begin(),
+						   row_lengths.end());
+#else
       const std::vector<signed int> int_row_lengths (row_lengths.begin(),
                                                      row_lengths.end());
+#endif
 
 //TODO: There must be a significantly better way to provide information about the off-diagonal blocks of the matrix. this way, petsc keeps allocating tiny chunks of memory, and gets completely hung up over this
+
       const int ierr
         = MatCreateMPIAIJ(communicator,
 			  local_rows, local_columns,
@@ -252,7 +258,7 @@ namespace PETScWrappers
                                        // set symmetric flag, if so requested
       if (is_symmetric == true)
         {
-#if (PETSC_VERSION_MAJOR <= 2) 
+#if DEAL_II_PETSC_VERSION_LT(3,0,0)
           const int ierr
             = MatSetOption (matrix, MAT_SYMMETRIC);
 #else
@@ -261,7 +267,7 @@ namespace PETScWrappers
 #endif
 
           AssertThrow (ierr == 0, ExcPETScError(ierr));
-        }    
+        }
     }
 
 
@@ -296,18 +302,30 @@ namespace PETScWrappers
         }
       const unsigned int
         local_row_end = local_row_start + local_rows_per_process[this_process];
+
+#if DEAL_II_PETSC_VERSION_LT(2,3,3)
+				       //old version to create the matrix, we
+				       //can skip calculating the row length
+				       //at least starting from 2.3.3 (tested,
+				       //see below)
+
       const unsigned int
         local_col_end = local_col_start + local_columns_per_process[this_process];
 
                                        // then count the elements in- and
                                        // out-of-window for the rows we own
-      std::vector<int> row_lengths_in_window (local_row_end - local_row_start);
-      std::vector<int> row_lengths_out_of_window (local_row_end - local_row_start);
+#ifdef PETSC_USE_64BIT_INDICES
+      std::vector<PetscInt>
+#else
+      std::vector<int>
+#endif
+	row_lengths_in_window (local_row_end - local_row_start),
+	row_lengths_out_of_window (local_row_end - local_row_start);
       for (unsigned int row = local_row_start; row<local_row_end; ++row)
         for (unsigned int c=0; c<sparsity_pattern.row_length(row); ++c)
           {
             const unsigned int column = sparsity_pattern.column_number(row,c);
-            
+
             if ((column >= local_col_start) &&
                 (column < local_col_end))
               ++row_lengths_in_window[row-local_row_start];
@@ -334,6 +352,27 @@ namespace PETScWrappers
                           &matrix);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
+#else //PETSC_VERSION>=2.3.3
+				       // new version to create the matrix. We
+				       // do not set row length but set the
+				       // correct SparsityPattern later.
+      int ierr;
+      
+      ierr = MatCreate(communicator,&matrix);
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
+      
+      ierr = MatSetSizes(matrix,
+			 local_rows_per_process[this_process],
+			 local_columns_per_process[this_process],
+			 sparsity_pattern.n_rows(),
+			 sparsity_pattern.n_cols());
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
+      
+      ierr = MatSetType(matrix,MATMPIAIJ);
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
+#endif
+      
+      
                                        // next preset the exact given matrix
                                        // entries with zeros, if the user
                                        // requested so. this doesn't avoid any
@@ -357,11 +396,14 @@ namespace PETScWrappers
                                        // available. if we don't have this, we
                                        // have to somehow clumsily work around
                                        // the whole thing:
-#if (PETSC_VERSION_MAJOR <= 2) && \
-    ((PETSC_VERSION_MINOR < 2) ||  \
-     ((PETSC_VERSION_MINOR == 2) && (PETSC_VERSION_SUBMINOR == 0)))      
+#if DEAL_II_PETSC_VERSION_LT(2,2,1)
 
-          std::vector<int> row_entries;
+#ifdef PETSC_USE_64BIT_INDICES
+          std::vector<PetscInt>
+#else
+          std::vector<int>
+#endif
+            row_entries;
           std::vector<PetscScalar> row_values;
           for (unsigned int i=0; i<sparsity_pattern.n_rows(); ++i)
             {
@@ -369,7 +411,7 @@ namespace PETScWrappers
               row_values.resize (sparsity_pattern.row_length(i), 0.0);
               for (unsigned int j=0; j<sparsity_pattern.row_length(i); ++j)
                 row_entries[j] = sparsity_pattern.column_number (i,j);
-              
+
               const int int_row = i;
               MatSetValues (matrix, 1, &int_row,
                             sparsity_pattern.row_length(i), &row_entries[0],
@@ -377,7 +419,7 @@ namespace PETScWrappers
             }
 
           compress ();
-          
+
 #else
 
                                            // first set up the column number
@@ -386,10 +428,13 @@ namespace PETScWrappers
                                            // dummy entry at the end to make
                                            // sure petsc doesn't read past the
                                            // end
-          std::vector<int> rowstart_in_window (local_row_end -
-                                               local_row_start + 1,
-                                               0);
-          std::vector<int> colnums_in_window;
+#ifdef PETSC_USE_64BIT_INDICES
+          std::vector<PetscInt>
+#else
+          std::vector<int>
+#endif
+	    rowstart_in_window (local_row_end - local_row_start + 1, 0),
+	    colnums_in_window;
           {
             unsigned int n_cols = 0;
             for (unsigned int i=local_row_start; i<local_row_end; ++i)
@@ -401,18 +446,29 @@ namespace PETScWrappers
               }
             colnums_in_window.resize (n_cols+1, -1);
           }
-          
+
                                            // now copy over the information
                                            // from the sparsity pattern.
           {
-            unsigned int index=0;
+#ifdef PETSC_USE_64BIT_INDICES
+	    PetscInt
+#else
+	    int
+#endif
+	      * ptr = & colnums_in_window[0];
+	      
             for (unsigned int i=local_row_start; i<local_row_end; ++i)
-              for (unsigned int j=0; j<sparsity_pattern.row_length(i);
-                   ++j, ++index)
-              colnums_in_window[index] = sparsity_pattern.column_number(i,j);
-            Assert (index == colnums_in_window.size()-1, ExcInternalError());
+	      {
+		typename SparsityType::row_iterator
+		  row_start = sparsity_pattern.row_begin(i),
+		  row_end = sparsity_pattern.row_end(i);
+
+		std::copy(row_start, row_end, ptr);
+		ptr += row_end - row_start;
           }
-          
+	  }
+
+
                                            // then call the petsc function
                                            // that summarily allocates these
                                            // entries:
@@ -421,6 +477,10 @@ namespace PETScWrappers
                                         &colnums_in_window[0],
                                         0);
 
+#if DEAL_II_PETSC_VERSION_LT(2,3,3)
+					   // this is only needed for old
+					   // PETSc versions:
+	  
                                            // for some reason, it does not
                                            // seem to be possible to force
                                            // actual allocation of actual
@@ -441,39 +501,55 @@ namespace PETScWrappers
             const std::vector<PetscScalar>
               values (sparsity_pattern.max_entries_per_row(),
                       1.);
-            
+
             for (unsigned int i=local_row_start; i<local_row_end; ++i)
               {
-                const int petsc_i = i;
+#ifdef PETSC_USE_64BIT_INDICES
+		PetscInt
+#else
+		int
+#endif
+		  petsc_i = i;
                 MatSetValues (matrix, 1, &petsc_i,
                               sparsity_pattern.row_length(i),
                               &colnums_in_window[rowstart_in_window[i-local_row_start]],
                               &values[0], INSERT_VALUES);
               }
           }
-          
+
           compress ();
 
                                            // set the dummy entries set above
                                            // back to zero
           *this = 0;
+#endif // version <=2.3.3
           compress ();
-          
+
 #endif
 
-				           // Now we won't insert any
-				           // further entries, so PETSc can
-				           // internally optimize some data
-				           // structures.
-#if (PETSC_VERSION_MAJOR <= 2) 
-          const int ierr =
-	    MatSetOption (matrix, MAT_NO_NEW_NONZERO_LOCATIONS);
-#else
-          const int ierr =
-	    MatSetOption (matrix, MAT_NEW_NONZERO_LOCATIONS, PETSC_FALSE);
-#endif
-	  
+				           // Tell PETSc that we are not
+				           // planning on adding new entries
+				           // to the matrix. Generate errors
+				           // in debugmode.
+#if DEAL_II_PETSC_VERSION_LT(3,0,0)
+          int ierr;
+#ifdef DEBUG
+	  ierr = MatSetOption (matrix, MAT_NEW_NONZERO_LOCATION_ERR);
 	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#else
+	  ierr = MatSetOption (matrix, MAT_NO_NEW_NONZERO_LOCATIONS);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#endif
+#else
+          int ierr;
+#ifdef DEBUG
+	  ierr = MatSetOption (matrix, MAT_NEW_NONZERO_LOCATION_ERR, PETSC_TRUE);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#else
+	  ierr = MatSetOption (matrix, MAT_NEW_NONZERO_LOCATIONS, PETSC_FALSE);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+#endif
+#endif
         }
     }
 
