@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: petsc_vector_base.cc 18534 2009-03-31 13:13:45Z young $
+//    $Id: petsc_vector_base.cc 21162 2010-06-07 20:47:13Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2004, 2005, 2006, 2007, 2008 by the deal.II authors
+//    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -13,13 +13,13 @@
 
 
 #include <lac/petsc_vector_base.h>
-#include <lac/petsc_vector.h>
-#include <lac/petsc_parallel_vector.h>
-
-#include <cmath>
 
 #ifdef DEAL_II_USE_PETSC
 
+#  include <lac/petsc_vector.h>
+#  include <lac/petsc_parallel_vector.h>
+#  include <cmath>
+
 DEAL_II_NAMESPACE_OPEN
 
 namespace PETScWrappers
@@ -30,77 +30,162 @@ namespace PETScWrappers
     {
       Assert (index < vector.size(),
               ExcIndexRange (index, 0, vector.size()));
-              
-                                       // this is clumsy: there is no simple
-                                       // way in PETSc to read an element from
-                                       // a vector, i.e. there is no function
-                                       // VecGetValue or so. The only way is
-                                       // to obtain a pointer to a contiguous
+
+				       // old versions of PETSc appear to be
+                                       // missing the function VecGetValues(),
+                                       // so the workaround consists of
+                                       // obtaining a pointer to a contiguous
                                        // representation of the vector and
-                                       // read from it. Subsequently, the
-                                       // vector representation has to be
-                                       // restored. In addition, we can only
-                                       // get access to the local part of the
-                                       // vector, so we have to guard against
-                                       // that
+                                       // read from it.  In addition, we can
+                                       // only get access to the local part of
+                                       // the vector, so we have to guard
+                                       // against that
       if (dynamic_cast<const PETScWrappers::Vector *>(&vector) != 0)
         {
+ #if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
           PetscScalar *ptr;
           int ierr
-            = VecGetArray (static_cast<const Vec &>(vector), &ptr);
+            = VecGetArray (vector.vector, &ptr);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
-          
+
           const PetscScalar value = *(ptr+index);
-          
-          ierr = VecRestoreArray (static_cast<const Vec &>(vector), &ptr);
+
+          ierr = VecRestoreArray (vector.vector, &ptr);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
-          
           return value;
+#else
+	  PetscInt idx = index;
+	  PetscScalar value;
+	  int ierr = VecGetValues(vector.vector, 1, &idx, &value);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+	  return value;
+#endif
         }
       else if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&vector) != 0)
         {
+	  int ierr;
+
+	  if (vector.ghosted)
+	    {
+#ifdef PETSC_USE_64BIT_INDICES
+	      PetscInt
+#else
+		int
+#endif
+		begin, end;
+	      ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
+	      AssertThrow (ierr == 0, ExcPETScError(ierr)); 
+
+	      Vec l;
+	      ierr = VecGhostGetLocalForm(vector.vector, &l);
+	      AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+	      PetscInt lsize;
+	      ierr = VecGetSize(l, &lsize);
+	      AssertThrow (ierr == 0, ExcPETScError(ierr));
+ 
+	      PetscScalar *ptr;
+	      ierr = VecGetArray(l, &ptr);
+	      AssertThrow (ierr == 0, ExcPETScError(ierr));
+ 
+	      PetscScalar value;
+
+	      if ( index>=static_cast<unsigned int>(begin)
+		  && index<static_cast<unsigned int>(end) )
+		{
+						   //local entry
+		  value=*(ptr+index-begin);
+		}
+	      else
+		{
+						   //ghost entry
+		  unsigned ghostidx
+		    = vector.ghost_indices.index_within_set(index);
+		  
+		  Assert(ghostidx+end-begin<(unsigned int)lsize, ExcInternalError());
+		  value=*(ptr+ghostidx+end-begin);
+		  
+		  
+		}
+	      
+	      
+	      ierr = VecRestoreArray(l, &ptr);
+	      AssertThrow (ierr == 0, ExcPETScError(ierr));
+	      
+	      ierr = VecGhostRestoreLocalForm(vector.vector, &l);
+	      AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+	      return value;
+	    }
+
+
                                            // first verify that the requested
                                            // element is actually locally
                                            // available
-          int ierr;
-          int begin, end;
-          ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
-                                       &begin, &end);
+          
+#ifdef PETSC_USE_64BIT_INDICES
+	  PetscInt
+#else
+	  int
+#endif
+	    begin, end;
+          ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
 
+
+	  
           AssertThrow ((index >= static_cast<unsigned int>(begin)) &&
                        (index < static_cast<unsigned int>(end)),
                        ExcAccessToNonlocalElement (index, begin, end-1));
 
+					   // old version which only work with
+					   // VecGetArray()...
+#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+	  
                                            // then access it
           PetscScalar *ptr;
-          ierr = VecGetArray (static_cast<const Vec &>(vector), &ptr);
+          ierr = VecGetArray (vector.vector, &ptr);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
-          
+
           const PetscScalar value = *(ptr+index-begin);
-          
-          ierr = VecRestoreArray (static_cast<const Vec &>(vector), &ptr);
+
+          ierr = VecRestoreArray (vector.vector, &ptr);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
-          
+
           return value;
+	  
+#else
+					   //new version with VecGetValues()
+	  PetscInt idx = index;
+	  PetscScalar value;
+	  ierr = VecGetValues(vector.vector, 1, &idx, &value);
+	  AssertThrow (ierr == 0, ExcPETScError(ierr));
+	  
+	  return value;
+#endif
+	  
         }
       else
                                          // what? what other kind of vector
                                          // exists there?
         Assert (false, ExcInternalError());
       return -1e20;
-    }  
+    }
   }
-  
+
   VectorBase::VectorBase ()
                   :
+		  ghosted(false),
                   last_action (LastAction::none)
   {}
-  
 
-  
+
+
   VectorBase::VectorBase (const VectorBase &v)
                   :
+		  Subscriptor (),
+		  ghosted(v.ghosted),
+		  ghost_indices(v.ghost_indices),
                   last_action (LastAction::none)
   {
     int ierr = VecDuplicate (v.vector, &vector);
@@ -110,7 +195,7 @@ namespace PETScWrappers
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
-  
+
   VectorBase::~VectorBase ()
   {
     const int ierr = VecDestroy (vector);
@@ -123,7 +208,7 @@ namespace PETScWrappers
   VectorBase::operator = (const PetscScalar s)
   {
 
-    Assert (numbers::is_finite(s), 
+    Assert (numbers::is_finite(s),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
                                      // flush previously cached elements. this
@@ -131,13 +216,13 @@ namespace PETScWrappers
                                      // 2.2.1, at least for parallel vectors
                                      // (see test bits/petsc_65)
     compress ();
-    
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecSet (&s, vector);
 #else
     const int ierr = VecSet (vector, s);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -150,9 +235,9 @@ namespace PETScWrappers
   {
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
-    
+
     PetscTruth flag;
-    
+
     const int ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -160,7 +245,7 @@ namespace PETScWrappers
   }
 
 
-  
+
   bool
   VectorBase::operator != (const VectorBase &v) const
   {
@@ -168,7 +253,7 @@ namespace PETScWrappers
             ExcDimensionMismatch(size(), v.size()));
 
     PetscTruth flag;
-    
+
     const int ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -176,11 +261,16 @@ namespace PETScWrappers
   }
 
 
-  
+
   unsigned int
   VectorBase::size () const
   {
-    int sz;
+#ifdef PETSC_USE_64BIT_INDICES
+    PetscInt
+#else
+    int
+#endif
+      sz;
     const int ierr = VecGetSize (vector, &sz);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -192,7 +282,12 @@ namespace PETScWrappers
   unsigned int
   VectorBase::local_size () const
   {
-    int sz;
+#ifdef PETSC_USE_64BIT_INDICES
+    PetscInt
+#else
+    int
+#endif
+      sz;
     const int ierr = VecGetLocalSize (vector, &sz);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -204,7 +299,12 @@ namespace PETScWrappers
   std::pair<unsigned int, unsigned int>
   VectorBase::local_range () const
   {
-    int begin, end;
+#ifdef PETSC_USE_64BIT_INDICES
+    PetscInt
+#else
+    int
+#endif
+      begin, end;
     const int ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
 					   &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -218,7 +318,9 @@ namespace PETScWrappers
   VectorBase::set (const std::vector<unsigned int> &indices,
                    const std::vector<PetscScalar>  &values)
   {
-    do_set_add_operation(indices, values, false);
+    Assert (indices.size() == values.size(),
+            ExcMessage ("Function called with arguments of different sizes"));
+    do_set_add_operation(indices.size(), &indices[0], &values[0], false);
   }
 
 
@@ -227,9 +329,32 @@ namespace PETScWrappers
   VectorBase::add (const std::vector<unsigned int> &indices,
                    const std::vector<PetscScalar>  &values)
   {
-    do_set_add_operation(indices, values, true);
+    Assert (indices.size() == values.size(),
+            ExcMessage ("Function called with arguments of different sizes"));
+    do_set_add_operation(indices.size(), &indices[0], &values[0], true);
   }
-  
+
+
+
+  void
+  VectorBase::add (const std::vector<unsigned int>    &indices,
+                   const ::dealii::Vector<PetscScalar>&values)
+  {
+    Assert (indices.size() == values.size(),
+            ExcMessage ("Function called with arguments of different sizes"));
+    do_set_add_operation(indices.size(), &indices[0], values.begin(), true);
+  }
+
+
+
+  void
+  VectorBase::add (const unsigned int  n_elements,
+		   const unsigned int *indices,
+		   const PetscScalar  *values)
+  {
+    do_set_add_operation(n_elements, indices, values, true);
+  }
+
 
 
   PetscScalar
@@ -245,7 +370,7 @@ namespace PETScWrappers
 
     return result;
   }
-  
+
 
 
   void
@@ -263,7 +388,7 @@ namespace PETScWrappers
   VectorBase::real_type
   VectorBase::norm_sqr () const
   {
-    const PetscScalar d = l2_norm();
+    const real_type d = l2_norm();
     return d*d;
   }
 
@@ -300,48 +425,48 @@ namespace PETScWrappers
                                        // add up remaining elements
       while (ptr != start_ptr+size())
         sum0 += *ptr++;
-  
-      mean = (sum0+sum1+sum2+sum3)/size();
+
+      mean = (sum0+sum1+sum2+sum3)/static_cast<PetscScalar>(size());
     }
-    
+
                                      // restore the representation of the
                                      // vector
     ierr = VecRestoreArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
     return mean;
   }
-  
+
 
 
   VectorBase::real_type
   VectorBase::l1_norm () const
   {
-    PetscScalar d;
+    real_type d;
 
     const int ierr = VecNorm (vector, NORM_1, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
   }
-  
+
 
 
   VectorBase::real_type
   VectorBase::l2_norm () const
   {
-    PetscScalar d;
+    real_type d;
 
     const int ierr = VecNorm (vector, NORM_2, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
   }
-  
+
 
 
   VectorBase::real_type
-  VectorBase::lp_norm (const PetscScalar p) const
+  VectorBase::lp_norm (const real_type p) const
   {
                                      // get a representation of the vector and
                                      // loop over all the elements
@@ -349,12 +474,12 @@ namespace PETScWrappers
     int ierr = VecGetArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-    PetscScalar norm = 0;
+    real_type norm = 0;
     {
-      PetscScalar sum0 = 0,
-                  sum1 = 0,
-                  sum2 = 0,
-                  sum3 = 0;
+      real_type sum0 = 0,
+		sum1 = 0,
+		sum2 = 0,
+		sum3 = 0;
 
                                        // use modern processors better by
                                        // allowing pipelined commands to be
@@ -363,24 +488,24 @@ namespace PETScWrappers
       const PetscScalar * eptr = ptr + (size()/4)*4;
       while (ptr!=eptr)
         {
-          sum0 += std::pow(std::fabs(*ptr++), p);
-          sum1 += std::pow(std::fabs(*ptr++), p);
-          sum2 += std::pow(std::fabs(*ptr++), p);
-          sum3 += std::pow(std::fabs(*ptr++), p);
-        };
+          sum0 += std::pow(numbers::NumberTraits<value_type>::abs(*ptr++), p);
+          sum1 += std::pow(numbers::NumberTraits<value_type>::abs(*ptr++), p);
+          sum2 += std::pow(numbers::NumberTraits<value_type>::abs(*ptr++), p);
+          sum3 += std::pow(numbers::NumberTraits<value_type>::abs(*ptr++), p);
+        }
                                        // add up remaining elements
       while (ptr != start_ptr+size())
-        sum0 += std::pow(std::fabs(*ptr++), p);
-  
+        sum0 += std::pow(numbers::NumberTraits<value_type>::abs(*ptr++), p);
+
       norm = std::pow(sum0+sum1+sum2+sum3,
-                      static_cast<PetscScalar>(1./p));
+                      1./p);
     }
-    
+
                                      // restore the representation of the
                                      // vector
     ierr = VecRestoreArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
     return norm;
   }
 
@@ -389,7 +514,7 @@ namespace PETScWrappers
   VectorBase::real_type
   VectorBase::linfty_norm () const
   {
-    PetscScalar d;
+    real_type d;
 
     const int ierr = VecNorm (vector, NORM_INFINITY, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -402,7 +527,7 @@ namespace PETScWrappers
   VectorBase::real_type
   VectorBase::normalize () const
   {
-    PetscScalar d;
+    real_type d;
 
     const int ierr = VecNormalize (vector, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -411,12 +536,24 @@ namespace PETScWrappers
   }
 
 
+  VectorBase::real_type
+  VectorBase::min ()  const
+  {
+    PetscInt  p;
+    real_type d;
+
+    const int ierr = VecMin (vector, &p, &d);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+    return d;
+  }
+
 
   VectorBase::real_type
   VectorBase::max ()  const
   {
-    PetscInt    p;
-    PetscScalar d;
+    PetscInt  p;
+    real_type d;
 
     const int ierr = VecMax (vector, &p, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -425,9 +562,8 @@ namespace PETScWrappers
   }
 
 
-
   VectorBase &
-  VectorBase::abs () 
+  VectorBase::abs ()
   {
     const int ierr = VecAbs (vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -438,7 +574,7 @@ namespace PETScWrappers
 
 
   VectorBase &
-  VectorBase::conjugate () 
+  VectorBase::conjugate ()
   {
     const int ierr = VecConjugate (vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -449,7 +585,7 @@ namespace PETScWrappers
 
 
   VectorBase &
-  VectorBase::mult () 
+  VectorBase::mult ()
   {
     const int ierr = VecPointwiseMult (vector,vector,vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -473,23 +609,44 @@ namespace PETScWrappers
     bool flag = true;
     while (ptr != eptr)
       {
-        if (*ptr != 0)
+        if (*ptr != value_type())
           {
             flag = false;
             break;
           }
         ++ptr;
       }
-    
+
                                      // restore the representation of the
                                      // vector
     ierr = VecRestoreArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
     return flag;
   }
 
 
+  namespace internal
+  {
+    template <typename T>
+    bool is_non_negative (const T &t)
+    {
+      return t >= 0;
+    }
+
+
+
+    template <typename T>
+    bool is_non_negative (const std::complex<T> &)
+    {
+      Assert (false,
+	      ExcMessage ("You can't ask a complex value "
+			  "whether it is non-negative."))
+      return true;
+    }
+  }
+
+
 
   bool
   VectorBase::is_non_negative () const
@@ -505,37 +662,37 @@ namespace PETScWrappers
     bool flag = true;
     while (ptr != eptr)
       {
-        if (*ptr < 0.0)
+        if (! internal::is_non_negative(*ptr))
           {
             flag = false;
             break;
           }
         ++ptr;
       }
-    
+
                                      // restore the representation of the
                                      // vector
     ierr = VecRestoreArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-      
+
     return flag;
   }
-  
+
 
 
   VectorBase &
   VectorBase::operator *= (const PetscScalar a)
   {
 
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecScale (&a, vector);
 #else
     const int ierr = VecScale (vector, a);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -547,15 +704,15 @@ namespace PETScWrappers
   VectorBase::operator /= (const PetscScalar a)
   {
 
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
     const PetscScalar factor = 1./a;
-    
-    Assert (numbers::is_finite(factor), 
+
+    Assert (numbers::is_finite(factor),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecScale (&factor, vector);
 #else
     const int ierr = VecScale (vector, factor);
@@ -571,13 +728,13 @@ namespace PETScWrappers
   VectorBase &
   VectorBase::operator += (const VectorBase &v)
   {
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
-    const PetscScalar one = 1.0;    
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
+    const PetscScalar one = 1.0;
     const int ierr = VecAXPY (&one, v, vector);
 #else
     const int ierr = VecAXPY (vector, 1, v);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -588,13 +745,13 @@ namespace PETScWrappers
   VectorBase &
   VectorBase::operator -= (const VectorBase &v)
   {
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
-    const PetscScalar minus_one = -1.0;    
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
+    const PetscScalar minus_one = -1.0;
     const int ierr = VecAXPY (&minus_one, v, vector);
 #else
     const int ierr = VecAXPY (vector, -1, v);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -606,18 +763,18 @@ namespace PETScWrappers
   VectorBase::add (const PetscScalar s)
   {
 
-    Assert (numbers::is_finite(s), 
+    Assert (numbers::is_finite(s),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecShift (&s, vector);
 #else
     const int ierr = VecShift (vector, s);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
-  
+
 
 
   void
@@ -625,7 +782,7 @@ namespace PETScWrappers
   {
     *this += v;
   }
-  
+
 
 
   void
@@ -633,18 +790,18 @@ namespace PETScWrappers
                    const VectorBase     &v)
   {
 
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecAXPY (&a, v, vector);
 #else
     const int ierr = VecAXPY (vector, a, v);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
-  
+
 
 
   void
@@ -654,15 +811,15 @@ namespace PETScWrappers
                    const VectorBase &w)
   {
 
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(b), 
+    Assert (numbers::is_finite(b),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
     const PetscScalar weights[2] = {a,b};
     Vec               addends[2] = {v.vector, w.vector};
-    
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecMAXPY (2, weights, vector, addends);
 #else
     const int ierr = VecMAXPY (vector, 2, weights, addends);
@@ -670,7 +827,7 @@ namespace PETScWrappers
 
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
-  
+
 
 
   void
@@ -678,18 +835,18 @@ namespace PETScWrappers
                     const VectorBase &v)
   {
 
-    Assert (numbers::is_finite(s), 
+    Assert (numbers::is_finite(s),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecAYPX (&s, v, vector);
 #else
     const int ierr = VecAYPX (vector, s, v);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
-  
+
 
 
   void
@@ -698,9 +855,9 @@ namespace PETScWrappers
                     const VectorBase     &v)
   {
 
-    Assert (numbers::is_finite(s), 
+    Assert (numbers::is_finite(s),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
                                      // there is nothing like a AXPAY
@@ -709,7 +866,7 @@ namespace PETScWrappers
     *this *= s;
     add (a,v);
   }
-  
+
 
 
   void
@@ -719,27 +876,27 @@ namespace PETScWrappers
                     const PetscScalar b,
                     const VectorBase     &w)
   {
- 
-    Assert (numbers::is_finite(s), 
+
+    Assert (numbers::is_finite(s),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(b), 
+    Assert (numbers::is_finite(b),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
                                      // there is no operation like MAXPAY, so
                                      // do it in two steps
     *this *= s;
-    
+
     const PetscScalar weights[2] = {a,b};
     Vec               addends[2] = {v.vector,w.vector};
-    
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecMAXPY (2, weights, vector, addends);
 #else
     const int ierr = VecMAXPY (vector, 2, weights, addends);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -754,14 +911,14 @@ namespace PETScWrappers
                     const PetscScalar c,
                     const VectorBase     &x)
   {
- 
-    Assert (numbers::is_finite(s), 
+
+    Assert (numbers::is_finite(s),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(b), 
+    Assert (numbers::is_finite(b),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(c), 
+    Assert (numbers::is_finite(c),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
                                      // there is no operation like MAXPAY, so
@@ -770,13 +927,13 @@ namespace PETScWrappers
 
     const PetscScalar weights[3] = {a,b,c};
     Vec               addends[3] = {v.vector, w.vector, x.vector};
-    
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecMAXPY (3, weights, vector, addends);
 #else
     const int ierr = VecMAXPY (vector, 3, weights, addends);
 #endif
-    
+
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -797,7 +954,7 @@ namespace PETScWrappers
                    const VectorBase &v)
   {
 
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
     Assert (size() == v.size(),
@@ -811,7 +968,7 @@ namespace PETScWrappers
 
     *this *= a;
   }
-  
+
 
 
   void
@@ -821,9 +978,9 @@ namespace PETScWrappers
                    const VectorBase &w)
   {
 
-    Assert (numbers::is_finite(a), 
+    Assert (numbers::is_finite(a),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-    Assert (numbers::is_finite(b), 
+    Assert (numbers::is_finite(b),
 	    ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
     Assert (size() == v.size(),
@@ -844,7 +1001,7 @@ namespace PETScWrappers
   VectorBase::ratio (const VectorBase &a,
                      const VectorBase &b)
   {
-#if (PETSC_VERSION_MAJOR <= 2) && (PETSC_VERSION_MINOR < 3)
+#if DEAL_II_PETSC_VERSION_LT(2,3,0)
     const int ierr = VecPointwiseDivide (a, b, vector);
 #else
     const int ierr = VecPointwiseDivide (vector, a, b);
@@ -877,29 +1034,29 @@ namespace PETScWrappers
 
     if (across)
       for (unsigned int i=0; i<size(); ++i)
-        out << static_cast<double>(val[i]) << ' ';
+        out << val[i] << ' ';
     else
       for (unsigned int i=0; i<size(); ++i)
-        out << static_cast<double>(val[i]) << std::endl;
+        out << val[i] << std::endl;
     out << std::endl;
 
                                      // restore the representation of the
                                      // vector
     ierr = VecRestoreArray (vector, &val);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
-    
+
     AssertThrow (out, ExcIO());
   }
 
 
-  
+
   void
   VectorBase::swap (VectorBase &v)
   {
     const int ierr = VecSwap (vector, v.vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
-  
+
 
 
   VectorBase::operator const Vec & () const
@@ -915,17 +1072,15 @@ namespace PETScWrappers
     AssertThrow(false, ExcNotImplemented() );
     return 0;
   }
-  
+
 
 
   void
-  VectorBase::do_set_add_operation (const std::vector<unsigned int> &indices,
-				    const std::vector<PetscScalar>  &values,
-				    const bool add_values)
+  VectorBase::do_set_add_operation (const unsigned int  n_elements,
+				    const unsigned int *indices,
+				    const PetscScalar  *values,
+				    const bool          add_values)
   {
-    Assert (indices.size() == values.size(),
-            ExcMessage ("Function called with arguments of different sizes"));
-
     if (last_action != VectorBase::LastAction::insert)
       {
         int ierr;
@@ -942,30 +1097,47 @@ namespace PETScWrappers
 				     // collective operation, so we
 				     // can skip the call if necessary
 				     // (unlike the above calls)
-    if (indices.size() != 0)
+    if (n_elements != 0)
       {
-#if (PETSC_VERSION_MAJOR <= 2) && \
-    ((PETSC_VERSION_MINOR < 2) ||  \
-     ((PETSC_VERSION_MINOR == 2) && (PETSC_VERSION_SUBMINOR == 0)))
-	const std::vector<int> petsc_indices (indices.begin(),
-					      indices.end());
+#ifdef PETSC_USE_64BIT_INDICES
+	std::vector<PetscInt> petsc_ind (n_elements);
+	for (unsigned int i=0; i<n_elements; ++i)
+	  petsc_ind[i] = indices[i];
+	const PetscInt *petsc_indices = &petsc_ind[0];
 #else
-	const std::vector<PetscInt> petsc_indices (indices.begin(),
-						   indices.end());
+	const int * petsc_indices = (const int*)indices;
 #endif
 
-	InsertMode mode = INSERT_VALUES;
-	if (add_values)
-	  mode = ADD_VALUES;
+	InsertMode mode = ADD_VALUES;
+	if (!add_values)
+	  mode = INSERT_VALUES;
 	const int ierr
-	  = VecSetValues (vector, indices.size(),
-			  &petsc_indices[0], &values[0],
+	  = VecSetValues (vector, n_elements, petsc_indices, values,
 			  mode);
 	AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
     last_action = LastAction::insert;
   }
+
+
+  void 
+  VectorBase::update_ghost_values() const
+  {
+				     // generate an error for not ghosted
+				     // vectors
+    if (!ghosted)
+	throw ExcInternalError();
+
+    int ierr;
+
+    ierr = VecGhostUpdateBegin(vector, INSERT_VALUES, SCATTER_FORWARD);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    ierr = VecGhostUpdateEnd(vector, INSERT_VALUES, SCATTER_FORWARD);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+}
+
+
   
 }
 
