@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: sparse_direct.cc 17243 2008-10-16 18:28:08Z allmaras $
+//    $Id: sparse_direct.cc 21126 2010-05-13 07:07:11Z young $
 //    Version: $Name$
 //
-//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 by the deal.II authors
+//    Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009 by the deal.II authors
 //
 //    This file is subject to QPL and may not be  distributed
 //    without copyright and license information. Please refer
@@ -23,7 +23,6 @@
 #include <list>
 #include <typeinfo>
 
-
 // this is a weird hack: on newer linux systems, some system headers
 // include /usr/include/linux/compiler.h which explicitly checks which
 // gcc is in use. in that file is also a comment that explains that
@@ -392,7 +391,7 @@ SparseDirectMA27::SparseDirectMA27 (const double LIW_factor_1,
 		LA_increase_factor (LA_increase_factor),
 		initialize_called (false),
 		factorize_called (false),
-		sparsity_pattern (0)
+		sparsity_pattern (0, typeid(*this).name())
 {}
 
 
@@ -455,6 +454,7 @@ SparseDirectMA27::initialize (const SparsityPattern &sp)
                                        // stdin
       pipe(detached_mode_data->server_client_pipe);
       pipe(detached_mode_data->client_server_pipe);      
+
                                        // fflush(NULL) is said to be a
                                        // good idea before fork()
       std::fflush(NULL);
@@ -463,7 +463,7 @@ SparseDirectMA27::initialize (const SparsityPattern &sp)
                                        // process
       detached_mode_data->child_pid = fork();
       if (detached_mode_data->child_pid == 0)
-                                         // child process starts here
+                                           // child process starts here
         {
                                            // copy read end of input
                                            // pipe to stdin, and
@@ -1073,7 +1073,7 @@ SparseDirectMA47::SparseDirectMA47 (const double LIW_factor_1,
 		LA_increase_factor (LA_increase_factor),
 		initialize_called (false),
 		factorize_called (false),
-		matrix (0)
+		matrix (0, typeid(*this).name())
 {}
 
 
@@ -1878,6 +1878,117 @@ SparseDirectUMFPACK::Tvmult_add (
   Assert(false, ExcNotImplemented());
 }
 
+#ifdef DEAL_II_USE_MUMPS
+SparseDirectMUMPS::SparseDirectMUMPS () 
+  :
+  initialize_called (false)
+{}
+
+SparseDirectMUMPS::~SparseDirectMUMPS () 
+{}
+
+template <class Matrix>
+void SparseDirectMUMPS::initialize (const SparseMatrix<double>& matrix, 
+				    const Vector<double>      & vector) 
+{
+                         // Check we haven't been here before:
+  Assert (initialize_called == false, ExcInitializeAlreadyCalled());
+
+                         // Initialize MUMPS instance:
+  id.job = -1;
+  id.par =  1;
+  id.sym =  0;
+
+                         // Use MPI_COMM_WORLD as communicator
+  id.comm_fortran = -987654;
+  dmumps_c (&id);
+   
+                         // Hand over matrix and right-hand side
+  if (Utilities::System::get_this_mpi_process (MPI_COMM_WORLD) == 0) 
+    {
+
+                         // Objects denoting a MUMPS data structure:
+                         //
+                         // Set number of unknowns
+      n   = vector.size ();
+
+                         // number of nonzero elements in matrix
+      nz  = matrix.n_actually_nonzero_elements ();
+
+                         // representation of the matrix 
+      a   = new double[nz];
+
+                         // matrix indices pointing to the row and
+                         // column dimensions respectively of the
+                         // matrix representation above (a): ie. a[k]
+                         // is the matrix element (irn[k], jcn[k])
+      irn = new unsigned int[nz];
+      jcn = new unsigned int[nz];
+      
+      unsigned int index = 0;
+      
+      for (SparseMatrix<double>::const_iterator ptr = matrix.begin (); 
+	   ptr != matrix.end (); ++ptr)
+	if (std::abs (ptr->value ()) > 0.0) 
+	  {
+	    a[index]   = ptr->value ();
+	    irn[index] = ptr->row () + 1;
+	    jcn[index] = ptr->column () + 1;
+	    ++index;
+	  }
+      
+      rhs = new double[n];
+      
+      for (unsigned int i = 0; i < n; ++i)
+	rhs[i] = vector (i);
+      
+      id.n   = n;
+      id.nz  = nz;
+      id.irn = irn;
+      id.jcn = jcn;
+      id.a   = a;
+      id.rhs = rhs;
+    }
+  
+                         // No outputs   
+  id.icntl[0] = -1;
+  id.icntl[1] = -1;
+  id.icntl[2] = -1;
+  id.icntl[3] =  0;
+
+                        // Exit by setting this flag:
+  initialize_called = true;
+}
+
+void SparseDirectMUMPS::solve (Vector<double>& vector) 
+{
+                        // Check that the solver has been initialized
+                        // by the routine above:
+  Assert (initialize_called == true, ExcNotInitialized());
+
+                        // and that the matrix has at least one
+                        // nonzero element:
+  Assert (nz != 0, ExcNotInitialized());
+
+                        // Start solver
+  id.job = 6;
+  dmumps_c (&id);
+  id.job = -2;
+  dmumps_c (&id);
+
+                        // Copy solution into the given vector
+  if (Utilities::System::get_this_mpi_process (MPI_COMM_WORLD) == 0) 
+    {
+      for (unsigned int i=0; i<n; ++i)
+	vector(i) = rhs[i];
+      
+      delete[] a;
+      delete[] irn;
+      delete[] jcn;
+      delete[] rhs;
+    }
+}
+#endif // DEAL_II_USE_MUMPS
 
 // explicit instantiations for SparseMatrixMA27 
 template
@@ -1911,4 +2022,13 @@ InstantiateUMFPACK(SparseMatrix<float>);
 InstantiateUMFPACK(BlockSparseMatrix<double>);
 InstantiateUMFPACK(BlockSparseMatrix<float>);
 
+// explicit instantiations for SparseDirectMUMPS
+#ifdef DEAL_II_USE_MUMPS
+template <class Matrix>
+void SparseDirectMUMPS::initialize (const SparseMatrix<double>& matrix, 
+				    const Vector<double>      & vector);
+
+void SparseDirectMUMPS::solve (Vector<double>& vector); 
+#endif
+
 DEAL_II_NAMESPACE_CLOSE
