@@ -1,8 +1,8 @@
 //---------------------------------------------------------------------------
-//    $Id: trilinos_precondition.cc 18724 2009-04-23 23:06:37Z bangerth $
+//    $Id: trilinos_precondition.cc 21411 2010-06-30 11:53:57Z bangerth $
 //    Version: $Name$
 //
-//    Copyright (C) 2008 by the deal.II authors
+//    Copyright (C) 2008, 2009, 2010 by the deal.II authors
 //
 //    This file is subject to QPL and may not be distributed
 //    without copyright and license information. Please refer
@@ -12,22 +12,20 @@
 //---------------------------------------------------------------------------
 
 
-#include <lac/vector.h>
-#include <lac/sparse_matrix.h>
 #include <lac/trilinos_precondition.h>
-#include <lac/trilinos_vector_base.h>
-#include <lac/trilinos_sparse_matrix.h>
 
 #ifdef DEAL_II_USE_TRILINOS
 
-#include <Ifpack.h>
-#include <Ifpack_Chebyshev.h>
-#include <Teuchos_ParameterList.hpp>
-#include <Epetra_Vector.h>
-#include <Epetra_MultiVector.h>
-#include <Epetra_Vector.h>
-#include <ml_include.h>
-#include <ml_MultiLevelPreconditioner.h>
+#  include <lac/vector.h>
+#  include <lac/sparse_matrix.h>
+#  include <lac/trilinos_sparse_matrix.h>
+
+#  include <Ifpack.h>
+#  include <Ifpack_Chebyshev.h>
+#  include <Teuchos_ParameterList.hpp>
+#  include <Epetra_MultiVector.h>
+#  include <ml_include.h>
+#  include <ml_MultiLevelPreconditioner.h>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -51,68 +49,23 @@ namespace TrilinosWrappers
 #ifdef DEAL_II_COMPILER_SUPPORTS_MPI
                     communicator (base.communicator),
 #endif
-		    map (new Epetra_Map(*base.map))
+		    vector_distributor (new Epetra_Map(*base.vector_distributor))
   {}
 
 
 
   PreconditionBase::~PreconditionBase()
+  {}
+
+
+
+  void PreconditionBase::clear ()
   {
-    preconditioner.release();
-  }
-
-
-
-  void
-  PreconditionBase::vmult (VectorBase       &dst,
-			   const VectorBase &src) const
-  {
-    Assert (dst.vector_partitioner().SameAs(preconditioner->OperatorRangeMap()),
-	    ExcNonMatchingMaps("dst"));
-    Assert (src.vector_partitioner().SameAs(preconditioner->OperatorDomainMap()),
-	    ExcNonMatchingMaps("src"));
-    
-    const int ierr = preconditioner->ApplyInverse (src.trilinos_vector(), 
-						   dst.trilinos_vector());
-    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-  }
-
-
-				        // For the implementation of
-				        // the <code>vmult</code>
-				        // function with deal.II data
-				        // structures we note that
-				        // invoking a call of the
-				        // Trilinos preconditioner
-				        // requires us to use Epetra
-				        // vectors as well. We do this
-				        // by providing a view, i.e.,
-				        // feed Trilinos with a
-				        // pointer to the data, so we
-				        // avoid copying the content
-				        // of the vectors during the
-				        // iteration (this function is
-				        // only useful when used in
-				        // serial anyway). In the
-				        // declaration of the right
-				        // hand side, we need to cast
-				        // the source vector (that is
-				        // <code>const</code> in all
-				        // deal.II calls) to
-				        // non-constant value, as this
-				        // is the way Trilinos wants
-				        // to have them.
-  void PreconditionBase::vmult (dealii::Vector<double>       &dst,
-				const dealii::Vector<double> &src) const
-  {
-    
-    Epetra_Vector LHS (View, preconditioner->OperatorDomainMap(),
-		       dst.begin());
-    Epetra_Vector RHS (View, preconditioner->OperatorRangeMap(),
-		       const_cast<double*>(src.begin()));
-  
-    const int ierr = preconditioner->ApplyInverse (RHS, LHS);
-    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+    preconditioner.reset();
+#ifdef DEAL_II_COMPILER_SUPPORTS_MPI
+    communicator = MPI_COMM_SELF;
+#endif
+    vector_distributor.reset();
   }
 
 
@@ -133,16 +86,17 @@ namespace TrilinosWrappers
   PreconditionJacobi::initialize (const SparseMatrix   &matrix,
 				  const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+				// release memory before reallocation
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("point relaxation",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   0));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("point relaxation", 
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    0));
-
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
@@ -150,9 +104,9 @@ namespace TrilinosWrappers
     parameter_list.set ("relaxation: sweeps", 1);
     parameter_list.set ("relaxation: type", "Jacobi");
     parameter_list.set ("relaxation: damping factor", additional_data.omega);
-    parameter_list.set ("relaxation: min diagonal value", 
+    parameter_list.set ("relaxation: min diagonal value",
 			additional_data.min_diagonal);
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -161,8 +115,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -185,16 +137,16 @@ namespace TrilinosWrappers
   PreconditionSSOR::initialize (const SparseMatrix   &matrix,
 				const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("point relaxation",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   additional_data.overlap));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("point relaxation",
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    additional_data.overlap));
-
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
@@ -202,10 +154,10 @@ namespace TrilinosWrappers
     parameter_list.set ("relaxation: sweeps", 1);
     parameter_list.set ("relaxation: type", "symmetric Gauss-Seidel");
     parameter_list.set ("relaxation: damping factor", additional_data.omega);
-    parameter_list.set ("relaxation: min diagonal value", 
+    parameter_list.set ("relaxation: min diagonal value",
 			additional_data.min_diagonal);
     parameter_list.set ("schwarz: combine mode", "Add");
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -214,8 +166,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -238,16 +188,16 @@ namespace TrilinosWrappers
   PreconditionSOR::initialize (const SparseMatrix   &matrix,
 			       const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("point relaxation",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   additional_data.overlap));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("point relaxation",
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    additional_data.overlap));
-
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
@@ -255,10 +205,10 @@ namespace TrilinosWrappers
     parameter_list.set ("relaxation: sweeps", 1);
     parameter_list.set ("relaxation: type", "Gauss-Seidel");
     parameter_list.set ("relaxation: damping factor", additional_data.omega);
-    parameter_list.set ("relaxation: min diagonal value", 
+    parameter_list.set ("relaxation: min diagonal value",
 			additional_data.min_diagonal);
     parameter_list.set ("schwarz: combine mode", "Add");
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -267,8 +217,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -293,25 +241,25 @@ namespace TrilinosWrappers
   PreconditionIC::initialize (const SparseMatrix   &matrix,
 			      const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("IC",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   additional_data.overlap));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("IC", 
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    additional_data.overlap));
-
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
     Teuchos::ParameterList parameter_list;
-    parameter_list.set ("fact: level-of-fill",(int)additional_data.ic_fill); 
-    parameter_list.set ("fact: absolute threshold",additional_data.ic_atol); 
-    parameter_list.set ("fact: relative threshold",additional_data.ic_rtol); 
+    parameter_list.set ("fact: level-of-fill",(int)additional_data.ic_fill);
+    parameter_list.set ("fact: absolute threshold",additional_data.ic_atol);
+    parameter_list.set ("fact: relative threshold",additional_data.ic_rtol);
     parameter_list.set ("schwarz: combine mode", "Add");
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -320,8 +268,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -346,25 +292,25 @@ namespace TrilinosWrappers
   PreconditionILU::initialize (const SparseMatrix   &matrix,
 			       const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("ILU",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   additional_data.overlap));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("ILU", 
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    additional_data.overlap));
-
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
     Teuchos::ParameterList parameter_list;
-    parameter_list.set ("fact: level-of-fill",(int)additional_data.ilu_fill); 
-    parameter_list.set ("fact: absolute threshold",additional_data.ilu_atol); 
-    parameter_list.set ("fact: relative threshold",additional_data.ilu_rtol); 
+    parameter_list.set ("fact: level-of-fill",(int)additional_data.ilu_fill);
+    parameter_list.set ("fact: absolute threshold",additional_data.ilu_atol);
+    parameter_list.set ("fact: relative threshold",additional_data.ilu_rtol);
     parameter_list.set ("schwarz: combine mode", "Add");
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -373,8 +319,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -401,26 +345,26 @@ namespace TrilinosWrappers
   PreconditionILUT::initialize (const SparseMatrix   &matrix,
 				const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("ILUT",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   additional_data.overlap));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("ILUT", 
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    additional_data.overlap));
-
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
     Teuchos::ParameterList parameter_list;
-    parameter_list.set ("fact: drop value",additional_data.ilut_drop);  
-    parameter_list.set ("fact: level-of-fill",(int)additional_data.ilut_fill);  
-    parameter_list.set ("fact: absolute threshold",additional_data.ilut_atol); 
-    parameter_list.set ("fact: relative threshold",additional_data.ilut_rtol); 
+    parameter_list.set ("fact: drop value",additional_data.ilut_drop);
+    parameter_list.set ("fact: level-of-fill",(int)additional_data.ilut_fill);
+    parameter_list.set ("fact: absolute threshold",additional_data.ilut_atol);
+    parameter_list.set ("fact: relative threshold",additional_data.ilut_rtol);
     parameter_list.set ("schwarz: combine mode", "Add");
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -429,8 +373,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -449,21 +391,22 @@ namespace TrilinosWrappers
   PreconditionBlockwiseDirect::initialize (const SparseMatrix   &matrix,
 					   const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+			  ("Amesos",
+			   const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()),
+			   additional_data.overlap));
 
-    ifpack = Teuchos::rcp (Ifpack().Create 
-			   ("Amesos", 
-			    const_cast<Epetra_CrsMatrix*>(&matrix.trilinos_matrix()), 
-			    additional_data.overlap));
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Preconditioner * ifpack = static_cast<Ifpack_Preconditioner*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
     Teuchos::ParameterList parameter_list;
     parameter_list.set ("schwarz: combine mode", "Add");
-    
+
     ierr = ifpack->SetParameters(parameter_list);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
@@ -472,8 +415,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -502,12 +443,13 @@ namespace TrilinosWrappers
   PreconditionChebyshev::initialize (const SparseMatrix   &matrix,
 				     const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    ifpack.release();
+    preconditioner.reset ();
+    preconditioner.reset (new Ifpack_Chebyshev (&matrix.trilinos_matrix()));
 
-    ifpack = Teuchos::rcp (new Ifpack_Chebyshev (&matrix.trilinos_matrix()));
-    Assert (&*ifpack != 0, ExcMessage ("Trilinos could not create this "
-				       "preconditioner"));
+    Ifpack_Chebyshev * ifpack = static_cast<Ifpack_Chebyshev*>
+      (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+				     "preconditioner"));
 
     int ierr;
 
@@ -533,8 +475,6 @@ namespace TrilinosWrappers
 
     ierr = ifpack->Compute();
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-    preconditioner = Teuchos::rcp (&*ifpack, false);
   }
 
 
@@ -544,6 +484,8 @@ namespace TrilinosWrappers
   PreconditionAMG::AdditionalData::
   AdditionalData (const bool                             elliptic,
 		  const bool                             higher_order_elements,
+		  const unsigned int                     n_cycles,
+		  const bool                             w_cycle,
 		  const double                           aggregation_threshold,
 		  const std::vector<std::vector<bool> > &constant_modes,
 		  const unsigned int                     smoother_sweeps,
@@ -552,6 +494,8 @@ namespace TrilinosWrappers
                   :
                   elliptic (elliptic),
 		  higher_order_elements (higher_order_elements),
+		  n_cycles (n_cycles),
+		  w_cycle (w_cycle),
 		  aggregation_threshold (aggregation_threshold),
 		  constant_modes (constant_modes),
 		  smoother_sweeps (smoother_sweeps),
@@ -565,16 +509,11 @@ namespace TrilinosWrappers
   PreconditionAMG:: initialize (const SparseMatrix   &matrix,
 				const AdditionalData &additional_data)
   {
-    preconditioner.release();
-    multilevel_operator.release();
-
     const unsigned int n_rows = matrix.m();
-    const unsigned int constant_modes_dimension = 
-      additional_data.constant_modes.size();
 
 				        // Build the AMG preconditioner.
     Teuchos::ParameterList parameter_list;
-  
+
     if (additional_data.elliptic == true)
       {
 	ML_Epetra::SetDefaults("SA",parameter_list);
@@ -595,7 +534,7 @@ namespace TrilinosWrappers
 				   // now we also just use Uncoupled,
 				   // but we should be aware that maybe
 				   // MIS might be needed
-				   // 
+				   //
 				   // TODO: Maybe there are any
 				   // other/better options?
 	if (additional_data.higher_order_elements)
@@ -612,38 +551,49 @@ namespace TrilinosWrappers
 	parameter_list.set("aggregation: type", "Uncoupled");
 	parameter_list.set("aggregation: block scaling", true);
       }
-  
-    parameter_list.set("smoother: sweeps", 
+
+    parameter_list.set("smoother: sweeps",
 		       static_cast<int>(additional_data.smoother_sweeps));
-    parameter_list.set("smoother: ifpack overlap", 
+    parameter_list.set("cycle applications",
+		       static_cast<int>(additional_data.n_cycles));
+    if (additional_data.w_cycle == true)
+      parameter_list.set("prec type", "MGW");
+    else
+      parameter_list.set("prec type", "MGV");
+
+    parameter_list.set("smoother: Chebyshev alpha",10.);
+    parameter_list.set("smoother: ifpack overlap",
 		       static_cast<int>(additional_data.smoother_overlap));
-    parameter_list.set("aggregation: threshold", 
+    parameter_list.set("aggregation: threshold",
 		       additional_data.aggregation_threshold);
-    
+
     if (additional_data.output_details)
       parameter_list.set("ML output", 10);
     else
-      parameter_list.set("ML output", 0);  
+      parameter_list.set("ML output", 0);
 
     const Epetra_Map & domain_map = matrix.domain_partitioner();
-    
-    Epetra_MultiVector distributed_constant_modes (domain_map, 
+
+    const unsigned int constant_modes_dimension =
+      additional_data.constant_modes.size();
+    Epetra_MultiVector distributed_constant_modes (domain_map,
 						   constant_modes_dimension);
-  
+
     if (constant_modes_dimension > 1)
       {
-	Assert (n_rows == additional_data.constant_modes[0].size(),
-		ExcDimensionMismatch(n_rows,
-				     additional_data.constant_modes[0].size()));
-	Assert (n_rows == 
+	const bool constant_modes_are_global =
+	  additional_data.constant_modes[0].size() == n_rows;
+	const unsigned int n_relevant_rows =
+	  constant_modes_are_global ? n_rows : additional_data.constant_modes[0].size();
+	const unsigned int my_size = domain_map.NumMyElements();
+	if (constant_modes_are_global == false)
+	  Assert (n_relevant_rows == my_size,
+		  ExcDimensionMismatch(n_relevant_rows, my_size));
+	Assert (n_rows ==
 		static_cast<unsigned int>(distributed_constant_modes.GlobalLength()),
 		ExcDimensionMismatch(n_rows,
 				     distributed_constant_modes.GlobalLength()));
 
-	const unsigned int my_size = domain_map.NumMyElements();
-	Assert (my_size == static_cast<unsigned int>(domain_map.MaxLID()+1),
-		ExcDimensionMismatch (my_size, domain_map.MaxLID()+1));
-	
 				        // Reshape null space as a
 				        // contiguous vector of
 				        // doubles so that Trilinos
@@ -651,60 +601,111 @@ namespace TrilinosWrappers
 	for (unsigned int d=0; d<constant_modes_dimension; ++d)
 	  for (unsigned int row=0; row<my_size; ++row)
 	    {
-	      int global_row_id = domain_map.GID(row);
-	      distributed_constant_modes.ReplaceMyValue(row, d, 
-		static_cast<double>(additional_data.constant_modes[d][global_row_id]));
+	      int global_row_id = constant_modes_are_global ? domain_map.GID(row) : row;
+	      distributed_constant_modes[d][row] = 
+		additional_data.constant_modes[d][global_row_id];
 	    }
-  
+
 	parameter_list.set("null space: type", "pre-computed");
-	parameter_list.set("null space: dimension", 
+	parameter_list.set("null space: dimension",
 			   distributed_constant_modes.NumVectors());
-	parameter_list.set("null space: vectors", 
+	parameter_list.set("null space: vectors",
 			   distributed_constant_modes.Values());
       }
 
-    multilevel_operator = Teuchos::rcp (new ML_Epetra::MultiLevelPreconditioner(
-					matrix.trilinos_matrix(), parameter_list,
-					true));
+    initialize (matrix, parameter_list);
 
     if (additional_data.output_details)
-      multilevel_operator->PrintUnused(0);
-
-    preconditioner = Teuchos::rcp (&*multilevel_operator, false);
+      {
+	ML_Epetra::MultiLevelPreconditioner * multilevel_operator = 
+	  dynamic_cast<ML_Epetra::MultiLevelPreconditioner*> (preconditioner.get());
+	Assert (multilevel_operator != 0, 
+		ExcMessage ("Preconditioner setup failed."));
+	multilevel_operator->PrintUnused(0);
+      }
   }
 
 
 
   void
-  PreconditionAMG::
-  initialize (const ::dealii::SparseMatrix<double> &deal_ii_sparse_matrix,
-	      const AdditionalData                 &additional_data,
-	      const double                          drop_tolerance)
+  PreconditionAMG::initialize (const SparseMatrix           &matrix,
+			       const Teuchos::ParameterList &ml_parameters)
   {
+    preconditioner.reset ();
+    preconditioner.reset (new ML_Epetra::MultiLevelPreconditioner
+			  (matrix.trilinos_matrix(), ml_parameters));
+  }
+
+
+
+  template <typename number>
+  void
+  PreconditionAMG::
+  initialize (const ::dealii::SparseMatrix<number> &deal_ii_sparse_matrix,
+	      const AdditionalData                 &additional_data,
+	      const double                          drop_tolerance,
+	      const ::dealii::SparsityPattern      *use_this_sparsity)
+  {
+    preconditioner.reset();
     const unsigned int n_rows = deal_ii_sparse_matrix.m();
-  
+
 				        // Init Epetra Matrix using an
 				        // equidistributed map; avoid
 				        // storing the nonzero
 				        // elements.
-    map.reset (new Epetra_Map(n_rows, 0, communicator));
+    vector_distributor.reset (new Epetra_Map(n_rows, 0, communicator));
 
-    Matrix.reset();
-    Matrix = std_cxx1x::shared_ptr<SparseMatrix> (new SparseMatrix());
+    if (trilinos_matrix.get() == 0)
+      trilinos_matrix.reset (new SparseMatrix());
 
-    Matrix->reinit (*map, deal_ii_sparse_matrix, drop_tolerance);
-    Matrix->compress();
+    trilinos_matrix->reinit (*vector_distributor, *vector_distributor,
+			     deal_ii_sparse_matrix, drop_tolerance, true,
+			     use_this_sparsity);
 
-    initialize (*Matrix, additional_data);
+    initialize (*trilinos_matrix, additional_data);
   }
-  
-  
-  void PreconditionAMG::
-  reinit ()
+
+
+
+  void PreconditionAMG::reinit ()
   {
+    ML_Epetra::MultiLevelPreconditioner * multilevel_operator = 
+      dynamic_cast<ML_Epetra::MultiLevelPreconditioner*> (preconditioner.get());
     multilevel_operator->ReComputePreconditioner();
   }
 
+
+
+  void PreconditionAMG::clear ()
+  {
+    PreconditionBase::clear();
+    trilinos_matrix.reset();
+  }
+
+
+
+  unsigned int PreconditionAMG::memory_consumption() const
+  {
+    unsigned int memory = sizeof(this);
+
+				// todo: find a way to read out ML's data
+				// sizes
+    if (trilinos_matrix.get() != 0)
+      memory += trilinos_matrix->memory_consumption();
+    return memory;
+  }
+
+
+
+
+				// explicit instantiations
+  template void PreconditionAMG::initialize (const ::dealii::SparseMatrix<double> &,
+					     const AdditionalData &, const double,
+					     const ::dealii::SparsityPattern*);
+  template void PreconditionAMG::initialize (const ::dealii::SparseMatrix<float> &,
+					     const AdditionalData &, const double,
+					     const ::dealii::SparsityPattern*);
+
 }
 
 DEAL_II_NAMESPACE_CLOSE
